Package: motmot
Title: Models of Trait Macroevolution on Trees
Version: 2.1.4
Depends: R (>= 2.10.0), ape (>= 3.0-7)
Description: Functions for fitting models of trait evolution
        on phylogenies for continuous traits. The majority of functions are
        described in Thomas and Freckleton (2012)
        <doi:10.1111/j.2041-210X.2011.00132.x> and allow tests of variation 
        in the rates of trait evolution.
Authors@R: c(
    person(given = "Mark", family = "Puttick",
            role = c("aut", "cph"),
            comment = c(ORCID = "0000-0002-1011-3442"),
            email = "marknputtick@gmail.com"),
    person(given = "Gavin", family = "Thomas",
            role = c("aut", "cph")),
    person(given = "Rob", family = "Freckleton",
            role = c("aut", "cph")),
    person(given = "Magnus", family = "Clarke",
            role = "ctb"),
    person(given = "Travis", family = "Ingram",
            role = "ctb"),
    person(given = "David", family = "Orme",
            role = "ctb"),
    person(given = "Emmanuel", family = "Paradis",
            role = "ctb"),
    person("Martin R.", "Smith", role = c("ctb", "cre"),
            email = "martin.smith@durham.ac.uk",
            comment = c(ORCID = "0000-0001-5660-1727"))
  )
License: GPL (>= 2)
URL: https://puttickbiology.wordpress.com/motmot/
BugReports: https://github.com/ms609/motmot/issues
LinkingTo: Rcpp
Imports: Rcpp, coda, ks, mvtnorm, caper, methods
Suggests: knitr, rmarkdown
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-04-15 15:51:36 UTC; pjjg18
Author: Mark Puttick [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-1011-3442>),
  Gavin Thomas [aut, cph],
  Rob Freckleton [aut, cph],
  Magnus Clarke [ctb],
  Travis Ingram [ctb],
  David Orme [ctb],
  Emmanuel Paradis [ctb],
  Martin R. Smith [ctb, cre] (ORCID:
    <https://orcid.org/0000-0001-5660-1727>)
Maintainer: Martin R. Smith <martin.smith@durham.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-16 07:00:20 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-04-17 09:22:20 UTC; unix
Archs: motmot.so.dSYM

Package: motmot
Title: Models of Trait Macroevolution on Trees
Version: 2.1.4
Depends: R (>= 2.10.0), ape (>= 3.0-7)
Description: Functions for fitting models of trait evolution
        on phylogenies for continuous traits. The majority of functions are
        described in Thomas and Freckleton (2012)
        <doi:10.1111/j.2041-210X.2011.00132.x> and allow tests of variation 
        in the rates of trait evolution.
Authors@R: c(
    person(given = "Mark", family = "Puttick",
            role = c("aut", "cph"),
            comment = c(ORCID = "0000-0002-1011-3442"),
            email = "marknputtick@gmail.com"),
    person(given = "Gavin", family = "Thomas",
            role = c("aut", "cph")),
    person(given = "Rob", family = "Freckleton",
            role = c("aut", "cph")),
    person(given = "Magnus", family = "Clarke",
            role = "ctb"),
    person(given = "Travis", family = "Ingram",
            role = "ctb"),
    person(given = "David", family = "Orme",
            role = "ctb"),
    person(given = "Emmanuel", family = "Paradis",
            role = "ctb"),
    person("Martin R.", "Smith", role = c("ctb", "cre"),
            email = "martin.smith@durham.ac.uk",
            comment = c(ORCID = "0000-0001-5660-1727"))
  )
License: GPL (>= 2)
URL: https://puttickbiology.wordpress.com/motmot/
BugReports: https://github.com/ms609/motmot/issues
LinkingTo: Rcpp
Imports: Rcpp, coda, ks, mvtnorm, caper, methods
Suggests: knitr, rmarkdown
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-04-15 15:51:36 UTC; pjjg18
Author: Mark Puttick [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-1011-3442>),
  Gavin Thomas [aut, cph],
  Rob Freckleton [aut, cph],
  Magnus Clarke [ctb],
  Travis Ingram [ctb],
  David Orme [ctb],
  Emmanuel Paradis [ctb],
  Martin R. Smith [ctb, cre] (ORCID:
    <https://orcid.org/0000-0001-5660-1727>)
Maintainer: Martin R. Smith <martin.smith@durham.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-16 07:00:20 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 07:10:50 UTC; unix
Archs: motmot.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(plot,timeSlice.ML)
S3method(plot,traitMedusa.model)
S3method(summary,traitMedusa)
export(ML.RatePhylo)
export(RatePhylo.allCI)
export(addFossilToPhy)
export(as.rateData)
export(as.rateMatrix)
export(calcCutOff)
export(chr.disp.lrt)
export(chr.disp.param)
export(chr.disp.sim)
export(contemporaryPhy)
export(dropTipPartial)
export(fairProportions)
export(likRatePhylo)
export(likTraitPhylo)
export(mcmc.plot)
export(node.descendents)
export(nodeTimes)
export(optim.likRatePhylo)
export(phyloCovar)
export(phyloMean)
export(phyloVar)
export(pic.pgls)
export(plot.timeSlice.ML)
export(plot.traitMedusa.model)
export(sampleShid)
export(sortTraitData)
export(summary.traitMedusa)
export(traitData.plot)
export(transformPhylo)
export(transformPhylo.MCMC)
export(transformPhylo.ML)
export(transformPhylo.ll)
export(transformPhylo.sim)
export(transformRateMatrix)
import(ape)
import(caper)
import(coda)
import(grDevices)
import(graphics)
import(ks)
import(methods)
import(mvtnorm)
import(stats)
import(utils)
importFrom(Rcpp,sourceCpp)
useDynLib(motmot)
useDynLib(motmot, .registration=TRUE)

# Motmot

<!---
    from urllib.parse import quote
    quote("python- {}-#4691C2.svg".format(
                " | ".join(["3.6", "3.7", "3.8", "3.9", "PyInstaller"])))
--->

![](https://img.shields.io/badge/python-%203.6%20%7C%203.7%20%7C%203.8%20%7C%203.9%20%7C%20PyInstaller-%234691C2.svg)


A sophisticated mesh class for analysing colourless [Polygon meshes] such as
an [STL file]
providing a seamless abstraction between raw *vectors* meshes or the more
efficient *vertices + faces* (a.k.a *vertices + polygons*) form.

* Free software: MIT license
* Source code: https://github.com/bwoodsend/motmot/
* Releases: https://pypi.org/project/motmot/
* Documentation: https://bwoodsend.github.io/motmot/index.html


### Related projects

This library focuses almost exclusively on analysing meshes.
It it highly likely that you will need to supplement it with other libraries
to read/write to a certain format or to simplify an existing mesh.

* Mesh read/write:

  * [numpy-stl]:
    Reads and writes STL files. This is a dependency of `motmot`.
  * [meshio](https://github.com/nschloe/meshio):
    Reads and writes a multitude of mesh formats.
  * [pymesh](https://github.com/taxpon/pymesh):
    Reads and writes STL and OBJ files.

* Mesh analysis:

  * [PyMesh](https://github.com/PyMesh/PyMesh):
    A highly sophisticated mesh library which unfortunately depends on some
    rather
    hairy C++ libraries, making it not generally installable.
    It's not even on PyPI.
  * [trimesh](https://github.com/mikedh/trimesh):
    Another general purpose mesh library. This one is pure Python and focuses
    strictly on triangular and preferably closed meshes.
    It also brings a few readers and writers with it.
    This library is very powerful.
    It's quite likely that you'd be better off using it instead of `motmot`.

* Mesh cleaning:

  * [quad_mesh_simplify](https://github.com/jannessm/quadric-mesh-simplification):
    Decimate (collapse redundant or near redundant vertices in) meshes to make
    the filesize much smaller with negligible impact on quality.
  * [Py_Fast-Quadric-Mesh-Simplification](https://github.com/Kramer84/Py_Fast-Quadric-Mesh-Simplification):
    Another mesh decimator. This one is much faster but not on PyPI (yet).


### Usage

The basic API for ``motmot`` is modelled off that of [numpy-stl] with a few
alterations.


#### Initialisation

Meshes can be :

1.  Constructed from scratch using a single *vectors* array.
    This array should be 3D with shape ``(n, k, 3)`` where:

    * ``n`` is the number of polygons in the mesh,
    * ``k`` is the number of corners each polygon has,
    * ``3`` corresponds to having 3 axes. i.e. ``x``, ``y`` and ``z``.

    ```python
    # vectors is a (n, 3, 3) numpy array.
    triangle_mesh = Mesh(vectors)

    # vectors is a (n, 4, 3) numpy array.
    square_mesh = Mesh(vectors)
    ```

2.  Or using the more memory efficient *vertices + faces* form.

    ```python
    # `vertices` is an array of points. It should contain no duplicates.
    # `faces` is an integer array indicating which vertices are used to construct
    # each polygon.
    mesh = Mesh(vertices, faces)
    ```

3.  Read from an STL file. This uses [numpy-stl] under the hood.
    Currently, STL is the only file format that `motmot` will read implicitly:

    ```python
    from motmot import Mesh
    mesh = Mesh("some-file.stl")
    ```

4.  Read from an lzma, gzip or bzip2 compressed STL file:

    ```python
    from motmot import Mesh

    # An lzma compressed STL file. Create using `xz some-file.stl` in bash.
    mesh = Mesh("some-file.stl.xz")
    # A gzip compressed STL file. Create using `gzip some-file.stl` in bash.
    mesh = Mesh("some-file.stl.gz")
    # A bzip2 compressed STL file. Create using `bzip2 some-file.stl` in bash.
    mesh = Mesh("some-file.stl.bz2")
    ```


5.  Stream from any subclass of ``io.RawIOBase``.
    From here you can read from arbitrary sources such as
    embedded files, streams, archives or other pseudo files.
    For example, the following reads an STL directly from a web request:

    ```python
    from urllib import request
    from motmot import Mesh

    # Pull an STL file from the internet and load it without an intermediate
    # temporary file.
    url = "https://raw.githubusercontent.com/bwoodsend/vtkplotlib/master/" \
          "vtkplotlib/data/models/rabbit/rabbit.stl"

    with request.urlopen(url) as req:
        mesh = Mesh(req)
    ```


#### Vertices + Faces meshes vs Vectors meshes

There are two forms of mesh.

1.  A *vectors* mesh is essentially a list of polygons where
    each polygon is a list of points (its corners) and
    each point is an ``(x, y, z)`` triplet.
    This form is simple but wasteful because points which appear in multiple
    polygons are written multiple times which wastes memory and rendering time.

2.  A *vertices+faces* mesh takes all the unique points from a *vectors* mesh,
    calling them the *vertices*, then replaces each point in *vectors* with its
    index from *vertices*, calling this *faces*.
    Note that *faces* is often also known as *facets* or *polygons*.

Motmot makes the two forms interchangeable.
Each of *vectors*, *vertices* and *faces* are all available as attributes on all
meshes but,
depending on how a mesh is constructed,
*vectors* may be internally derived from *vertices* and *faces* or vice-versa.

```python
import numpy as np
from motmot import Mesh

# Define the 8 vertices in a cuboid.
vertices = np.array([
    [0., 0., 0.],
    [3., 0., 0.],
    [0., 5., 0.],
    [3., 5., 0.],
    [0., 0., 9.],
    [3., 0., 9.],
    [0., 5., 9.],
    [3., 5., 9.],
])

# Define the 6 sides of a cube or cuboid.
faces = np.array([
    # Draw a square using vertices[6], vertices[2], vertices[0] and vertices[4]
    [6, 2, 0, 4],
    # Draw a square using vertices[0], vertices[1], vertices[5] and vertices[4]
    [0, 1, 5, 4],
    # And so on...
    [0, 2, 3, 1],
    [5, 1, 3, 7],
    [3, 2, 6, 7],
    [4, 5, 7, 6],
])

# Construct a vertices+faces mesh.
mesh = Mesh(vertices, faces)
# This attribute is set to True to signify that this was originally a faces mesh.
mesh.is_faces_mesh
# Although `vectors` can still be derived automatically.
mesh.vectors

# Construct a vectors mesh.
mesh = Mesh(vertices[faces])
# This attribute is set to False to signify that this was originally a vectors
# mesh.
mesh.is_faces_mesh
# But `vertices` and `faces` can still be derived automatically.
mesh.vertices, mesh.faces
```


#### Mesh properties

This is just a brief summary of what is available.
See the corresponding entry in the
[the API reference]
for more information on each property.

```python
# Outward normal to each polygon:
>>> mesh.normals
array([[-45.,   0.,   0.],
       [ -0., -27.,  -0.],
       [ -0.,  -0., -15.],
       [ 45.,   0.,  -0.],
       [ -0.,  27.,   0.],
       [  0.,   0.,  15.]])

# Normalised (magnitude of 1.0) outward normal to each polygon:
>>> mesh.units
array([[-1.,  0.,  0.],
       [ 0., -1.,  0.],
       [ 0.,  0., -1.],
       [ 1.,  0.,  0.],
       [ 0.,  1.,  0.],
       [ 0.,  0.,  1.]])

# Area of each polygon.
>>> mesh.areas
array([45., 27., 15., 45., 27., 15.])

# Total surface area (just a sum of mesh.areas).
>>> mesh.area
174.0

# The number of times each vertex is used (which admittedly
# isn't particularly interesting for a cuboid):
>>> mesh.vertex_counts
array([3, 3, 3, 3, 3, 3, 3, 3], dtype=int32)

# A mapping of which other vertices each vertex is directly connect to.
>>> mesh.vertex_map
RaggedArray.from_nested([
    [1, 7, 3],  # vertices[0] connects to vertices[[1, 7, 3]].
    [2, 6, 0],  # vertices[1] connects to vertices[[2, 6, 0]].
    [4, 1, 3],  # and so on...
    [5, 0, 2],
    [5, 6, 2],
    [4, 7, 3],
    [1, 4, 7],
    [0, 5, 6],
])

# Because this mesh's vertices appear the same number of times,
# this example slightly trivialises the problem. Consider instead
# a mesh with only the first three faces. Not all vertices have
# the same number of neighbours.
>>> mesh[:3].vertex_map
RaggedArray.from_nested([
    [1, 3],
    [2, 6, 0],
    [4, 1, 3],
    [0, 2, 5],
    [5, 2, 6],
    [3, 4],
    [4, 1],
])

# If you prefer to use raw vertices rather than vertex IDs then
# use the connected_vertices() method.
>>> mesh.connected_vertices(mesh.vertices[0])
array([[0., 5., 0.],
       [3., 5., 9.],
       [0., 0., 9.]])

# Similarly, `polygon_map` maps every polygon to each of its neighbours.
# Read the first line of the following as *polygon 0 shares an edge each with
# polygons 4, 2, 1 and 5*.
>>> mesh.polygon_map
array([[4, 2, 1, 5],
       [2, 3, 5, 0],
       [0, 4, 3, 1],
       [1, 2, 4, 5],
       [2, 0, 5, 3],
       [1, 3, 4, 0]])
```


#### Vertex Lookup

`motmot` leverages two libraries for looking up vertices.

* [hirola.HashTable](https://hirola.readthedocs.io/en/latest/) for [exact lookup](#exact-lookup)
* [pykdtree.kdtree.KDTree](https://github.com/storpipfugl/pykdtree) for [fuzzy lookup](#approximate-lookup)


##### Exact lookup

It is easy to convert vertex IDs to real vertices.
Simply pass them as indices to `mesh.vertices`.

```python
>>> ids = [0, 4, 5, 2]
>>> points = mesh.vertices[ids]
>>> points
array([[0., 0., 0.],
       [0., 0., 9.],
       [3., 0., 9.],
       [0., 5., 0.]])
```

Go the other way by indexing the `vertex_table` attribute.

```python
>>> mesh.vertex_table[points]
array([0, 4, 5, 2], dtype=int64)
```

Some things to be aware of:

*   The `dtype` of the points queried must match `mesh.dtype`.

*   As with regular floats in a regular Python `dict`,
    even the smallest deviation will cause lookup to fail.

    ```python
    >>> mesh.vertex_table[[3., 0., 9.]]
    5
    >>> mesh.vertex_table[[3., 0, 9.00000000001]]
    KeyError: 'key = array([3., 0., 9.]) is not in this table.'
    ```


##### Approximate lookup

To find *nearest points*, `motmot` uses a [KDTree].
The API here is very shallow and it is quite likely that you may wish to
create and use `KDTree`s directly rather than use `motmot`'s methods.

A KDTree, fitted to `mesh.centers` (the middle of each polygon),
is found at the `mesh.kdtree` attribute.

Given a set of points defined as:
```python
points = np.array([[2., 3.5, 4.2], [2.3, 4.2, 1.1]], mesh.dtype)
```

Find the nearest point on the mesh surface to each point:

```python
>>> mesh.closest_point(points)
array([[3. , 3.5, 4.2],
       [2.3, 4.2, 0. ]])
```
Or to restrict the output to only `mesh.centers` without interpolating between
them:

```python
>>> mesh.closest_point(points, interpolate=False)
array([[3. , 2.5, 4.5],
       [1.5, 2.5, 0. ]])
```

For anything else, use `mesh.kdtree` directly.


#### Laziness

A `motmot.Mesh` *lazy loads* its properties using a backport of
[@functools.cached_property].
This allows them to be calculated when only you need them so that no time is
ever wasted calculating something which you do not use.
Take for example, [mesh.normals].
Nothing is calculated on
`mesh = Mesh(vertices, faces)` so that if the normals are never used then they are
never calculated.
Accessing the attribute `mesh.normals` initialises and returns
them making `mesh.normals` look like a regular attribute on the outside.
The value is cached so that the calculation never runs more than once.
i.e. `mesh.normals is mesh.normals`.

Caches should be reset after a mesh is modified.
Most of this is done automatically.
Mesh modifier methods such as `rotate()`, `translate()` or `crop(in_place=True)`
will all invalidate affected caches themselves.
Similarly, setting any of the `vertices`, `faces` or `vectors` attributes will
reset all caches.
Writing in place to those arrays (e.g. `mesh.vectors[:] = x`) however
is undetectable to `motmot`.
Call `mesh.reset()` after doing an in place modification.


[numpy-stl]: https://github.com/wolph/numpy-stl
[@functools.cached_property]: https://docs.python.org/3/library/functools.html#functools.cached_property
[the API reference]: https://motmot.readthedocs.io/en/latest/stubs/mesh.html
[mesh.normals]: https://motmot.readthedocs.io/en/latest/stubs/mesh.html#motmot.Mesh.normals
[KDTree]: https://github.com/storpipfugl/pykdtree
[Polygon meshes]: https://en.wikipedia.org/wiki/Polygon_mesh
[Triangle mesh]: https://en.wikipedia.org/wiki/Triangle_mesh
[STL file]: https://en.wikipedia.org/wiki/STL_(file_format)

## ----echo=FALSE, warning=FALSE, message=FALSE---------------------------------
library(knitr)
opts_chunk$set(tidy.opts=list(width.cutoff=60))

## ----global_options, include=FALSE--------------------------------------------
knitr::opts_chunk$set(fig.pos = 'H')

## ----warning=FALSE, message=FALSE, eval=FALSE---------------------------------
# install.packages("motmot")

## ----warning=FALSE, message=FALSE---------------------------------------------
library(motmot)

## -----------------------------------------------------------------------------
data(anolis.tree)
data(anolis.data)
attach(anolis.data)
anolis.tree

## -----------------------------------------------------------------------------
sortedData <- sortTraitData(phy=anolis.tree, y=anolis.data, data.name="Male_SVL", pass.ultrametric = TRUE)
phy <- sortedData$phy
male.length <- sortedData$trait

## ----plot1, fig.cap = "Figure 1. TraitData showing the relative male snout-vent length at the tips", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200----
traitData.plot(y=male.length, phy, lwd.traits=2, col.label="#00008050", tck=-0.01, mgp=c(0,0.2,0), cex.axis=0.5, show.tips=FALSE)

## -----------------------------------------------------------------------------
## uncomment to view the tree
# plot(phy, show.tip.label=FALSE, no.margin=TRUE, edge.col="grey20")
# nodelabels(182, 182, bg="black", col="white")
phy.clade <- extract.clade(phy, 182)
male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, rownames(male.length)),])

## -----------------------------------------------------------------------------
bm.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="bm")
bm.ml

## -----------------------------------------------------------------------------
lambda.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="lambda")
lambda.ml

## ----plot2, fig.cap = "Figure 2. Profile plot of ML estimation for Pagel's lambda", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200----
lambda.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="lambda", profilePlot=TRUE)

## -----------------------------------------------------------------------------
p.value <- 1 - pchisq(lambda.ml$MaximumLikelihood - bm.ml$logLikelihood, 1)
p.value

## -----------------------------------------------------------------------------
bm.ml$AICc- lambda.ml$AICc

## -----------------------------------------------------------------------------
delta.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="delta")
delta.ml

## ----plot3, fig.cap = "Figure 3. Comparison of BM and Kappa transformed trees.", echo = T, fig.height = 5, fig.width = 5, , fig.path='figures/', dev='png', dpi=200----
kappa.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="kappa", profilePlot=FALSE, returnPhy=TRUE)
par(mfrow=c(1,2))
plot.phylo(phy.clade, show.tip.label=FALSE, no.margin=TRUE)
mtext("Original phylogeny", 3, cex=0.7, line=-1)
plot.phylo(kappa.ml$kappaPhy, show.tip.label=FALSE, no.margin=TRUE)
mtext("Kappa model phylogeny", 3, cex=0.7, line=-1)
mtext("Kappa = 1e-8", 3, cex=0.7, line=-2)

## ----plot4, fig.cap = "Figure 4. Profile plot to estimate alpha", echo = T, fig.height = 5, fig.width = 5, , fig.path='figures/', dev='png', dpi=200----
ou.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="OU", profilePlot=TRUE, upperBound=2)
ou.ml

## -----------------------------------------------------------------------------
p.value <- 1 - pchisq(ou.ml$MaximumLikelihood - bm.ml$logLikelihood, 1)
p.value
bm.ml$AICc- ou.ml$AICc

## ----plot5, fig.cap = "Figure 5. Profile plot to estimate the ACDC parameter", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200----
acdc.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="ACDC", profilePlot=TRUE)
acdc.ml

## -----------------------------------------------------------------------------
p.value.2 <- 1 - pchisq(acdc.ml$MaximumLikelihood - bm.ml$logLikelihood , 1)
p.value.2

## -----------------------------------------------------------------------------
transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="ACDC", profilePlot=FALSE, upperBound=-1e-6, print.warning=FALSE)

## ----plot6, fig.cap = "Figure 6. Profile plot to estimate the psi parameter", echo = TRUE, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200----
psi.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="psi", profilePlot=TRUE)
psi.ml

## -----------------------------------------------------------------------------
p.value.psi <- 1 - pchisq(psi.ml$MaximumLikelihood - bm.ml$logLikelihood , 1)
p.value.psi

## -----------------------------------------------------------------------------
psi_ext.est <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="psi", profilePlot=FALSE, hiddenSpeciation=TRUE, full.phy=phy)
all.equal(psi.ml, psi_ext.est)

## ----plot7, fig.cap = "Figure 7. Two clades used in the multipsi model", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200----
plot(phy.clade, no.margin=TRUE, cex=0.8)
two.clade.labels <- c(rep("a", 17), rep("b",37))
edgelabels(two.clade.labels, col=c(rep("blue", 17), rep("red", 37)), bg="white")

## -----------------------------------------------------------------------------
transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="multipsi", branchLabels=c(rep("a", 17), rep("b",37)), hiddenSpeciation=TRUE, full.phy=phy)

## -----------------------------------------------------------------------------
acdc.ml.lambda <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="ACDC", lambdaEst=TRUE)
# original ACDC model
acdc.ml
# ACDC model plus lambda
acdc.ml.lambda

## -----------------------------------------------------------------------------
# p value of the ACDC and ACDC+lambda models. No significant improvement
1 - pchisq(acdc.ml.lambda$MaximumLikelihood - acdc.ml$MaximumLikelihood , df=1)
# p value of the BM and ACDC+lambda model comparison. No significant improvement
1 - pchisq(acdc.ml.lambda$MaximumLikelihood - bm.ml$logLikelihood, df=2)

## ----plot8, fig.cap = "Figure 8. Lineages with different rates of evolution", echo = T, fig.height = 5, fig.width = 5, , fig.path='figures/', dev='png', dpi=200----
plot(phy.clade, show.tip.label=FALSE, no.margin=TRUE, edge.col="grey20")
nodelabels(c(32, 49), c(32, 49), bg="black", col="white")

## -----------------------------------------------------------------------------
cladeRate.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="clade", nodeIDs=c(32, 49))
cladeRate.ml

## -----------------------------------------------------------------------------
# tm1 algorithm not run
# tm1.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="tm1", minCladeSize=2, nSplits=3)
# trait.medusa.tm1.summary <- summary.traitMedusa(tm1.ml, cutoff=2, AICc=T)
# tm2 model
tm2.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="tm2", minCladeSize=5, nSplits=2)

## ----plot9, fig.cap = "Figure 9. The subset of the tree showing the rate heterogeneity estimated from the traitMedusa model", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200----
trait.medusa.tm2.summary <- summary(tm2.ml, cutoff=2, AICc=TRUE)
trait.medusa.tm2.summary
colour_motmot <- plot(x = trait.medusa.tm2.summary, reconType = "rates", type = "fan", cex=0.5, edge.width=2)

## -----------------------------------------------------------------------------
## uncomment to run
# set.seed(203);
# calcCutOff(phy.clade, n=1000, model="tm2", minCladeSize=5, nSplits=1);
##      95% 
## 5.698198 

## -----------------------------------------------------------------------------
summary(tm2.ml, cutoff=5.698198, AICc=TRUE)$Rates

## -----------------------------------------------------------------------------
timeSlice.10.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="timeSlice", splitTime=10)

## ----plot10, fig.cap = "Figure 10. TimeSlice plot with a split at 10 Ma", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200----
outputSummary <- plot(timeSlice.10.ml, cutoff=0.001, cex=0.55, edge.width=2, cex.plot=0.8, colour.ramp=c("blue", "red"), label.offset=0.5)

## -----------------------------------------------------------------------------
outputSummary$RatesCI

## -----------------------------------------------------------------------------
timeSlice.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="timeSlice", nSplits=1, boundaryAge=8)

## ----plot11, fig.cap = "Figure 11. TimeSlice plot with Maximum likelihood estimation of split time", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200----
outputSummary <- plot(timeSlice.ml, cutoff=1, cex=0.2, edge.width=2, cex.plot=0.8, colour.ramp=c("blue", "red"), label.offset=0.5)

## -----------------------------------------------------------------------------
modeSlice.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="modeSlice", splitTime=c(40, 30), mode.order=c("ACDC", "OU", "BM"), rate.var=TRUE, acdcScalar=TRUE)
modeSlice.ml$AICc
bm.ml$AICc

## -----------------------------------------------------------------------------
bm.model <- transformPhylo.ML(male.length.clade, phy=phy.clade, model="bm")
nested.acdc <- transformPhylo.ML(male.length.clade, phy=phy.clade, model="ACDC", nodeIDs=44)
nested.ou <- transformPhylo.ML(male.length.clade, phy=phy.clade, model="OU", nodeIDs=44)

1 - pchisq(nested.acdc$MaximumLikelihood - bm.model$logLikelihood, 1)
1 - pchisq(nested.ou$MaximumLikelihood - bm.model$logLikelihood, 1)

## ----results="hide"-----------------------------------------------------------
set.seed(12) # set seed so run will be identical - for example use only
lambda.mcmc <- transformPhylo.MCMC(y=male.length.clade, phy=phy.clade, model="lambda", mcmc.iteration=2000, burn.in=0.25, random.start=FALSE, sample.every=1)

## -----------------------------------------------------------------------------
lambda.mcmc[1:4]

## ----plot12, fig.cap = "Figure 12. MCMC trace for Pagel's lambda", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200----
mcmc.plot(lambda.mcmc)

## -----------------------------------------------------------------------------
data(finches)
emp.tree <- finch.tree
emp.data <- finch.data
param.simulation <- chr.disp.param(emp.tree, n.sim = 100, max.sigma = 8, max.a = 8, ntraits=1, mc.cores = 1)

## -----------------------------------------------------------------------------
chr.disp.lrt(emp.tree=emp.tree, emp.data=emp.data, param.out=param.simulation, posteriorSize=75)

## -----------------------------------------------------------------------------
# Data and phylogeny
data(anolis.tree)
anolis.tree$node.label <- NULL
set.seed(3492)
lm.data <- transformPhylo.sim(phy=anolis.tree, n=2, model="bm")
dat <- data.frame(x = lm.data[,1], y = lm.data[,2], names = anolis.tree$tip, row.names = anolis.tree$tip)
# pgls from CAPER with matrix inversion
library(caper)
comp.dat <- comparative.data(anolis.tree, dat, names)
time.now <- Sys.time()
matrix.inv.caper <- pgls( y ~ x, data = comp.dat, lambda="ML")
pgls.time <- Sys.time() - time.now
pgls.time
time.now <- Sys.time()
picModel <- pic.pgls(formula=y ~  x, phy=anolis.tree, y = dat, lambda="ML", return.intercept.stat=FALSE)
pic.time <- Sys.time() - time.now
pic.time

## -----------------------------------------------------------------------------
# from caper
summary(matrix.inv.caper)
# from MOTMOT
picModel

#' @title Drop tips from a phylogenetic tree while preserving deleted nodes
#' @description Wrapper for the \code{ape} function \code{drop.tip} that preserves the number of nodes affecting each branch. For use with the \code{psi} and \code{multipsi} models.
#' @param phy Phylogenetic tree in \code{phylo} format
#' @param tip A vector of mode numeric or character specifying the tips to delete, to be passed to \code{drop.tip}
#' @return Phylogenetic tree in \code{phylo} format, with an added element \code{Shid}, a vector of numbers of observed but "missing" speciation events per branch, in the same order as the branches in the \code{phylo} object
#' @references Ingram, T. 2011. Speciation along a depth gradient in a marine adaptive radiation. Proc. R. Soc. B 278: 613-618.
#' @author Travis Ingram
#' @seealso \code{\link{transformPhylo.ML}}
#' @examples
#' ## Read in phylogeny and data from Thomas et al. (2009)
#' data(anolis.tree)
#' data(anolis.data)
#' ## identify tips to drop
#' tips.to.go <- anolis.tree$tip.label[1:30]
#' dropTipPartial(phy=anolis.tree, tip=tips.to.go)
#' @export

dropTipPartial <- function (phy, tip) 
{
    phy1 <- phy
    phy1$edge.length <- phy$edge.length ^ 0
    phy1 <- drop.tip(phy1, tip)
    phy2 <- drop.tip(phy, tip)
    phy2$orig.node <- phy1$edge.length
    phy2
}

#' @title prune tree and data to lineages present in a time bin in the past 
#'
#' @description the function takes a full tree and returns a pruned phylogeny with only tips and lineages found within a time bin preserved. If trait data are supplied the function will return tip states based either on the original tips found in the bin, or tip states inferred from ancestral states
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param maxBin the start age (older time in myr from present) of the time bin in which lineages are preserved
#' @param minBin the final age (younger time in myr from present) of the time bin in which lineages are preserved
#' @param reScale if the most recent tip is not from the present, the age needed to add so the phylogeny is in 'real time'
#' @param allTraits a vector of trait data corresponding to the phy$edge object. The trait data represent tip and internal node data for the phylogeny
#' @param closest.min Logical. Should new tip values for lineages that span the bin be taken from the node nearest the \code{minBin} age (\code{closest.min=TRUE}, default) or the \code{maxBin} age \code{closest.min=FALSE}
#' @param traits.from.tip Logical. Should tip values for pendant edges in the bin be taken from the original tip value or the reconstructed node value (if it is closer than the tip value)
#' @return the pruned phylogeny. The object \code{descendants} refers to the lineages the branch in the time bin gave rise to before it was pruned. If traits are included a vector of trait values representing species at the tips.
#' @references Puttick, M. N., Kriwet, J., Wen, W., Hu, S., Thomas, G. H., & Benton, M. J. (2017). Body length of bony fishes was not a selective factor during the biggest mass extinction of all time. Palaeontology, 60, 727-741.
#' @author Mark Puttick
#' @examples
#' ## prune a random tree to taxa present between 4 and 2 units before present
#' # generate tree
#' set.seed(20)
#' tree <- rtree(20)
#' # generate traits
#' traits <- rnorm(20)
#' # plot tree and timeframe
#' plot(tree)
#' max.age <- nodeTimes(tree)[1,1]
#' abline(v=max.age - c(4, 2))
#' # prune tree to timeframe
#' cont.tree <- contemporaryPhy(phy=tree, maxBin=4, minBin=2, allTraits=traits)
#' plot(cont.tree$phy)
#' @export

contemporaryPhy <- function(phy, maxBin, minBin, reScale=0, allTraits, closest.min=TRUE, traits.from.tip=TRUE) {
	
	node.in <- match(unique(phy$edge[,1]), phy$edge[,1])
	node.times <- nodeTimes(phy)[node.in,1]
	names(node.times) <- unique(phy$edge[,1])
	
	startBin <- maxBin - reScale
	endBin <- minBin - reScale
	nodePreBin <- as.numeric(names(node.times)[which(node.times < endBin)])
	
	tip.times <- nodeTimes(phy)[which(phy$edge[,2] <= Ntip(phy)), 2]
	tipsInBin <- intersect(which(tip.times >= endBin), which(tip.times <= startBin))
	
	int.node <- which(phy$edge[,2] > Ntip(phy))
	descendantAsTrait <- rep(0, dim(phy$edge)[1])
	for(rr in int.node) descendantAsTrait[rr] <- length(node.descendents(phy$edge[rr,2], phy, T)[[2]])
	
	tipsInMat <- which(phy$edge[,2] <= Ntip(phy))
	terminalTips <- which(descendantAsTrait == 0)
	loseTips <- match(tipsInMat[tipsInBin], terminalTips)
	descendantAsTrait[terminalTips[-loseTips]] <- 1
	names(descendantAsTrait) <- phy$edge[,2]
	
	success <- timeTravelPhy(phy=phy, node=nodePreBin, nodeEstimate=descendantAsTrait, timeCut=endBin)
	suc <- removeNonBin(success$phy, success$tipData, keepByTime=startBin - endBin)
	
	out <- list()
	out <- suc
	names(out) <- c("phy", "descendants")
	
	if(!is.null(allTraits)) {
		
		first.times <- nodeTimes(phy)
		
		new.phylo <- suc$prunedPhy
		time.new.phylo <- nodeTimes(new.phylo) + reScale
	
		new.tip.names <- new.phylo$tip.label
		identify.tip <- regexpr("node_", new.phylo$tip.label)
		original.tip <- which(identify.tip == -1)
		same.as.original <- new.tip.names[original.tip]
		which.tips <- match(match(same.as.original, phy$tip.label), phy$edge[,2])
		new.spp <- which(identify.tip != -1)
		location.new.spp <- match(new.spp, new.phylo$edge[,2])
	
		node.original <- as.numeric(gsub("node_", "", new.phylo$tip.label[new.spp]))
		which.nodes <- match(node.original, phy$edge[,2])
		on.orig <- sort(c(which.nodes, which.tips))
	
		if (closest.min) closest.bin <- minBin else closest.bin <- maxBin
	
		trait.mat.new <- sapply(1:length(on.orig), function(i) {
			here.node <- on.orig[i]
			diff.age <- abs(first.times[here.node, ] - closest.bin)
			smallest.distance <- which.min(diff.age)
			if(smallest.distance == 2) {
				allTraits[here.node]
				} else {
				older.node <- match(phy$edge[here.node,1], phy$edge[,2])
				allTraits[older.node]
				}
			}
		)
		
		if(traits.from.tip)	trait.mat.new[original.tip] <- allTraits[which.tips]
		out$traits <- trait.mat.new
	}
	
	return(out)
}

#' @title Bayesian MCMC for models of trait evolution
#' @description Fits Bayesian models for various models of continuous character evolution using a Metropolis-Hastings Markov Chain Monte Carlo (MCMC) approach
#' @param y A matrix of trait values.
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param model The model of trait evolution (see details).
#' @param mcmc.iteration Integer - the number of generations for which to run the MCMC chain
#' @param hiddenSpeciation Logical. If TRUE the psi model will include nodes that are on the 'full.phy' but not the tree pruned of trait data
#' @param useMean Logical. Use the branch-based estimates of extinction of mean (TRUE, default) for the "psi" and "multispi" models only applicable if "hiddenSpeciation" = TRUE
#' @param full.phy The full phylogeny containing the species that do not contain trait data so are not included in 'phy'
#' @param burn.in The proportion of the chain (as given by mcmc.iteration) which to discard as 'burn-in'
#' @param lowerBound Minimum value for parameter estimates
#' @param upperBound Maximum value for parameter estimates
#' @param random.start Use a random starting value for the MCMC run (TRUE), or use the environment set.seed() value
#' @param meserr A vector (or matrix) of measurement error for each tip. This is only applicable to univariate analyses
#' @param covPIC Logical. For multivariate analyses, allow for co-variance between traits rates (TRUE) or no covariance in trait rates (FALSE). If FALSE, only the trait variances not co-variances are used.
#' @param lambdaEst Logical.  Estimate lambda alongside parameter estimates to reduce data noise. Only applicable for models "kappa", "delta", "OU", "psi", "multispi", and "ACDC". Default=FALSE.
#' @param nodeIDs Integer - ancestral nodes of clades applicable to rate heterogenous and nested models of evolution (see details)
#' @param branchLabels Branches on which different psi parameters are estimated in the "multipsi" model
#' @param acdcScalar Logical.  For nested EB rate model, simultaneously estimated a rate scalar alongside EB model. Default=FALSE.
#' @param sample.every Number specifying the every nth that is sampled in the MCMC chain (default = 1).
#' @details The method estimates posterior probabilities using a Metropolis-Hastings MCMC approach that places a prior bounded uniform distribution on all parameters with an independence sampler. These prior distributions can be altered by changing the upperBound and lowerBound arguments. The MCMC model will estimate the posterior probability for the following models:
#' \describe{
#' \item{model="kappa"}{fits Pagel's kappa by raising all branch lengths to the power kappa. As kappa approaches zero, trait change becomes focused at branching events. For complete phylogenies, if kappa approaches zero this infers speciational trait change. Default bounds from ~0 - 1.}
#' \item{model="lambda"}{fits Pagel's lambda to estimate phylogenetic signal by multiplying all internal branches of the tree by lambda, leaving tip branches as their original length (root to tip distances are unchanged). Default bounds from ~0 - 1.}
#' \item{model="delta"}{fits Pagel's delta by raising all node depths to the power delta. If delta <1, trait evolution is concentrated early in the tree whereas if delta >1 trait evolution is concentrated towards the tips. Values of delta above one can be difficult to fit reliably. Default bounds from ~0 - 5.}
#' \item{model="OU"}{fits an Ornstein-Uhlenbeck model - a random walk with a central tendency proportional to alpha. High values of alpha can be interpreted as evidence of evolutionary constraints, stabilising selection or weak phylogenetic signal. It is often difficult to distinguish among these possibilities. Default bounds from ~0 - 10.}
#' \item{model="psi"}{fits a acceleration-deacceleration model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate (Ingram 2010).}
#' \item{model="ACDC"}{fits a model to in which rates can exponentially increased or decrease through time (Blomberg et al. 2003). If the upper bound is < 0, the model is equivalent to the 'Early Burst' model of Harmon et al. 2010. Default rate parameter bounds from ln(1e-10) ~ ln(20) divided by the root age.}
#' }
#' @return median The median estimate of the posterior for the parameter 
#' @return 95.HPD The 95 percent Highest Posterior Density for the parameter
#' @return ESS Effective Sample Size for the posterior
#' @return acceptance.rate The ratio for which new proposals were accepted during the MCMC chain
#' @return mcmc.chain Full MCMC chain containing all iterations (including burn-in)
#' @author Mark Puttick, Gavin Thomas
#' @seealso \code{\link{transformPhylo.ML}}, \code{\link{transformPhylo.ll}}, \code{\link{transformPhylo}}
#' @import coda
#' @examples
#' data(anolis.tree)
#' data(anolis.data)
#' attach(anolis.data)
#' male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
#' sortedData <- sortTraitData(anolis.tree, male.length)
#' phy <- sortedData$phy
#' male.length <- sortedData$trait
#' phy.clade <- extract.clade(phy, 182)
#' male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, rownames(male.length)),])
#' ## please note, this model will be need to run for longer to achieve convergence
#' lambda.mcmc <- transformPhylo.MCMC(y=male.length.clade, phy=phy.clade, 
#' model="lambda", mcmc.iteration=100, burn.in=0.1)
#' @export 

transformPhylo.MCMC <- function(y, phy, model, mcmc.iteration=1000, burn.in=0.1, hiddenSpeciation = FALSE, full.phy=NULL, lowerBound = NULL, upperBound = NULL, useMean = FALSE, random.start=TRUE, meserr = NULL, covPIC=TRUE, lambdaEst=FALSE, nodeIDs = NULL, branchLabels = NULL, acdcScalar = FALSE, sample.every=10) {

	if(length(model) > 1) stop("please provide one model only")
	if(ncol(y) > 1 && !is.null(meserr)) stop("meserr only applicable to univariate analyses")
	
	controlList=c(fnscale=-1, maxit=100, factr=1e-7, pgtol=0, type=2, lmm=5)
	
	model <- tolower(model)
	all.models <- c("lambda", "delta", "kappa", "ou", "acdc", "psi")
	if(is.na((match(model, all.models)))) stop(paste(model, "not recognised - please provide one of", paste0(all.models, collapse=", ")))
	if(random.start) set.seed(as.numeric(Sys.time()))

	bounds <- matrix(c(1e-08, 1, 1e-08, 1, 1e-08, 5, 1e-08, 20, 0, 1, 1e-08, 10, 1e-10, 100000), 7, 2, byrow = TRUE)
	rownames(bounds) <- c("kappa", "lambda", "delta", "alpha", "psi", "rate", "acdcRate")

	if(model == "lambda") {
		if (is.null(lowerBound)) {
			lowerBound <- bounds["lambda", 1]
			}
		if (is.null(upperBound)) {
			upperBound <- bounds["lambda", 2]
		}
		
		input.value <- runif(1, lowerBound, upperBound)
		lik.model <- function(pram) {
			lambda.phy <- transformPhylo(phy, model="lambda", y=y, lambda=pram, meserr=meserr)
			return(likTraitPhylo(y, lambda.phy, covPIC=covPIC)[[2]])
		}
		name.param <- c("Lambda")
	}

	if(model == "delta") {
		if (is.null(nodeIDs)) {
        		nodeIDs <- Ntip(phy) + 1
    		} else {
        		nodeIDs <- nodeIDs
    			}
		if (is.null(lowerBound)) {
			lowerBound <- bounds["delta", 1]
			}
		if (is.null(upperBound)) {
			upperBound <- bounds["delta", 2]
			}
		if (lambdaEst) {
    			lowerBound <- c(lowerBound, bounds["lambda", 1])
    			upperBound <- c(upperBound, bounds["lambda", 2])
    		    	}
		
		input.value <- sapply(1:length(lowerBound), function(x) runif(1, lowerBound[x], upperBound[x]))
		if(length(input.value) == 2) input.value[2] = 1
		lik.model <- function(pram) {
			if (lambdaEst) lambda <- tail(pram, 1) else lambda <- 1
        		delta.est <- pram[1]
        		lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, model = "lambda", meserr = meserr)
			delta.phy <- transformPhylo(lambdaPhy, model="delta", y=y, delta=delta.est, meserr=meserr, nodeIDs = nodeIDs)
			return(likTraitPhylo(y, delta.phy, covPIC=covPIC)[[2]])
			}
		
		if (lambdaEst) {
				name.param <- c("Delta", "Lambda")
			} else {
				name.param <- c("Delta")
			}
			
		}

		if(model == "kappa") {
			if (is.null(nodeIDs)) {
        			nodeIDs <- Ntip(phy) + 1
      		} else {
        			nodeIDs <- nodeIDs
      			}
			if (is.null(lowerBound)) {
				lowerBound <- bounds["kappa", 1]
				}
			if (is.null(upperBound)) {
				upperBound <- bounds["kappa", 2]
				}
		
			if (lambdaEst) {
       			lowerBound <- c(lowerBound, bounds["lambda", 1])
       			upperBound <- c(upperBound, bounds["lambda", 2])
        			}
		
			input.value <- sapply(1:length(lowerBound), function(x) runif(1, lowerBound[x], upperBound[x]))
					
			lik.model <- function(pram) {
				if (lambdaEst) lambda <- tail(pram, 1) else lambda <- 1
         		kappa.est <- pram[1]
         		lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, model = "lambda", meserr = meserr)
				kappa.phy <- transformPhylo(phy, model="kappa", y=y, kappa=kappa.est, meserr=meserr, nodeIDs = nodeIDs)
				return(likTraitPhylo(y, kappa.phy, covPIC=covPIC)[[2]])
				}
		
			if (lambdaEst) {
				name.param <- c("Kappa", "Lambda")
			} else {
				name.param <- c("Kappa")
			}
		}
	
		if(model == "ou") {
			if (is.null(nodeIDs)) {
        			nodeIDs <- Ntip(phy) + 1
      		} else {
        			nodeIDs <- nodeIDs
      			}
			if (is.null(lowerBound)) {
				lowerBound <- bounds["alpha", 1]
				}
			if (is.null(upperBound)) {
				upperBound <- bounds["alpha", 2]
				}
			input.value <- runif(1, lowerBound , upperBound)
			if(!is.ultrametric(phy)) {
				stop("non-ultrametric OU model not available in transformPhylo.MCMC at current, sorry")
				}
			if (lambdaEst) {
       			lowerBound <- c(lowerBound, bounds["lambda", 1])
       			upperBound <- c(upperBound, bounds["lambda", 2])
        		}
			input.value <- upperBound[1] + 1
			while(!(input.value <= upperBound[1] && input.value >= lowerBound[1])) input.value <- rexp(1, upperBound[1])
			if(lambdaEst) input.value[2] <- runif(1, lowerBound[2], upperBound[2])
		
			lik.model <- function(pram) {
				if (lambdaEst) lambda <- tail(pram, 1) else lambda <- 1
				ou.est <- pram[1]
        			lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, model = "lambda", meserr = meserr)
        			alpha.phy <- transformPhylo(phy, model="ou", y=y, alpha=ou.est, meserr=meserr)
				return(likTraitPhylo(y, alpha.phy, covPIC=covPIC)[[2]])
				}
	
		if (lambdaEst) {
				name.param <- c("alpha", "Lambda")
			} else {
				name.param <- c("alpha")
				}	
		}

		if(model == "acdc") {
			if (is.null(nodeIDs)) {
        			nodeIDs <- Ntip(phy) + 1
      		} else {
        			nodeIDs <- nodeIDs
      			}
			rootBranchingTime <- nodeTimes(phy)[1,1]
			if (is.null(lowerBound)) {
				lowerBound <- log(bounds["acdcRate", 1]) / rootBranchingTime
				}
			if (is.null(upperBound)) {
				upperBound <- log(bounds["acdcRate", 2]) / rootBranchingTime
				}
			if (acdcScalar) {
				upperBound[1] <- -1e-6
      			if(is.na(lowerBound[2])) {
      				lowerBound[2] <- 1
      				}
      			if(is.na(upperBound[2])) {
      				upperBound[2] <- 5
      			}
       		}
			if (lambdaEst) {
       			lowerBound <- c(lowerBound, bounds["lambda", 1])
       			upperBound <- c(upperBound, bounds["lambda", 2])
        			} 
			input.value <- sapply(1:length(lowerBound), function(x) runif(1, lowerBound[x], upperBound[x]))
			if (acdcScalar) {
        			lik.model <- function(pram) {
        				if (lambdaEst) lambda <- tail(pram, 1) else lambda <- 1
          			acdc.est <- pram[1]
          			scalarRate <- pram[2]
          			lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, model = "lambda", meserr = meserr)
          			acdc.phy <- transformPhylo(lambdaPhy, model="acdc", acdcRate=acdc.est, meserr=meserr, nodeIDs = nodeIDs, cladeRates = scalarRate)
          			return(likTraitPhylo(y, acdc.phy, covPIC=covPIC)[[2]])
        				}
				} else {
					lik.model <- function(pram) {
					if (lambdaEst) lambda <- tail(pram, 1) else lambda <- 1
         			acdc.est <- pram[1]
         			scalarRate <- 1
         			lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, model = "lambda", meserr = meserr)
         			acdc.phy <- transformPhylo(lambdaPhy, model="acdc", y=y, acdcRate=acdc.est, meserr=meserr)
					return(likTraitPhylo(y, acdc.phy, covPIC=covPIC)[[2]])
					}
				}
		
			if (lambdaEst) {
				name.param <- c("ACDC.rate", "Lambda")
				if (acdcScalar) name.param <- c("ACDC.rate", "ACDC.scalar", "Lambda")
			} else {
				name.param <- c("ACDC.rate")
				if (acdcScalar) name.param <- c("ACDC.rate", "ACDC.scalar")
				}
		}

		if(model == "psi") {
			if (is.null(nodeIDs)) {
    		    		nodeIDs <- Ntip(phy) + 1
    		  	} else {
        			nodeIDs <- nodeIDs
      			}
			if (is.null(lowerBound)) {
				lowerBound <- bounds["psi", 1]
				}
			if (is.null(upperBound)) {
				upperBound <- bounds["psi", 2]
				}
			if (lambdaEst) {
       			lowerBound <- c(lowerBound, bounds["lambda", 1])
       			upperBound <- c(upperBound, bounds["lambda", 2])
        			}	
			if (hiddenSpeciation) {
				if (is.null(full.phy)) stop("please provide a full phylogeny")
				full.data.match <- match(full.phy$tip.label, rownames(y))
				tips.no.data <- full.phy$tip.label[which(is.na(full.data.match))]
				phy <- dropTipPartial(full.phy, tips.no.data)
				}
	 		if(is.ultrametric(phy)) {
     	 		phy.bd <- birthdeath(phy)
     		 } else {
     		 	phy.bd <- birthdeath_motmot(phy)
     		 	}
     	 
      		mu_over_lambda <- phy.bd[[4]][1]
      		lambda_minus_mu <- phy.bd[[4]][2]
      		lambda.sp <- as.numeric(lambda_minus_mu / (1 - mu_over_lambda))
      		mu.ext <- as.numeric(lambda_minus_mu / (1 / mu_over_lambda - 1))			
			if (mu.ext > 0) {
				phy <- sampleHiddenSp(phy, lambda.sp = lambda.sp, mu.ext = mu.ext, useMean=useMean)
			} else {
				phy$hidden.speciation <- NULL
			}
			
			input.value <- sapply(1:length(lowerBound), function(x) runif(1, lowerBound[x], upperBound[x]))

			lik.model <- function(pram) {
				if (lambdaEst) lambda <- tail(pram, 1) else lambda <- 1
         		psi.est <- pram[1]
         		lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, model = "lambda", meserr = meserr)
         		psi.phy <- transformPhylo(phy, model="psi", y=y, psi=psi.est, meserr=meserr, lambda.sp = lambda.sp, nodeIDs = nodeIDs)
				return(likTraitPhylo(y, psi.phy, covPIC=covPIC)[[2]])
				}

			if (lambdaEst) {
				name.param <- c("Psi", "Lambda")
			} else {
				name.param <- c("Psi")
			}
	}
		
		if(model == "multispi") {
   		   if (is.null(branchLabels)) stop("for 'multipsi' model must provide branchLabels giving state for each branch")
      		states <- levels(factor(branchLabels))
      		
      		if (is.null(lowerBound)) {
        			lowerBound <- bounds[rep("psi", length(states)), 1]
        			if (lambdaEst) lowerBound[length(states) + 1] <- bounds["lambda", 1]
      			}
      		if (is.null(upperBound)) {
        			upperBound <- bounds[rep("psi", length(states)), 2]
        			if (lambdaEst) upperBound[length(states) + 1] <- bounds["lambda", 2]
      			}
      		
      		input.value <- sapply(1:length(lowerBound), function(x) runif(1, lowerBound[x], upperBound[x]))

      		if (hiddenSpeciation) {
        			if (is.null(full.phy)) stop("please provide a full phylogeny")
        			full.data.match <- match(full.phy$tip.label, rownames(y))
        			tips.no.data <- full.phy$tip.label[which(is.na(full.data.match))]
        			phy <- dropTipPartial(full.phy, tips.no.data)
      			}
			
			if(is.ultrametric(phy)) {
	      		phy.bd <- birthdeath(phy)
	      	} else {
	      		phy.bd <- birthdeath_motmot(phy)
	      		}
      	
      		mu_over_lambda <- phy.bd[[4]][1]
      		lambda_minus_mu <- phy.bd[[4]][2]
      		lambda.sp <- as.numeric(lambda_minus_mu / (1 - mu_over_lambda))
      		mu.ext <- as.numeric(lambda_minus_mu / (1 / mu_over_lambda - 1))

      		if (mu.ext > 0) {
      			phy <- sampleHiddenSp(phy, lambda.sp = lambda.sp, mu.ext = mu.ext, useMean = useMean)
      		} else {
      			phy$hidden.speciation <- NULL
      			}
		
			var.funmultipsi <- function(param) {
        			all.param <- length(param)
        			if (lambdaEst) {
        				lambda <- param[all.param]
          			psi <- param[-all.param]
        			} else {
        		  		psi <- param
        		  		lambda <- 1
        			}
        			lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, model = "lambda", meserr = meserr)
        			return(transformPhylo.ll(y = y, phy = lambdaPhy, branchLabels = branchLabels, psi = psi, model = "multipsi", meserr = meserr, covPIC = covPIC, lambda.sp = lambda.sp)[[2]])
      		}
      	
      		vo <- optim(start, var.funmultipsi, method = "L-BFGS-B", lower = lowerBound, upper = upperBound, control = controlList)

			lik.model <- function(pram) {
				all.param <- length(pram)
				if (lambdaEst) {
		 			lambda <- pram[all.param]
		 			psi <- pram[-all.param]
		 		} else {
		 			psi <- pram
		 			lambda <- 1
		 			}
       			lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, model = "lambda", meserr = meserr)
       			psi.phy <- transformPhylo(phy = lambdaPhy, branchLabels = branchLabels, psi = psi, model = "multipsi", meserr = meserr, lambda.sp = lambda.sp)
				return(likTraitPhylo(y, psi.phy, covPIC=covPIC)[[2]])
				}
	
			if (lambdaEst) {
				name.param <- c(rep("multipsi", length(states)), "Lambda")
				} else {
				name.param <- rep("multipsi", length(states))
				}
		}
	
	
		prior.calc <- function(pram) {
			prior.uni <- sapply(1:length(pram), function(x) dunif(pram[x], lowerBound[x], upperBound[x]))
			return(sum(prior.uni))
			}

		model.posterior <- function(pram) return (lik.model(pram) + prior.calc(pram))	
	
		motmot.mcmc <- function(input.value, iterations, silent=FALSE) {
			propose.mcmc <- function(pram) {
				return(runif(length(pram), min=lowerBound, max=upperBound))
				}
			
		mcmc.chain <- matrix(input.value, nrow=1)
    			for (i in 1:iterations) {
    				proposed.move <- propose.mcmc(mcmc.chain[i, ])
     	   		chain.prob <- exp(model.posterior(proposed.move) - model.posterior(mcmc.chain[i,]))
     	   		if (runif(1) < chain.prob) {
     	   			mcmc.chain <- rbind(mcmc.chain, proposed.move)
        			} else {
     	   			mcmc.chain <- rbind(mcmc.chain, mcmc.chain[i,])
     	   		}
			if(!silent) {
				cat("\r", paste0("MCMC progress (%): ", sprintf("%.4f", i/iterations * 100)))
				if(i/iterations == 1) {
					cat("\n", " ")
					cat("\r", "finished MCMC")
					cat("\r", "")
					}
				}	
    			}
    			if(!silent) cat("\n", " ")
    			return(mcmc.chain)
		}
	
		chain <- motmot.mcmc(input.value, iterations=mcmc.iteration)
		burnIn <- ceiling(mcmc.iteration * burn.in)
		post.burn.in <- as.matrix(chain[-c(1:burnIn), ])
		post.burn.in <- as.matrix(post.burn.in[seq(1, nrow(post.burn.in), by=sample.every), ])
		rownames(post.burn.in) <- NULL
		
		acceptance.1 <- 1 - mean(apply(as.matrix(post.burn.in), 2, function(x) duplicated(signif(x, 7))))
		
		if(dim(chain)[2] == 1) {
			ess.mcmc <- coda::effectiveSize(post.burn.in)
			median.mcmc <- median(post.burn.in)
			hpd.mcmc <- coda::HPDinterval(as.mcmc(post.burn.in))
			hpd.mcmc <- as.numeric(hpd.mcmc)
			names(ess.mcmc) <- names(median.mcmc) <- name.param
			names(hpd.mcmc) <- c("lower 95% HPD", "upper 95% HPD")
		} else {
			ess.mcmc <- apply(post.burn.in, 2, coda::effectiveSize)
			median.mcmc <- apply(chain[-c(1:burnIn), ], 2, median)
			hpd.mcmc <- apply(chain[-c(1:burnIn), ], 2, function(x) {
				class(x) <- "mcmc"
				as.numeric(coda::HPDinterval(x))
				}
			)
			names(ess.mcmc) <- names(median.mcmc) <- name.param
			colnames(hpd.mcmc) <- name.param
		}

		cat("\n")
		output.mcmc <- list(median.mcmc, hpd.mcmc, ess.mcmc, acceptance.1, post.burn.in)
		names(output.mcmc) <- c("median", "95.HPD", "ESS", "acceptance.rate", "mcmc.chain")
		class(output.mcmc) <- "motmot.mcmc"
		print(output.mcmc[1:4])
		invisible(output.mcmc)
	}

#' @title Confidence intervals for rate parameters
#' @description Calculates approximate confidence intervals for all rate parameters. CIs are esimated for one rate parameters while fixing others at a given value (usually the maximum likelihood estimate).
#'@description These are reliable (given the asympotic assumptions of the chi-square distribution) if only two groups are being compared but should be regarded only as a rough approximation for =>3 different rate categories. If the rates are correlated the CIs may be underestimated. 
#' @param rateData an object of class \code{rateData}
#' @param MLrate a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal. Normally these will be the maximum likelihood rate estimates.
#' @param fixed A vector stating whether each parameter should be allowed to vary (either \code{FALSE} which results in a start value of 1, or a numeric start value) or should be fixed (\code{TRUE}).
#' @param rateMIN Minimum value for the rate parameters
#' @param rateMAX Maximum value for the rate parameters
#' @param common.mean a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=FALSE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates.
#' @param lambda.est Logical. Estimate Pagel's lambda.
#' @return rateLci Lower confidence interval for rate estimate
#' @return rateUci Upper confidence interval for rate estimate
#' @references Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.
#' @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @author Gavin Thomas, Rob Freckleton
#'@examples
#' data(anolis.data)
#' data(anolis.tree)
#' ## Convert data to class rateData with a rateMatrix object as input
#' anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)
#'
#' anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
#' rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)
#'
#' # A model with a different rate in each of the four groups. The 'fixed' command is used to determine
#' # whether a particular rate is to be constrained or not. Use '1' to fix a group and 'FALSE' to show 
#' # that the parameter is not fixed and should be estimated. The values should be entered in the same
#' # order as the ranking of the groups. That is, group 0 (small islands) takes position one in the 
#' # fixed vector, group 1 (large island trunk crown and trunk ground) takes position 2 and so on. The 
#' # default is to allow each group to take a different mean. 
#'
#' anole.ML <- optim.likRatePhylo(rateData=anolis.rateData, rate=NULL,
#' fixed=c(FALSE,FALSE,FALSE, FALSE),
#' common.mean=FALSE, lambda.est=FALSE)
#'
#' # Confidence intervals for the first two parameters
#'
#' RatePhylo.allCI(rateData=anolis.rateData, MLrate = anole.ML$MLRate, 
#' fixed=c(FALSE, TRUE, TRUE, TRUE), rateMIN = 0.001, rateMAX = 50, 
#' common.mean = FALSE)
#' @export

RatePhylo.allCI <-
function(rateData, MLrate=NULL, fixed=NULL, rateMIN = 0.001, rateMAX = 50, common.mean=FALSE, lambda.est=TRUE) {

	if(is.null(MLrate))  { MLrate <- c(rep(1,length(rateData$Vmat))) } else { MLrate <- MLrate }	
	if(is.null(fixed))  { fixed <- c(rep(FALSE,length(rateData$Vmat) - 1), FALSE) } else { op <- fixed }	

	
	
	n.rate <- length(MLrate)	
	all.CIs <- matrix(nrow=n.rate, ncol=2, dimnames=list(c(1:n.rate), c("Lci", "Uci")))
	
	for(i in 1:n.rate) {
			fix.now <- c(rep(TRUE, n.rate))
			fix.now[i] <- FALSE
	
		if(fixed[i] == FALSE) { 
				CI <- RatePhylo.CI(rateData, MLrate, fix.now, common.mean=common.mean, lambda.est=lambda.est)
				all.CIs[i,1] <- CI[1]
				all.CIs[i,2] <- CI[2]
				} else { all.CIs[i,1] <- NA
					all.CIs[i,2] <- NA }
					}
	
	return(all.CIs)
	}

#' Anolis phenotype data
#'
#' Data on anolis phenotype data from Thomas et al. 2009
#'
#' @docType data
#'
#' @usage data(anolis.data)
#'
#' @format An object of class \code{"data.frame"}.
#'
#' @keywords datasets
#'
#' @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @examples
#' data(anolis.data)
#' head(anolis.data)
"anolis.data"

#' @title Get times for nodes and tips
#' @description Produces branching and tip times for ultrametric and non-ultrametric trees
#' @note \code{nodeTimes} is essentially a re-written version of the \pkg{ape} \code{branching.times}.
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @return Returns a matrix corresponging the phy "edge" matrix showning internal and external node times
#' @author Mark Puttick, Emmanuel Paradis
#' @examples
#' ## Read in phylogeny from Thomas et al. (2009)
#' data(anolis.tree)
#' anolis.node.times <- nodeTimes(phy=anolis.tree)
#' @export

nodeTimes <- function(phy) {
	
	phy <- reorder(phy)
	depBranches <- node.depth.edgelength(phy)
	all <- cbind(depBranches[phy$edge[,1]], depBranches[phy$edge[,2]])
	return(max(depBranches) - all)
	
}

#' @title Phylogenetic tree transformations
#' @description Transforms the branch lengths of a phylo object according to a model of trait evolution (see details).
#' @param y A matrix of trait values.
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param model The model of trait evolution (see details).
#' @param kappa Value of kappa transform.
#' @param lambda Value of lambda transform.
#' @param delta Value of delta transform.
#' @param alpha Value of alpha (OU) transform.
#' @param psi Value of psi transform.
#' @param acdcRate Value of ACDC transform.
#' @param lambda.sp Estimate of speciation (lambda) for the psi models
#' @param nodeIDs Integer - ancestral nodes of clades.
#' @param splitTime A split time (measured from the present, or most recent species) at which a shift in the rate occurs for the "timeSlice" model
#' @param timeRates The rates (from ancient to recent) for the timeSlice model
#' @param rateType If model="clade", a vector specifying if rate shift occurs in a clade ("clade") or on the single branch leading to a clade ("branch").
#' @param branchRates Numeric vector specifying relative rates for individual branches
#' @param cladeRates Numeric vector specifying telative rates for clades or logical to indicate scalar is included in the 'modeslice' model (the scalar is included in the mode.param argument with the 'modeslice' model).
#' @param branchLabels A vector of length equal to the number of internal branches, signifying the which "multiPsi" class it belongs to
#' @param meserr A vector (or matrix) of measurement error for each tip. This is only applicable to univariate analyses. Largely untested - please use cautiously
#' @param cophenetic.dist a cophenetic distance matrix showing the absolute distance between taxa - only applicable for OU models run on non-ultrmetric trees. If null will be calculated internally, but supplying the data can speed up run time
#' @param vcv.matrix a variance-covariance matrix - only applicable for OU models run on non-ultrmetric trees. If null will be calculated internally, but supplying the data can speed up run time
#' @param mode.order The order of modes for the 'modeslice' model. Any combination of 'BM', 'OU', 'acdc', and 'kappa'
#' @param rate.var Allows rate variation in BM modes in the 'modeslice' model
#' @param mode.param Parameters for the modes of evoluton in the 'modeslice' model
#' @details Transforms the branch lengths of a phylo object according to one of the following models:
#' - `model="bm"`: Brownian motion (constant rates random walk)
#' - `model="kappa"`: fits Pagel's kappa by raising all branch lengths to the power kappa. As kappa approaches zero, trait change becomes focused at branching events. For complete phylogenies, if kappa approaches zero this infers speciational trait change.
#' - `model="lambda"`: fits Pagel's lambda to estimate phylogenetic signal by multiplying all internal branches of the tree by lambda, leaving tip branches as their original length (root to tip distances are unchanged).
#' - `model="delta"`: fits Pagel's delta by raising all node depths to the power delta. If delta <1, trait evolution is concentrated early in the tree whereas if delta >1 trait evolution is concentrated towards the tips. Values of delta above one can be difficult to fit reliably.
#' - `model="free"`: fits Mooers et al's (1999) free model where each branch has its own rate of trait evolution. This can be a useful exploratory analysis but it is slow due to the number of parameters, particularly for large trees.
#' - `model="clade"`: fits a model where particular clades are a priori hypothesised to have different rates of trait evolution (see O'Meara et al. 2006; Thomas et al. 2006, 2009). Clades are specified using nodeIDs and are defined as the mrca node. Unique rates for each clade are specified using cladeRates. rateType specifies whether the rate shift occurs in the stem clade or on the single branch leading to the clade.
#' - `model="OU"`: fits an Ornstein-Uhlenbeck model - a random walk with a central tendency proportional to alpha. High values of alpha can be interpreted as evidence of evolutionary constraints, stabilising selection or weak phylogenetic signal.
#' - `model="psi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate (Ingram 2010). Note that 'original nodes' from the full phylogeny can be included as an element on the phylogeny (e.g., phy$orig.node) as well as estimates of 'hidden' speciation (e.g., phy$hidden.speciation) if estimates of extinction (mu) are > 0.
#' - `model="multiPsi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate but allows seperate values of psi fitted to seperate branches (Ingram 2010; Ingram et al. 2016). Note that 'original nodes' from the full phylogeny can be included as an element on the phylogeny (e.g., phy$orig.node) as well as estimates of 'hidden' speciation (e.g., phy$hidden.speciation) if estimates of extinction (mu) are > 0.
#' - `model="ACDC"`: fits a model to in which rates can exponentially increased or decrease through time (Blomberg et al. 2003). If the upper bound is < 0, the model is equivalent to the 'Early Burst' model of Harmon et al. 2010. If a nodeIDs is supplied, the model will fit a ACDC model nested within a clade, with a BM fit to the rest of the tree.
#' - `model="timeSlice"`: A model in which all branch rates change at time(s) in the past.
#' - `model="modeSlice"`: A model in which all branch modes change at a time or times set a priori by the user.
#' @return phy  A phylo object with branch lengths scaled according to the given model and parameters
#' @seealso \code{\link{transformPhylo.ML}}, \code{\link{transformPhylo.ll}}, \code{\link{transformPhylo.MCMC}}
#' @references Ingram T. 2011. Speciation along a depth gradient in a marine adaptive radiation. Proc. Roy. Soc. B. 278, 613-618.
#' @references Ingram T,  Harrison AD, Mahler L, Castaneda MdR, Glor RE, Herrel A, Stuart YE, and Losos JB. Comparative tests of the role of dewlap size in Anolis lizard speciation. Proc. Roy. Soc. B. 283, 20162199.
#' @references Mooers AO, Vamosi S, & Schluter D. 1999. Using phylogenies to test macroevolutionary models of trait evolution: sexual selection and speciation in Cranes (Gruinae). American Naturalist 154, 249-259.
#' @references O'Meara BC, Ane C, Sanderson MJ & Wainwright PC. 2006. Testing for different rates of continuous trait evolution using likelihood. Evolution 60, 922-933
#' @references Pagel M. 1997. Inferring evolutionary processes from phylogenies. Zoologica Scripta 26, 331-348.
#' @references Pagel M. 1999 Inferring the historical patterns of biological evolution. Nature 401, 877-884.
#' @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @author Gavin Thomas, Mark Puttick
#' @examples
# Apply delta transform to anolis tree
#' data(anolis.tree)
#' anolis.treeDelta <- transformPhylo(phy=anolis.tree, model="delta", delta=0.5)
#' @export
#' @import caper

transformPhylo <- function (phy, model = NULL, y = NULL, meserr=NULL, kappa = NULL, lambda = NULL, delta = NULL, alpha = NULL, psi = NULL, lambda.sp = NULL, nodeIDs = NULL, rateType = NULL, branchRates = NULL, cladeRates = NULL,  splitTime = NULL, timeRates = NULL, acdcRate=NULL,  branchLabels = NULL, cophenetic.dist=NULL, vcv.matrix=NULL, mode.order=NULL, mode.param=NULL, rate.var=NULL) {

    n <- length(phy$tip.label)
	
   if (is.null(meserr) == FALSE) {
        if (dim(y)[2] > 1) {
            meserr <- NULL
            (stop("Measurement error can only be included for univariate models. Set meserr to NULL."))
        }
    }
    
    model <- tolower(model)
		
    switch(model, bm = {
		   if (is.null(meserr) == FALSE) {
		   height <- nodeTimes(phy)[1,1]
		   interns <- which(phy$edge[, 2] > n)
		   externs <- which(phy$edge[, 2] <= n)
		   phy$edge.length[externs] <- phy$edge.length[externs] + (meserr ^ 2) / (var(y) / height)[1]
		   } else {
		   phy <- phy
		   }
		   
		   }, kappa = {
		   	
		   	if(is.null(nodeIDs)) {
		   		node <- Ntip(phy) + 1	
		   		} else {
		   		node <- nodeIDs
		   		}
		   	
		   	if (is.null(meserr) == FALSE) {
		   		height <- nodeTimes(phy)[1,1]
		   		interns <- which(phy$edge[, 2] > n)
		   		externs <- which(phy$edge[, 2] <= n)
		   		}
		   		
		   	if(node == (Ntip(phy) + 1)) {
		   		relations_num <- 1:(Nnode(phy) + Ntip(phy))
		   	} else {
		   		relations_num <- c(node.descendents(node, phy))
		   	}
		   	
		   	branches <- match(relations_num, phy$edge[,2])
		   	if(any(is.na(branches))) branches <- branches[complete.cases(branches)]
		   	branches <- sort(branches)
		   	phy$edge.length[branches] <- phy$edge.length[branches] ^ kappa
		   	if (is.null(meserr) == FALSE) {
		   		phy$edge.length[externs] <- phy$edge.length[externs] + (meserr^2)/(var(y)/height)[1]
        		}
		   
		   }, lambda = {
		   if (is.null(meserr) == FALSE) {
		   height <- nodeTimes(phy)[1,1]
		   interns <- which(phy$edge[, 2] > n)
		   externs <- which(phy$edge[, 2] <= n)
		   }
		   if (is.ultrametric(phy)) {
		   rootOrig <- nodeTimes(phy)[1,1]
		   tips <- match(c(1:Ntip(phy)), phy$edge[, 2])
		   phy$edge.length <- phy$edge.length * lambda
		   phy$edge.length[tips] <- phy$edge.length[tips] + (rootOrig * (1 - lambda))
		   }
		   if (is.ultrametric(phy) == FALSE) {
		   tips <- match(c(1:Ntip(phy)), phy$edge[, 2])
		   cladeMat <- caper::clade.matrix(phy)
		   branchHeights <- sapply(1:Ntip(phy), function(x) sum(cladeMat$edge.length[cladeMat$clade.matrix[, x] == 1]))
		   phy$edge.length <- phy$edge.length * lambda
		   phy$edge.length[tips] <- phy$edge.length[tips] + (branchHeights * (1 - lambda))
		   }
		   if (is.null(meserr) == FALSE)	{
		   	phy$edge.length[externs] <- phy$edge.length[externs] +  (meserr^2)/(var(y)/height)[1]
		   	}
		   
		   
		   }, delta = {
		   
		   if(is.null(nodeIDs)) {
		   	node <- Ntip(phy) + 1
		   	} else { 
		   	node <- nodeIDs
		   	}
		   allTimes <- nodeTimes(phy)
		   if(node == phy$edge[1, 1]) {
		   	 height <- allTimes[1,1]
		   } else {
		   	height <- allTimes[which(node == phy$edge[,2]), 2]
		   }
		   	
	   if (is.null(meserr) == FALSE) {
		   	interns <- which(phy$edge[, 2] > n)
		   	externs <- which(phy$edge[, 2] <= n)
		   }
		   
		   times <- allTimes[match(c((Ntip(phy) + 1) : (Nnode(phy) + Ntip(phy))), phy$edge[,1]) , 1]
		   names(times) <- (Ntip(phy) + 1):(Ntip(phy) + Nnode(phy))
		   
		   if(node == (Ntip(phy) + 1)) {
		   		relations_num <- 1:(Nnode(phy) + Ntip(phy))
		   	} else {
		   		relations_num <- c(node.descendents(node, phy))
			}
		   	
		   originTime <- times[which(names(times) == node)][1]
		   branches <- match(relations_num, phy$edge[,2])
		   if(any(is.na(branches))) branches <- branches[complete.cases(branches)]
			
		   times <- originTime - times
		   tips <- Ntip(phy)
		   res <- phy
		   branches <- sort(branches)
		   		   
		   res$edge.length[branches] <- sapply(branches, function(x) {
				bl <- phy$edge.length[x]
		   		age <- times[phy$edge[x, 1] - tips]
		   		(age + bl)^delta - age^delta
		   		}
		   	)
		   	
		   phy <- res
		   if (is.null(meserr) == FALSE)	{
		   	phy$edge.length[externs] <- phy$edge.length[externs] + (meserr^2)/(var(y)/height)[1]
		   	}
		   	
		   # allTimes2 <- nodeTimes(phy)
		   # if(node == phy$edge[1, 1]) {
		   	# height2 <- allTimes2[1,1]
		   # } else {
		   	# height2 <- allTimes2[which(node == phy$edge[,2]), 2]
		   # }
		   # height2 <- allTimes[1,1]
		   # phy$edge.length[branches] <- phy$edge.length[branches] * (height/ height2)
		   		   
		   }, free = {
		   if (is.null(meserr) == FALSE) {
		   height <- nodeTimes(phy)[1,1]
		   interns <- which(phy$edge[, 2] > n)
		   externs <- which(phy$edge[, 2] <= n)
		   }
		   branchRates <- branchRates + (1 - min(branchRates))
		   phy$edge.length <- phy$edge.length * branchRates
		   if (is.null(meserr) == FALSE) {
		   	phy$edge.length[externs] <- phy$edge.length[externs] + (meserr^2)/(var(y)/height)[1]
		   	}
		      
		   }, clade = {
		   if (is.null(meserr) == FALSE) {
		   height <- nodeTimes(phy)[1,1]
		   interns <- which(phy$edge[, 2] > n)
		   externs <- which(phy$edge[, 2] <= n)
		   }
		   if (is.null(rateType)) {
		   rateType <- rep("clade", length(nodeIDs))
		   } else {
		   rateType <- rateType
		   }
		   branchShiftNms <- branchShiftID <- cladeShiftNms <- cladeShiftID <- NULL
		   cladeMembers <- matrix(0, ncol = length(nodeIDs), nrow = length(phy$edge[, 1]))
		   shiftType <- data.frame(rateType, nodeIDs, cladeRates)
		   colnms <- paste(shiftType[, 1], shiftType[, 2], sep = "")
		   shiftType <- data.frame(shiftType, colnms)
		   if (sum(shiftType[, 1] == "clade") > 0) {
		   cladeShiftID <- shiftType[shiftType[, 1] == "clade", 2]
		   cladeShiftNms <- as.character(shiftType[shiftType[, 1] == "clade", 4])
		   cladeMembers[, 1:length(cladeShiftID)] <- cladeIdentity(phy = phy, nodeIDs = cladeShiftID) #cladeMembersObj=cladeMembersObj)
		   }
		   if (sum(shiftType[, 1] == "branch") > 0) {
		   branchShiftID <- shiftType[shiftType[, 1] == "branch", 2]
		   branchShiftNms <- as.character(shiftType[shiftType[, 1] == "branch", 4])
		   }
		   if (is.null(branchShiftNms) == FALSE) { 
		   	colnames(cladeMembers) <- c(cladeShiftNms, rep(NA, length(branchShiftNms))) 
		   	} else  { 
		   	colnames(cladeMembers) <- cladeShiftNms
		   	}
		   	
		   if (sum(shiftType[, 1] == "branch") > 0) {
		   for (i in 1:length(branchShiftID)) {
			   branchID <- which(phy$edge[, 2] == branchShiftID[i])
			   cladeMembers[branchID, ] <- 0
			   cladeMembers[branchID, length(cladeShiftID) + i] <- 1
			   colnames(cladeMembers)[length(cladeShiftID) + i] <- branchShiftNms[i]
			   }
		   }
		   cladeMembers <- as.matrix(cladeMembers[, match(shiftType[, 4], colnames(cladeMembers))])
		   for (i in 1:length(cladeRates)) {
		   	phy$edge.length[cladeMembers[, i] == 1] <- phy$edge.length[cladeMembers[, i] == 1] * cladeRates[i]
		   	}
		   if (is.null(meserr) == FALSE) {
            phy$edge.length[externs] <- phy$edge.length[externs] +(meserr^2)/(var(y)/height)[1]
        	}
		   
		   }, ou = {
		   	
		   is.contemp <- is.ultrametric(phy)
		   
		   if(is.null(nodeIDs)) { 
		   	node <- Ntip(phy) + 1
		   	} else { 
		   	node <- nodeIDs
		   	}
		   	
		   if (is.null(meserr) == FALSE) {
		   		interns <- which(phy$edge[, 2] > n)
		   		externs <- which(phy$edge[, 2] <= n)
		   }
		   
		   times <- nodeTimes(phy)
   		   height <- times[1,1]
		   times <- times[match(c((Ntip(phy) + 1) : (Nnode(phy) + Ntip(phy))), phy$edge[,1]) , 1]
		   names(times) <- (Ntip(phy) + 1):(Ntip(phy) + Nnode(phy))
		   
			if(node == (Ntip(phy) + 1)) {
		   		relations_num <- 1:(Nnode(phy) + Ntip(phy))
		   	} else {
		   		relations_num <- c(node.descendents(node, phy))
		   	}

		   originTime <- times[which(names(times) == node)][1]
		   branches <- match(relations_num, phy$edge[,2])
		   branches <- sort(branches)
		   if(any(is.na(branches))) {
		   	branches <- branches[complete.cases(branches)]
		   	}
			times2 <- nodeTimes(phy)[,1]
		
			phy2 <- phy
			
			if(is.contemp) {
				edge.length.branches <- sapply(branches, function(i) {
					Tmax <- originTime
					bl <- phy$edge.length[i]
					age <- times2[i]
					t1 = originTime - age
					t2 = t1 + bl
					(1/(2 * alpha)) * exp(-2 * alpha * (Tmax - t2)) * (1 - exp(-2 * alpha * t2)) - (1/(2 * alpha)) * exp(-2 * alpha * (Tmax - t1)) * (1 - exp(-2 * alpha * t1))
					}
				)
				phy2$edge.length[branches] <- edge.length.branches
				phy <- phy2
				if (is.null(meserr) == FALSE) {
					phy$edge.length[externs] <- phy$edge.length[externs] + (meserr^2)/(var(y)/height)[1]
		   		}
		   		
			} else {
   	
   				if(is.null(cophenetic.dist)) {
   					t_ij <- cophenetic(phy)
	   				} else {
    				t_ij <- cophenetic.dist
    				}
    			if(is.null(vcv.matrix)) {
    				c_ij <- vcv(phy)
    				} else {
    				c_ij <- vcv.matrix
    				}
    			vcv.out <- (1 / (2 * alpha)) * exp(-alpha * t_ij) * (1 - exp(-2 * alpha * c_ij))
    			if (is.null(meserr) == FALSE) { 
    				diag(vcv.out) <- diag(vcv.out) + (meserr^2)/(var(y)/height)[1]
    			}
    			phy <- vcv.out
			}
		   
		 	
		   		   
		}, psi = {
        if (is.null(meserr) == FALSE) {
            height <- nodeTimes(phy)[1,1]
            interns <- which(phy$edge[, 2] > n)
            externs <- which(phy$edge[, 2] <= n)
        }
		if (is.null(phy$orig.node)) {
			orig.node <- rep(1, length(phy$edge.length))
		} else {
			orig.node <- phy$orig.node
		}
		if (!is.null(phy$hidden.speciation)) orig.node <- orig.node + phy$hidden.speciation
        phy2 <- phy
        phy2$edge.length <- (psi / ( 2* lambda.sp)) * orig.node + (1 - psi) * phy$edge.length
        phy <- phy2
        if (is.null(meserr) == FALSE) {
            phy$edge.length[externs] <- phy$edge.length[externs] + (meserr^2) / (var(y) / height)[1]
        }
        
        
         }, multipsi = {
        if (is.null(meserr) == FALSE) {
            height <- nodeTimes(phy)[1,1]
            interns <- which(phy$edge[, 2] > n)
            externs <- which(phy$edge[, 2] <= n)
        }
		if(is.null(phy$orig.node)){
			orig.node <- rep(1, length(phy$edge.length))
		}else{
			orig.node <- phy$orig.node
		}
		if(!is.null(phy$hidden.speciation)) orig.node <- orig.node + phy$hidden.speciation
        phy2 <- phy
		states <- levels(factor(branchLabels))
		for(i in 1:length(states)) {
            phy2$edge.length[branchLabels==states[i]] <- (psi[i]/(2*lambda.sp)) * orig.node[branchLabels==states[i]] + (1 - psi[i]) * phy$edge.length[branchLabels==states[i]]
		}
        phy <- phy2
        if (is.null(meserr) == FALSE) {
            phy$edge.length[externs] <- phy$edge.length[externs] + (meserr^2)/(var(y)/height)[1]
        	}
        }, 
        
        
        timeslice = {
		   if (is.null(meserr) == FALSE) {
		   height <- nodeTimes(phy)[1,1]
		   interns <- which(phy$edge[, 2] > n)
		   externs <- which(phy$edge[, 2] <= n)
		   }

		   splitTime <- nodeTimes(phy)[1,1] - sort(splitTime, TRUE)
		   sliceLengths <- sliceTree(phy, splitTime)
		   phy$edge.length <- as.vector(timeRates %*% t(sliceLengths))
		   if (is.null(meserr) == FALSE) {
		   	phy$edge.length[externs] <- phy$edge.length[externs] + (meserr ^2 )/(var(y) / height)[1]
		   	}
		   } ,
		   
		   
		modeslice = {
		  		   
		  		   
		  phy.in <- phy
		   if(!is.null(splitTime)) {
				inputSplitTime <- splitTime
				splitTime <- nodeTimes(phy)[1,1] - sort(splitTime, TRUE)
				sliceLengths <- sliceTree(phy, splitTime)
				slice.phy <- lapply(1:length(mode.order), function(sx) {
					phy2 <- phy
					phy2$edge.length <-  sliceLengths[,sx]
					phy2
					}
				)
				all.splits <- c(max(nodeTimes(phy)), inputSplitTime)	
		 	} else {
		 		slice.phy <- list()
		 		slice.phy[[1]] <- phy
		 		all.splits <- max(nodeTimes(phy))	
		 		
		 	}
		 
		 
		all.vcvs <- list()
		count <- 1 	
		for(x in 1:length(mode.order)) {
			
			#bm
			if(mode.order[x] == "bm") {
				if(rate.var) {
					phy2 <- slice.phy[[x]]
					all.vcvs[[x]] <- vcv(phy2) * mode.param[count]
					count <- count + 1
				} else {
					phy2 <- slice.phy[[x]]
					all.vcvs[[x]] <- vcv(phy2)
				}
				
			}
			
			# OU 
			if(mode.order[x] == "ou") {
				alpha <- mode.param[count]
				phy2 <- slice.phy[[x]]
				t_ij <- cophenetic(phy2)
				c_ij <- vcv(phy2)
				all.vcvs[[x]] <- (1 / (2 * alpha)) * exp(-alpha * t_ij) * (1 - exp(-2 * alpha * c_ij))							
				count <- count + 1
			}
			
			

			
			if(mode.order[x] == "kappa") {
				kappa <- mode.param[count]
				phy2 <- slice.phy[[x]]
				phy2$edge.length <- phy2$edge.length ^ kappa
				all.vcvs[[x]] <- vcv(phy2) 
				count <- count + 1
			}
			
			if(mode.order[x] == "acdc") {
				phy2 <- slice.phy[[x]]
				if(isTRUE(cladeRates)) {
					phy2$edge.length <- phy2$edge.length * mode.param[count]
					count <- count + 1
					}
				acdcRate <- mode.param[count]
				originTime <-  all.splits[x]
				if(is.null(originTime)) originTime <- nodeTimes(phy)[1,1]
				edges <- phy2$edge.length
				if(!is.null(splitTime)) {
					times <- sliceLengths[,x]
				} else {
					times <- nodeTimes(phy)[,1]
					}
				
				edgeACDC <- sapply(1:length(edges), function(i) {
					bl <- edges[i]
					age <- times[i]
					t1 <- originTime - age
					t2 <- t1 + bl
					(exp(acdcRate * t2) - exp(acdcRate * t1)) / (acdcRate)
					}
				)
				phy2$edge.length <- edgeACDC
				all.vcvs[[x]] <- vcv(phy2)
				count <- count + 1
				}
			}


		
			
		phy <- all.vcvs[[1]]
		if(length(all.vcvs) > 1) for(u in 2:length(all.vcvs)) phy <- phy + all.vcvs[[u]]
		
		if (is.null(meserr) == FALSE) {
			height <- nodeTimes(phy.in)[1,1]
		   	diag(phy) <- diag(phy) + (meserr ^2 )/(var(y) / height)[1]
		   	}
		   } ,
		   
		   
		acdc = {
		   	
			if(is.null(nodeIDs)) node <- Ntip(phy) + 1 else node <- nodeIDs
			times <- nodeTimes(phy)[,1]
			if (is.null(meserr) == FALSE) {
				height <- times[1]
				interns <- which(phy$edge[, 2] > n)
				externs <- which(phy$edge[, 2] <= n)
				}
			
			names(times) <- phy$edge[,1]

			if(node == (Ntip(phy) + 1)) {
				relations_num <- 1:(Nnode(phy) + Ntip(phy))
				} else {
				relations_num <- c(node, node.descendents(node, phy))
				}

			originTime <- times[which(names(times) == node)][1]
			branches <- match(relations_num, phy$edge[,2])
			if(any(is.na(branches))) branches <- branches[complete.cases(branches)]
			branches <- sort(branches)
			times <- nodeTimes(phy)[,1]
			
			if(is.null(cladeRates) == FALSE) phy$edge.length[branches] <- phy$edge.length[branches] * cladeRates
			edgeACDC <- sapply(branches, function(i) {
				bl <- phy$edge.length[i]
				age <- times[i]
				t1 <- originTime - age
				t2 <- t1 + bl
				(exp(acdcRate * t2) - exp(acdcRate * t1))/(acdcRate)
				})
				
			phy$edge.length[branches] <- edgeACDC	
			if (is.null(meserr) == FALSE)	 {
				phy$edge.length[externs] <- phy$edge.length[externs] + (meserr^2)/(var(y)/height)[1]
					}
				}
		)
		return(phy)
}

#' Models Of Trait Macroevolution On Trees
#'
#' Functions for fitting models of trait evolution on phylogenies for continuous
#' traits. The majority of functions described in Thomas and Freckleton (2012)
#' and include functions that allow for tests of variation in the rates of trait
#' evolution.
#'
#' @name motmot-package
#' @aliases motmot
#'
#' @docType package
#'
#' @author Mark Puttick <marknputtick@gmail.com>.
#' @author Gavin Thomas.
#' @author Travis Ingram.
#' @author Magnus Clarke.
#' @author Rob Freckleton.
#' @author David Orme.
#' @author Emmanuel Paradis.
#'
#' @keywords package
#'
#' @references
#' Thomas G. H., & Freckleton R. 2012.
#'  MOTMOT: models of trait macroevolution on trees.
#'  _Methods in Ecology and Evolution_ 3, 145-151.
#'  \doi{10.1111/j.2041-210X.2011.00132.x}
#'
#' @importFrom Rcpp sourceCpp
#' @useDynLib motmot, .registration=TRUE
"_PACKAGE"

#' @title Calculate multiple-test cut-off
#' @description Calculate the log-likelihood, AIC, or AICc cut-off necessary for type-one error to reach acceptable levels
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param n Number of simulations
#' @param mc.cores Number of cores for parallel processing for linux-type systems (not applicable to Windows)
#' @param model Evolutionary model, typically "tm1", "tm2", or "timeSlice", which is used to test the empirical data
#' @param measure Measure used to summarise the model. One of "lnL" (log-likelihood), "AIC", or "AICc"
#' @param alpha.error Target for the desired type-one error rate for the model (default 0.05)
#' @param ... Arguments to be passed to \code{\link{transformPhylo.ML}} which should be identical to the model applied to empirical data
#' @return The cut-off requred to produce an type-one error rate equal to quantile.cut.off (default = 0.05) when data are simulated under Brownian motion, and these data are analysed under the appropriate model.
#' @author Mark Puttick
#' @seealso \code{\link{transformPhylo.ML}}, \code{\link{transformPhylo.ll}}, \code{\link{transformPhylo}}, \code{\link{transformPhylo.MCMC}}
#' @examples
#' data(anolis.tree)
#' set.seed(393)
#' # calculated necessary AICc cut-off to reduce type-one error to 5% 
#' # for a timeSlice model with a split at 30Ma (only 5 simulations used,
#' # it's recommend to use 1000 for analyses)
#' calcCutOff(anolis.tree, n=5, model="timeSlice", splitTime=30)
#' @export

calcCutOff <- function(phy, n=1000, mc.cores=1, model, measure="AICc", alpha.error=0.05, ...) {

	quiet <- function(x) { 
	  sink(tempfile()) 
	  on.exit(sink()) 
	  invisible(force(x)) 
	} 
	
	sim.data <- transformPhylo.sim(phy, n=n, model="bm")
	y.out <- quiet(
				parallel::mclapply(1:n, mc.cores=mc.cores, function(xx) {
					model.fit <- transformPhylo.ML(as.matrix(sim.data[, xx]), phy, model = model, ...)
					if(model == "timeSlice") {
						model.fit <- model.fit[[1]]
						model.fit[, measure]
					} else {
						model.fit <- c(transformPhylo.ML(as.matrix(sim.data[, xx]), phy, model = "BM")[[measure]], model.fit[[measure]])
						model.fit
            		}
            	}	
			)
		)
	
	y.out <- matrix(unlist(y.out), nrow=2)
	aicc.add <- quantile(-apply(y.out, 2, diff), (1-alpha.error))
	return(aicc.add)
	}

#' @title Sort data and remove missing entries for tree and trait data
#' @description Plots a phylogeny with lines representing the value of a continuous trait
#' @param y A matrix of trait values with taxon names as rownames. Missing values should be NA
#' @param phy An object of class \code{phylo} or \code{multiPhylo}  (see \pkg{ape})
#' @param data.name If null the first column of y is assummed as the trait, otherwise if y is a matrix with more than one column either the name of the column or the number of the column must be supplied by data.name
#' @param log.trait Logical. If \code{TRUE}, data are log-transformed
#' @param pass.ultrametric Although trees that are believed to be ultrametric to pass the function \code{is.ultrametric} in \pkg{ape}
#' @return phy Tree with missing data pruned
#' @return trait Rearranged data with missing species removed
#' @author Mark Puttick
#' @examples 
#' data(anolis.tree)
#' data(anolis.data)
#' attach(anolis.data)
#' male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
#' any(is.na(male.length[,1]))
#' data.sorted <- sortTraitData(anolis.tree, male.length)
#' phy <- data.sorted[[1]]
#' male.length <- data.sorted[[2]]
#' @export

sortTraitData <- function (phy, y, data.name=NULL, log.trait = TRUE, pass.ultrametric=FALSE) 
{
	
	trait.names <- rownames(y)
	if(is.matrix(y) || is.data.frame(y)) {
		if(dim(y)[2] > 1) {
			if(dim(y)[2] > 1 && is.null(data.name)) stop("not sure which column to take - please provide a data.name")
			trait.data <- as.matrix(y[,data.name], dimnames=list(rownames(y)), nrow=nrow(y))
			} else {
			trait.data <- y	
			}
		} else {
			trait.data <- y
		}

    if (inherits(phy, "multiPhylo")) {
        tree <- phy
        phy <- phy[[1]]
        multi.phy <- TRUE
    } else {
        multi.phy <- FALSE
    }
    phy.names <- phy$tip.label
    missing <- apply(trait.data, 2, function(x) which(is.na(x)))
    missing <- c(unique(unlist(missing)))
    if(length(missing) > 0) {
    	dat.trait <- matrix(trait.data[-missing, ], ncol = ncol(trait.data), dimnames = list(trait.names[-missing]))
    } else {
    	dat.trait <- matrix(trait.data, ncol = ncol(trait.data), dimnames = list(trait.names))
    	}
    rm.tip <- match(phy.names, rownames(dat.trait))
    if (multi.phy) {
        red.phy <- lapply(tree, function(x) drop.tip(x, phy.names[which(is.na(rm.tip))]))
        class(red.phy) <- "multiPhylo"
        trait <- dat.trait[match(red.phy[[1]]$tip.label, rownames(dat.trait)), 
            ]
    } else {
        red.phy <- drop.tip(phy, phy.names[which(is.na(rm.tip))])
        trait <- dat.trait[match(red.phy$tip.label, rownames(dat.trait)), ]
    }
    
    if(pass.ultrametric == TRUE)  {
		outer <- red.phy$edge[,2]
		node.times <- nodeTimes(red.phy)
		externs <- which(outer <= Ntip(red.phy))
		red.phy$edge.length[externs] <- red.phy$edge.length[externs] + (node.times[externs,2] - 1e-7)
	}

    if (log.trait) trait <- log(trait)
    out <- list()
    out$phy <- red.phy
    out$trait <- as.matrix(trait)
    return(out)
}

#' @title Calculation of Brownian (co)variance using independent contrasts.
#' @description Calculates the Brownian variance (single trait) or variance-covariance matrix (mutliple traits) using phylogenetically independent contrasts.
#' @param x A continuous trait
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param estimator Should Brownian variance (or covariance) be based on the unbiased ("unbiased" - default) or maximum likelihood ("ML") estimator.
#' @return brownianVariance Brownian variance (or covariance for multiple traits) given the data and phylogeny
#' @references Felsenstein J. 1973. Maximum-likelihood estimation of evolutionary trees from continuous characters. Am. J. Hum. Genet. 25, 471-492.
#' @references Felsenstein J. 1985. Phylogenies and the comparative method. American Naturalist 125, 1-15.
#' @references Freckleton RP & Jetz W. 2009. Space versus phylogeny: disentangling phylogenetic and spatial signals in comparative data. Proc. Roy. Soc. B 276, 21-30.
#' @author Gavin Thomas, Rob Freckleton
#' @examples 
#' data(anolis.tree)
#' data(anolis.data)
#' ## calculate Brownian variance of female SVL
#' female.svl <- matrix(anolis.data[,"Female_SVL"], 
#' dimnames=list(rownames(anolis.data)))
#' input.data <- sortTraitData(phy=anolis.tree, y=female.svl, log.trait=TRUE)
#' phyloCovar(x=input.data$trait, phy = input.data$phy)
#' @export

phyloCovar <- function(x, phy, estimator="unbiased") {
		
		if (is.matrix(x)==FALSE) { stop("Trait data must be a matrix with taxon names as row names")}
		
		n <- length(phy$tip.label)
		phy <- reorder(phy, order = "pruningwise")
	
		x <- as.matrix(x[phy$tip.label,])
		
		contrasts <- apply(x, MARGIN=2, FUN=pic.motmot, phy=phy)
		rawVariances <- c(contrasts[[1]]$contr[,2], contrasts[[1]]$V)
		rawContrasts <- matrix(NA, nrow=n, ncol=ncol(x))
		
		for (i in 1:ncol(x)) {
			rawContrasts[,i] <- c(contrasts[[i]]$contr[,1],0)
			}
			
		brCov <- matrix(NA, nrow=ncol(x), ncol=ncol(x))
		
	if (estimator=="unbiased") {
		for (i in 1:ncol(x)) {
			for (k in 1:ncol(x)) {	
				brCov[k,i] <- brCov[i,k] <- crossprod(rawContrasts[,k]/sqrt(rawVariances), rawContrasts[,i]/sqrt(rawVariances)) / (n-1)
								}}
	}
	
	if (estimator=="ML") {
		for (i in 1:ncol(x)) {
			for (k in 1:ncol(x)) {	
				brCov[k,i] <- brCov[i,k] <- crossprod(rawContrasts[,k]/sqrt(rawVariances), rawContrasts[,i]/sqrt(rawVariances)) / (n)
			}}
	}
		
		return(brCov)
		}
		

#' @title Fast PLGS estimation based on contrasts
#' @description Estimates regression parameters for a phylogenetic generalised least-squares analysis using the fast constrasts method (Felsenstein 1973; 1985; Freckleton 2012). This implementation is applicable for continuous traits only and not factors
#' @param formula A model formula with continuous variables only
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param y A matrix of trait values with rownames corresponding to the phy tip labels, and column names corresponding to the formula variable names
#' @param lambda Default is "ML" meaning the phylogenetic signal of the response variable will be estimated using restricted Maximum likelihood. If a numeric value between 0-1 is provided this will be used in the calculation of regression coefficients
#' @param return.intercept.stat Logical. If \code{TRUE} the standard errors, t value, and p value of the estimated Intercept is provided for comparison with output from \code{\link[caper]{pgls}} from \pkg{caper} etc.,. Default is \code{FALSE} as this slows the function as it involves constructing and calculating the inverse of the phy variance-covariance matrix, and based on contrasts the design matrix column of ones would have zero contrasts.
#' @param meserr A vector (or matrix) of measurement error for each tip. This is only applicable to univariate analyses.
#' @return A list containing the model, model summary, intercept, estimate of Lambda, model log-Likelihood, model AIC
#' @useDynLib motmot
#' @importFrom Rcpp sourceCpp
#' @seealso \code{\link[caper]{pgls}}
#' @author Mark N Puttick and Rob Freckleton
#' @examples
#' # Data and phylogeny
#' data(anolis.tree)
#' anolis.tree$node.label <- NULL
#' lm.data <- transformPhylo.sim(phy=anolis.tree, n=2, model="bm")
#' dat <- data.frame(x = lm.data[,1], y = lm.data[,2],
#' names = anolis.tree$tip, row.names = anolis.tree$tip)
#' picModel <- pic.pgls(formula=y ~  x,
#' phy=anolis.tree, y = dat, lambda=1,
#' return.intercept.stat=FALSE)
#' @export

pic.pgls <- function(formula, phy, y, lambda="ML", return.intercept.stat=FALSE, meserr=NULL) {
	
	formula.break <- strsplit(as.character(formula), "~")
	
	response <- attr(terms.formula(formula, data=y), "term.labels")
	model.data <- model.frame(formula, y, na.action = na.pass)
	class.data <- !all(sapply(model.data, function(x) is.numeric(x)[1]))
	if(class.data) stop("please provide numerical variables only")
	
	
	yy <- strsplit(as.character(formula), "~")[[2]]
	response.xx <- paste(attr(terms.formula(formula, data=y), "term.labels"), collapse=" + ")
	if(response.xx == "") response.xx <- 1
	formula <- as.formula(paste(yy, " ~ 0 + ", response.xx))	
	
	response.var <- as.matrix(model.data[,1])
	explanatory.var <- as.matrix(model.data[,-1])
	rownames(explanatory.var) <- rownames(response.var) <- rownames(model.data)
	model.var <- cbind(response.var, explanatory.var)
	
	if(lambda == "ML") {	
		lambda.model <- transformPhylo.ML(y=response.var, phy=phy, model='lambda', returnPhy=TRUE, modelCIs=FALSE, meserr=meserr)
		phy.lambda <- lambda.model$lambdaPhy
		lambda <- lambda.model$Lambda[1]
	} else {
		lambda.model <- transformPhylo.ll(y=response.var, phy=phy, model='lambda', lambda=lambda, meserr=meserr)		
		phy.lambda <- transformPhylo(phy=phy, model='lambda', lambda=lambda)		
		}
	
	pic.data <- data.frame(apply(model.var, 2, function(x) pic(x, phy.lambda)))
	colnames(pic.data) <- colnames(model.data)
	model <- lm(formula, pic.data, y = TRUE, x=TRUE)
	model.y <- transformPhylo.ML(y=as.matrix(model.var[,1]), phy.lambda, model="bm", meserr=meserr)
	rst.y <- model.y$root.state
	rst.x <- 0
	if(dim(model.var)[2] > 1) for(u in 2:dim(model.var)[2]) rst.x[u-1] <- transformPhylo.ML(y=as.matrix(model.var[,u]), phy.lambda, model="bm", meserr=meserr)$root.state
	intercept <- rst.y - sum(rst.x *  model[[1]])
	vars <- pic(model.var[,1], phy.lambda, var.contrasts = TRUE)[,2]
	sigma2 <- model.y$brownianVariance
	V <- pic.motmot(x=model.var[,1], phy=phy.lambda)$V
	u <- resid(model)
	n <- length(u)
	k <- length(coef(model)) 
	logLik <- -0.5 * ((n + 1) * log( 2 * pi * sigma2) + sum(log(vars)) + sum(u ^ 2) / sigma2)
	ll.model <- logLik - 0.5 * log(V)
	
	
	output <- list()
	output$model <- model
	output$model.summary <- summary(model)
	output$intercept <- intercept
	output$lambda <- lambda
	output$logLikelihood <- ll.model
	output$AIC <- (-2 * ll.model) + k
	
	if(return.intercept.stat) {	
		n <- dim(y)[1]
		x.values <- matrix(c(rep(1, n), model.var[,-1]), ncol=dim(y)[2])
		errMat <- t(x.values) %*% solve(vcv(phy.lambda)) %*% x.values
		errMat <- solve(errMat) * summary(model)[[6]] ^ 2
		sterr <- diag(errMat)
		sterr <- sqrt(sterr)
		intercept.error <- sterr[1]
		t.value <- abs((0-intercept)/intercept.error)
		p.value <- pt(t.value, dim(pic.data)[1] - dim(pic.data)[2], lower.tail=FALSE) * 2
		output$intercept <- c("intercept"=intercept, "std.error"=intercept.error, "t.value"=t.value, "p.value"=p.value)
		
	}
	return(output)
	print(output$model.summary)
}

#' Finch phenotype data
#'
#' Finch data from Clarke et al. 2017
#'
#' @docType data
#'
#' @usage data(finches)
#'
#' @format An object of class \code{"data.frame"}.
#'
#' @keywords datasets
#'
#' @references Clarke M, Thomas GH, Freckleton RP. 2017. Trait evolution in adaptive radiations: modelling and measuring interspecific competition on phylogenies. The American Naturalist. 189, 121-137.
#' @examples
#' data(finches)
#' head(finch.data)
"finch.data"

#' @title Conversion among data and phylogeny objects
#' @description Transforms the expected variance and covariances among species according to hypotheses of rate variation between lineages.
#' @param rateData an object of class \code{rateData}
#' @param rate a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal.
#' @return retMat Rate-transformed variance covariance matrix
#' @references Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.
#' @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @author Gavin Thomas
#' @examples
#'  data(anolis.tree)
#'  data(anolis.data)
#'  ## Convert data to class rateData with a rateMatrix object as input
#'  anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)
#'  anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
#'  rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)
#'  
#'  # Tranform the expected variance covariance matrix so that the rates in the first and last 
#'  # categories are equal (both 1) whereas the rate in the second category is twice as fast (2) and 
#'  # the rate in the third category is ten times slower.
#'  
#'  trans.anolis.rateData <- transformRateMatrix(rateData=anolis.rateData, rate = c(1,2,0.1,1))
#' @export


transformRateMatrix <- function(rateData, rate=NULL) {
			V <- rateData$Vmat
			
			if(is.null(rate))  { rate <- c(rep(1,length(rateData$Vmat))) } else { rate <- rate }	
			
			nV <- length(rate)
			
			if(length(rate) != length(rateData$Vmat)){stop("The number of rates defined differs from the number of rate matrices")}
			
			v1 <- V[[1]]		
			rateMats <- vector(mode="list", length = nV)
			retMat <- matrix(0, nrow = dim(v1)[1], ncol = dim(v1)[2])
			
			for(i in 1:nV) {
			   rateMats[[i]] <- rate[i] * V[[i]]  
			   retMat <- retMat + rateMats[[i]]
					}
	
	return(retMat)}

#' @title Phylogenetic tree transformations
#' @description Simulates trait data on a tree using a specified model of evolution (see details).
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param n Number of simulations
#' @param x Vector, matrix or data.frame (with taxon names as names or rownames) of categories for each species. Only applicable if model="mixedRate" 
#' @param model The model of trait evolution (see details).
#' @param returnNodes Logical. If TRUE, alongside the tip values all node values are returned corresponding to APE's edge.matrix for the tree.
#' @param kappa Value of kappa transform.
#' @param lambda Value of lambda transform.
#' @param delta Value of delta transform.
#' @param alpha Value of alpha (OU) transform.
#' @param psi Value of psi transform.  Note that 'original nodes' from the full phylogeny can be included as an element on the phylogeny (e.g., phy$orig.node) as well as estimates of 'hidden' speciation (e.g., phy$hidden.speciation) if estimates of extinction (mu) are > 0.
#' @param lambda.sp Estimate of speciation (lambda) for the psi models
#' @param splitTime A split time (measured from the present, or most recent species) at which a shift in the rate occurs for the "timeSlice" model
#' @param timeRates The rates (from ancient to recent) for the timeSlice model
#' @param nodeIDs Integer - ancestral nodes of clades.
#' @param rateType If model="clade", a vector specifying if rate shift occurs in a clade ("clade") or on the single branch leading to a clade ("branch").
#' @param acdcRate Value of ACDC transform.
#' @param trend value of the expectation mean change through time
#' @param trend.anc.state the expected ancestal state for the trend model (default is 0)
#' @param branchLabels Branches on which different psi parameters are estimated in the "multipsi" model.
#' @param branchRates Numeric vector specifying relative rates for individual branches
#' @param rate a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. 
#' @param group.means a vector of the relative difference in means between rate categories, expressed as a scalar applied to the expected standard deviation (see Ricklefs 2006)
#' @param cladeRates Numeric vector specifying telative rates for clades or logical to indicate scalar is included in the 'modeslice' model (the scalar is included in the mode.param argument with the 'modeslice' model).
#' @param rate.var Allows rate variation in BM modes in the 'modeslice' model
#' @param mode.order The order of modes for the 'modeslice' model. Any combination of 'BM', 'OU', 'acdc', and 'kappa'

#' @return Returns a matrix of simulated dated with taxon names as rownames (number of columns=n).
#' @references Ricklefs RE. 2006. Time, species, and the generation of trait variation in clades. Systematic Biology 55, 151-159.
#' @references Ricklefs RE. 2006. Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030
#' @author Gavin Thomas, Mark Puttick
#' @seealso \code{\link{transformPhylo.ML}}, \code{\link{transformPhylo.ll}}, \code{\link{transformPhylo}}, \code{\link{transformPhylo.MCMC}}
#' @import mvtnorm
#' @examples
#' data(anolis.tree)
#' data(anolis.data)
#' 
#' # Simulate 10 sets of data with kappa=0.1 using the anolis tree
#' sim.dat1 <- transformPhylo.sim(phy=anolis.tree, n=10, model="kappa", kappa=0.1)
#' 
#' # Simulate 10 sets of data where rates and means differ between to the categories defined by "x"
#' x <- anolis.data$geo_ecomorph
#' names(x) <-  rownames(anolis.data)
#' sim.dat2 <- transformPhylo.sim(phy=anolis.tree, n=10, x=x, model="mixedRate", rate=c(1,1,2,4),
#' group.means=c(0,5,0,0))
#' @export
#' @import caper

transformPhylo.sim <- function(phy, n=1, x=NULL, model=NULL, returnNodes=FALSE, kappa=NULL, lambda=NULL, delta=NULL, alpha=NULL, psi=NULL, acdcRate=NULL, lambda.sp = NULL, trend=NULL, trend.anc.state=0, nodeIDs=NULL, rateType=NULL, cladeRates=NULL, branchRates=NULL, rate=NULL, group.means=NULL, splitTime=NULL, timeRates=NULL, branchLabels = NULL, rate.var=NULL, mode.order=NULL) {
	
	  model <- tolower(model)
	all.models <- c("bm", "trend", "kappa", "lambda", "delta", "free", "clade", "ou", "acdc", "psi", "multipsi", "timeslice", "mixedrate")
	if (any(is.na((match(model, all.models))))) stop(paste(model, "not recognised - please provide one of", paste0(all.models, collapse = ", ")))
	
	y.sim.function <- function(phy, return.node=FALSE) {	
		lengths <- phy$edge.length
		empty.mat <- matrix(NA, nrow=nrow(phy$edge), ncol=2)
		start.nodes <- which(phy$edge[,1] == Ntip(phy) + 1)
		empty.mat[start.nodes,1] <- rnorm(1)
	
		while(length(start.nodes) != 0) {
			next.node <- phy$edge[start.nodes,2]
			var.to.add <- rnorm(length(start.nodes), sd=sqrt(lengths[start.nodes]))
			empty.mat[start.nodes,2] <- empty.mat[start.nodes,1] + var.to.add
			start.nodes.int <- c()
			for(xx in 1:length(next.node)) {
				n.n <- which(phy$edge[,1] == next.node[xx])
				empty.mat[n.n,1] <- empty.mat[start.nodes[xx], 2]
				}
			start.nodes <- intersect(which(complete.cases(empty.mat[, 1])), which(!complete.cases(empty.mat[, 2])))
			}	
		tips <- which(phy$edge[, 2] <= Ntip(phy))
		y.out <- list()
		y.out$y <- matrix(empty.mat[tips, 2], dimnames=list(phy$tip.label))
		if(return.node) y.out$all.nodes <- empty.mat
		return(y.out)
		}

	switch(model,		  
		   
		   "bm" = {
					transformPhy <- phy
					ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
					ydum <- sapply(ydum.all, function(re) re[[1]])
					rownames(ydum) <- phy$tip.label
					if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
					},
			
			 "trend" = {
					transformPhy <- phy
					phyMat <- caper::VCV.array(transformPhy)
					attr(phyMat, "class") <- "matrix"
					tip.distance <- diag(vcv(transformPhy))
					trend.mean <- trend.anc.state + (tip.distance * trend)
					ydum <- as.matrix(t(mvtnorm::rmvnorm(n, mean=trend.mean, sigma = phyMat)))
					rownames(ydum) <- rownames(phyMat)
					if(returnNodes) warning("returnNodes not applicable to trend model, sorry")
					},
		   
		   "kappa" = {
					transformPhy <- transformPhylo(phy=phy, model="kappa", kappa=kappa, nodeIDs=nodeIDs)
					ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
					ydum <- sapply(ydum.all, function(re) re[[1]])
					rownames(ydum) <- phy$tip.label
					if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
					},
		   
		   "lambda" = {
					transformPhy <- transformPhylo(phy=phy, model="lambda", lambda=lambda)
					ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
					ydum <- sapply(ydum.all, function(re) re[[1]])
					rownames(ydum) <- phy$tip.label
					if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
					},
		   
		   "delta" = {
					transformPhy <- transformPhylo(phy=phy, model="delta", delta=delta, nodeIDs=nodeIDs)
					ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
					ydum <- sapply(ydum.all, function(re) re[[1]])
					rownames(ydum) <- phy$tip.label
					if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
					},
					
		   "free" = {
					transformPhy <- transformPhylo(phy=phy, model="free", branchRates=branchRates)
					ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
					ydum <- sapply(ydum.all, function(re) re[[1]])
					rownames(ydum) <- phy$tip.label
					if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
					},
		   
		   "clade" = {
					transformPhy <- transformPhylo(phy=phy, model="clade", nodeIDs=nodeIDs, cladeRates=cladeRates, rateType=rateType)
					ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
					ydum <- sapply(ydum.all, function(re) re[[1]])
					rownames(ydum) <- phy$tip.label
					if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
					},
		   
		   "ou" = {
		   	
		   			if (!is.ultrametric(phy)) {
      			        cophenetic.dist <- cophenetic.phylo(phy)
      			        vcv.matrix <- caper::VCV.array(phy)
      			        phyMat <- transformPhylo(phy=phy, model="OU", alpha=alpha, nodeIDs=nodeIDs, cophenetic.dist=cophenetic.dist, vcv.matrix=vcv.matrix)
      			        class(phyMat) <- "matrix"
      			        ydum <- as.matrix(t(mvtnorm::rmvnorm(n, sigma = phyMat)))
      			        rownames(ydum) <- rownames(phyMat)
      			        if(returnNodes) warning("returnNodes not applicable to OU model with non-ultrametric trees, sorry")
      			        } else {
      			        transformPhy <- transformPhylo(phy=phy, model="OU", alpha=alpha, nodeIDs=nodeIDs)
						ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
						ydum <- sapply(ydum.all, function(re) re[[1]])
						rownames(ydum) <- phy$tip.label
						if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
						}
					},
			"acdc" = {
					transformPhy <- transformPhylo(phy=phy, model="ACDC", acdcRate=acdcRate, nodeIDs=nodeIDs, cladeRates=cladeRates)
					ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
					ydum <- sapply(ydum.all, function(re) re[[1]])
					rownames(ydum) <- phy$tip.label
					if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
					},
		   "psi" = {
					transformPhy <- transformPhylo(phy = phy, model = "psi", psi = psi, lambda.sp = lambda.sp)
					ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
					ydum <- sapply(ydum.all, function(re) re[[1]])
					rownames(ydum) <- phy$tip.label
					if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
					},
			"multipsi" = {
       				transformPhy <- transformPhylo(phy = phy, model = "multipsi", psi = psi, lambda.sp = lambda.sp, branchLabels = branchLabels)
       				ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
					ydum <- sapply(ydum.all, function(re) re[[1]])
					rownames(ydum) <- phy$tip.label
					if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
    				},
					
			"timeslice" = {
				phy2 <- phy	   
		   		transformPhy <- transformPhylo(phy=phy, model="timeSlice", splitTime=splitTime, timeRates=timeRates)
		   		ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
				ydum <- sapply(ydum.all, function(re) re[[1]])
				rownames(ydum) <- phy$tip.label
				if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
		   		} ,
		   		
		   	"modeslice" = {
				phy2 <- phy	   
		   		transformPhy <- transformPhylo(phy=phy, model="modeSlice", mode.order=mode.order, splitTime=splitTime, cladeRates=cladeRates, rate.var=rate.var)
		   		ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
				ydum <- sapply(ydum.all, function(re) re[[1]])
				rownames(ydum) <- phy$tip.label
				if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
		   		} ,

			"mixedrate" = {
        			x <- as.matrix(x)
		        dat <- data.frame(x = x, y = rep(0, length(x[, 1])))
		        rateData <- as.rateData(y = "y", x = "x", rateMatrix = NULL, phy = phy, data = dat)
		        V <- transformRateMatrix(rateData, rate = rate)
		        expect.sd <- sqrt(mean(V[upper.tri(V)]))
		        if (is.null(group.means)) {
		            ydum.all <- lapply(1:n, function(ix) y.sim.function(transformPhy, return.node=returnNodes))
					ydum <- sapply(ydum.all, function(re) re[[1]])
					rownames(ydum) <- phy$tip.label
					if(returnNodes) ydum <- list(trait.values=ydum, all.nodes=lapply(ydum.all, function(re) re[[2]]))
		        } else {
		            x.means <- unique(rateData$x)
		            n.means <- length(x.means)
		            samp.means <- rep(NA, length(rateData$x))
		            ydum <- vector(mode = "list", length = length(group.means))
		            for (i in 1:n.means) {
		                samp.means[which(rateData$x == (i - 1))] <- rep(0 + (expect.sd * group.means[i]), length(which(rateData$x == (i - 1))))
		            }
		            ydum <- as.matrix(t(mvtnorm::rmvnorm(n, mean = samp.means, sigma = (V))))
		            rownames(ydum) <- rownames(V)
		       		if(returnNodes) warning("returnNodes not applicable to mixedrate model with non-ultrametric trees unique nodes, sorry")
		       		}
		       	}
		   )
	return(ydum)
}

#' @title Simulate character displacement data
#' @description Simulates data under a Brownian motion or character displacement model
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param n.steps Number of time steps the for the simulation (default = 1000 time steps).
#' @param sigma The value of Brownian variance in the simulation
#' @param a The strength of competition between inter-specific lineages
#' @param ntraits Number of traits to be simulated
#' @param sympatry an optional matrix giving the time that each pair of species starts to interact
#' @param allopatry an optional matrix giving the times when species stop interacting
#' @param trait.lim an optional parameter that puts limits on the available trait-space, preventing trait values with magnitude greater than the value of lim
#' @return A list containing the the simulated data (tval) showing the sigma, a, mean gap and gap standard deviation. Additionally, if used, the user input sympatry (symp) and/or allopatry (allo) matrices
#' @useDynLib motmot
#' @importFrom Rcpp sourceCpp
#' @seealso \code{\link{chr.disp.param}}, \code{\link{chr.disp.lrt}}
#' @references Clarke M, Thomas GH, Freckleton RP. 2017. Trait evolution in adaptive radiations: modelling and measuring interspecific competition on phylogenies. The American Naturalist. 189, 121-137.
#' @author Magnus Clarke and Mark Puttick
#' @examples
#' ## import finch data form Clarke et al. (2017)
#' data(finches)
#' emp.tree <- finch.tree
#' emp.data <- finch.data
#' ## simulate small amount of data 
#' ## (example only - many more datasets are required for accuracy)
#' sim.data <- chr.disp.sim(emp.tree, n.steps=100,
#' sigma=1, a=2, ntraits=1, sympatry=NA, allopatry=NA, trait.lim=NA)
#' @export

chr.disp.sim <- function(phy, n.steps=1000, sigma=1, a=0, ntraits=1, sympatry=NA, allopatry=NA, trait.lim=NA) {
    
    phy.names <- phy$tip.label
    phy.chr.disp <- checktree(phy)
    num_tips <- length(phy.chr.disp$data_order)
    splitting_nodes <- phy.chr.disp$splitting_nodes - 1
    times <- phy.chr.disp$times
    tval <- rep(0, num_tips * ntraits)
    dt <- nodeTimes(phy)[1,1] / n.steps

    symp <- vectortime(sympatry, 0, num_tips)
    allo <- vectortime(allopatry, 99e9, num_tips )
    if(is.na(trait.lim)) trait.lim <- 9e99

    result <- .C("pathsim", ntip=as.integer(num_tips), dt=as.double(dt), rate = as.double(sigma^2), a=as.double(a), r_intervals=as.double(times), splitters=as.integer(splitting_nodes), tval = as.double(tval), ntraits=as.integer(ntraits), symp=as.double(symp), allo=as.double(allo), lim=as.numeric(trait.lim))

    result$tval <- as.matrix(reorder_data(phy.chr.disp, result$tval, ntraits))
    rownames(result$tval) <- phy.names
    if(all(result$symp == 0)) result$symp <- NULL
    if(all(result$allo == 0)) result$allo <- NULL
    result
}

#' @title Calculation of Brownian variance.
#' @description This function calculates the phylogenetic variance (Brownian variance, or rate) of the data given the tree and model of evolution 
#' @param rateData an object of class \code{rateData}
#' @param rate a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal.
#' @param common.mean a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=FALSE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates.
#' @param lambda.est Logical. Fit Pagel's lambda.
#' @param lambda Numeric value for lambda from 0-1.
#' @param meserr Logical. Include measurement error.
#' @return phylo.var  phylogenetic variance (Brownian variance)
#' @references Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.
#'  @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @author Gavin Thomas, Rob Freckleton
#' @examples
#'  ## Read in phylogeny and data from Thomas et al. (2009)
#' data(anolis.tree)
#' data(anolis.data)
#' 
#' ## Convert data to class rateData with a rateMatrix object as input
#' anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)
#' 
#' anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
#' rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)
#' 
#' # A model with a different rate in each of the four groups. The 'fixed' command is used to determine
#' # whether a particular rate is to be constrained or not. Use '1' to fix a group and 'FALSE' to show
#' # that the parameter is not fixed and should be estimated. The values should be entered in the same 
#' # order as the ranking of the groups. That is, group 0 (small islands) takes position one in the 
#' # fixed vector, group 1 (large island trunk crown and trunk ground) takes position 2 and so on. 
#' # The default is to allow each group to take a different mean. 
#'
#' phyloVar(anolis.rateData, rate=c(1,2,1,1), common.mean=FALSE)
#' # common mean for all groups
#' phyloVar(anolis.rateData, rate=c(1,2,1,1), common.mean=TRUE)
#' @export

phyloVar <-
function(rateData, rate=NULL, common.mean=FALSE, lambda.est=TRUE, lambda=1, meserr=FALSE) {
	
		if(is.null(rate))  { rate <- c(rep(1,length(rateData$Vmat))) } else { rate <- rate }	
	
		if(length(rate) != length(rateData$Vmat)){stop("The number of rates defined differs from the number of rate matrices")}
			
		y <- rateData$y
		x <- as.factor(rateData$x)
						
		if (common.mean==FALSE) {k <- nlevels(x)} else {k <- 1}
			
		V <- transformRateMatrix(rateData, rate)
		if (lambda.est & !meserr) {
			v.temp <- V
			diag(v.temp) <- rep(0, dim(V)[1])
			V.lam <- lambda*v.temp
			diag(V.lam) <- diag(V)
			V <- V.lam
		}
		
		x <- make.anc(y, x)
			
		if(common.mean==FALSE) {x <- x} else { x <- rep(1, length(x[,1]))}
		
		mu <- phyloMean(rateData, rate, common.mean=common.mean, lambda.est, lambda)
			
			iV <- solve(V)
			e <- y - x %*% mu
			s2 <- crossprod(e, iV %*% e)
			n <- length(y) 
			phylo.var <- ( s2 / (n - k) )
			return(phylo.var)
			}

# allCladeMembers (internal function)
#
# This is an internal function to generate an \code{allCladeMembersMatrix}
# phy phylogeny in \pkg{ape} \code{phylo} format
# cladeMembersMatrix
# Gavin Thomas

allCladeMembers <- function(phy) {
	
	nodeIDs <- c(1:Ntip(phy), (Ntip(phy)+2):(Ntip(phy) + Nnode(phy)))
	k <- length(nodeIDs)
   cladeMembersMatrix <-  sapply(nodeIDs, function(k) {
    	nodeShiftID <- c(k, node.descendents(x = k , phy = phy))
    	as.numeric(phy$edge[, 2] %in% nodeShiftID)
    	}
    )
    return(cladeMembersMatrix)
}

# Identify branches (including tips) descended from a node (internal function).
# description Internal function to get presence absence of descendent branches from a vector of node numbers. The descendents include the branch leading to the focal node (i.e. node defines the stem group no crown group
# phy An object of class \code{phylo} (see \pkg{ape}).
# nodeIDs Vector of node numbers (positive integers).
# cladeMembersObj Matrix of clade membership
# The function returns a matrix of unique presences given the selected node. If the selected nodes are nested then presences are only recorded for the least inclusive node.
# return matrix Matrix of unique presences for each node id
# author Gavin Thomas
# examples
# ## Read in phylogeny and data from Thomas et al. (2009)
# data(anolis.tree)
# data(anolis.data)
# cladeIdentityMatrix <- cladeIdentity(phy=anolis.tree, nodeIDs=170)

cladeIdentity <- function (phy, nodeIDs, cladeMembersObj=NULL) 
{
	
	k <- length(nodeIDs)
	
    if(is.null(cladeMembersObj)) {
		cladeMembers <- matrix(NA, ncol = k, nrow = length(phy$edge[, 
														   1]))
		for (i in 1:k) {
			nodeShiftID <- c(nodeIDs[i], node.descendents(x = nodeIDs[i], phy = phy))
			cladeMembers[, i] <- as.numeric(phy$edge[, 2] %in% nodeShiftID)
		}
	}
	
	if (is.null(cladeMembersObj)==FALSE) {
		allNodes <- c(1:Ntip(phy), (Ntip(phy)+2):(length(phy$edge.length)+1))   #######
		cladeMembers <- as.matrix(cladeMembersObj[,match(nodeIDs, allNodes)])
	}
	
    originalOrder <- colSums(cladeMembers)
    richnessOrder <- sort(originalOrder, decreasing = FALSE, 
						  index.return = TRUE)
    cladeMembersOrdered <- matrix(cladeMembers[, richnessOrder$ix], 
								  ncol = length(nodeIDs))
	
	if (k>1) {
		for (i in 2:k){
			if (i ==2 ) {	cladeMembersOrdered[,i] <- cladeMembersOrdered[,i] - cladeMembersOrdered[,1:(i-1)]}
			else {cladeMembersOrdered[,i] <- cladeMembersOrdered[,i] - rowSums(cladeMembersOrdered[,1:(i-1)])}
		}
	}
	
    cladeMembers <- cladeMembersOrdered[, sort(richnessOrder$ix, index.return = TRUE)$ix]
    cladeMembers <- matrix(cladeMembers, ncol = length(nodeIDs))
    return(cladeMembers)
}


# Create design matrix (internal function)
# This is an internal function to generate the design matrix required to define different means for each hypothesised rate category.
# y The response variable - typically a continuous trait.
# x The explanatory (discrete) variable used to define the hypothesised rate categories. Can be specified as a column number or column name.
# data A data frame containing (minimally) the x and y variables as columns with species names as rownames.
# common.mean a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=FALSE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates..
# author Gavin Thomas

make.anc <-
function(y, x, data=NULL, common.mean=FALSE) {
	
	if(is.factor(x) == "FALSE"){
		stop("The discrete trait must be a factor")	
	}
	
	m <- model.frame(y ~ x, data)
	x <- model.matrix(y ~ x, m)
	return(x)
}


#  Name check
# This is an internal function to check if names in a trait vector or matrix match a tree
# phy a phylogeny in \pkg{ape} phylo format
# data A matrix or vector of named trait data corresponding to species in phy
# Gavin Thomas

name.check <- function (phy, data) 
{
    if (is.vector(data)) {
        data.names <- names(data)
    }
    else {
        data.names <- rownames(data)
    }
    t <- phy$tip.label
    r1 <- t[is.na(match(t, data.names))]
    r2 <- data.names[is.na(match(data.names, t))]
    r <- list(sort(r1), sort(r2))
    names(r) <- cbind("tree_not_data", "data_not_tree")
    if (length(r1) == 0 && length(r2) == 0) 
	return("OK")
    else return(r)
}

# Phylogenetically independent contrasts (internal)
# Calculates phylogenetically independent contrasts.
# param x A matrix of trait values with taxon names as rownames.
# param phy An object of class \code{phylo} (see \pkg{ape}).
# details Extracts values for contrasts, expected variances using contrasts by calling \code{pic}.
# return contr A matrix with two columns containing raw contrasts in the first column and their expected variances in the second column.
# return root.v Expected variances of branches either side of root
# return V Expected variance at the root 
# references Felsenstein J. 1985. Phylogenies and the comparative method. American Naturalist 125, 1-15.
# author Gavin Thomas, Rob Freckleton, Emmanuel Paradis

pic.motmot <- function (x, phy) {
out <- pic(x, phy, rescaled.tree=TRUE, var.contrasts=TRUE, scaled=FALSE)
   contr <- out[[1]]	
	nb.tip <- length(phy$tip.label)
	idx <- which(out[[2]]$edge[,1] == (nb.tip+1) )
	
	root.v <- out[[2]]$edge.length[idx]
	V <- prod(root.v)/(sum(root.v))
  
  return(list(contr = contr, root.v = root.v, V = V))
}

# remove species occuring before time in the past (internal function) 
# removes tips and lineages after a time in the past
# param phy An object of class \code{phylo} (see \pkg{ape}).
# param traitData data associated with the species
# param keepByTime an age at which to keep preserve tips before the present (time = 0)
# param tol edge length precision in time cut (default = 1e-08)
# return a list with the prunedPhylogeny and a prunedData
# Mark Puttick

removeNonBin <- function(phy, traitData, keepByTime=0, tol=1e-08){

    all.times <- nodeTimes(phy)
    tip.time <- all.times[which(phy$edge[,2] <= Ntip(phy)) , 2]
	non.bin <- which(tip.time > (keepByTime - tol))
	traitLess <- traitData[ - non.bin]
   	phylo <- drop.tip(phy, non.bin)
	return(list(prunedPhy=phylo, prunedData=traitLess))
}



# Sample hidden speciation events along branches of a tree (internal function)
# Uses estimated speciation and extinction rates to sample the number of speciation events 'hidden' by subsequent extinction on each branch of a tree following Bokma (2008). For use with the \code{psi} and \code{multipsi} models. 
# param phy An object of class \code{phylo} (see \pkg{ape}).
# param lambda.sp Estimate of the rate of speciation "lambda"
# param mu.ext Estimate of the rate of extinction "mu"
# param useMean A logical indicating whether to output the average or expected number of hidden speciation events per branch, which may be non-integer (if \code{TRUE}), or to sample an integer number on each branch from a Poisson distribution (if \code{FALSE}, the default)
# details The expected number of hidden speciation events are calculated for each branch given its start and end times, and estimates of lambda and mu which are assumed to be constant across the tree. To properly account for uncertainty in the effect of extinction on the number of nodes affecting each branch of a tree, it may be appropriate to repeat model-fitting on many realizations of \code{Sobs} on the tree of interest (similar to evaluating phylogenetic uncertainty)
# eturn Phylogenetic tree in \code{phylo} format, with an added element \code{Sobs}, a vector of numbers of hidden speciation events per branch, in the same order as the branches in the \code{phylo} object
# author Travis Ingram

sampleHiddenSp <- function (phy, lambda.sp = NULL, mu.ext = NULL, useMean = FALSE) 
{
    if (is.null(lambda.sp) | is.null(mu.ext)) stop("Please provide values for lambda and mu ")  
    phy$node.label <- NULL
    edge.node.times <- nodeTimes(phy)
    to <- edge.node.times[ , 1]
    te <- edge.node.times[ , 2]
    te[te < 0] <- 0
    if (mu.ext > 0) {
        if (lambda.sp == mu.ext) {
            expSh <- 2 * lambda.sp * (to - te) + 2 * log((1 + lambda.sp * te) / (1 + lambda.sp * to))
        } else {
            expSh <- 2 * lambda.sp * (to - te) + 2 * log((lambda.sp * exp(te *  (lambda.sp - mu.ext)) - mu.ext) / (lambda.sp * exp(to * (lambda.sp - mu.ext)) - mu.ext))
        }
        if (useMean) {
            hidden.speciation <- expSh
        } else {
            hidden.speciation <- rpois(length(expSh), expSh)
        }
	} else {
        hidden.speciation <- rep(0, length(to))
    }
    phy$hidden.speciation <- hidden.speciation
    phy
}

# Slice tree (internal function)
# Split tree for the time slice function 
# phy Phylogenetic tree in phylo format
# splitTime Split time in the past
# Branch lengths of sliced tree
# Mark Puttick

sliceTree <- function(phy, splitTime) {
  branchLengths <- c()
  node.times.2 <- node.times <- nodeTimes(phy)
  startTime <- node.times[1, 1]
  split.these.times <- c(splitTime, startTime)
  treeBrLength <- phy$edge.length
  
  for (p in 1:length(split.these.times)) {
    startTime.2 <- startTime - split.these.times[p]
    br.l <- rep(0, dim(node.times)[1])
    these.br <- which(node.times[, 1] > startTime.2)
    br.l[these.br] <- node.times[these.br, 1] - startTime.2
    treeBrLength <- node.times[, 1] - br.l
    intNode <- which(node.times[these.br, 2] != 0)
    test.finish <-
      which(treeBrLength[these.br][intNode] <= node.times[these.br, 2][intNode])
    node.times[these.br, 1] <- treeBrLength[these.br]
    if (length(test.finish) > 0) {
      br.l[these.br][intNode][test.finish] <-
        node.times.2[these.br, 1][intNode][test.finish] - node.times[these.br, 2][intNode][test.finish]
      node.times[these.br, 1][intNode][test.finish] <- 0
    }
    node.times.2 <- node.times
    branchLengths <- cbind(branchLengths, br.l)
  }
  return(branchLengths)
}


# birth death from APE for non-ultrametric trees

birthdeath_motmot <- function(phy) {
	if (!inherits(phy, "phylo")) 
        stop("object \"phy\" is not of class \"phylo\"")
    N <- length(phy$tip.label)
    x <- c(NA, nodeTimes(phy)[match(unique(phy$edge[,1]),phy$edge[,1]),1])
    dev <- function(a, r) {
        if (r < 0 || a > 1) 
            return(1e+100)
        -2 * (lfactorial(N - 1) + (N - 2) * log(r) + r * sum(x[3:N]) + 
            N * log(1 - a) - 2 * sum(log(exp(r * x[2:N]) - a)))
    }
    out <- nlm(function(p) dev(p[1], p[2]), c(0.1, 0.2), hessian = TRUE)
    if (out$estimate[1] < 0) {
        out <- nlm(function(p) dev(0, p), 0.2, hessian = TRUE)
        para <- c(0, out$estimate)
		inv.hessian <- try(solve(out$hessian))
		se <- if (inherits(inv.hessian, "try-error")) 
			NA
		else sqrt(diag(inv.hessian))
        se <- c(0, se)
    } else {
        para <- out$estimate
		inv.hessian <- try(solve(out$hessian))
		se <- if (inherits(inv.hessian, "try-error")) 
			c(NA, NA)
		else sqrt(diag(inv.hessian))
    }
    Dev <- out$minimum
    foo <- function(which, s) {
        i <- 0.1
        if (which == 1) {
            p <- para[1] + s * i
            bar <- function() dev(p, para[2])
        } else {
            p <- para[2] + s * i
            bar <- function() dev(para[1], p)
        }
        while (i > 1e-09) {
            while (bar() < Dev + 3.84) p <- p + s * i
            p <- p - s * i
            i <- i/10
        }
        p
    }
    CI <- mapply(foo, c(1, 2, 1, 2), c(-1, -1, 1, 1))
    dim(CI) <- c(2, 2)
    names(para) <- names(se) <- rownames(CI) <- c("d/b", "b-d")
    colnames(CI) <- c("lo", "up")
    obj <- list(tree = deparse(substitute(phy)), N = N, dev = Dev, 
        para = para, se = se, CI = CI)
    class(obj) <- "birthdeath"
    obj
    }
    
    
    	# my function to get edge lengths from a vcv matrix
	vcv2edge.length <- function(vcv, tre) {
		mrca.phy <- mrca(tre)
		inner <- tre$edge[,1]
		outer <- tre$edge[,2]
		out.len <- c()
		for(i in 1:length(tre$edge[,1])) {
			vcv.in <- inner[i]
			vcv.out <- outer[i]
			if(vcv.in == (Ntip(tre) + 1)) {
				out.len[i] <- vcv[which(mrca.phy == vcv.out)][1]
			} else {
				out.len[i] <- vcv[which(mrca.phy == vcv.out)][1] - vcv[which(mrca.phy == vcv.in)][1]
				}
			}
		return(out.len)
	}
	

# @title Internal function
# @description Internal function. Constructor to allow fixing of rate parameters.
# @param rateData an object of class \code{rateData}
# @param fixed A vector stating whether each parameter should be allowed to vary (either \code{FALSE} which results in a start value of 1, or a numeric start value) or should be fixed (\code{TRUE}).
# @param common.mean a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=FALSE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates..
# @param lambda.est Logical - Logical. Fit Pagel's lambda.
# @param meserr an object of class "rateData"
# @return Returns a function to be passed to \code{optim.likRatePhylo}
# @author Gavin Thomas
# @export

make.likRatePhylo <-
function(rateData, fixed, common.mean=FALSE, lambda.est, meserr) {
	
	op <- c(fixed, !lambda.est, !meserr)
	
	function(params){
		
	
		op[c(!fixed, lambda.est, meserr)] <- params[c(!fixed, lambda.est, meserr)]
		

		y <- rateData$y
		x <- as.factor(rateData$x)
		if (common.mean==FALSE) {k <- nlevels(x)} else {k <- 1}
		
		V <- rateData$Vmat
		v1 <- V[[1]]
		nV <- length(rateData$Vmat)
		
		rateMats <- vector(mode="list", length = nV)
		retMat <- matrix(0, nrow = dim(v1)[1], ncol = dim(v1)[2])
		
		for(i in 1:nV) {
			rateMats[[i]] <- op[i] * V[[i]]  
			retMat <- retMat + rateMats[[i]]
		}
										
			x <- make.anc(y, x)
			
			if(common.mean==FALSE) {
				x <- x} else { x <- rep(1, length(x[,1]))}
									
				V <- retMat
				v.temp <- V
				diag(v.temp) <- rep(0, dim(V)[1])
			
				V.lam <- op[nV+1]*v.temp
				diag(V.lam) <- diag(V)
				V <- V.lam
				if (meserr==TRUE) {	diag(V) <- diag(V) + rateData$meserr/op[nV+2] } 

				logDetV <- determinant(V)$modulus
				
				iV <- solve(V)
				xVix <- crossprod(x, iV %*% x)
				xViy <- crossprod(x, iV %*% y)
				mu <- solve(xVix) %*% xViy 
				
				e <- y - x %*% mu
				s2 <- crossprod(e, iV %*% e)
				n <- length(y) 
				phylo.var <- ( s2 / (n - k) )
				
				n <- length(y)
				ll <- -n / 2.0 * log( 2 * pi) - n / 2.0 * log(phylo.var) - logDetV / 2.0 - (n - k)/2.0
				
				lik.RatePhylo <- ( list(ll = ll, mu = mu, phylo.var = phylo.var, lambda=op[nV+2]) )
				return(-1 * lik.RatePhylo$ll)	
		}
	
	}


# Confidence intervals for rate parameters
# Calculates approximate confidence intervals for all rate parameters. CIs are esimated for one rate parameters while fixing others at a given value (usually the maximum likelihood estimate).
# These are reliable (given the asympotic assumptions of the chi-square distribution) if only two groups are being compared but should be regarded only as a rough approximation for =>3 different rate categories. If the rates are correlated the CIs may be underestimated.
# param rateData an object of class \code{rateDate}
# param MLrate a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal. Normally these will be the maximum likelihood rate estimates.
# param fixed A vector stating whether each parameter should be allowed to vary (either \code{FALSE} which results in a start value of 1, or a numeric start value) or should be fixed (\code{TRUE}).
# param rateMIN Minimum value for the rate parameters
# param rateMAX Maximum value for the rate parameters
# param common.mean a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=FALSE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates.
# param lambda.est Logical. Estimate Pagel's lambda.
# return rateLci Lower confidence interval for rate estimate
# return rateUci Upper confidence interval for rate estimate
# Gavin Thomas, Rob Freckleton
# data(anolis.tree)
# data(anolis.data)
# 
# ## Convert data to class rateData with a rateMatrix object as input
# anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)
# anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
# rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)
#
# # A model with a different rate in each of the four groups. The 'fixed' command is used to determine
# # determine whether a particular rate is to be constrained or not. Use '1' to fix a group and
# # 'FALSE' to show that the parameter is not fixed and should be estimated. The values
# #  should be entered in the same order as the ranking of the groups. That is, group
# # 0 (small islands) takes position one in the fixed vector, group 1 (large island trunk crown 
# # and trunk ground) takes position 2 and so on. The default is to allow each group 
# # to take a different mean. 
#
#anole.ML <- optim.likRatePhylo(rateData=anolis.rateData, rate=NULL, 
#fixed=c(FALSE,FALSE,FALSE, FALSE), common.mean=FALSE, lambda.est=FALSE) # ML estimates
#
# # Confidence intervals for the first parameters
#
# RatePhylo.CI(rateData=anolis.rateData, MLrate = anole.ML$MLRate, 
# fixed=c(FALSE, TRUE, TRUE, TRUE), rateMIN = 0.001, rateMAX = 50, 
# common.mean = FALSE)

RatePhylo.CI <-
function(rateData, MLrate=NULL, fixed, rateMIN = 0.001, rateMAX = 50, common.mean=FALSE, lambda.est=TRUE) {

	if(is.null(MLrate))  { MLrate <- c(rep(1,length(rateData$Vmat))) } else { MLrate <- MLrate }	
	
	MLrate <- as.numeric(format(MLrate))

	ML <- likRatePhylo(rateData, MLrate, common.mean=common.mean, lambda.est=lambda.est)$ll
	
		fixed.rates <- data.frame(MLrate, fixed)
		use.rate.ind <- which(fixed.rates$fixed==FALSE)
		use.rate <- fixed.rates$MLrate[use.rate.ind]
	
		var.fun <- function(vary.rate) {
		
				fixed.rates$MLrate[use.rate.ind] <- vary.rate
				test.rate <- fixed.rates$MLrate
				ll <- likRatePhylo(rateData, test.rate, common.mean=common.mean, lambda.est=lambda.est)$ll
		
		return( ll - ML + 1.92)
		 }
		 
	if(var.fun(rateMIN) < 0) { 
			rateLci <- uniroot(var.fun, interval = c(rateMIN, use.rate))$root 
			}
	if(var.fun(rateMAX) < 0) {
			rateUci <- uniroot(var.fun, interval = c(use.rate, rateMAX))$root
			}
			return(c(rateLci=rateLci, rateUci=rateUci))
			}


# #' @title timeTravelPhy (internal function) 
# #'
# #' @description removes tips and lineages after a time in the past
# #' @param phy An object of class "phylo" (see ape package).
# #' @param node nodes arising more recently than the cut time
# #' @param nodeEstimate trait the number of descendants arising from the nodes
# #' @param timeCut position at which to cut the phylogeny
# #' @param traits Logical. Include trait values in the output tree
# #' @return the pruned phylogeny and a 'tipObject' of the number of lineages found in the pruned branches
# #' @author Mark Puttick

timeTravelPhy <- function(phy, node, nodeEstimate, timeCut, traits=TRUE){

	countDesAll <- c()
	node.in <- match(unique(phy$edge[,1]), phy$edge[, 1])
	originalNodeTimes <- nodeTimes(phy)[node.in, 1]
  	names(originalNodeTimes) <- unique(phy$edge[ ,1])
	internal <- phy$edge[, 1]
	external <- phy$edge[, 2]

	if(length(node) > 0) {

		for (i in 1:length(node)) {
			branchOne <- which(internal == node[i])
			countDes <- ext <- external[branchOne]
			isTerminal <- match(internal, ext)
			terminalCheck <- which(is.na(isTerminal) == FALSE)
			loopStop <- any(terminalCheck)
			
			while (loopStop == TRUE) {
					ext <- external[terminalCheck]
					countDes <- c(countDes, ext)
					isTerminal <- match(internal, ext)
					terminalCheck <- which(is.na(isTerminal) == FALSE)
					loopStop <- any(terminalCheck)
				}

			alreadyCounted <- which(is.na(match(countDes, countDesAll)) == FALSE)
			if (sum(alreadyCounted) > 0) countDes <- countDes[-alreadyCounted]
			countDesAll <- c(countDes, countDesAll)
			}

	tre2 <- phy
	tipsInSample <- countDesAll[which(countDesAll <= Ntip(phy))]
	desInMat <- match(countDesAll, phy$edge[,2])
	tre2$edge <- phy$edge[-desInMat, ]
	keepN <- match(tre2$edge[, 2], node)
	nodeToTips <- tre2$edge[complete.cases(keepN), 2]
	nodalPosition <- which(is.na(keepN) != TRUE)
	originalNodes <- unique(tre2$edge[, 1])
	for (k in 1:length(node)) tre2$edge[which(tre2$edge[ ,2] == node[k]), 2] <- NA
		
	naMat <- which(is.na(tre2$edge[, 2]))
	tipsRemain <- which(tre2$edge[, 2] <= Ntip(phy))
	keeperTips <- tre2$edge[tipsRemain, 2]

	
	tre2$edge[which(tre2$edge[, 2] <= Ntip(phy)), 2] <- NA
	tre2$edge.length <- phy$edge.length[-desInMat]
	internals <- tre2$edge[ ,1]
	tipsInTre2 <- length(which(is.na(tre2$edge[ , 2])))
	n_node <- tipsInTre2 + 1
	nodeTodal <- seq(tre2$edge[1,1], tre2$edge[1,1] + dim(tre2$edge)[1] / 2 - 1, by=1)
	tre2NodeTotal <- match(tre2$edge[,1], nodeTodal)
	nodeTodalNode <- match(nodeTodal, tre2$edge[,1])
	notFoundNode <- nodeTodal[which(is.na(nodeTodalNode))]
	uniqueTre2Int <- unique(tre2$edge[which(is.na(tre2NodeTotal)), 1])
	for(y in 1:length(uniqueTre2Int)) tre2$edge[which(tre2$edge[,1] == uniqueTre2Int[y]), 1] <- notFoundNode[y]
	
	takeValue <- tre2$edge[1, 1] - n_node
	tre2$edge[ ,1] <- tre2$edge[,1] - takeValue
	addOnNodes <- dim(tre2$edge)[1] / 2 - 1
	internalNewTree <- rep(seq(n_node, n_node + addOnNodes, 1), each=2)
	externalNew <- rep(unique(tre2$edge[ ,1]), each=2)
	internalNewMatch <- match(tre2$edge[ ,1], externalNew)
	tre2$edge[,1] <- internalNewTree[internalNewMatch]
	buildInt <- which(complete.cases(tre2$edge[ ,2]) == T)
	extNode <- n_node + 1
	desNodes <- seq(from=extNode, to=extNode + length(buildInt) - 1, 1)
	tre2$edge[buildInt, 2] <- desNodes
	tre2$edge[which(is.na(tre2$edge[,2])), 2] <- 1:tipsInTre2
	apeTree <- list(edge=tre2$edge, tip.label=c(1:tipsInTre2), Nnode=tipsInTre2-1, edge.length=tre2$edge.length)
	class(apeTree) <- "phylo"
	
	toKeep <- apeTree$edge[naMat, 2]
	tipsToKeep <- match(c(1:tipsInTre2), toKeep)
	
	new.tip <- 1:Ntip(apeTree)
	new.tip[-apeTree$edge[naMat, 2]] <- phy$tip.label[-tipsInSample]
	new.tip[apeTree$edge[naMat, 2]] <- paste0("node_", unique(phy$edge[match(nodeToTips, phy$edge[,2]), 2]))
	apeTree$tip.label <- new.tip
	
	if(traits == TRUE) {
		
		keepTipPhy <- match(keeperTips, phy$edge[,2])
		keepTipPhyValue <- nodeEstimate[keepTipPhy]
		newTips <- match(nodeToTips, phy$edge[,2])
		tipObject <- rep(NA, Ntip(apeTree))
		tipObject[which(is.na(tipsToKeep))] <- as.numeric(keepTipPhyValue)	
		needNewTips <- which(is.na(tipObject))
		ageOld <- match(nodeToTips, as.numeric(names(originalNodeTimes)))

		for(p in 1:length(needNewTips)){
			origNode_n <- originalNodeTimes[ageOld][p]		
			nodeN <- nodeToTips[p]
			pastNode <- phy$edge[which(phy$edge[,2] == nodeN), 1]
			pastLength <- phy$edge.length[which(phy$edge[,2] == nodeN)]
			ancAge <- origNode_n + pastLength			
			decAge <- origNode_n
			findNear <- which(c(ancAge - timeCut, - (decAge - timeCut)) == min(c(ancAge - timeCut, - (decAge - timeCut))))
			nearestNode <- which(phy$edge[,2] == c(pastNode, nodeN)[findNear])
			tipObject[needNewTips[p]] <- as.numeric(nodeEstimate[nearestNode])[1]
			}
		
		names(tipObject) <- NULL
		}
	
	} else {
	
	apeTree <- phy	
	tipObject <- nodeEstimate[which(phy$edge[,2] <= Ntip(phy))]
	
	}
	
	node.in.ape <- match(unique(apeTree$edge[,1]), apeTree$edge[,1])
	all.ages.ape <- nodeTimes(apeTree)
	nodeTimesApe <- all.ages.ape[node.in.ape,1]
  	names(nodeTimesApe) <- unique(apeTree$edge[,1])
	tipTimesApe <- all.ages.ape[which(apeTree$edge[,2] <= Ntip(apeTree)), 2] 
	names(tipTimesApe) <- apeTree$edge[which(apeTree$edge[,2] <= Ntip(apeTree)), 2]
	cutOffAge <- nodeTimes(phy)[1,1] - timeCut
	cutOff <- nodeTimes(apeTree)[1,1] - cutOffAge
	preCutOff <- which(tipTimesApe < cutOff)
	preCutOffTips <- match(as.numeric(names(preCutOff)), apeTree$edge[ ,2])
	tipsPreCut <- tipTimesApe[preCutOff] - cutOff
	tipsInApe <- match(preCutOff, apeTree$edge[,2])
	
	apeTree$edge.length[preCutOffTips] <- apeTree$edge.length[preCutOffTips] + tipsPreCut

	if(traits == TRUE) return(list(phy=apeTree, tipData=tipObject))
	if(traits == FALSE) return(phy=apeTree)
}


reml.lik <- function(node.state, bm.rate, vcv.phylo, y) {
	n <- ncol(vcv.phylo)
	s.b <- (t(y - rep(node.state, n)) %*% (solve(vcv.phylo)) %*% (y - rep(node.state, n)) / bm.rate)[1,1]
	ll.in <- -0.5 * ((n) * log(2 * pi * bm.rate) + log(det(vcv.phylo)) + s.b)
	return(ll.in )
}
      
mu.mean <- function(vcv.phy, y.in) {
	x <- rep(1, ncol(vcv.phy))
	iV <- solve(vcv.phy)
	xVix <- crossprod(x, iV %*% x)
	xViy <- crossprod(x, iV %*% y.in)
	mu <- solve(xVix) %*% xViy
	return(mu)
	}

sig.sq <- function(mu, vcv.phy, y) {
	n <- ncol(vcv.phy)
	return((1 / (n - 1)) * (t(y - rep(mu, n)) %*% (solve(vcv.phy)) %*% (y - rep(mu, n))))
	}
		

#' Calculate fair proportions phylogenetic diversity metric
#'
#' Calculate fair proportions phylogenetic diversity metric
#' Note that \code{as.rateMatrix} calls the CAIC function \code{vcv.array} multiple times and this can be slow for large phylogenies (though faster than using the "ape" equivalent \code{vcv.phylo}).
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param nodeCount Logical - should root to tip node counts be returned (default is \code{FALSE})
#' @return Returns a matrix of fair proportion for all tips in phylogeny and node counts if selected.
#' @references Redding, D.W. and Mooers, A.O. (2006). Incorporating evolutionary measures into conservation prioritisation. Conservation Biology, 20, 1670-1678.
#' @references Isaac, N.J.B., Turvey, S.T., Collen, B., Waterman, C. and Baillie, J.E.M. (2007). Mammals on the EDGE: conservation priorities based on threat and phylogeny. PLoS ONE, 2, e296.
#' @author Gavin Thomas
#' @examples
#' data(anolis.tree)
#' 
#' fp <- fairProportions(anolis.tree)
#' fpNodes <- fairProportions(anolis.tree, nodeCount=TRUE)
#' @export
#' @import caper

fairProportions <- function (phy, nodeCount=FALSE) {
	
	treeMatrix <- caper::clade.matrix(phy)
	fpEdgeVals <- treeMatrix$edge.length / apply(treeMatrix$clade.matrix, 1, sum) 
	fpTips <- as.matrix(apply(fpEdgeVals * treeMatrix$clade.matrix, 2, sum))

	if (nodeCount == TRUE) {	
		nodeCount <- apply(treeMatrix$clade.matrix, 2, sum) - 1
		fpTips <- cbind(fpTips, nodeCount)
		colnames(fpTips) <- c("FP", "NodeCount")
		rownames(fpTips) <- phy$tip.label 
		} else {
		rownames(fpTips) <- phy$tip.label
		}

	rm(treeMatrix)
	return(fpTips)
}

#' @title Maximum likelihood rate estimation for traits and phylogenies
#' @description Full function for maximum likelihood estimation of rate parameters and comparison to a single rate model.
#' @param rateData an object of class \code{rateData}
#' @param rate a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal.
#' @param fixed A vector stating whether each parameter should be allowed to vary (either \code{FALSE} which results in a start value of 1, or a numeric start value) or should be fixed (\code{TRUE}).
#' @param pretty Display the output nicely (\code{pretty=TRUE}) or as a list (\code{pretty=FALSE})
#' @param rateMIN Minimum value for the rate parameters.
#' @param rateMAX Maximum value for the rate parameters
#' @param common.mean a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=TRUE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates.
#' @param lambda.est Logical. Estimate Pagel's lambda.
#' @param est.CI Logical. Estimate approximate confidence intervals for rate parameters.
#' @param meserr Logical. Incorporate measurement error.
#' @param file File string for output. Only used if \code{pretty=TRUE}.
#' @return If \code{pretty=FALSE}, returns a list containing:
#' \describe{
#' \item{MLRate}{Maximum likelihood estimates of the rate parameters}
#' \item{Lambda}{Maximum likelihood estimate of lambda}
#' \item{LCI}{Approximate lower confidence intervals for rate}
#' \item{UCI}{Approximate upper confidence intervals for rate parameters}
#' \item{means}{Means for each category}
#' \item{nParam}{Number of parameters in the model (how many means and rate categories)}
#' \item{Max.lik}{Maximum (log) likeihood}
#' \item{AIC}{for maximum likelihood model}
#' \item{AICc}{for maximum likelihood model}
#' \item{LambdaSingle}{Maximum likelihood estimate of lambda for the single rate model}
#' \item{Lik1}{Likelihood of the equivalent single rate model}
#' \item{Likelihood}{ratio statistic of "Max.lik" vs "Lik1"}
#' \item{P}{P values for the LR statistic}
#' \item{df}{Degrees of freedom for the LR statistic}
#' \item{AIC.rate1}{AIC for single rate model}
#' \item{AICc.rate1}{AICc for single rate model}
#' }
#' @return If \code{pretty=TRUE}, prints a nice version of the list to screen. If \code{file} is specified the pretty output will be sent to file, not the console.
#' @note Unlike phyloMean and likRatePhylo (that use treatment contrasts), the means reported here are the actual values
#' @references Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.
#' @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @author Gavin Thomas, Rob Freckleton
#' @examples ## Read in phylogeny and data from Thomas et al. (2009)
#' @examples data(anolis.tree)
#' @examples data(anolis.data)
#' @examples ## Convert data to class rateData with a rateMatrix object as input
#' @examples anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph",
#' @examples data=anolis.data)
#' @examples anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
#' @examples rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)
#' @examples # A model with a different rate in one of the four groups. The 'fixed' command is used to determine
#' @examples # whether a particular rate is to be constrained or not. Use '1' to fix a group and 'FALSE' to show
#' @examples # that the parameter is not fixed and should be estimated. The values should be entered in the same 
#' @examples # order as the ranking of the groups. That is, group 0 (small islands) takes position one in the 
#' @examples # fixed vector, group 1 (large island trunk crown and trunk ground) takes position 2 and so on. 
#' @examples # The default is to allow each group to take a different mean. 
#' @examples ML.RatePhylo(anolis.rateData, fixed=c(1, FALSE, FALSE, FALSE), pretty=TRUE, lambda.est = FALSE)
#' @export

ML.RatePhylo <-
function(rateData, rate=NULL, fixed = NULL, pretty = TRUE, rateMIN = 0.001, rateMAX = 50, common.mean=FALSE, lambda.est=TRUE, est.CI=FALSE, meserr=FALSE, file=NULL) {
						
		if(is.null(rate))  { rate <- c(rep(1,length(rateData$Vmat))) } else { rate <- rate }	
	
		if(is.null(fixed))  { op <- fixed <- c(rep(FALSE,length(rateData$Vmat) - 1), TRUE) } else { op <- fixed }		
		
				ovp <- optim.likRatePhylo(rateData, rate, fixed, rateMIN, rateMAX, common.mean=common.mean, lambda.est=lambda.est, meserr=meserr)
				max.lik.rate <- ovp$MLRate
				lambda <- ovp$Lambda
				max.lik <- ovp$Max.lik
				singlerate <- optim.likRatePhylo(rateData, rate, rep(TRUE, length(rateData$Vmat)), rateMIN, rateMAX, common.mean=common.mean, lambda.est=lambda.est, meserr=meserr)
				lik1 <- singlerate$Max.lik
				lambda_single <- singlerate$Lambda
				D <- 2 * (max.lik - lik1)
	
	
				mlparams <- likRatePhylo(rateData, rate=max.lik.rate, common.mean=common.mean, lambda.est=lambda)
	
	
				if (common.mean==TRUE) { mu <- mlparams$mu } else {
					
					mu <- rep(NA, length(mlparams$mu))
					mu[1] <- mlparams$mu[1]
					for (i in 2:length(mlparams$mu)) { mu[i] <- mlparams$mu[1] + mlparams$mu[i] }
				}
		
														
		
				
				if (length(op) == length(which(op==FALSE)))  { k2 <- length(op) - 1 
						} else { k2 <- length(which(op==FALSE)) }
				
				if(length(op)!=length(which(op==FALSE))) {
					if(common.mean==TRUE) {k <- 2 + length(which(op==FALSE)) + lambda.est + meserr
							} else { k <- (length(which(op==FALSE)) +1 + length(op)) + lambda.est + meserr}
							
							} else {
					
					if(common.mean==TRUE) {k <- 1 + length(which(op==FALSE)) + lambda.est + meserr
							} else { k <- (length(which(op==FALSE)) + length(op)) + lambda.est + meserr}
							}
								
				pval <- 1- pchisq(D, k2)
				n <- length(rateData$y)
				
				
	if (est.CI) { CIs.rate <- RatePhylo.allCI(rateData, max.lik.rate, fixed=rep("FALSE", length(rateData$Vmat)), common.mean=common.mean)
	} else { CIs.rate <- matrix(NA, ncol=2, nrow=1) }
				aic <- -2 * max.lik + 2 * k
				aicc <- -2 * max.lik + 2 * k + ((2*k*(k+1))/(n-k-1))
				
				if(common.mean==TRUE) {
						aic.rate1 <- -2 * lik1 + 2 * 2
						aicc.rate1 <- -2 * lik1 + 2 * 2 + ((2*2*(2+1))/(n-2-1))
					} else {
						aic.rate1 <- -2 * lik1 + 2 * (1 + length(op))
						aicc.rate1 <- -2 * lik1 + 2 * (1 + length(op)) + ((2*(1 + length(op))*((1 + length(op))+1))/(n-(1 + length(op))-1))
						}
				
				if (is.null(file)) { file <- "" }
				if(pretty == TRUE) {
					cat("____________________________\n", file=paste(file), append=TRUE)
					cat("Maximum likelihood estimation: rates:\n\n", file=paste(file), append=TRUE)
					cat("Lambda:          ", lambda, "\n", file=paste(file), append=TRUE)
					cat("Brownian variance (rate): ", mlparams$s2, "\n", file=paste(file), append=TRUE)
					cat("ML estimates of group relative rates :          ", max.lik.rate, "\n", file=paste(file), append=TRUE)
					cat("Lower confidence intervals for rates:", CIs.rate[,1], "\n", file=paste(file), append=TRUE)
					cat("Upper confidence intervals for rates:", CIs.rate[,2], "\n", file=paste(file), append=TRUE)
					cat("ML estimates of group means : ", mu, "\n\n", file=paste(file), append=TRUE)
					cat("Number of parameters: ", k, "\n", file=paste(file), append=TRUE)
					cat("Maximised log likelihood: ", max.lik, "\n", file=paste(file), append=TRUE)
					cat("  AIC = ", aic, "  \n", file=paste(file), append=TRUE)
					cat("  AICc = ", aicc, "  \n\n", file=paste(file), append=TRUE)
					cat("____________________________\n", file=paste(file), append=TRUE)
					cat("Comparison with single rate model\n", file=paste(file), append=TRUE)
					cat("Lambda (single rate model):          ", lambda_single, "\n", file=paste(file), append=TRUE)
					cat("Log likelihood (single rate): ", lik1, "\n", file=paste(file), append=TRUE)
					cat("LR statistic (ML rates vs single rate):", D, file=paste(file), append=TRUE)
					cat("  P = ", pval , file=paste(file), append=TRUE)
					cat(" df = ", k2, " \n", file=paste(file), append=TRUE)
					cat("  Single rate AIC = ", aic.rate1, "  \n", file=paste(file), append=TRUE)
					cat("  Single rate AICc = ", aicc.rate1, "  \n", file=paste(file), append=TRUE)
					cat("____________________________\n", file=paste(file), append=TRUE)
					}
				if(pretty == FALSE) {
					max.lik.rate.list <- list(BRvar=mlparams$s2, MLRate = max.lik.rate, LCI = CIs.rate[,1], UCI = CIs.rate[,2],  means=mu, nParam = k, lambda=lambda, Max.lik = max.lik, AIC = aic, AICc=aicc, LambdaSingle = lambda_single, Lik1 = lik1, LR = D, P = pval, df = k2, AIC.rate1=aic.rate1, AICc.rate1=aicc.rate1)
					return(max.lik.rate.list) }	
	}

#' Blomberg's K
#' Estimate Blomberg's K (Blomberg et al. 2003)
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param y A matrix of trait values.
#' @return The estimate of the K statistic
#' @useDynLib motmot
#' @importFrom Rcpp sourceCpp
#' @seealso \code{\link{transformPhylo.ML}}, the Picante package
#' @references Blomberg SP, Garland T, & Ives AR. 2003. Testing for phylogenetic signal in comparative data: behavioral traits are more labile. Evolution 57, 717-745.

blomberg.k <- function(phy, y) {
	bm.phy <- transformPhylo.ML(phy, model="bm", y=y )
	ntip <- Ntip(phy)
	nodetimes <- nodeTimes(phy)
	vcv.int <- vcv(phy)
	vcv.phy <- sum(solve(vcv.int))
	diag.vcv <- sum(diag(vcv.int))
	mean.sq.error.0  <- bm.phy$brownianVariance[[1]]
	mat.state <- y[,1] - bm.phy$root.state
	transpose.mat <- t(mat.state)
	mean.sq.error <- transpose.mat %*% mat.state / (ntip - 1)
	mean.sq.error.div <- 1 / (ntip - 1) * ((diag.vcv - ntip / vcv.phy))

	obs <- mean.sq.error / mean.sq.error.0
	exp <- mean.sq.error.div
	k.out <- obs / exp
	return(k.out)
	}
	
	
	

#' @title Character displacement likelihood ratio test
#' @description Conducts a likelihood ratio test between empirical data (phylogeny and trait data), and simumlations from the function chr.disp.sim using an approximate Bayesian computation (ABC) approach (Clarke et al. 2017)
#' @param emp.tree An empirical phylogeny - a object of class \code{phylo} (see \pkg{ape}).
#' @param emp.data Continuous trait data matrix
#' @param param.out simulated data from the function \code{chr.disp.sim}
#' @param posteriorSize The number of samples to use in the likelihood-ratio test
#' @return List containing element of 'estimates' with the estimates of sigma and a, with the Brownian motion (a = 0) summarised in column one and the character displacement (a > 0) in column two. 'likelihood' contains the likelihood of the Brownian motion model and the character displacement model, and the likelihood ratio test estimate. If used, there is an estimate of Blomberg's K for the empirical and simulated data.
#' @useDynLib motmot
#' @importFrom Rcpp sourceCpp
#' @import ks
#' @seealso \code{\link{chr.disp.sim}}, \code{\link{chr.disp.param}}
#' @references Clarke M, Thomas GH, Freckleton RP. 2017. Trait evolution in adaptive radiations: modelling and measuring interspecific competition on phylogenies. The American Naturalist. 189, 121-137.
#' @author Magnus Clarke and Mark Puttick
#' @examples
#' ## import finch data form Clarke et al. (2017)
#' data(finches)
#' ## simulate small amount of data 
#' ## (example only - many more datasets are required for accuracy)
#' param.simulation <- chr.disp.param(finch.tree, n.sim = 100, n.steps=100,
#' max.sigma = 8, max.a = 8, ntraits=1, 
#' allopatry=as.matrix(allopatric.data), mc.cores = 1)
#' chr.disp.lrt(finch.tree, finch.data, param.simulation, 50)
#' @export

chr.disp.lrt <- function(emp.tree, emp.data, param.out, posteriorSize=500) {

    sig <- param.out[[1]][ ,1]
    atry <- param.out[[1]][ ,2]
    sstat <- param.out[[1]][ ,-(1:2)]
    
    input.match <- param.out$input.arguments
    max.sigma <- input.match$max.sigma
	max.a <- input.match$max.a
	est.blomberg.k <- input.match$est.blomberg.k
	n.sim <- input.match$n.sim
	n.traits <- input.match$n.traits
	trait.lim <- input.match$trait.lim
	# sympatry <- param.out$sympatry
	# allopatry <- param.out$allopatry
	

    # Get summary stats for the true data, and distance to sims
    tstat <- summary_stats(phy=emp.tree, est.blomberg.k=est.blomberg.k, y=emp.data)
    diff <- colSums(abs(t(sstat)[-3,] - unlist(tstat[-3])) ^ 2)

    # Get simulations from nth closest to closest 
    h.1_post <- order(diff)[1:posteriorSize]
    u.sig <- sig[h.1_post]
    u.atry <- atry[h.1_post]
    h.1_post <- matrix(ncol=2, nrow=length(u.sig))
    h.1_post[,1] <- u.sig
    h.1_post[,2] <- u.atry

    k.out <- ks::kde(h.1_post, xmin=c(0, 0), xmax=c(max.sigma, max.a), binned=FALSE)
    k.0.out <- ks::kde(h.1_post, xmin=c(0, 0), xmax=c(max.sigma, 0), binned=FALSE)

    # Use kernel smoothing to estimate likelihood maxima with and without competition.
    k.max.index <- which(k.out$estimate == max(k.out$estimate), arr.ind = TRUE)
    h.1.lik <- k.out$estimate[k.max.index[1], k.max.index[2]]
    h.1.est <- c(unlist(k.out$eval.points)[k.max.index[1]], unlist(k.out$eval.points)[length(k.out$estimate[,1]) + k.max.index[2]])

    k.0.max.index <- which(k.0.out$estimate == max(k.0.out$estimate), arr.ind = TRUE)
    h.0.lik <- k.0.out$estimate[k.0.max.index[1], k.0.max.index[2]]
    h.0.est <- c(unlist(k.0.out$eval.points)[k.0.max.index[1]], unlist(k.0.out$eval.points)[length(k.0.out$estimate[,1]) + k.0.max.index[2]])
    
    likelihood.ratio.test <- -2 * log(h.0.lik / h.1.lik )
    p.value <- pchisq(likelihood.ratio.test, 1)
   
    output <- list()
    output$estimates <- data.frame(h.0.est, h.1.est)
    rownames(output$estimates) <- c("sigma", "a")
    colnames(output$estimates) <- c("Brownian motion", "Character displacement model")
    output$likelihood <- data.frame(log(h.0.lik), log(h.1.lik), likelihood.ratio.test, p.value)
	if(est.blomberg.k) output$blomberg.k <- c("empirical.blomberg.k"=unlist(tstat[3]), "simulated.mean.blomberg.k"=mean(param.out[[1]][,4]))
	return(output)
	
}

#' @title plot a univariate continuous trait data on a phylogeny
#' @description Plots a phylogeny with lines representing the value of a continuous trait
#' @param y A matrix of trait values with taxon names as rownames.
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param col.label colour labels for the traits at the tips and in the histogram
#' @param col.tree colour for the edge labels on the tree
#' @param include.hist Logical. Include a histrogram alongside the plot of the tree?
#' @param col.hist colour of the histogram
#' @param cex.plot Numeric. The size of labels for the histogram axis labels
#' @param show.tips Logical. If FALSE (default), no tip labels are shown. If TRUE, tip labels are shown
#' @param cex.tips Numeric. The size of the phylogeny tip labels
#' @param n.split Numeric. The number of splits for the axis labels and shading for the trait values
#' @param lwd.traits Line widths of traits shown on the plot
#' @param axis.text text shown above the trait label axis. If NULL (default), nothing is displayed
#' @param transform.axis.label If the data are provided as logarithms the labels for trait axis can be transformed to their original values by calculating the exponential function of the natural (transform.axis.label="exp") or base 10 logarithm. The default (NULL) leaves the labels un-transformed. 
#' @param show.axis Logical. If TRUE, shows the axis of trait values on the plot.
#' @param at Axis tick point locations for if show.axis is TRUE.
#' @param labels Axis labels locations for if show.axis is TRUE.
#' @param axis.text.line The location of the label for the trait axis beside the plot.
#' @param offset.bars The distance of trait plot lines from the phylogeny.
#' @param ... further arguments passed to the axis function
#' @return A plot with the trait values shown at the tips, and a histrogram of the trait values
#' @author Mark Puttick
#' @examples
#' data(anolis.tree)
#' data(anolis.data)
#' attach(anolis.data)
#' male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
#' sortedData <- sortTraitData(anolis.tree, male.length)
#' phy <- sortedData$phy
#' male.length <- sortedData$trait
#' traitData.plot(y=male.length, phy)
#' @export



traitData.plot <- function(y, phy, col.label="red", col.tree="black", col.hist="navy", cex.plot=0.7, cex.tips=0.7, 
  show.tips=FALSE, include.hist=FALSE, n.split=5, lwd.traits=1, show.axis = TRUE,  axis.text=NULL, transform.axis.label=NULL, at = NULL, labels = NULL, axis.text.line = 1, offset.bars = 1, ...) {

  if (include.hist) {
    par(mfrow = c(1, 2), mar=c(3, 3, 3, 3), oma = c(1, 1, 1, 1))
  } else {
    par(mfrow = c(1, 1), mar=c(4, 4, 4, 4), oma=c(0, 0, 0, 0))	
  }
  
  tree.height <- nodeTimes(phy)[1,1] * offset.bars
  times <- tree.height / (max(y))
  range.fun <- function(x) (x - min(x)) / (max(x) - min(x))
  if (is.null(at)) {
  	splits <- (seq(0, max(y), length.out = n.split + 1) * times * 0.9) + (tree.height)
  } else {
  	splits <- (at * times * 0.9) + (tree.height)
  }
  
  if (is.null(labels)) {
  	label.splits <- seq(min(y), max(y), length.out=n.split + 1)
  } else {
  	label.splits <- labels
  }
  
  plot(ladderize(phy), show.tip.label = FALSE, edge.col = col.tree, x.lim = c(0, tree.height * 2))
  lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
  x_cord <- lastPP$xx
  y_cord  <- lastPP$yy
  tip.x.orig <- tip.x <- x_cord[1:Ntip(phy)]
  tip.y.orig <- tip.y <- y_cord[1:Ntip(phy)]
  t.data.scale.orig <- t.data.scale <- max(x_cord) + (range.fun(y) * (tree.height * 0.9))
  recycle.color <- rep(col.label, Ntip(phy))
  
  ord.y <- order(tip.y)
  tip.y <- tip.y[ord.y]
  tip.x <- tip.x[ord.y]
  recycle.color <- recycle.color[ord.y]
  t.data.scale <- t.data.scale[ord.y]
  zero.height <- tip.y[ - length(tip.y)] + diff(tip.y) / 2

  all.y <- rep(c(tip.y, zero.height), each = 3)
  all.x <- rep(tip.x[1], length(all.y))
  all.x[seq(2, length(all.y), 6)] <- t.data.scale
  for(uu in seq(0, length(all.x) - 7, 6)) 
    all.x[c(4:6) + uu] <- NA
  ord.y2 <- order(all.y)
  all.y <- all.y[ord.y2]
  recycle.color <- rep(recycle.color, 6)
  for(x in seq(1, length(splits), 2))
    polygon(rep(splits[(x):(x+1)], each = 2), c(Ntip(phy), 1, 1, Ntip(phy)),
      col = "#00000020", border = FALSE, xpd = TRUE)
  polygon(all.x * offset.bars, all.y, xpd = TRUE, col = NA, border = recycle.color, lwd = lwd.traits)
  if (!is.null(transform.axis.label)) {
    if (transform.axis.label == "exp")
      label.splits <- exp(label.splits)
	if (transform.axis.label == "exp10")
	  label.splits <- 10 ^ (label.splits)
  }
  
  if (show.axis)
    axis(3, at = splits, labels = signif(label.splits, 3), xpd = TRUE, line = -0.5, ...)
    
  if (show.tips)
    text(t.data.scale.orig, tip.y.orig, phy$tip.label, pos = 4, cex = cex.tips, xpd = TRUE)
  if (!is.null(axis.text))
    mtext(axis.text, 3, at = mean(splits), line = axis.text.line, cex = cex.plot)
    
  if(include.hist) {
    par(mar=c(3, 3, 3, 3))
    if (is.null(colnames(y))) {
      name.trait <- "trait"
    } else {
      name.trait <- colnames(y)
    }
    hist(y, col = col.hist, xaxs = "i", yaxt = "n", border = "white", main = "", 
      xlab = "", ylab = "", cex.axis = cex.plot, las = 2, axes = TRUE, mgp = c(3, 1, 0))	
    box()
	# axis(1, tick = FALSE, line = -1.5, cex.axis = cex.plot, las = 2, mgp = c(3, 1, 0))
	axis(2, cex.axis = cex.plot, mgp = c(3, 1, 0))
	mtext("frequency", 2, line = axis.text.line, cex = cex.plot)
	mtext(name.trait, 1, line = axis.text.line, cex = cex.plot)
  }
}	

#' @title Tree plotting for rates
#' @description Plots trees with colours based on rates of trait evolution. Also provides simple coloured plotting for trait values using the \code{\link[ape]{ace}} function in the \pkg{ape} library.
#' @param y A matrix of trait values.
#' @param x Output from \code{\link{summary.traitMedusa}}.
#' @param reconType Colour branches according to rate shifts ("rates" - requires traitMedusaObject) or ancestral state reconstruction ("picReconstruction"  - requires x).
#' @param palette Defines the colour scheme with four options: hotspot.colors (red to blue), heat.colors (yellow to red), cool.colors (blues), combi.colors (yellows to reds and blues)
#' @param ... Other functions to pass to \code{\link[ape]{plot.phylo}} 
#' @return Returns a data frame of colours used in plot along with rate (or ancestral state) range for each colour. 
#' @seealso \code{\link{transformPhylo.ML}}, \code{\link{summary.traitMedusa}}.
#' @author Gavin Thomas, Mark Puttick
#' @examples
#' # Data and phylogeny
#' data(anolis.tree)
#' data(anolis.data)
#'
#' # female SVL data
#' female.svl <- matrix(anolis.data[,"Female_SVL"], 
#' dimnames=list(rownames(anolis.data)))
#' input.data <- sortTraitData(phy=anolis.tree, y=female.svl, log.trait=TRUE)
#' 
#' # arbitarily reduce data size for speed in this example
#' phy.clade <- extract.clade(input.data[[1]], 182)
#' male.length.clade <- as.matrix(input.data[[2]][match(input.data[[1]]$tip.label, 
#' rownames(input.data[[2]])),])
#' # Identify rate shifts and print and plot results with up to one rate shifts 
#' # and minimum clade size of 10.
#' anolisSVL_MEDUSA <- transformPhylo.ML(male.length.clade, phy=phy.clade, 
#' model="tm1",minCladeSize=10, nSplits=1)
#' anolisSVL_MEDUSA_out <- summary(anolisSVL_MEDUSA, cutoff=1, AICc=FALSE)
#' colours <- plot(x = anolisSVL_MEDUSA_out,
#' reconType = "rates", type = "fan", cex=0.6, edge.width=3)
#' @import grDevices
#' @export plot.traitMedusa.model
#' @export

plot.traitMedusa.model <- function(x, y = NULL, ..., reconType="rates", palette="hotspot.colors") {
  
  if (!inherits(x, "traitMedusa.model"))
    stop("Please sapply output from transformPhylo.ML with model = 'tm1' or model = 'tm2'.")
	
  phy <- x$original.phy
  traitMedusaObject <- x
  
  cool.colors <- function (n, alpha = 1)
    grDevices::rainbow(n, start = 3/6, end = 4/6, alpha = alpha)
  heat.colors <- function (n, alpha = 1)
    grDevices::rainbow(n, start = 0, end = 1/6, alpha = alpha)
  hotspot.colors <- function (n, alpha = 1)
    grDevices::rainbow(n, start = 0, end = 4/6, alpha = alpha)
    
  switch (reconType,
    "picReconstruction" = {
       if (is.vector(y)==FALSE)
         stop("trait must be a vector with taxon names")
       a <- ace(y, phy, method="pic")
       reconTrait <- rep(NA, length(phy$edge[,2]))
       # internal nodes   
       reconTrait[match(names(a$ace)[2:length(a$ace)], phy$edge[,2])] <- a$ace[2:length(a$ace)]
       # tips
       idx <- match(names(y), phy$tip.label)
       reconTrait[match(idx, phy$edge[,2])] <- y
         
       ncolours <- Ntip(phy) + Nnode(phy) -1
       edgeColours <- rep(NA, ncolours)
         
       if (palette=="hotspot.colors")
         use.colors <- rev(hotspot.colors(ncolours))
       if (palette=="heat.colors")
         use.colors <- rev(heat.colors(ncolours))
       if (palette=="cool.colors")
         use.colors <- rev(cool.colors(ncolours))
       if (palette=="combi.colors")
         use.colors <- c(cool.colors(ncolours / 2), rev(heat.colors(ncolours / 2)))
         
       binsize <- (max(reconTrait) - min(reconTrait)) / length(reconTrait)
       color.bin <- matrix(NA, ncol=length(reconTrait), nrow=2)
       rownames(color.bin) <- c("min", "max")
         
       for (i in 1:length(reconTrait)) {
         color.bin[1, i] <- min(reconTrait) + (i - 1) * binsize
         color.bin[2, i] <- min(reconTrait) + i* binsize
         }
           
       color.bin[2,length(reconTrait)] <- max(reconTrait)	# necessary due to rounding problems
         
       for (i in 1: ncolours) {
         color.trait <- (reconTrait[i] >= color.bin["min",]) & (reconTrait[i] <= color.bin["max",])
         edgeColours[i] <- use.colors[which(color.trait==TRUE)]
       }
     },
		
	"rates" = {
	  if(traitMedusaObject$Rates[1,1] == "0")
	    stop("BM model - no shifts to plot")
	  cladeRates <- traitMedusaObject$Rates[,3]
	  nodes <- traitMedusaObject$Rates[,1]
	  rateType <- traitMedusaObject$Rates[,2]
	  ncolours <- Ntip(phy) + Nnode(phy) -1
	  nodeBranch <- data.frame(as.numeric(nodes), rateType)
	  nodes <- nodeBranch[nodeBranch[,"rateType"]=="clade",1]
	  branches <- nodeBranch[nodeBranch[,"rateType"]=="branch",1]
   	  cladeMembers <- cladeIdentity(phy, nodes)
	  edgeColours <- rep("black", length(phy$edge[,1]))
	
	  lgcladeRates <- log(as.numeric(cladeRates))
	  lgcladeRatesNodes <- lgcladeRates[nodeBranch[,"rateType"]=="clade"]
	  lgcladeRatesBranches <- lgcladeRates[nodeBranch[,"rateType"]=="branch"]
		
	  if (palette=="hotspot.colors")
	    use.colors <- rev(hotspot.colors(ncolours))
	  if (palette=="heat.colors")
	    use.colors <- rev(heat.colors(ncolours))
	  if (palette=="cool.colors")
	    use.colors <- rev(cool.colors(ncolours))
	  if (palette=="combi.colors")
	    use.colors <- c(cool.colors(ncolours / 2), rev(heat.colors(ncolours / 2)))
	  
	  binsize <- (max(abs(lgcladeRates)) - -max(abs(lgcladeRates))) / length(edgeColours)
	  color.bin <- matrix(NA, ncol=length(edgeColours), nrow=2)
	  rownames(color.bin) <- c("min", "max")
	  
	  for (i in 1:length(edgeColours)) {
	    color.bin[1,i] <- -max(abs(lgcladeRates)) + (i - 1) * binsize
	    color.bin[2,i] <- -max(abs(lgcladeRates)) + i * binsize
	    }
	  
	  color.bin[2, length(edgeColours)] <- max(abs(lgcladeRates))	# necessary due to rounding problems
	  color.bin[1, 1] <- -max(abs(lgcladeRates))	# necessary due to rounding problems

      for (i in 1:length(nodes)) {
      	color.trait <- (lgcladeRatesNodes[i] >= color.bin["min", ]) & (lgcladeRatesNodes[i] <= color.bin["max", ]) 
      	edgeColours[cladeMembers[,i]==1] <- use.colors[which(color.trait == TRUE)]
      	}
     
     for (i in 1:length(branches)) {
       color.trait <- (lgcladeRatesBranches[i] >= color.bin["min", ]) & (lgcladeRatesBranches[i] <= color.bin["max", ]) 
       edgeColours[which(phy$edge[,2]==branches[i])] <- use.colors[which(color.trait==TRUE)]
       }
     }
   )
   
   plot.phylo(phy, edge.color = edgeColours, ...)
   invisible(as.data.frame(t(rbind(use.colors, exp(color.bin)))))
   }

#' @title Log-likelhood for models of trait evolution. 
#' @description Fits likelihood models for various models of continuous character evolution.
#' @param y A matrix of trait values.
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param model The model of trait evolution (see details).
#' @param meserr A vector (or matrix) of measurement error for each tip. This is only applicable to univariate analyses.
#' @param kappa Value of kappa transform.
#' @param lambda Value of lambda transform.
#' @param delta Value of delta transform.
#' @param alpha Value of alpha (OU) transform.
#' @param psi Value of psi transform.
#' @param lambda.sp Speciation rate estimate for the tree.
#' @param branchLabels Branches on which different psi parameters are estimated in the "multipsi" model.
#' @param nodeIDs Integer - ancestral nodes of clades.
#' @param rateType If model="clade", a vector specifying if rate shift occurs in a clade ("clade") or on the single branch leading to a clade ("branch").
#' @param branchRates Numeric vector specifying relative rates for individual branches
#' @param timeRates The rates (from ancient to recent) for the timeSlice model
#' @param splitTime A split time (measured from the present, or most recent species) at which a shift in the rate occurs for the "timeSlice" model
#' @param acdcRate Value of ACDC transform
#' @param cladeRates Numeric vector specifying relative rates for clades or logical to indicate scalar is included in the 'modeslice' model (the scalar value is included in the mode.param argument with the 'modeslice' model).
#' @param covPIC Logical. For multivariate analyses, allow for co-variance between traits rates (TRUE) or no covariance in trait rates (FALSE). If FALSE, only the trait variances not co-variances are used.
#' @param cophenetic.dist a cophenetic distance matrix showing the absolute distance between taxa - only applicable for OU models run on non-ultrmetric trees. If null will be calculated internally, but supplying the data can speed up run time
#' @param vcv.matrix a variance-covariance matrix - only applicable for OU models run on non-ultrmetric trees. If null will be calculated internally, but supplying the data can speed up run time
#' @param mode.order The order of modes for the 'modeslice' model. Any combination of 'BM', 'OU', 'acdc', and 'kappa'
#' @param rate.var Allows rate variation in BM modes in the 'modeslice' model
#' @param mode.param Parameters for the modes of evoluton in the 'modeslice' model
#' @param mu Phylogenetic mean estimate, Mainly for internal use when using the variance-covariance method to calculate likelihood for non-ultrametric trees with the OU model
#' @param sigma.sq Brownian variance estimate, mainly for internal use when using the variance-covariance method to calculate likelihood for non-ultrametric trees with the OU model
#' @details This function fits likelihood models (see below) for continuous character evolution where the parameter values are set a priori. The function returns the log-likihood and the Brownian variance (or variance covariance matrix).
#' - `model="bm"`: Brownian motion (constant rates random walk).
#' - `model="kappa"`: fits Pagel's kappa by raising all branch lengths to the power kappa. As kappa approaches zero, trait change becomes focused at branching events. For complete phylogenies, if kappa approaches zero this infers speciational trait change. Default bounds from ~0 - 1.
#' - `model="lambda"`: fits Pagel's lambda to estimate phylogenetic signal by multiplying all internal branches of the tree by lambda, leaving tip branches as their original length (root to tip distances are unchanged). Default bounds from ~0 - 1.
#' - `model="delta"`: fits Pagel's delta by raising all node depths to the power delta. If delta <1, trait evolution is concentrated early in the tree whereas if delta >1 trait evolution is concentrated towards the tips. Values of delta above one can be difficult to fit reliably. If a nodeIDs is supplied, the model will fit a delta model nested within a clade, with a BM fit to the rest of the tree. Default bounds from ~0 - 5.
#' - `model="OU"`: fits an Ornstein-Uhlenbeck model - a random walk with a central tendency proportional to alpha. High values of alpha can be interpreted as evidence of evolutionary constraints, stabilising selection or weak phylogenetic signal. It is often difficult to distinguish among these possibilities. If a nodeIDs is supplied, the model will fit a OU model nested within a clade, with a BM fit to the rest of the tree. For OU models, alternative optimisation are performed with different starting values (1e-8, 0.01, 0.1, 1, 5). Default bounds from ~0 - 10.
#' - `model="ACDC"`: fits a model to in which rates can exponentially increased or decrease through time (Blomberg et al. 2003). If the upper bound is < 0, the model is equivalent to the 'Early Burst' model of Harmon et al. 2010. If a nodeIDs is supplied, the model will fit a ACDC model nested within a clade, with a BM fit to the rest of the tree. Default rate parameter bounds from ln(1e-10) ~ ln(20) divided by the root age. Note this process starts on the stem branch leading to the MRCA of the common node, unlike the other methods that start at the common node.
#' - `model="trend"`: fits a model in which the expectated mean change through time is non-zero, signifying a directional evolution to a larger or smaller trait value. This model is only appliacble to non-ultrametric trees.
#' - `model="psi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate (Ingram 2010). Note that the algorithm will automatically estimate speciation and extinction estimates, and will incorporate estimates of 'hidden' speciation if death estimates are greater than 0.
#' - `model="multipsi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate but allows seperate values of psi fitted to seperate branches (Ingram 2010; Ingram et al. 2016). Note that the algorithm will automatically estimate speciation and extinction estimates, and will incorporate estimates of 'hidden' speciation if death estimates are greater than 0.
#' - `model="free"`: fits Mooers et al's free model where each branch has its own rate of trait evolution. This can be a useful exploratory analysis but it is slow due to the number of parameters, particularly for large trees. Default rate parameter bounds from ~0 - 200.
#' - `model="clade"`: fits a model where particular clades are a priori hypothesised to have different rates of trait evolution (see O'Meara et al. 2006; Thomas et al. 2006, 2009). Clades are specified using nodeIDs and are defined as the mrca node. Default rate parameter bounds from ~0 - 200.
#' - `model="tm1"`: fits "clade" models without any a priori assertion of the location of phenotypic diversification rate shifts. It uses the same AIC approach as the runMedusa function in the geiger package (runMedusa tests for shifts in the rate of lineage diversification). The algorithm first fits a constant-rate Brownian model to the data, it then works iteratively through the phylogeny fitting a two-rate model at each node in turn. Each two-rate model is compared to the constant rate model and the best two-rate model is retained. Keeping the location of this rate shift intact, it then repeats the procedure for a three-rate model and so on. The maximum number of rate shifts can be specified a priori using nSplits. Limits can be applied to the size (species richness) of clades on which to infer new rate shifts using minCladeSize. This can be useful to enable large trees to be handled but should be used cautiously since specifiying a large minimum clade size may result in biologically interesting nested rate shifts being missed. Equally, very small clade sizes may provide poor estimates of rate that may not be informative. Limits on the search can also be placed using restrictNode. This requires a list where each element of the list is a vector of tip names that define monophyletic groups. Rate shifts will not be searched for within any of the defined groups. Default rate parameter bounds from ~0 - 1000.
#' - `model="tm2"`: this model is similar to "tm1", however, at each node it assesses the fit of two models. The first model is exactly as per "tm1". The second model infers a rate shift on the single branch descending directly from a node but not on any of the descending branches thereafter. Only the best fitting single-branch or whole clade model is retained for the next iteration. If a single-branch shift is favoured, this infers either that there was a rapid shift in trait value along the stem leading to the crown group, or that the members of the clade have undergone parallel shifts. In either case, this can be considered as a change in mean, though separating a single early shift from a clade-parallel shift is not possible with this method.
#' - `model="timeSlice"`: A model in which all branch rates change at a time or times set a priori by the user. If splitTime=NULL, all 1 Ma (as defined by test Age) intervals from the root of the tree - 10 and the youngest tip + 10 will be included in the search. The +/- 10 Ma age can be modified using the argument boundaryAge. At each stage the best fitting model will be stored, and the search will continue until n shifts, with n shifts defined by nSplits. If a single value or vector is used for splitTime, only these ages are included in the search.
#' - `model="modeslice"`: A model in which all branch modes change at a time or times set a priori by the user.
#' @return brownianVariance Brownian variance (or covariance for multiple traits) given the data and phylogeny
#' @return logLikelihood The log-likelihood of the model and data
#' @seealso \code{\link{transformPhylo.ML}},\code{\link{transformPhylo.MCMC}}, \code{\link{transformPhylo.sim}}
#' @references Felsenstein J. 1973. Maximum-likelihood estimation of evolutionary trees from continuous characters. Am. J. Hum. Genet. 25, 471-492.
#' Felsenstein J. 1985. Phylogenies and the comparative method. American Naturalist 125, 1-15.
#' Freckleton RP & Jetz W. 2009. Space versus phylogeny: disentangling phylogenetic and spatial signals in comparative data. Proc. Roy. Soc. B 276, 21-30. 
#' @references Ingram T. 2011. Speciation along a depth gradient in a marine adaptive radiation. Proc. Roy. Soc. B. 278, 613-618.
#' @references Ingram T,  Harrison AD, Mahler L, Castaneda MdR, Glor RE, Herrel A, Stuart YE, and Losos JB. 2016. Comparative tests of the role of dewlap size in Anolis lizard speciation. Proc. Roy. Soc. B. 283, 20162199. #' Mooers AO, Vamosi S, & Schluter D. 1999. Using phylogenies to test macroevolutionary models of trait evolution: sexual selection and speciation in Cranes (Gruinae). American Naturalist 154, 249-259.
#' O'Meara BC, Ane C, Sanderson MJ & Wainwright PC. 2006. Testing for different rates of continuous trait evolution using likelihood. Evolution 60, 922-933
#' Pagel M. 1997. Inferring evolutionary processes from phylogenies. Zoologica Scripta 26, 331-348.
#' Pagel M. 1999 Inferring the historical patterns of biological evolution. Nature 401, 877-884.
#' Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @author Gavin Thomas, Mark Puttick
#' @examples
#' # Data and phylogeny
#' data(anolis.tree)
#' data(anolis.data)
#' 
#' # anolis.data is not matrix and contains missing data so put together matrix of
#' # relevant traits (here female and male snout vent lengths) and remove species 
#' # with missing data from the matrix and phylogeny
#' sorted.traits <- sortTraitData(anolis.tree, anolis.data,
#' c("Female_SVL", "Male_SVL"), log.trait=TRUE, pass.ultrametric=TRUE)
#'
#' tree <- sorted.traits$phy
#' traits <- sorted.traits$trait
#' 
#' # log likelihood of kappa = 0.1 or 1
#' transformPhylo.ll(traits, phy=tree, model="kappa", kappa=0.1)
#' transformPhylo.ll(traits, phy=tree, model="kappa", kappa=1)
#' 
#' # log likelihood of lambda = 0.01 or 1
#' transformPhylo.ll(traits, phy=tree, model="lambda", lambda=0.01)
#' transformPhylo.ll(traits, phy=tree, model="lambda", lambda=1)
#' 
#' # log likelihood of delta = 1.5 or 1
#' transformPhylo.ll(traits, phy=tree, model="delta", delta=1.5)
#' transformPhylo.ll(traits, phy=tree, model="delta", delta=1)
#' 
#' # log likelihood of alpha = 0.001 or 2
#' transformPhylo.ll(traits, phy=tree, model="OU", alpha=0.001)
#' transformPhylo.ll(traits, phy=tree, model="OU", alpha=2)
#'
#' @export
#' @import mvtnorm

transformPhylo.ll <- function(y=NULL, phy, model=NULL, meserr=NULL, kappa=NULL, lambda=NULL, delta=NULL, alpha=NULL, psi=NULL, lambda.sp = NULL, nodeIDs=NULL, rateType=NULL, branchRates=NULL, cladeRates=NULL, timeRates=NULL, splitTime=NULL, branchLabels = NULL, acdcRate=NULL,  covPIC = TRUE, cophenetic.dist=NULL, vcv.matrix=NULL, mode.order=NULL, mode.param=NULL, rate.var=NULL, mu=NULL, sigma.sq=NULL) {
	
		model <- tolower(model)
  all.models <- c("bm", "kappa", "lambda", "delta", "ou", "acdc", "psi", "multipsi", "free", "clade", "timeslice", "modeslice")
  if (any(is.na((match(model, all.models))))) stop(paste(model, "not recognised - please provide one of", paste0(all.models, collapse = ", ")))
		
		contemp.ou <- TRUE
		if(model=="ou") contemp.ou <- is.ultrametric(phy)
		
	switch(model,		  
		   
		   "bm" = {
		   transformPhy <- transformPhylo(phy=phy, model="bm", meserr = meserr, y=y)
		   },
		   
		   "kappa" = {
		   transformPhy <- transformPhylo(phy=phy, model="kappa", kappa=kappa, nodeIDs=nodeIDs, meserr = meserr, y=y)
		   },
		   
		   "lambda" = {
		   transformPhy <- transformPhylo(phy=phy, model="lambda", lambda=lambda, meserr = meserr, y=y)
		   },
		   
		   "delta" = {
		   transformPhy <- transformPhylo(phy=phy, model="delta", delta=delta, nodeIDs=nodeIDs, meserr = meserr, y=y)
		   },
		   
		   "free" = {
		   transformPhy <- transformPhylo(phy=phy, model="free", branchRates=branchRates, meserr = meserr, y=y)
		   },
		   
		   "clade" = {
		   transformPhy <- transformPhylo(phy=phy, model="clade", nodeIDs=nodeIDs, cladeRates=cladeRates, rateType=rateType, meserr = meserr, y=y)
		   },
		   
		   "ou" = {
		   	if(contemp.ou) {
		   		transformPhy <- transformPhylo(phy=phy, model="OU", alpha=alpha, nodeIDs=nodeIDs, meserr = meserr, y=y)
		   	} else {
		   		transformPhy <- transformPhylo(phy=phy, model="OU", alpha=alpha, nodeIDs=nodeIDs, meserr = meserr, y=y, cophenetic.dist=cophenetic.dist, vcv.matrix=vcv.matrix)
		   		}
		   },
		   
		   "psi" = {
		   transformPhy <- transformPhylo(phy=phy, model="psi", psi=psi, meserr = meserr, y=y, lambda.sp=lambda.sp)
		   },
		   
		   "multipsi" = {
        	transformPhy <- transformPhylo(phy = phy, branchLabels = branchLabels, model = "multipsi", psi = psi, meserr = meserr, y = y, lambda.sp = lambda.sp)
		   },
		   
		   "timeslice" = {
		   transformPhy <- transformPhylo(phy=phy, model="timeSlice", timeRates=timeRates,  splitTime=splitTime, meserr = meserr, y=y)
		   },
		   
		   	"acdc" = {
		   transformPhy <- transformPhylo(phy=phy, model="acdc", acdcRate=acdcRate, nodeIDs=nodeIDs, cladeRates=cladeRates, y=y, meserr = meserr)
		   },
		   
		   	"modeslice"= {
	   		transformPhy <- transformPhylo(phy=phy, model="modeslice", mode.order=mode.order, mode.param=mode.param, meserr = meserr, splitTime=splitTime, rate.var=rate.var, cladeRates=cladeRates, y=y)
		  }
		 )
	
		if(methods::is(transformPhy)[1] == "phylo") {
			return(likTraitPhylo(y=y, phy=transformPhy, covPIC = covPIC))
		} else {
			transformPhy <- transformPhy * sigma.sq
			if(length(mu) == 1) {
				mean.est <- rep(mu, ncol(transformPhy))
			} else {
				mean.est <- mu
			}
	   		return(mvtnorm::dmvnorm(y[,1], mean=mean.est, sigma=transformPhy, log=TRUE))	
			}
		}

#' @title Identify shifts in the rate of trait diversification through time
#' @description Summarises phenotypic rate variation on phylogenies through 
#' @param x Output of a timeSlice analysis in \code{\link{transformPhylo.ML}}
#' @param ... Other functions to pass to \code{\link[ape]{plot.phylo}} 
#' @param cutoff Value for differences in AIC scores when comparing models. More complex models with an AIC score more than this number of units lower than simpler models are retained (as per runMedusa in the geiger package).
#' @param AICc If \code{TRUE}, AICc is used instead of AIC.
#' @param lowerBound Minimum value for parameter estimates.
#' @param upperBound Maximum value for parameter estimates.
#' @param phylo.plot Logical. If \code{TRUE}, the phylogeny is plotted
#' @param cex.plot Character expansion for the plot of rates through time
#' @param colour.ramp The colours signifying different rates from low (first colour) to high (second colour)
#' @param model.average Logical only applicable to "timeSlice" models. Will return the model averaged timeSlice for models in which multiple shifts were considered (i.e, when splitTime is NULL). If TRUE, the function returns a plot showing the relative weights of each shift time and the model-averaged rates through time that are weighted by their relative weights. If TRUE, plot.phylo is ignored.
#' @details This functions summarises the output of a "timeSlice" model in \code{\link{transformPhylo.ML}} (see below). The best overall model is chosen based on AIC (or AICc if AICc=TRUE). The cut-off point for improvement in AIC score between successively more complex models can be defined using cutoff. The default cutoff is 4 but this is somewhat arbitrary and a "good" cut-off may well vary between data sets so it may well be worth exploring different cutoffs.
#' @return ModelFit Summary of the best optimal rate shift model or the model average of each split time (if model averaging was used).
#' @return Rates Summary of the rate parameters from the best rate shift model or the model averaged rates through time.
#' @return optimalTree A phylo object with branch lengths scaled relative to rate and a plot of estimated rates through time with their associated CIs.
#' @references To Add
#' @author Mark Puttick
#' @seealso \code{\link{transformPhylo.ML}}
#' @import grDevices
#' @examples
#' data(anolis.tree)
#' data(anolis.data)
#' attach(anolis.data)
#' male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
#' sortedData <- sortTraitData(anolis.tree, male.length)
#' phy <- sortedData$phy
#' male.length <- sortedData$trait
#' phy.clade <- extract.clade(phy, 182)
#' male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, 
#' rownames(male.length)),])
#' timeSlice.10.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="timeSlice", 
#' splitTime=c(10))
#' outputSummary <- plot(timeSlice.10.ml, cutoff=0.001, cex=0.5, 
#' colour.ramp=c("blue", "red"))
#' @export plot.timeSlice.ML
#' @export

plot.timeSlice.ML <- function(x, ..., cutoff=4, AICc=TRUE, lowerBound=1e-8, upperBound=1000, 
  phylo.plot = TRUE, colour.ramp=c("blue", "red"), cex.plot=1, model.average = FALSE) {
  	
  if (!inherits(x, "timeSlice.ML"))
	  stop("Please apply output from transformPhylo.ML with model = 'timeSlice'.")
  
  timeSliceObject <- x
  phy <- timeSliceObject$phy
  y <- timeSliceObject$y
  meserr <- timeSliceObject$meserr
  covPIC <- timeSliceObject$covPIC
  aic.fun <- function(likelihood, k) return(-2 * likelihood + 2 * k)
  aicc.fun <- function(likelihood, k, n) return(-2 * likelihood + 2 * k + ((2 * k * (k + 1)) / (n - k - 1)))
  model.full <- timeSliceObject$all.models
  phy <- timeSliceObject$phy
  y <- timeSliceObject$y
  meserr <- timeSliceObject$meserr
  covPIC <- timeSliceObject$covPIC
    
  if (AICc) {
  	diff.AICc <- diff(timeSliceObject$timeSlice[,"AICc"])
  } else {
  	diff.AICc <- diff(timeSliceObject$timeSlice[,"AIC"])
  }
  best.model <- which(diff.AICc < -cutoff)
  
  if (length(best.model) == 0) {
  	best.mod <- 1
  	model.best <- "BM"
  } else {
    best.mod <- max(best.model) + 1
    model.best <- paste("split", best.mod - 1)
  }
  return.top <- timeSliceObject$timeSlice[best.mod, ]
  na.test <- which(is.na(return.top))
  if (length(na.test) > 0) {
    model.param <- return.top[-which(is.na(return.top))]
  } else {
    model.param <- return.top
  }
  
  if (best.mod > 1) {
  	rate.location <- which(unlist(gregexpr("rate", names(model.param))) != -1)
  	rates <- model.param[rate.location]
  	time.shift.location <- which(unlist(gregexpr("time", names(model.param))) != -1)
  	shift.time <- model.param[time.shift.location]
  	lnL <- model.param[1]
  	phy.time.slice <- transformPhylo(phy, model="timeSlice", timeRates=rates, splitTime=shift.time)
  	LCI <- NULL
  	UCI <- NULL
  	
  	for(i in 1:length(rates)) {
  	  ratesSingle <- rates[i]
  	  foo <- function(param) {
  	  	pp <- rates
  	  	pp[i] <- param
  	  	ll <- transformPhylo.ll(y=y, phy=phy, model="timeSlice", timeRates=pp, splitTime=shift.time,
  	  	  meserr = meserr, covPIC = covPIC)$logLikelihood
  	  	return(as.numeric(ll - lnL + 1.92))
  	  }
	  
	  if (!any(ratesSingle == lowerBound, ratesSingle == upperBound)) {
	  	if (foo(lowerBound) < 0) {
	  	  LCI[i] <- uniroot(foo, interval = c(lowerBound, ratesSingle))$root
	  	} else {
	  	  LCI[i] <- NA
	  	}
	  	if (foo(10000) < 0) {
	  	  UCI[i] <- uniroot(foo, interval = c(ratesSingle, upperBound))$root
	  	} else {
	  	  UCI[i] <- NA
	  	}
	  }
	}
	rates.output <- cbind(rates, cbind(LCI, UCI))
  } else {
    shift.time <- NA
    phy.time.slice <- phy
    rates <- 1
  }
  
  if (model.average) {
    shifts.previous <- timeSliceObject[[1]][1:nrow(timeSliceObject[[1]]), "time.split1"]
	if (length(timeSliceObject$all.models) == 0) 
	  stop("Model Averaging not possible on models with only one shift configuration.")
	all.models.shift.aicc <- lapply(timeSliceObject$all.models, function(current.model) {
	  param <- nrow(current.model) + 1
	  all.aiccs <- aicc.fun(current.model[nrow(current.model), ], param, Ntip(phy))
	  delta.aicc <- all.aiccs - min(all.aiccs)
	  akaike.weights <- exp(-0.5 * delta.aicc) / sum(exp(-0.5 * delta.aicc))
	  akaike.weights
	  }
	)
	
	rem.small <- function(kl) {
	  weights.now <- kl
	  if (any(weights.now < 1e-8))
	    weights.now[which(weights.now < 1e-8)] <- NA
	  return(weights.now)
	}
	
	iterations <- length(timeSliceObject$all.models)
	if (!is.null(timeSliceObject$testShiftTimes))
	  iterations <- 1
	output.rates <- list()
	
	for(xn in 1:iterations) {
	  aic.now <- all.models.shift.aicc[[xn]]
	  model.rates.now <- model.full[[xn]][- nrow(model.full[[xn]]), ]
	  weighted.rates <- t(t(model.rates.now) * aic.now)
	  old.shifts <- shifts.previous[xn]
	  all.bins <- c(max(nodeTimes(phy)), as.numeric(colnames(model.full[[1]])), 0)
	  ma <- matrix(NA, ncol = length(all.bins), nrow = ncol(model.rates.now))
	  colnames(ma) <- all.bins
	  for(i in 1:length(colnames(model.rates.now))) {
	    if (!is.na(old.shifts)) {
	      ages.to.here <- sort(as.numeric(c(colnames(model.rates.now)[i], old.shifts)), decreasing = TRUE)
	    } else {
	      ages.to.here <- colnames(model.rates.now)[i]
	    }
	  initial.rate.to.here <- c(1, match(ages.to.here, colnames(ma)), ncol(ma))
	  for (yy in 1:(length(initial.rate.to.here) - 1)) {
	    ma[i, initial.rate.to.here[yy]:initial.rate.to.here[yy + 1]] <- rem.small(weighted.rates[yy, i])
	    }
	  }
	  output.rates[[xn]] <- rates.averaged <- apply(ma, 2, sum, na.rm = TRUE)
	}
	
	plot.ind <- 2 * iterations
	if (!is.null(timeSliceObject$testShiftTimes))
	  plot.ind <- 1
	
	par(mfrow = c(2, iterations), oma = c(0, 0, 0, 0), mar = rep(4.5, 4))
	for(plot.x in 1: iterations) {
	  aicss <- all.models.shift.aicc[[plot.x]]
	  plot(as.numeric(names(aicss)), aicss, type = "h", ylim = c(0 , 1), xlim = c(max(nodeTimes(phy)), 0), las = 1,
	    ylab = "Akaike weights", xlab = "Time of shift (Ma)", ...)
	  if (plot.x == 1) {
	    mtext("1 shift model", 3, cex = 0.8)
	  } else {
	  	mtext(paste0(plot.x, " shift model"), 3, cex = 0.8, line = 2)
	  	prev.shift <- paste0(signif(shifts.previous[2:plot.x], 4), collapse = ", ")
	  	mtext(paste0("Fixed shift(s):", prev.shift, 4), 3, cex = 0.8, line = 1) 	
	  }
	}
	for(plot.x in 1: iterations) {
	  to.plot <- output.rates[[plot.x]]
	  plot(as.numeric(names(to.plot)), to.plot, type = "s", xlim = c(max(nodeTimes(phy)), 0), las = 1,
	    ylab = "Model averaged rates", xlab = "Time (Ma)", ...)
	}
  } else {
    if (phylo.plot) {
      	
      if (model.best == "BM") {
        "Trees not plotted as BM model is supported."
      } else {      	
        start.time <- nodeTimes(phy)[1, 1]
        all.times <- start.time - c(start.time, sort(shift.time, TRUE), 0)
        all.times[1] <- -1
        time.poly <- seq(1, length(all.times))
        gradientFun <- grDevices::colorRampPalette(c(colour.ramp[1], colour.ramp[2]))
        col.in <- gradientFun(length(rates))[rank(rates, ties.method = "first")]
        col.in <- paste0(col.in, "75")
        par(mfrow = c(3, 1), mar=c(1, 1, 1, 1))
        plot.phylo(phy, ...)
        for(time.x in time.poly)
          polygon(c(all.times[time.x], all.times[time.x+1], all.times[time.x+1], all.times[time.x]), 
            c(-1, -1, Ntip(phy)+1, Ntip(phy)+1), col=col.in[time.x], border=FALSE)
        par(new = TRUE)
        plot.phylo(phy, ...)
        all.times.temp <- start.time - c(start.time, sort(shift.time, TRUE), 0)
        all.times2 <- c(0, cumsum(diff(all.times.temp) * rates))
        all.times2[1] <- -1 / (nodeTimes(phy)[1, 1] / nodeTimes(phy.time.slice)[1, 1])
        if (tail(all.times2, 1) != nodeTimes(phy.time.slice)[1, 1])
          all.times2[length(all.times2)] <- nodeTimes(phy.time.slice)[1,1]
        plot.phylo(phy.time.slice, ...)
        for (time.x in time.poly)
          polygon(c(all.times2[time.x], all.times2[time.x+1], all.times2[time.x+1], all.times2[time.x]),
            c(-1, -1, Ntip(phy) + 1, Ntip(phy) + 1), col=col.in[time.x], border=FALSE)
        par(new = TRUE)
	    plot.phylo(phy.time.slice, ...)
	    if (any(rates == lowerBound))
	      abline(v = all.times2[which(rates == lowerBound)  + 1], col = col.in[1], lwd = 2)
	    relative.rates <- rates / rates[1]
	    relative.LCI <- LCI / rates[1]
	    relative.UCI <- UCI / rates[1]
	    na.lci <- which(is.na(relative.LCI))
	    na.uci <- which(is.na(relative.UCI))
	    if (length(na.lci) > 0) 
	      relative.LCI[na.lci] <- relative.rates[na.lci]
	    if (length(na.uci) > 0)
	      relative.UCI[na.uci] <- relative.rates[na.uci]
	    max.y <- max(max(relative.UCI, na.rm = TRUE), max(relative.rates, na.rm = TRUE)) * 1.05
	    times.x <- c(start.time, sort(shift.time, TRUE), 0)
	    par(new = TRUE)
	    par(oma = c(5, 5, 5, 5))
	    plot(times.x, c(relative.rates, tail(relative.rates, 1)), type = "s", ylim = c(0, max.y),
	      xlim = c(start.time, 0), las = 1, xaxs = "i", yaxs = "i", mgp = c(0, 0.7, 0), cex.axis = cex.plot, 
	      ylab = "", xlab = "")
	    nn <- (length(times.x) - 1)
	    for(x in 1:nn)
	      polygon(c(times.x[x], times.x[x + 1], times.x[x + 1], times.x[x]),
	        c(relative.LCI[x], relative.LCI[x], relative.UCI[x], relative.UCI[x]), border = FALSE, col = col.in[x])
	    mtext("relative rates", 2, line = 2, cex = cex.plot)
	    mtext("time (Ma)", 1, line = 2, cex = cex.plot)
	    names.leg <- c()
	    for(kk in 1:nn)
	      names.leg <- c(names.leg, paste0("rate ", kk, ": ", signif(relative.rates[kk], 3),
	        "x (", signif(times.x[kk], 3), "-", signif(times.x[kk+1], 3), " Ma)"))
	    legend("topleft", legend = names.leg, col = col.in, pch = 15, cex = cex.plot)
	    }
	  }
	}
  output <- list()
  output$ModelFit <- model.best
  output$Rates <- model.param
  if (length(best.model) > 0)
    output$RatesCI <- rates.output
  output$optimalTree <- phy.time.slice
  if (model.average) {
    output$output.rates <- output.rates
    output$model.support <- all.models.shift.aicc
  }
  return(output)
}

#' Log-likelihood estimation for traits and phylogenies
#'
#' This function calculates the log-likelihood and Brownian (co)variance for a trait(s) and a phylogeny using phylogenetically independent contrasts
#' Note that \code{as.rateMatrix} calls the CAIC function \code{vcv.array} multiple times and this can be slow for large phylogenies (though faster than using the "ape" equivalent \code{vcv.phylo}).
#' @param y A matrix of trait data. Rownames must be included and match the taxon names in the phylogeny. Can accept single traits (calculates variance) or multiple traits (calculates variance-covariance matrix).
#' @param phy An object of class "phylo" (see \pkg{ape}).
#' @param covPIC Logical - allow for covariance between multivariate traits (\code{TRUE}), or assume not covariance (\code{FALSE}). Only applicable to multivariate traits
#' @param brCov If \code{NULL} (the default), Brownian covariance is analytically estimated. If a user-supplied numerical value is suppied the likelihood is calculate given this value
#' @details The \code{phylo} object must be rooted and fully dichotomous
#' @return brownianVariance Brownian variance (or covariance for multiple traits) given the data and phylogeny
#' @return logLikelihood The log-likelihood of the model and data
#' @references Felsenstein J. 1973. Maximum-likelihood estimation of evolutionary trees from continuous characters. Am. J. Hum. Genet. 25, 471-492.
#' @references Felsenstein J. 1985. Phylogenies and the comparative method. American Naturalist 125, 1-15.
#' @references Freckleton RP & Jetz W. 2009. Space versus phylogeny: disentangling phylogenetic and spatial signals in comparative data. Proc. Roy. Soc. B 276, 21-30. 
#' @author Gavin Thomas, Rob Freckleton
#' @examples 
#' data(anolis.tree)
#' data(anolis.data)
#' ## calculate Brownian variance log-likelihood of female SVL
#' female.svl <- matrix(anolis.data[,"Female_SVL"], 
#' dimnames=list(rownames(anolis.data)))
#' input.data <- sortTraitData(phy=anolis.tree, y=female.svl, log.trait=TRUE)
#' likTraitPhylo(phy = input.data$phy, y=input.data$trait)
#' @export

likTraitPhylo<-function (y, phy, covPIC = TRUE, brCov=NULL)
{
    if (is.matrix(y) == FALSE) {
        stop("Trait data must be a matrix with taxon names as row names")
    }
    n <- length(phy$tip.label)
    k <- ncol(y)
    phy <- reorder(phy, order = "pruningwise")
    y <- as.matrix(y[phy$tip.label, ])
    contrasts <- apply(y, 2, pic.motmot, phy = phy)
    rawVariances <- c(contrasts[[1]]$contr[, 2], contrasts[[1]]$V)   
    rawContrasts <- sapply(contrasts, function(k) c(k$contr[, 1], 0))
  	t.mat <- apply(rawContrasts, 2, function(x) x / sqrt(rawVariances))
    if(is.null(brCov)) {
    		brCov <- crossprod(t.mat, t.mat) / (n-1)
    } else {
    		brCov <- as.matrix(brCov)
    }
    if(covPIC == FALSE) {
    		brCov[upper.tri(brCov)] <- 0
    		brCov[lower.tri(brCov)] <- 0
    }
    iW <- solve(brCov)
    addCon.mat <- apply(rawContrasts, 1, function(con) crossprod(con, iW %*% con)) / rawVariances
    addCons <- sum(addCon.mat)
	logLikelihood <- -0.5 * (n * k * log(2 * pi) + n * log(det(brCov)) + k * sum(log(rawVariances)) + addCons)
    return(list(brownianVariance = brCov, logLikelihood = logLikelihood))
}

#' @title plot the output from transformPhylo.MCMC
#' @description Plots a histogram of the estimated parameter and a trace of the results
#' @param mcmc.input an object of class "mcmc.motmot" output from \code{\link{transformPhylo.MCMC}}
#' @param y.limit the limits for the y axes for the plots
#' @param x.limit the limits for the x axes for the plots
#' @param label.text the labels for the two plots defaults to '(a)' etc., for the histogram and '(b)' etc., for the trace plot
#' @param cex.axis character expansion for the plot axis labels
#' @param cex.labels character expansion for the plot axis names
#' @param col.hist colour for the histogram bars
#' @param col.trace colour for the trace plot
#' @return Two plots showing the histogram of the estimated parameter value and a trace of the MCMC estimation
#' @author Mark Puttick
#' @examples
#' library(motmot)
#' data(anolis.tree)
#' data(anolis.data)
#' attach(anolis.data)
#' male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
#' sortedData <- sortTraitData(anolis.tree, male.length)
#' phy <- sortedData$phy
#' male.length <- sortedData$trait
#' phy.clade <- extract.clade(phy, 182)
#' male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, rownames(male.length)),])
#' ## not run
#' # please note, this model will be need to run for longer to achieve convergence
#' # lambda.mcmc <- transformPhylo.MCMC(y=male.length.clade, phy=phy.clade, 
#' # model="lambda", mcmc.iteration=100, burn.in=0.1)
#' # mcmc.plot(lambda.mcmc)
#' @export


mcmc.plot <- function(mcmc.input, y.limit=NULL, x.limit=NULL, label.text=NULL, cex.axis=1, cex.labels=0.7, col.hist="green4", col.trace="navy") {
	if (!inherits(mcmc.input, "motmot.mcmc")) stop("please supply object of class motmot.mcmc")
	n.param <- ncol(mcmc.input$mcmc)
	names.param <- names(mcmc.input[[1]])
	
	hist.l <- lapply(1:n.param, function(x) suppressWarnings(hist(mcmc.input$mcmc[,x], xlim=x.limit, ylim=y.limit, xaxs="i", yaxs="i", las=1, ylab="density", xlab=names.param[x], plot=FALSE)))
	
	if(is.null(y.limit)) y.limit <- sapply(hist.l, function(x) c(0, max(x$counts) * 1.05))
	if(is.null(x.limit)) x.limit <- sapply(hist.l, function(x) {
		if(x$breaks[1] > 0) {
			diffs <- diff(range(x$breaks)) / 20
			out <- c(0, max(x$breaks) + diffs)
			} else {
			diffs <- diff(range(x$breaks)) / 20
			out <- c(min(x$breaks) - diffs, max(x$breaks) + diffs)
			}
			out
		}
	)
	
	if(is.null(label.text))  label.text <- c("(a)", "(b)", "(c)", "(d)", "(e)", "(f)", "(g)", "(h)", "(i)")
	par(mfrow=c(n.param,2))
	count <- 1
	for(u in 1:n.param) {
		plot(hist.l[[u]], xaxs="i", yaxs="i", las=1, main="", xlab=names.param[u], col=col.hist, border="white", xlim=x.limit[,u], ylim=y.limit[,u], cex.axis=cex.axis)
		mtext(label.text[count], 3, at=0, cex=cex.labels, font=2)
		count <- count + 1
		box()
		plot(mcmc.input$mcmc[,u], ylim=x.limit[,u], xaxs="i", yaxs="i", las=1, ylab=paste0(names.param[u]), xlab="generations", type="l", cex.axis=cex.axis, col=col.trace)
		mtext(label.text[count], 3, at=0, cex=cex.labels, font=2)
		count <- count + 1
		}
}

#' Log-likelihood rate estimation for traits and phylogenies
#'
#' This function calculates the log-likelihood, phylogenetic mean, and Brownian variance for a trait and a phylogeny transformed according to variation in relative rates. 
#' @param rateData an object of class \code{rateData}
#' @param rate a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal.
#' @param common.mean a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=FALSE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates.#' 
#' @param lambda.est Logical. Fit Pagel's lambda.
#' @param lambda Logical. Numeric value for lambda from 0-1.
#' @param meserr Logical. Logical. Include measurement error.
#' @param sigmaScale Logical. Scalar for measurement error relative to tree.
#' @return ll log-likelihood of the model
#' @return mu phylogenetically corrected mean(s)
#' @return s2 Brownian variance
#' @references Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.
#' @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @author Gavin Thomas, Rob Freckleton
#' @note The means are output as treatment contrasts.
#' @examples
#' data(anolis.tree)
#' data(anolis.data)
#' 
#' ## Convert data to class rateData with a rateMatrix object as input
#' anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)
#' 
#' anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
#' rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)
#' 
#' ## Calculate phylogenetic mean, variance, log likelihood for a model where the first 
# rate category (small islands) takes position one in the rate vector, group 1
# (large island trunk crown and trunk ground) takes position 2 and so on.
# The rates in the first and last categories are equal (both 1) whereas the rate in the
# second category is twice as fast (2) and the rate in the third category is ten times slower.
# Means are allowed to differ. 
#' 
#' # mean only
#' phyloMean(rateData=anolis.rateData, rate = c(1,2,0.1,1), common.mean = FALSE)
#' 
#' # variance only
#' phyloVar(rateData=anolis.rateData, rate = c(1,2,0.1,1), common.mean = FALSE) 
#' 
#' # mean, variance and log-likelihood
#' likRatePhylo(rateData=anolis.rateData, rate = c(1,2,0.1,1), common.mean = FALSE) 
#' @export

likRatePhylo <-
function(rateData, rate=NULL, common.mean=FALSE, lambda.est=TRUE, lambda=1, meserr=FALSE, sigmaScale=NULL) {
	
		if(is.null(rate))  { rate <- c(rep(1,length(rateData$Vmat))) } else { rate <- rate }	

			y <- rateData$y
			x <- as.factor(rateData$x)
			if (common.mean==FALSE) {k <- nlevels(x)} else {k <- 1}
	
			V <- transformRateMatrix(rateData, rate)

			x <- make.anc(y, x)
						
			if (!lambda.est & !meserr) {
				logDetV <- determinant(V)$modulus
				mu <- phyloMean(rateData, rate, common.mean, lambda.est, lambda)
				s2 <- phyloVar(rateData, rate, common.mean, lambda.est, lambda)
				n <- length(x[,1])
				ll <- -n / 2.0 * log( 2 * pi) - n / 2.0 * log(s2) - logDetV / 2.0 - (n - k)/2.0
				lik.RatePhylo <- ( list(ll = ll, mu = mu, s2 = s2) )
			}

			if (lambda.est & !meserr) {
				v.temp <- V
				diag(v.temp) <- rep(0, dim(V)[1])
				V.lam <- lambda*v.temp
				diag(V.lam) <- diag(V)
				V <- V.lam
				logDetV <- determinant(V)$modulus
				mu <- phyloMean(rateData, rate, common.mean, lambda.est, lambda)
				s2 <- phyloVar(rateData, rate, common.mean, lambda.est, lambda)
				n <- length(x[,1])
				ll <- -n / 2.0 * log( 2 * pi) - n / 2.0 * log(s2) - logDetV / 2.0 - (n - k)/2.0
				lik.RatePhylo <- ( list(ll = ll, mu = mu, s2 = s2) )
			}

	
			if (!lambda.est & meserr) {
				
				if (is.null(sigmaScale)) { stop("Estimate of sigmaScale required") }
				
				if(common.mean==FALSE) {x <- x} else { x <- rep(1, length(x[,1]))}

				diag(V) <- diag(V) + rateData$meserr/sigmaScale
				logDetV <- determinant(V)$modulus
				
				iV <- solve(V)
				xVix <- crossprod(x, iV %*% x)
				xViy <- crossprod(x, iV %*% y)
				mu <- solve(xVix) %*% xViy 

				e <- y - x %*% mu
				s2 <- crossprod(e, iV %*% e)
				n <- length(y) 
				phylo.var <- ( s2 / (n - k) )
				
				
				ll <- -n / 2.0 * log( 2 * pi) - n / 2.0 * log(phylo.var) - logDetV / 2.0 - (n - k)/2.0
				lik.RatePhylo <- ( list(ll = ll, mu = mu, s2 = phylo.var) )
			}
	
	
			if (lambda.est & meserr) {
		
				if (is.null(sigmaScale)) { stop("Estimate of sigmaScale required") }
		
				if(common.mean==FALSE) {x <- x} else { x <- rep(1, length(x[,1]))}
		
				
				v.temp <- V
				diag(v.temp) <- rep(0, dim(V)[1])
				V.lam <- lambda*v.temp
				diag(V.lam) <- diag(V)
				V <- V.lam
				diag(V) <- diag(V) + rateData$meserr/sigmaScale
				logDetV <- determinant(V)$modulus
		
				iV <- solve(V)
				xVix <- crossprod(x, iV %*% x)
				xViy <- crossprod(x, iV %*% y)
				mu <- solve(xVix) %*% xViy 
		
				e <- y - x %*% mu
				s2 <- crossprod(e, iV %*% e)
				n <- length(y) 
				phylo.var <- ( s2 / (n - k) )
		
		
				ll <- -n / 2.0 * log( 2 * pi) - n / 2.0 * log(phylo.var) - logDetV / 2.0 - (n - k)/2.0
				lik.RatePhylo <- ( list(ll = ll, mu = mu, s2 = phylo.var, lambda=lambda) )
			}
	
	
				return(lik.RatePhylo)
	}

#' Conversion among data and phylogeny objects
#'
#' Function to generate a \code{rateMatrix} object containing a set of variance covariance matrices. 
#' Note that \code{as.rateMatrix} calls the CAIC function \code{vcv.array} multiple times and this can be slow for large phylogenies (though faster than using the \pkg{ape} equivalent \code{vcv.phylo}).
#' @param phy An object of class "phylo" (see \pkg{ape}).
#' @param x The explanatory (discrete) variable used to define the hypothesised rate categories. Can be specified as a column number or column name
#' @param data The explanatory (discrete) variable used to define the hypothesised rate categories. Can be specified as a column number or column name
#' @return rateMatrix An object of class "rateMatrix" - a list of matrices describing the expected variances and covariances of between species. Each matrix refers to the variances and covariances for a given state of x (see Thomas et al. 2006).
#' @references Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.
#' @author Gavin Thomas
#' @export
#' @examples
#' ## Read in phylogeny and data from Thomas et al. (2009)
#' data(anolis.tree)
#' data(anolis.data)
#'
#' ## Convert data to class rateMatrix
#' anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)
#' @export
#' @import caper

as.rateMatrix <- function(phy, x, data) {
	
	if (is.numeric(x)) { x <- colnames(data)[x] } else { x <- x }

	# Remove species with NA for the discrete trait from the phylogeny and dataframe
	idxphy <- which(is.na(data[,x]) == TRUE)
	if(length(idxphy)>0) { 
		
		dropnms <- rownames(data)[idxphy]
		phy <- drop.tip(phy, tip=dropnms) 
		mch.miss <- match(rownames(data), dropnms)
		keep.idx.miss <- is.na(mch.miss)
		data <- data[keep.idx.miss,]	
		
		cat("Dropped species from data and tree due to missing data: ", dropnms, "\n")
		}
	
		# Check that names in taxa and tree match using geiger name.check function
		nmch <- name.check(phy, data)
		if(nmch[1]!="OK") {cat("Dropping species due to mismatch between tree and data:", "\n")
		
			phy <- drop.tip(phy, nmch[[1]])
			mch <- match(rownames(data), nmch[[2]])
			keep.idx <- is.na(mch)
			data <- data[keep.idx, ]
			cat("Dropped tips from tree: ", nmch[[1]], "\n")
			cat("Dropped rows from data: ", nmch[[2]], "\n")
		}
	
		# Check for node labels
	
		if( is.null(phy$node.label) ) {
			cat("\n")
			cat("Tree has no node labels. Reconstructing using discrete equal rates model with ace.", "\n")
			cat("You are strongly advised to consider whether this model is suitable for your data.", "\n")
			xanc <- data[,x]
			names(xanc) <- rownames(data)
			anc <- ace(xanc, phy, type="discrete", CI=TRUE) 
            foo.anc <- function(x) {
                out <- which(x == max(x))
                if (length(out)>1) {out <- sample(out, 1)}
                return(out)
            }
            ml_anc <- apply(anc$lik.anc, 1, foo.anc)
			ml_anc <- colnames(anc$lik.anc)[ml_anc]
			phy$node.label <- ml_anc
			}
	
		# alphabetise tip labels with index return
		speciesPhylo <- sort(phy$tip.label, index.return=TRUE)
	
		# make data frame of discrete trait with species as rownames
		discreteTrait <- data.frame(data[,x], row.names=rownames(data))
	
		# alphabetise species names for trait data with index return
		speciesData <- sort(rownames(discreteTrait), index.return = TRUE) 
	
		if(sum(speciesPhylo$x != speciesData$x) > 0){stop("Taxon names in phylogeny do not match those in data frame")}	# check names

		# data frame of tip states in species alphabetical order
		speState <- data.frame(tip.index = speciesPhylo$ix, data.index = speciesData$ix, row.names=rownames(discreteTrait)[speciesData$ix], tipState=discreteTrait[speciesData$ix, ]) 

		# combines vector of internal nodes with numeric vector of ancestral states taken from node labels excluding root
		ancState <- data.frame(node = c((length(phy$tip.label)+2):max(phy$edge)), ancState = as.numeric(phy$node.label[c(2:length(phy$node.label))]))

		# Put together node and tip indices and ancestral states then reorder
		takeCol <- which(regexpr("tipState", colnames(speState)) != -1)[1]
		state<- data.frame(edge.index = c(ancState$node, speState$tip.index), state = c(ancState$ancState, speState[,takeCol]))
		state <- state[sort(state$edge.index, index.return = TRUE)$ix, ]
	
		# Put tree edge in node order and then put it all together and order according to the tree edge index
        phyEdge <- sort(phy$edge[,2], index.return = TRUE)
        phyEdgeState <- data.frame(state, phyEdge = phyEdge$x, phyEdgeix = phyEdge$ix)
        phyEdgeState <- phyEdgeState[order(phyEdgeState$phyEdgeix),]
	
		# get numeric vector of branch states (use as factor first that factor levels are scored from 1-i in either numerical of alphabetical order depending on input
		ancestor <- as.numeric(as.factor(phyEdgeState$state)) 
		
		# how many factor levels?
		nlevels <- sort(unique(ancestor))
	
		# set up list to contain vcv matrices for each state
		rateMatrix <- vector(mode="list", length = length(nlevels))
	
		# create state vcv matrices 
		for(i in nlevels) {
				x.anc <- ifelse(ancestor == i, 1, 0)
				state.edge <- x.anc * phy$edge.length
				state.phy <- list(edge=phy$edge, edge.length=state.edge, Nnode=phy$Nnode, tip.label=phy$tip.label)
				class(state.phy) <- "phylo"
			state.matrix <- caper::VCV.array(state.phy)
			class(state.matrix) <- "matrix"
			rateMatrix[[i]] <- state.matrix
		}		
	
	class(rateMatrix) <- "rateMatrix"
	return(rateMatrix)
}

#' @title Simulate character displacement data wrapper
#' @description Simulates phylogenetic trait data under a character displacement model (Clarke et al. 2017) in which traits interact inter-specifically, with competition between sympatric lineages driving trait values apart
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param n.steps Number of time steps the for the simulation (default = 1000 time steps).
#' @param n.sim Number of replications to simulate data
#' @param max.sigma The maximum value of Brownian variance in the simulation sampled from a U(0, max.sigma) distribution for each iteration
#' @param max.a The maximum value of the strength of competition between inter-specific lineages sampled from a U(0, max.a) distribution for each iteration
#' @param est.blomberg.k Logical. If TRUE, Blomberg's K is simultaneously estimated
#' @param ntraits Number of traits to be simulated
#' @param sympatry an optional matrix giving the time that each pair of species starts to interact
#' @param allopatry an optional matrix giving the times when species stop interacting
#' @param trait.lim an optional parameter that puts limits on the available trait-space, preventing trait values with magnitude greater than the value of lim
#' @param mc.cores Numeric. The number of parallel cores to be used in simulations. Only applicable on Linux and Mac systems
#' @return List containing the simulated data 'simulated.param': a matrix with each row represented an iteration, the sigma (Brownian variance) used in the iteration, the 'a' value used in each iteration, the mean and standard deviation between neighbouring trait values. The 'input.arguments' from the model, the 'input.phy' from the model, and the input 'sympatry' and 'allopatry' matrices. 
#' @useDynLib motmot
#' @importFrom Rcpp sourceCpp
#' @seealso \code{\link{chr.disp.sim}}, \code{\link{chr.disp.lrt}}
#' @references Clarke M, Thomas GH, & Freckleton RP. 2017. Trait evolution in adaptive radiations: modeling and measuring interspecific competition on phylogenies. The American Naturalist 189, 121-137.
#' @author Magnus Clarke and Mark Puttick
#' @examples
#' ## import finch data form Clarke et al. (2017)
#' data(finches)
#' ## simulate small amount of data 
#' ## (example only - many more datasets are required for accuracy)
#' param.simulation <- chr.disp.param(finch.tree, n.sim = 3, n.steps=100,  
#' max.sigma = 8, max.a = 8, ntraits=1, 
#' allopatry=as.matrix(allopatric.data), mc.cores = 1)
#' @export

chr.disp.param <- function (phy, n.sim = 100, n.steps=1000, max.sigma = 8, max.a = 8, est.blomberg.k = FALSE, ntraits=1, sympatry=NA, allopatry=NA, trait.lim=NA, mc.cores = 1) {
	
    sig <- runif(n.sim, 0, max.sigma)
    atry <- runif(n.sim, 0, max.a)
    if(!all(is.na(allopatry))) allopatry <- as.matrix(allopatry)
	if(!all(is.na(sympatry))) sympatry <- as.matrix(sympatry)    
    param.spl <- parallel::mclapply(1:n.sim, mc.cores = mc.cores, function(i) {
        d.out <- chr.disp.sim(phy = phy, n.steps=n.steps, a = atry[i], sigma = sig[i], ntraits=ntraits, sympatry=sympatry, allopatry=allopatry, trait.lim=trait.lim)$tval
        stats.sim <- summary_stats(phy = phy, y = as.matrix(d.out), 
            est.blomberg.k = est.blomberg.k)
        c(sig[i], atry[i], unlist(stats.sim))
    })
    
    if(!est.blomberg.k) {
    	    n.col <- 4
     	output.prm <- matrix(unlist(param.spl), ncol=n.col, byrow=TRUE)
    		colnames(output.prm) <- c("sigma", "a", "mean.gap", "sd.gap")
    	} else {
    		n.col <- 5
     	output.prm <- matrix(unlist(param.spl), ncol=n.col, byrow=TRUE)
    		colnames(output.prm) <- c("sigma", "a", "mean.gap", "sd.gap", "blomberg.k")
    	}
    input.pram <- data.frame(max.sigma = 8, max.a = 8, est.blomberg.k = as.logical(est.blomberg.k), n.sim = n.sim, n.traits=ntraits, trait.lim=trait.lim)
    return(list(simulated.param = output.prm, input.arguments = input.pram, 
        input.phy = phy, sympatry=sympatry, allopatry=allopatry))
}

# Functions for converting ape-format phyogenies to
# 'peth': a format better suited for time-forward simulation.

## Convert a tree in ape format to one in new format (return a new 'peth' tree object). Only works for ntip > 4

ape2peth <- function(tree) {
    edge <- tree$edge
    edge.length <- tree$edge.length
    nedge <- length(edge[,1])
    ntip <- Ntip(tree)

    # Set up ape-to-peth map and vector to keep track of running branches.
    running <- rep(FALSE, 2 * ntip - 2)
    map <- matrix(ncol=2, nrow=nedge + 1)

    # Assign root
    map[1,1] <- root <- edge[1,1]
    map[1,2] <- 1

    # splitting gives the NODE, as labelled in edge, which is speciating
    splitting <- root
	edge_counter <- 1

    # Keep track of how many nodes we've already assigned.
    node_counter <- 1

    # Produce vector of nodes to split for future simulations (peth numbering) and Produce vector of times between speciation events
    nts <- tts <- c()

    # Loop over speciation events
    for(i in seq(2, 2*(ntip-1), by=2)) {
        
        # 'splitting' is the splitting node in ape format
        # 'peth_split' is the splitting node in peth format
        peth_split <- map[ which(map[,1] == splitting), 2 ][1]
        nts <- c(nts, peth_split)

        # The branches coming from this node are now running
        running[ which(edge[,1]==splitting) ] <- TRUE
        running[ which(edge[,2]==splitting) ] <- FALSE

        # We're at a speciation event. Find out which nodes come from it.
        new_edges <- which(edge[,1] == splitting)
        new_nodes <- edge[new_edges, 2]

        # Get which of the new nodes comes from the shortest branch
        # We want this one to inherit the label-number of its parent.
        new_node1 <- new_nodes[order(edge.length[new_edges])[1]]
        new_node2 <- new_nodes[order(edge.length[new_edges])[2]]

        # First new node keeps ancestor number (splitting node), 
        # other new node takes node_counter+1
        map[i,1] <- new_node1
        map[i,2] <- peth_split
        map[i+1,1] <- new_node2
        map[i+1,2] <- node_counter + 1

        node_counter <- node_counter + 1

        # time_to_split = time to next split = shortest branch running.
        time_to_split <- min(edge.length[running])
        tts <- c(tts, time_to_split)

        # Subtract time to next split from running branches.
        edge.length[running] <- edge.length[running] - time_to_split

        # Which node splits next? 
        # Of the running branches, take the shortest one and then the splitting node 
        # is the second elemtn of edge for that branch (i.e. the child)
        el_temp <- edge.length
        el_temp[!running] <- 9e99
        splitting <- edge[ which.min(el_temp),2 ]
    }
    
    # Need to know what order simulated data will be in, relative to ape's expectatons
    ordered_map <- order(map[,1])
    want <- ordered_map[1:ntip]
    dord <- rev(map[want,2])

    # Need to know what order simulated data will be in, relative to ape's expectatons. This is given by working from bottom up through map
    dord <-  1:ntip
    for(i in 1:ntip) {
        want <- which(map[,1]==i)
        dord[i] <- map[want , 2]
    }

    ptree <- list(map, dord, nts, tts)
    class(ptree) = 'pethtree'
    names(ptree) = c('map', 'data_order', 'splitting_nodes', 'times')
    return(ptree)
}

## Convert ape to peth, or pass on peth tree, or return error if incorrect

checktree <- function(phy) {
    if (inherits(phy, "phylo")) {
        phy = ape2peth(phy)
    } else if (!inherits(phy, "pethtree")) {
        stop("Tree incorrectly formatted.")
    }
    phy
}

# Convert time-matrices of when species become sympatric/allopatric to vectors
# If matrix is NA then all elements are set to constant number n

vectortime <- function(sympatric, degree, n.tips) {
    if(all(is.na(sympatric))) {
        sympatric.out = replicate(n.tips ^ 2, degree)
    } else if(is.matrix(sympatric)) {
        sympatric.out = as.vector(sympatric)
    } else {
        sympatric.out = as.vector(sympatric)
    }
    sympatric.out
}

## Reorder a dataset to match the order of tips in an ape-format tree

reorder_data <- function(tree, y, ntraits) {
    data.re.ord.out <- matrix(ncol=ntraits, nrow=length(tree$data_order))
    for (i in 1:ntraits) {
        data.re.ord.out[,i] <- y[seq(i, length(y), by=ntraits)]
        data.re.ord.out[,i] <- data.re.ord.out[ ,i][tree$data_order]
    }
    data.re.ord.out
}

## Generate a matrix of the times at which lineages come into sympatry if each new lineage starts interacting after a delay

symp_matrix <- function(tree, delay=0) {
    
    tree.check = checktree(tree)
    ntip <- length(tree.check$data_order)
    sympatry.out <- matrix(nrow=ntip, ncol=ntip, 0)
    if (inherits(tree, "phylo")) rownames(s) <- colnames(s) <- tree$tip.label

    # find ages (time from root) of tip lineages
    age <- 1:ntip
    age[1] <- age[2] <- 0
    for(i in 1:ntip) age[i+2] = sum(t$times[1:i])

    # delay measured in terms of mean time between speciation events.
    time_between_splits <- mean(abs(diff(tree.check$times)))
    delay <- delay * time_between_splits

    # apply starttimes and delay to symp-matrix s
    for (i in 1:ntip) 
    {
        for (j in 1:ntip)
        {
            sympatry.out[i,j] = max(c(age[i], age[j])) + delay
        }
    }
    return(sympatry.out)
}

## Get summary statistics for a given tree and dataset

summary_stats <- function(phy, y, est.blomberg.k=FALSE) {
    difs <- as.matrix(dist(y))            # Euclidian distance
    difs[which(difs == 0)] <- NA               # Ignore matrix diagonal
    gap <- apply(difs, 1, min, na.rm=T)  
    y <- as.matrix(y)
    ntraits <- length(y[1,])
    output <- list(mean.gap=mean(gap), sd.gap=sd(gap))

    # Summary statistics: mean and sd of gaps between neighbours. Plus Blomberg's K optionally.
    if(est.blomberg.k) {
        k.est <- sapply(1:ntraits, function(k.int) {
        		y.nit <- as.matrix(y[, k.int])
        		rownames(y.nit) <- phy$tip.label
        		blomberg.k(y=y.nit, phy=phy)
        		}
        	)
        k.est <- mean(k.est)
        output$blom.k <- k.est
    }
    return(output)
  }


   

#' Conversion among data and phylogeny objects
#'
#' Function to generate a "rateData" object containing the discrete explanatory variable, continuous response variable and set of variance co-variance matrices. It loads the trait data and removes species with missing data from the data and vcv matrix. 
#' \code{as.rateData} requires either a set of matrices in rateMatrix format created using \code{as.rateMatrix} or, if no rateMatrix object is input then it requires a phylogeny in "phylo" format. If  a "phylo" object is used  \code{as.rateData} will call \code{as.rateMatrix} internally.
#' \code{as.rateMatrix} calls the "ape" function \code{vcv.phylo} multiple times and this can be slow for large phylogenies. It will often be more efficient to use \code{as.rateMatrix} first to create a "rateMatrix" object to pass to \code{as.rateData}, particularly if there are many response traits of interest to be fitted to the same phylogeny and set of reconstructed ancestral states.
#' @param y The response variable - typically a continuous trait. Specified as a column name or index
#' @param x The explanatory (discrete) variable used to define the hypothesised rate categories. Specified as a column name or index.
#' @param rateMatrix A "rateMatrix" object or NULL
#' @param phy An object of class "phylo" (see \pkg{ape}).
#' @param data A data frame containing (minimally) the x and y variables as columns with species names as rownames.
#' @param meserr.col Column name or index containing measurement error for species means.
#' @param meserr.propn Single value specifying the proportional measurement to be applied across all species.
#' @param log.y Logical, natural log transform response variable.
#' @param report_prune Logical. Prints a list of dropped species if TRUE
#' @return rateData An object of class "rateData" which is a list containing the response (y) and explanatory (x) variable along with a list of variance-covaraince matrices.
#' @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @author Gavin Thomas
#' @export
#' @examples
#' ## Read in phylogeny and data from Thomas et al. (2009)
#' data(anolis.tree)
#' data(anolis.data)
#'
#' ## Convert data to class rateData with a rateMatrix object as input
#' anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)
#' 
#' anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
#' rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)  
#'
#' ## Convert data to class rateData with a phylo object as input 
#' anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
#' rateMatrix = NULL, phy=anolis.tree, data=anolis.data, log.y=TRUE)
#' @export

as.rateData <-
function(y, x, rateMatrix=NULL, phy=NULL, data, meserr.col=NULL, meserr.propn=NULL, log.y=FALSE, report_prune=FALSE) {
	
		 if (is.numeric(y) && length(y) == 1) { y <- colnames(data)[y] } else { y <- y }
		 if (is.numeric(x) && length(x) == 1) { x <- colnames(data)[x] } else { x <- x }
		if (is.numeric(meserr.col) ) { meserr.col <- colnames(data)[meserr.col] } else { meserr.col <- meserr.col }

		if (!is.null(meserr.col) & !is.null(meserr.propn) ) {
				stop("Only one source of measurement error should be specified. Use messerr.col to specify a column of measurement error 
						OR use meserr.propn to specify a single numeric value as a proportion error (e.g. enter 0.05 for 5% measurement error.")
						}
	
		if (is.null(rateMatrix))	{ rateMatrix <- as.rateMatrix(phy=phy, x=x, data=data) } else { rateMatrix <- rateMatrix }
	
		if (is.null(meserr.col)) { dat <- data.frame(x=data[,x], y=data[,y], row.names = rownames(data)) }
					 
		if (!is.null(meserr.col)) { dat <- data.frame(x=data[,x], y=data[,y], meserr=data[,meserr.col], row.names = rownames(data)) }
		if (!is.null(meserr.propn)) { dat <- data.frame(x=data[,x], y=data[,y], meserr=rep(meserr.propn, dim(data)[1]), row.names = rownames(data)) }
					 
			 
	
	
		if(sum(sort(rownames(dat)) != sort(rownames(rateMatrix[[1]]))) > 0){warning("Length or names of phenotypic and of phylogenetic data do not match - non-matching taxa will be dropped")}	# check names

		
		# get names of species common to data frame and phylogeny
		sharedSpecies <- intersect(rownames(rateMatrix[[1]]), rownames(dat))
	
	
		# subset data to only those species in phylogeny and alphabetise
		dat <- dat[match(sharedSpecies ,rownames(dat)),] 
		dat<- dat[sort(rownames(dat), index.return = TRUE)$ix, ]
		
		# index missing data for y
		ccdat <- complete.cases(dat)
		idx <- which(ccdat)
		missing.dat <- rownames(dat)[which(!ccdat)]

	
		# Suppress dropped species list
		if(report_prune==TRUE) {
			if(length(missing.dat > 0)) {
				cat("Dropping species due to missing data (x, y, or meserr):", "\n")
				cat("Dropped species: ", missing.dat, "\n")
			}
			}
			
		# alphabetise and prune rate matrices to remove data with missing species
		Vmat <- vector(mode="list", length = length(rateMatrix))
		sumMat <- matrix(0, dim(rateMatrix[[1]])[1], dim(rateMatrix[[1]])[2])
		for(i in 1:length(rateMatrix)) {
			Vmatrix <- rateMatrix[[i]]
			sumMat <- sumMat + Vmatrix
			nms <- rownames(Vmatrix)
			snms <- sort(nms, index.return = TRUE)
			Vmatrix <- Vmatrix[snms$ix, snms$ix]
			Vmatrix <- Vmatrix[idx, idx]
			Vmat[[i]] <- Vmatrix
			
			}
		
	
			xdat <- as.matrix(dat[,1])
			if (log.y==TRUE) {ydat <- log(dat[idx, "y"]) } else {ydat <- dat[idx, "y"]}
			xdat <- xdat[idx, ]
			
			if (is.null(meserr.col) & is.null(meserr.propn)) {traits <- list(y = ydat, x = xdat, Vmat = Vmat, meserr=rep(0,length(ydat)))} 
			if (!is.null(meserr.col)) {	meserr <- dat[idx,"meserr"]^2
				traits <- list(y = ydat, x = xdat, Vmat = Vmat, meserr=meserr)}
			
			if (!is.null(meserr.propn)) {	meserr <- (dat[idx,"meserr"]*ydat)^2
				traits <- list(y = ydat, x = xdat, Vmat = Vmat, meserr=meserr)}
	
	
			attr(traits, "class") <- "rateData"
			return(traits)
			}

#' Pairwise sympatry matrix for finch species
#'
#' A 13 by 13 numeric matrix indicating sympatry between finch species included in this package.
#'
#' @docType data
#'
#' @usage data(finches)
#'
#' @format A numeric matrix with dimensions 13 x 13. Row and column names are species labels corresponding to `finch.data` and `finch.tree`. Entries are integers (typically 1 for sympatry, 0 for absence of sympatry).
#'
#' @details This matrix provides pairwise sympatry information (presence/absence) among the finch species supplied with the package and can be used with functions that require sympatry/allopatry matrices.
#'
#' @keywords datasets
#'
#' @references Clarke M, Thomas GH, Freckleton RP. 2017. Trait evolution in adaptive radiations: modelling and measuring interspecific competition on phylogenies. The American Naturalist. 189, 121-137.
#'
#' @examples
#' data(finches)
#' head(sympatric.data)
"sympatric.data"

#' @title Maximum likelihood for models of trait evoluion
#' @description Fits likelihood models for various models of continuous character evolution. Model fitting is based on maximum-likelihood evaluation using phylogenetically independent contrasts. This is exactly equivalent to, but substantially faster than, GLS approaches.
#' @param y A matrix of trait values.
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param model The model of trait evolution (see details).
#' @param modelCIs Logical - estimate confidence intervals for parameter estimates.
#' @param nodeIDs Integer - ancestral nodes of clades applicable to rate heterogenous and nested models of evolution (see details)
#' @param rateType If model="clade", a vector specifying if rate shift occurs in a clade ("clade") or on the single branch leading to a clade ("branch").
#' @param minCladeSize Integer - minimum clade size for inferred rate shift (where model="medusa").
#' @param nSplits Integer - number of rate shifts to apply for model="medusa" and "timeSlice".
#' @param splitTime A split time (measured from the present, or most recent species) at which a shift in the rate occurs for the "timeSlice" model. If splitTime=NULL, then all ages between 1 million year intervals from the root age - 10 Ma to the present + 10 Ma will be included in the search. The best model will be retained in each search, and will be used as a fixed age in the next search. The model will calculate the likelihood for the number of shifts defined by 'nSplits'.
#' @param boundaryAge Only applicable if splitTime=NULL, the age distance from the tips and and youngest tip for which to search for rate shifts. For example, if boundaryAge=10, only ages between the root age - 10 and the latest tip + 10 will be included in the search. If one value is given this will be used for upper and lower ages, but if a vector with two ages is provided the first is used for the upper age boundary and the second for the lower age boundary. Set to zero to allow testing of all ages.
#' @param testAge If splitTime=NULL, the interval between ages to be tested. For example, if testAge=1, all 1 Ma ages between the ages defined by 'boundaryAge' will be tested. If you would like to sequentially test specific shift times only, please use the argument "specificShiftTimes".
#' @param saveAll Logical. If TRUE, saves all the outputs from traitMedusa search in timeSlice (i.e, the log-likelihood and rate estimates for all considered shifts, not just the best fitting shift model). This can be used for model averaging with the function \code{\link{plot.timeSlice.ML}}
#' @param testShiftTimes A vector of times to be used in the search for split times. For use in the timeSlice model when splitTime=NULL. This allows users to specify ages that are test suquentially, rather than all shifts optimised simultaneously as is done when ages are provided in the argument 'splitTime'.
#' @param restrictNode List defining monophyletic groups within which no further rate shifts are searched.
#' @param lambdaEst Logical.Estimate lambda alongside parameter estimates to reduce data noise. Only applicable for models "kappa", "delta", "OU", "psi", "multispi", and "ACDC". Default=FALSE.
#' @param acdcScalar Logical.For nested EB rate model, simultaneously estimated a rate scalar alongside EB model. Default=FALSE. Only applicable to 'nested mode' and 'modeSlice' models. 
#' @param branchLabels Branches on which different psi parameters are estimated in the "multipsi" model
#' @param hiddenSpeciation Logical. If TRUE the psi model will include nodes that are on the 'full.phy' but not the tree pruned of trait data
#' @param full.phy The full phylogeny containing the species that do not contain trait data so are not included in 'phy'
#' @param useMean Logical. Use the branch-based estimates of extinction of mean (TRUE, default) for the "psi" and "multispi" models. Only applicable if "hiddenSpeciation"=TRUE. If FALSE, this will generate a single realisation of the numbers of hidden speciation events on each branch
#' @param profilePlot Logical. For the single parameter models "kappa", "lambda", "delta", "OU", "psi", "multipsi", and "ACDC", plot the profile of the likelihood.
#' @param lowerBound Minimum value for parameter estimates
#' @param upperBound Maximum value for parameter estimates
#' @param returnPhy Logical. In TRUE the phylogeny with branch lengths transformed by the ML model parameters is returned
#' @param tol Tolerance (minimum branch length) to exclude branches from trait MEDUSA search. Primarily intended to prevent inference of rate shifts at randomly resolved polytomies.
#' @param covPIC Logical. For multivariate analyses, allow for co-variance between traits rates (TRUE) or no covariance in trait rates (FALSE). If FALSE, only the trait variances not co-variances are used.
#' @param meserr A vector (or matrix) of measurement error for each tip. This is only applicable to univariate analyses.
#' @param n.cores Integer. Set number of computing cores when running model="traitMedusa" (tm1 and tm2 models)
#' @param controlList List. Specify fine-tune parameters for the optim likelihood search
#' @param print.warnings Logical. If TRUE, warnings are issued if confidence intervals fall outside upper or lower bounds
#' @param mode.order The order of modes for the 'modeslice' model. Any combination of 'BM', 'OU', 'acdc', and 'kappa'
#' @param rate.var Allows rate variation in BM modes in the 'modeslice model'
#' @details This function finds the maximum likelihood parameter values for continuous character evolution. For "kappa", "delta", "OU", "multipsi", and "ACDC" it is possible to fit a 'nested' model of evolution in which the ancestral rate of BM switches to a different node, as specified by nodeIDs or branchLabels for multipsi. The function returns the maximum-likelihood parameter estimates for the following models.
#' - `model="bm"`: Brownian motion (constant rates random walk).
#' - `model="kappa"`: fits Pagel's kappa by raising all branch lengths to the power kappa. As kappa approaches zero, trait change becomes focused at branching events. For complete phylogenies, if kappa approaches zero this infers speciational trait change. Default bounds from ~0 - 1.
#' - `model="lambda"`: fits Pagel's lambda to estimate phylogenetic signal by multiplying all internal branches of the tree by lambda, leaving tip branches as their original length (root to tip distances are unchanged). Default bounds from ~0 - 1.
#' - `model="delta"`: fits Pagel's delta by raising all node depths to the power delta. If delta <1, trait evolution is concentrated early in the tree whereas if delta >1 trait evolution is concentrated towards the tips. Values of delta above one can be difficult to fit reliably. If a nodeIDs is supplied, the model will fit a delta model nested within a clade, with a BM fit to the rest of the tree. Default bounds from ~0 - 5.
#' - `model="OU"`: fits an Ornstein-Uhlenbeck model - a random walk with a central tendency proportional to alpha. High values of alpha can be interpreted as evidence of evolutionary constraints, stabilising selection or weak phylogenetic signal. It is often difficult to distinguish among these possibilities. If a nodeIDs is supplied, the model will fit a OU model nested within a clade, with a BM fit to the rest of the tree. For OU models, alternative optimisation are performed with different starting values (1e-8, 0.01, 0.1, 1, 5). Default bounds from ~0 - 10.
#' - `model="ACDC"`: fits a model to in which rates can exponentially increased or decrease through time (Blomberg et al. 2003). If the upper bound is < 0, the model is equivalent to the 'Early Burst' model of Harmon et al. 2010. If a nodeIDs is supplied, the model will fit a ACDC model nested within a clade, with a BM fit to the rest of the tree. Default rate parameter bounds from ln(1e-10) ~ ln(20) divided by the root age. Note this process starts on the stem branch leading to the MRCA of the common node, unlike the other methods that start at the common node.
#' - `model="trend"`: fits a model in which the expectated mean change through time is non-zero, signifying a directional evolution to a larger or smaller trait value. This model is only appliacble to non-ultrametric trees.
#' - `model="psi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate (Ingram 2010). Note that the algorithm will automatically estimate speciation and extinction estimates, and will incorporate estimates of 'hidden' speciation if death estimates are greater than 0.
#' - `model="multipsi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate but allows seperate values of psi fitted to seperate branches (Ingram 2010; Ingram et al. 2016). Note that the algorithm will automatically estimate speciation and extinction estimates, and will incorporate estimates of 'hidden' speciation if death estimates are greater than 0.
#' - `model="free"`: fits Mooers et al's free model where each branch has its own rate of trait evolution. This can be a useful exploratory analysis but it is slow due to the number of parameters, particularly for large trees. Default rate parameter bounds from ~0 - 200.
#' - `model="clade"`: fits a model where particular clades are a priori hypothesised to have different rates of trait evolution (see O'Meara et al. 2006; Thomas et al. 2006, 2009). Clades are specified using nodeIDs and are defined as the mrca node. Default rate parameter bounds from ~0 - 200.
#' - `model="tm1"`: fits "clade" models without any a priori assertion of the location of phenotypic diversification rate shifts. It uses the same AIC approach as the runMedusa function in the geiger package (runMedusa tests for shifts in the rate of lineage diversification). The algorithm first fits a constant-rate Brownian model to the data, it then works iteratively through the phylogeny fitting a two-rate model at each node in turn. Each two-rate model is compared to the constant rate model and the best two-rate model is retained. Keeping the location of this rate shift intact, it then repeats the procedure for a three-rate model and so on. The maximum number of rate shifts can be specified a priori using nSplits. Limits can be applied to the size (species richness) of clades on which to infer new rate shifts using minCladeSize. This can be useful to enable large trees to be handled but should be used cautiously since specifiying a large minimum clade size may result in biologically interesting nested rate shifts being missed. Equally, very small clade sizes may provide poor estimates of rate that may not be informative. Limits on the search can also be placed using restrictNode. This requires a list where each element of the list is a vector of tip names that define monophyletic groups. Rate shifts will not be searched for within any of the defined groups. Default rate parameter bounds from ~0 - 1000.
#' - `model="tm2"`: this model is similar to "tm1", however, at each node it assesses the fit of two models. The first model is exactly as per "tm1". The second model infers a rate shift on the single branch descending directly from a node but not on any of the descending branches thereafter. Only the best fitting single-branch or whole clade model is retained for the next iteration. If a single-branch shift is favoured, this infers either that there was a rapid shift in trait value along the stem leading to the crown group, or that the members of the clade have undergone parallel shifts. In either case, this can be considered as a change in mean, though separating a single early shift from a clade-parallel shift is not possible with this method.
#' - `model="timeSlice"`: A model in which all branch rates change at a time or times set a priori by the user. If splitTime=NULL, all 1 Ma (as defined by test Age) intervals from the root of the tree - 10 and the youngest tip + 10 will be included in the search. The +/- 10 Ma age can be modified using the argument boundaryAge. At each stage the best fitting model will be stored, and the search will continue until n shifts, with n shifts defined by nSplits. If a single value or vector is used for splitTime, only these ages are included in the search.
#' - `model="modeslice"`: A model in which all branch modes change at a time or times set a priori by the user.
#' @return Returns the maximum log-likelihood and parameter estimates (with 95 percent confidence intervals if specified). If model="bm" instead returns the Brownian (co)variance and log-likelihood. Also returned are the root estimate, the AIC, and AICc.
#' @return traitMedusaObject A list in which the first element contains a matrix summarising the parameter estimates and node ids, log-likelihoods, number of parameters (k), AIC and AICc for the best one-rate model, two-rate model, three rate model and so on. The second element is a sub-list where the first element contains all two-rate models, the second element contains all three-rate models and so on. This can be summarised using traitMedusaSummary. The third element is the input trait data. The fourth element is the input phylogeny.
#' @note Confidence intervals are based on the assumption of an asymptotic Chi-square distribution. For multi-parameter models (e.g. rate shift models with more than two rates) the confidence intervals are approximate and are calculated for each parameter in turn while holding all other parameters at their maximum likelihood value.
#' @seealso \code{\link{transformPhylo.MCMC}}, \code{\link{transformPhylo}}, \code{\link{transformPhylo.ll}}, \code{\link{blomberg.k}}
#' @references Alfaro ME, Santini F, Brock CD, Alamillo H, Dornburg A, Carnevale G, Rabosky D & Harmon LJ. 2009. Nine exceptional radiations plus high turnover explain species diversity in jawed vertebrates. PNAS 106, 13410-13414.
#' @references Blomberg SP, Garland T & Ives AR 2003. Testing for phylogenetic signal in comparative data: behavioral traits are more labile. Evolution 57, 717-745.
#' @references Felsenstein J. 1973. Maximum-likelihood estimation of evolutionary trees from continuous characters. Am. J. Hum. Genet. 25, 471-492.
#' @references Felsenstein J. 1985. Phylogenies and the comparative method. American Naturalist 125, 1-15.
#' @references Freckleton RP & Jetz W. 2009. Space versus phylogeny: disentangling phylogenetic and spatial signals in comparative data. Proc. Roy. Soc. B 276, 21-30.
#' @references Harmon LJ et al. 2010. Early bursts of body size and shape evolution are rare in comparative data. Evolution 57, 717-745.
#' @references Ingram T. 2011. Speciation along a depth gradient in a marine adaptive radiation. Proc. Roy. Soc. B. 278, 613-618.
#' @references Ingram T,Harrison AD, Mahler L, Castaneda MdR, Glor RE, Herrel A, Stuart YE, and Losos JB. 2016. Comparative tests of the role of dewlap size in Anolis lizard speciation. Proc. Roy. Soc. B. 283, 20162199.
#' @references Mooers AO, Vamosi S, & Schluter D. 1999. Using phylogenies to test macroevolutionary models of trait evolution: sexual selection and speciation in Cranes (Gruinae). American Naturalist 154, 249-259.
#' @references O'Meara BC, Ane C, Sanderson MJ & Wainwright PC. 2006. Testing for different rates of continuous trait evolution using likelihood. Evolution 60, 922-933
#' @references Pagel M. 1997. Inferring evolutionary processes from phylogenies. Zoologica Scripta 26, 331-348.
#' @references Pagel M. 1999 Inferring the historical patterns of biological evolution. Nature 401, 877-884.
#' @references Pagel M. 1999 Inferring the historical patterns of biological evolution. Nature 401, 877-884.
#' @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @author Gavin Thomas, Mark Puttick
#' @import stats
#' @import coda
#' @import ape
#' @import caper
#' @import stats
#' @import utils
#' @import graphics
#' @import mvtnorm
#' @import ks
#' @import methods
#' @useDynLib motmot, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @examples
#' # Data and phylogeny
#' data(anolis.tree)
#' data(anolis.data)
#' sortedData <- sortTraitData(anolis.tree, anolis.data,
#' data.name="Male_SVL", log.trait=TRUE, pass.ultrametric=TRUE)
#' phy <- sortedData$phy
#' male.length <- sortedData$trait
#' phy.clade <- extract.clade(phy, 182)
#' male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, rownames(male.length)),])
#'
#' # Brownian motion model
#' transformPhylo.ML(male.length.clade , phy=phy.clade, model="bm")
#'
#' # Delta
#' transformPhylo.ML(male.length.clade , phy=phy.clade, model="delta", upperBound=2)
#'
#' # The upper confidence interval for kappa is outside the bounds so try increasing
#' # the upper bound
#'
#' transformPhylo.ML(male.length.clade , phy=phy.clade, model="delta", upperBound=5)
#'
#' # Test for different rates in different clades - here with 2 hypothesised
#' # unusual rates compared to the background
#'
#' # This fits the non-censored model of O'Meara et al. (2006)
#' phy.clade$node.label[which(phy.clade$node.label == "3")] <- 2
#' transformPhylo.ML(male.length.clade, phy=phy.clade, model="clade", nodeIDs=c(49, 54))
#'
#' # Identify rate shifts and print and plot results with upto three rate shifts
#' # and minimum clade size of 20.
#' anolisSVL_MEDUSA <- transformPhylo.ML(male.length.clade, phy=phy.clade, model="tm1",
#' minCladeSize=10, nSplits=2)
#' @export

transformPhylo.ML <- function (y, phy, model = NULL, modelCIs = TRUE, nodeIDs = NULL, 
    rateType = NULL, minCladeSize = 1, nSplits = 2, splitTime = NULL, 
    boundaryAge = 10, testAge = 1, restrictNode = NULL, lambdaEst = FALSE, 
    acdcScalar = FALSE, branchLabels = NULL, hiddenSpeciation = FALSE, 
    full.phy = NULL, useMean = FALSE, profilePlot = FALSE, lowerBound = NULL, 
    upperBound = NULL, covPIC = TRUE, n.cores = 1, tol = NULL, 
    meserr = NULL, controlList = c(fnscale = -1, maxit = 100, 
        factr = 1e-07, pgtol = 0, type = 2, lmm = 5), returnPhy = FALSE, 
    print.warnings = FALSE, mode.order = NULL, rate.var = FALSE, 
    testShiftTimes = NULL, saveAll = TRUE) 
{
    model <- tolower(model)
    all.models <- c("bm", "kappa", "lambda", "delta", "ou", "acdc", 
        "psi", "multipsi", "trend", "free", "clade", "tm1", "tm2", 
        "timeslice", "modeslice")
    if (any(is.na((match(model, all.models))))) 
        stop(paste(model, "not recognised - please provide one of", 
            paste0(all.models, collapse = ", ")))
    bounds <- matrix(c(1e-08, 1, 1e-08, 1, 1e-08, 5, 1e-08, 20, 
        0, 1, 1e-08, 1000, 1e-10, 20), 7, 2, byrow = TRUE)
    rownames(bounds) <- c("kappa", "lambda", "delta", "alpha", 
        "psi", "rate", "acdcrate")
    lower.function.warning <- function() if (print.warnings) 
        warning("Confidence limits fall outside parameter bounds - consider changing lowerBound")
    upper.function.warning <- function() if (print.warnings) 
        warning("Confidence limits fall outside parameter bounds - consider changing upperBound")
    aic.fun <- function(likelihood, k) return(-2 * likelihood + 
        2 * k)
    aicc.fun <- function(likelihood, k, n) return(-2 * likelihood + 
        2 * k + ((2 * k * (k + 1))/(n - k - 1)))
    if (acdcScalar && !is.null(nodeIDs)) 
        upperBound <- -1e-06
    x <- NULL
    switch(model, bm = {
        phy <- transformPhylo(phy = phy, model = "bm", meserr = meserr, 
            y = y)
        out <- likTraitPhylo(y, phy, covPIC = covPIC)
        out$logLikelihood <- out[[2]]
        out$brownianVariance <- out[[1]]
        out$root.state <- apply(y, 2, function(col.y) as.numeric(ace(col.y, 
            phy, method = "pic")[[1]][1]))
        out$AIC <- aic.fun(out$logLikelihood, 2)
        out$AICc <- aicc.fun(out$logLikelihood, 2, Ntip(phy))
        if (returnPhy) out$bmPhy <- phy
        class(out) <- "bm.ML"
    }, lambda = {
        if (is.null(lowerBound)) {
            lowerBound <- bounds["lambda", 1]
        }
        if (is.null(upperBound)) {
            upperBound <- bounds["lambda", 2]
        }
        lambda <- runif(1, lowerBound, upperBound)
        var.funlambda <- function(lambda) {
            return(transformPhylo.ll(y = y, phy = phy, lambda = lambda, 
                model = "lambda", meserr = meserr, covPIC = covPIC)[[2]])
        }
        vo <- optim(lambda, var.funlambda, method = "L-BFGS-B", 
            lower = lowerBound, upper = upperBound, control = controlList)
        if (modelCIs == TRUE) {
            lambda.fun <- function(param) {
                ll <- transformPhylo.ll(y, phy, model = "lambda", 
                  lambda = param, meserr = meserr, covPIC = covPIC)$logLikelihood
                return(ll - vo$value + 1.92)
            }
            if (lambda.fun(lowerBound) < 0) {
                LCI <- uniroot(lambda.fun, interval = c(lowerBound, 
                  vo$par))$root
            } else {
                LCI <- lowerBound
                lower.function.warning()
            }
            if (lambda.fun(upperBound) < 0) {
                UCI <- uniroot(lambda.fun, interval = c(vo$par, 
                  upperBound))$root
            } else {
                UCI <- upperBound
                upper.function.warning()
            }
            out <- list()
            out$MaximumLikelihood <- vo$value
            out$Lambda <- matrix(c(vo$par, LCI, UCI), 1, 3, byrow = TRUE)
            colnames(out$Lambda) <- c("MLLambda", "LowerCI", 
                "UpperCI")
        } else {
            out <- list()
            out$MaximumLikelihood <- vo$value
            out$Lambda <- matrix(vo$par, 1, 1, byrow = TRUE)
        }
        if (profilePlot) {
            par(mar = c(5, 5, 5, 5), oma = c(0, 0, 0, 0))
            lambdaCurve <- Vectorize(lambda.fun)
            curve(lambdaCurve(x), from = lowerBound[1], to = upperBound[1], 
                xlab = expression(paste("Pagel's ", lambda)), 
                ylab = "log-likelihood", las = 1, main = "profile plot", 
                lwd = 2)
            if (modelCIs) {
                abline(v = c(LCI, vo$par[1], UCI), lty = c(3, 
                  2, 3), lwd = 2, col = "#00000090")
            }
        }
        lambdaPhy <- transformPhylo(y = y, phy = phy, model = "lambda", 
            lambda = vo$par[1], meserr = meserr)
        out$brownianVariance <- likTraitPhylo(y = y, phy = lambdaPhy, 
            covPIC = covPIC)$brownianVariance
        out$root.state <- apply(y, 2, function(col.y) as.numeric(ace(col.y, 
            phy = lambdaPhy, method = "pic")[[1]][1]))
        out$AIC <- aic.fun(out$MaximumLikelihood, 3)
        out$AICc <- aicc.fun(out$MaximumLikelihood, 3, Ntip(lambdaPhy))
        if (returnPhy) out$lambdaPhy <- lambdaPhy
        class(out) <- "lambda.ML"
    }, kappa = {
        if (is.null(nodeIDs)) {
            nodeIDs <- Ntip(phy) + 1
        } else {
            nodeIDs <- nodeIDs
        }
        if (is.null(lowerBound)) {
            lowerBound <- bounds["kappa", 1]
            if (lambdaEst) lowerBound[2] <- bounds["lambda", 
                1]
        }
        if (is.null(upperBound)) {
            upperBound <- bounds["kappa", 2]
            if (lambdaEst) upperBound[2] <- bounds["lambda", 
                2]
        }
        kappa <- runif(1, lowerBound[1], upperBound[1])
        if (lambdaEst) kappa[2] <- runif(1, lowerBound[2], upperBound[2])
        var.funkappa <- function(param) {
            if (length(param) != 2) {
                lambda <- 1
            } else {
                lambda <- param[2]
            }
            kappa <- param[1]
            lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, 
                model = "lambda", meserr = meserr)
            return(transformPhylo.ll(y = y, phy = lambdaPhy, 
                kappa = kappa, nodeIDs = nodeIDs, model = "kappa", 
                meserr = meserr, covPIC = covPIC)[[2]])
        }
        vo <- optim(kappa, var.funkappa, method = "L-BFGS-B", 
            lower = lowerBound, upper = upperBound, control = controlList)
        if (lambdaEst) {
            lambda <- vo$par[2]
        } else {
            lambda <- 1
        }
        lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, 
            model = "lambda", meserr = meserr)
        if (modelCIs == TRUE) {
            kappa.fun <- function(param, chiSq = TRUE) {
                ll <- transformPhylo.ll(y = y, phy = lambdaPhy, 
                  kappa = param, model = "kappa", meserr = meserr, 
                  covPIC = covPIC)$logLikelihood
                if (chiSq) return(ll - vo$value + 1.92) else return(ll)
            }
            if (kappa.fun(lowerBound[1]) < 0) {
                LCI <- uniroot(kappa.fun, interval = c(lowerBound[1], 
                  vo$par[1]))$root
            } else {
                LCI <- lowerBound[1]
                lower.function.warning()
            }
            if (kappa.fun(upperBound[1]) < 0) {
                UCI <- uniroot(kappa.fun, interval = c(vo$par[1], 
                  upperBound[1]))$root
            } else {
                UCI <- upperBound[1]
                upper.function.warning()
            }
        }
        if (profilePlot) {
            par(mar = c(5, 5, 5, 5), oma = c(0, 0, 0, 0))
            kappaCurve <- Vectorize(kappa.fun)
            curve(kappaCurve(x, FALSE), from = lowerBound[1], 
                to = upperBound[1], xlab = expression(kappa), 
                ylab = "log-likelihood", las = 1, main = "profile plot", 
                lwd = 2)
            if (modelCIs) {
                abline(v = c(LCI, vo$par[1], UCI), lty = c(3, 
                  2, 3), lwd = 1, col = "#00000090")
            }
        }
        out <- list()
        out$MaximumLikelihood <- vo$value[1]
        if (modelCIs) {
            out$Kappa <- matrix(NA, 1, 3, byrow = TRUE)
            out$Kappa[1, ] <- c(vo$par[1], LCI, UCI)
            colnames(out$Kappa) <- c("MLKappa", "LowerCI", "UpperCI")
        } else {
            out$Kappa <- matrix(NA, 1, 1, byrow = TRUE)
            out$Kappa[1, ] <- c(vo$par[1])
            colnames(out$Kappa) <- c("MLKappa")
        }
        kappaPhy <- transformPhylo(y = y, phy = lambdaPhy, model = "kappa", 
            kappa = vo$par[1], meserr = meserr)
        out$brownianVariance <- likTraitPhylo(y = y, phy = kappaPhy, 
            covPIC = covPIC)$brownianVariance
        out$root.state <- apply(y, 2, function(col.y) as.numeric(ace(col.y, 
            kappaPhy, method = "pic")[[1]][1]))
        param <- 3
        if (lambdaEst) {
            out$lambda <- vo$par[2]
            param <- 4
        }
        out$AIC <- aic.fun(out$MaximumLikelihood, param)
        out$AICc <- aicc.fun(out$MaximumLikelihood, param, Ntip(phy))
        if (returnPhy) out$kappaPhy <- kappaPhy
        class(out) <- "kappa.ML"
    }, delta = {
        if (is.null(nodeIDs)) {
            nodeIDs <- Ntip(phy) + 1
        } else {
            nodeIDs <- nodeIDs
        }
        if (is.null(lowerBound)) {
            lowerBound <- bounds["delta", 1]
            if (lambdaEst) lowerBound[2] <- bounds["lambda", 
                1]
        }
        if (is.null(upperBound)) {
            upperBound <- bounds["delta", 2]
            if (lambdaEst) upperBound[2] <- bounds["lambda", 
                2]
        }
        delta <- runif(1, lowerBound[1], upperBound[1])
        if (lambdaEst) delta[2] <- runif(1, lowerBound[2], upperBound[2])
        var.fundelta <- function(param) {
            if (length(param) != 2) {
                lambda <- 1
            } else {
                lambda <- param[2]
            }
            lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, 
                model = "lambda", meserr = meserr)
            return(transformPhylo.ll(y = y, phy = lambdaPhy, 
                delta = param[1], nodeIDs = nodeIDs, model = "delta", 
                meserr = meserr, covPIC = covPIC)[[2]])
        }
        vo <- optim(delta, var.fundelta, method = "L-BFGS-B", 
            lower = lowerBound, upper = upperBound, control = controlList)
        if (lambdaEst) {
            lambda <- vo$par[2]
        } else {
            lambda <- 1
        }
        lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, 
            model = "lambda", meserr = meserr)
        if (modelCIs == TRUE) {
            delta.fun <- function(param, chiSq = TRUE) {
                ll <- transformPhylo.ll(y = y, phy = lambdaPhy, 
                  delta = param, model = "delta", nodeIDs = nodeIDs, 
                  meserr = meserr, covPIC = covPIC)$logLikelihood
                if (chiSq) return(ll - vo$value + 1.92) else return(ll)
            }
            if (delta.fun(lowerBound[1]) < 0) {
                LCI <- uniroot(delta.fun, interval = c(lowerBound[1], 
                  vo$par[1]))$root
            } else {
                LCI <- lowerBound[1]
                lower.function.warning()
            }
            if (delta.fun(upperBound[1]) < 0) {
                UCI <- uniroot(delta.fun, interval = c(vo$par[1], 
                  upperBound[1]))$root
            } else {
                UCI <- upperBound[1]
                upper.function.warning()
            }
        }
        if (profilePlot) {
            par(mar = c(5, 5, 5, 5), oma = c(0, 0, 0, 0))
            deltaCurve <- Vectorize(delta.fun)
            curve(deltaCurve(x, FALSE), from = lowerBound[1], 
                to = upperBound[1], xlab = expression(delta), 
                ylab = "log-likelihood", las = 1, main = "profile plot", 
                lwd = 2)
            if (modelCIs) {
                abline(v = c(LCI, vo$par[1], UCI), lty = c(3, 
                  2, 3), lwd = 2, col = "#00000090")
            }
        }
        out <- list()
        out$MaximumLikelihood <- vo$value[1]
        if (modelCIs) {
            out$Delta <- matrix(c(vo$par[1], LCI, UCI), 1, 3, 
                byrow = TRUE)
            colnames(out$Delta) <- c("MLDelta", "LowerCI", "UpperCI")
        } else {
            out$Delta <- matrix(vo$par[1], 1, 1, byrow = TRUE)
            colnames(out$Delta) <- c("MLDelta")
        }
        deltaPhy <- transformPhylo(y = y, phy = lambdaPhy, model = "delta", 
            delta = vo$par[1], meserr = meserr)
        out$brownianVariance <- likTraitPhylo(phy = deltaPhy, 
            y = y, covPIC = covPIC)$brownianVariance
        out$root.state <- apply(y, 2, function(col.y) as.numeric(as.numeric(ace(col.y, 
            phy = deltaPhy, method = "pic")[[1]][1])))
        param <- 3
        if (lambdaEst) {
            out$lambda <- vo$par[2]
            names(out)[4] <- "Lambda"
            param <- 4
        }
        out$AIC <- aic.fun(out$MaximumLikelihood, param)
        out$AICc <- aicc.fun(out$MaximumLikelihood, param, Ntip(phy))
        if (returnPhy) {
            out$deltaPhy <- deltaPhy
        }
        class(out) <- "delta.ML"
    }, ou = {
        if (is.null(nodeIDs)) nodeIDs <- Ntip(phy) + 1 else nodeIDs <- nodeIDs
        if (is.null(lowerBound)) {
            lowerBound <- bounds["alpha", 1]
            if (lambdaEst) lowerBound[2] <- bounds["lambda", 
                1]
        }
        if (is.null(upperBound)) {
            upperBound <- bounds["alpha", 2]
            if (lambdaEst) upperBound[2] <- bounds["lambda", 
                2]
        }
        alpha <- 0.01
        if (lambdaEst) alpha[2] <- runif(1, lowerBound[2], upperBound[2])
        n.par <- length(lowerBound)
        if (!is.ultrametric(phy)) {
            if (ncol(y) > 1) stop("non-ultrametric phy and OU model only applicable for single traits, sorry")
            print("non-ultrametric phy and OU model - using variance-covariance matrix, not tree-transformation")
            cophenetic.dist <- cophenetic.phylo(phy)
            vcv.matrix <- vcv(phy)
            alpha[1] <- log(alpha[1])
            lowerBound[1] <- log(lowerBound[1])
            upperBound[1] <- log(upperBound[1])
            if (lambdaEst) stop("non-ultrametric phy and OU model not applicable with lambda model, sorry")
            anc.loc <- 3
            bm.loc <- 2
            alpha[c(bm.loc, anc.loc)] <- log(c(0.1, 0.1))
            lowerBound[c(bm.loc, anc.loc)] <- log(c(1e-08, NA))
            upperBound[c(bm.loc, anc.loc)] <- c(log(10), NA)
        }
        if (is.ultrametric(phy)) {
            var.funOU <- function(param) {
                if (length(param) != 2) {
                  lambda <- 1
                } else {
                  lambda <- param[2]
                }
                alpha.int <- param[1]
                lambdaPhy <- transformPhylo(y = y, phy = phy, 
                  lambda = lambda, model = "lambda")
                return(transformPhylo.ll(y = y, phy = lambdaPhy, 
                  alpha = alpha.int, nodeIDs = nodeIDs, model = "OU", 
                  meserr = meserr, covPIC = covPIC)[[2]])
            }
            vo <- optim(as.numeric(alpha), var.funOU, method = "L-BFGS-B", 
                lower = as.numeric(lowerBound), upper = as.numeric(upperBound), 
                control = controlList)
        } else {
            var.funOU <- function(param) {
                vcv.phy <- vcv(phy)
                co.phy <- cophenetic(phy)
                return(transformPhylo.ll(y = y, phy = phy, model = "OU", 
                  alpha = exp(param[1]), meserr = meserr, mu = exp(param[anc.loc]), 
                  sigma.sq = exp(param[bm.loc]), covPIC = covPIC, 
                  vcv.matrix = vcv.phy, cophenetic.dist = co.phy))
            }
            start <- 200
            count <- 0
            no.solution <- TRUE
            while (no.solution) {
                vo <- try(optim(as.numeric(alpha), var.funOU, 
                  method = "L-BFGS-B", lower = as.numeric(lowerBound), 
                  upper = as.numeric(upperBound), control = controlList), 
                  silent = TRUE)
                if (is(vo)[1] != "try-error") {
                  no.solution <- FALSE
                } else {
                  start <- start - count
                  upperBound[c(bm.loc, anc.loc)] <- c(log(start), 
                    log(start))
                  count <- count + 10
                }
            }
        }
        if (lambdaEst) {
            lambda <- vo$par[2]
        } else {
            lambda <- 1
        }
        lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, 
            model = "lambda")
        if (!is.ultrametric(phy)) {
            ou.tr <- function(alpha) {
                vcv.matrix <- transformPhylo(y = y, phy = phy, 
                  alpha = alpha, nodeIDs = nodeIDs, model = "OU", 
                  meserr = meserr)
                mu <- mu.mean(vcv.matrix, y)[1, 1]
                sigma.sq <- sig.sq(mu, vcv.matrix, y)[1, 1]
                reml.lik <- mvtnorm::dmvnorm(y[, 1], mean = rep(mu, 
                  ncol(vcv.matrix)), sigma = vcv.matrix * sigma.sq, 
                  log = TRUE)
                return(list(reml.lik = reml.lik, reml.sigma.sq = sigma.sq, 
                  reml.mu = mu))
            }
            reml.out <- ou.tr(alpha = exp(vo$par[1]))
            vo$value[1] <- reml.out[[1]]
        }
        if (modelCIs == TRUE) {
            if (is.ultrametric(phy)) {
                ou.fun <- function(param, chiSq = TRUE) {
                  ll <- transformPhylo.ll(y, lambdaPhy, model = "OU", 
                    alpha = param, nodeIDs = nodeIDs, meserr = meserr, 
                    covPIC = covPIC)$logLikelihood
                  if (chiSq) {
                    return(ll - vo$value + 1.92)
                  } else {
                    return(ll)
                  }
                }
            } else {
                ou.fun <- function(param, chiSq = TRUE) {
                  ll <- ou.tr(alpha = exp(param))[[1]]
                  if (chiSq) {
                    return(ll - reml.out[[1]] + 1.92)
                  } else {
                    return(ll)
                  }
                }
            }
            lower.attempt <- try(uniroot(ou.fun, interval = c(lowerBound[1], 
                vo$par[1]))$root, silent = TRUE)
            if (is.numeric(lower.attempt)) {
                LCI <- uniroot(ou.fun, interval = c(lowerBound[1], 
                  vo$par[1]))$root
            } else {
                LCI <- lowerBound[1]
                lower.function.warning()
            }
            upper.attempt <- try(uniroot(ou.fun, interval = c(vo$par[1], 
                upperBound[1]))$root, silent = TRUE)
            if (is.numeric(upper.attempt)) {
                UCI <- uniroot(ou.fun, interval = c(vo$par[1], 
                  upperBound[1]))$root
            } else {
                UCI <- upperBound[1]
                upper.function.warning()
            }
        }
        if (!is.ultrametric(phy)) {
            vo$par[1] <- exp(vo$par[1])
            if (modelCIs == TRUE) {
                UCI <- exp(UCI[1])
                LCI <- exp(LCI[1])
            }
            lowerBound[1] <- exp(lowerBound[1])
            upperBound[1] <- exp(upperBound[1])
        }
        if (profilePlot) {
            par(mar = c(5, 5, 5, 5), oma = c(0, 0, 0, 0))
            if (is.ultrametric(phy)) {
                ouCurve <- Vectorize(ou.fun)
                curve(ouCurve(x, FALSE), from = lowerBound[1], 
                  to = upperBound[1], xlab = expression(alpha), 
                  ylab = "log-likelihood", las = 1, main = "profile plot", 
                  lwd = 2)
                if (modelCIs) abline(v = c(LCI, vo$par[1], UCI), 
                  lty = c(3, 2, 3), lwd = 2, col = "#00000090")
            } else {
                ou.fun.profile <- function(param) {
                  ll <- ou.tr(param)[[1]]
                  return(ll)
                }
                ouCurve <- Vectorize(ou.fun.profile)
                curve(ouCurve(x), from = lowerBound[1], to = upperBound[1], 
                  xlab = expression(alpha), ylab = "log-likelihood", 
                  las = 1, main = "profile plot", lwd = 2)
                if (modelCIs) abline(v = c(LCI[1], vo$par[1], 
                  UCI[1]), lty = c(3, 2, 3), lwd = 2, col = "#00000090")
            }
        }
        out <- list()
        out$MaximumLikelihood <- vo$value[1]
        if (modelCIs) {
            out$Alpha <- matrix(c(vo$par[1], LCI, UCI), 1, 3, 
                byrow = TRUE)
            colnames(out$Alpha) <- c("MLAlpha", "LowerCI", "UpperCI")
        } else {
            out$Alpha <- matrix(vo$par[1], 1, 1, byrow = TRUE)
            colnames(out$Alpha) <- c("MLAlpha")
        }
        if (is.ultrametric(phy)) {
            ouPhy <- transformPhylo(y = y, phy = lambdaPhy, model = "OU", 
                alpha = vo$par[1], meserr = meserr)
            out$brownianVariance <- likTraitPhylo(y = y, phy = ouPhy, 
                covPIC = covPIC)$brownianVariance
            out$root.state <- apply(y, 2, function(col.y) as.numeric(as.numeric(ace(col.y, 
                phy = ouPhy, method = "pic")[[1]][1])))
        } else {
            out$brownianVariance <- reml.out$reml.sigma.sq
            out$root.state <- reml.out$reml.mu
        }
        names(out) <- c("MaximumLikelihood", "Alpha", "brownianVariance", 
            "root.state")
        param <- 3
        if (lambdaEst) {
            out$lambda <- vo$par[2]
            param <- 4
        }
        out$AIC <- aic.fun(out$MaximumLikelihood, param)
        out$AICc <- aicc.fun(out$MaximumLikelihood, param, Ntip(phy))
        if (returnPhy) out$ouPhy <- ouPhy
        class(out) <- "ou.ML"
    }, acdc = {
        if (is.null(nodeIDs)) {
            nodeIDs <- Ntip(phy) + 1
        } else {
            nodeIDs <- nodeIDs
        }
        rootBranchingTime <- nodeTimes(phy)[1, 1]
        if (is.null(lowerBound)) {
            lowerBound <- log(bounds["acdcrate", 1])/rootBranchingTime
        }
        if (is.null(upperBound)) {
            upperBound <- log(bounds["acdcrate", 2])/rootBranchingTime
        }
        if (acdcScalar) {
            if (is.na(lowerBound[2])) {
                lowerBound[2] <- 1
            }
            if (is.na(upperBound[2])) {
                upperBound[2] <- 5
            }
        }
        if (lambdaEst) {
            lowerBound <- c(lowerBound, bounds["lambda", 1])
            upperBound <- c(upperBound, bounds["lambda", 2])
        }
        acdcRate <- runif(1, lowerBound[1], upperBound[1])
        if (acdcScalar && lambdaEst) {
            acdcRate[2] <- runif(1, lowerBound[2], upperBound[2])
            acdcRate[3] <- runif(1, lowerBound[3], upperBound[3])
        }
        if (!acdcScalar && lambdaEst || acdcScalar && !lambdaEst) {
            acdcRate[2] <- runif(1, lowerBound[2], upperBound[2])
        }
        if (acdcScalar) {
            var.funACDC <- function(param) {
                if (lambdaEst) lambda <- tail(param, 1) else lambda <- 1
                acdc.est <- param[1]
                scalarRate <- param[2]
                lambdaPhy <- transformPhylo(y = y, phy = phy, 
                  lambda = lambda, model = "lambda", meserr = meserr)
                return(transformPhylo.ll(lambdaPhy, acdcRate = acdc.est, 
                  nodeIDs = nodeIDs, model = "ACDC", y = y, cladeRates = scalarRate, 
                  meserr = meserr, covPIC = covPIC)[[2]])
            }
        } else {
            var.funACDC <- function(param) {
                if (lambdaEst) lambda <- param[2] else lambda <- 1
                acdc.est <- param[1]
                lambdaPhy <- transformPhylo(y = y, phy = phy, 
                  lambda = lambda, model = "lambda", meserr = meserr)
                return(transformPhylo.ll(lambdaPhy, acdcRate = acdc.est, 
                  nodeIDs = nodeIDs, model = "ACDC", y = y, cladeRates = 1, 
                  meserr = meserr, covPIC = covPIC)[[2]])
            }
        }
        vo <- optim(acdcRate, var.funACDC, method = "L-BFGS-B", 
            lower = lowerBound, upper = upperBound, control = controlList)
        if (lambdaEst) lambda <- tail(vo$par, 1) else lambda <- 1
        lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, 
            model = "lambda", meserr = meserr)
        if (acdcScalar) cladeRateEst <- vo$par[2] else cladeRateEst <- 1
        if (modelCIs) {
            ACDC.fun <- function(param, chiSq = TRUE) {
                ll <- transformPhylo.ll(y, lambdaPhy, acdcRate = param, 
                  nodeIDs = nodeIDs, model = "ACDC", cladeRates = cladeRateEst, 
                  meserr = meserr, covPIC = covPIC)$logLikelihood
                if (chiSq) return(ll - vo$value + 1.92) else return(ll)
            }
            if (ACDC.fun(lowerBound[1]) < 0) {
                LCI <- uniroot(ACDC.fun, interval = c(lowerBound[1], 
                  vo$par[1]))$root
            } else {
                LCI <- lowerBound[1]
                lower.function.warning()
            }
            if (ACDC.fun(upperBound[1]) < 0) {
                UCI <- uniroot(ACDC.fun, interval = c(vo$par[1], 
                  upperBound[1]))$root
            } else {
                UCI <- upperBound[1]
                upper.function.warning()
            }
        }
        if (profilePlot) {
            par(mar = c(5, 5, 5, 5), oma = c(0, 0, 0, 0))
            acdcCurve <- Vectorize(ACDC.fun)
            curve(acdcCurve(x, FALSE), from = lowerBound[1], 
                to = upperBound[1], xlab = "ACDC rate", ylab = "log-likelihood", 
                las = 1, main = "profile plot", lwd = 2)
            if (modelCIs) {
                abline(v = c(LCI, vo$par[1], UCI), lty = c(3, 
                  2, 3), lwd = 2, col = "#00000090")
            }
        }
        out <- list()
        out$MaximumLikelihood <- vo$value[1]
        if (modelCIs) {
            out$ACDC <- matrix(c(vo$par[1], LCI, UCI), 1, 3, 
                byrow = TRUE)
            colnames(out$ACDC) <- c("MLacdc", "LowerCI", "UpperCI")
        } else {
            out$ACDC <- matrix(vo$par[1], 1, 1, byrow = TRUE)
            colnames(out$ACDC) <- c("MLacdc")
        }
        if (acdcScalar) out$scalar <- scaleClade <- vo$par[2] else scaleClade <- 1
        acdcPhy <- transformPhylo(y = y, phy = lambdaPhy, model = "ACDC", 
            acdcRate = vo$par[1], nodeIDs = nodeIDs, cladeRates = scaleClade, 
            meserr = meserr)
        out$brownianVariance <- likTraitPhylo(phy = acdcPhy, 
            y = y, covPIC = covPIC)$brownianVariance
        out$root.state <- apply(y, 2, function(col.y) as.numeric(as.numeric(ace(col.y, 
            phy = acdcPhy, method = "pic")[[1]][1])))
        param <- length(vo$par) + 2
        if (lambdaEst) {
            out$lambda <- tail(vo$par, 1)
            param <- param + 1
        }
        n <- Ntip(phy)
        out$AIC <- aic.fun(out$MaximumLikelihood, param)
        out$AICc <- aicc.fun(out$MaximumLikelihood, param, Ntip(phy))
        if (returnPhy) out$acdcPhy <- acdcPhy
        class(out) <- "acdc.ML"
    }, psi = {
        if (is.null(lowerBound)) {
            lowerBound <- bounds["psi", 1]
            if (lambdaEst) {
                lowerBound[2] <- bounds["lambda", 1]
            }
        }
        if (is.null(upperBound)) {
            upperBound <- bounds["psi", 2]
            if (lambdaEst) {
                upperBound[2] <- bounds["lambda", 2]
            }
        }
        psi <- runif(1, lowerBound[1], upperBound[1])
        if (lambdaEst) psi[2] <- runif(1, lowerBound[2], upperBound[2])
        if (hiddenSpeciation) {
            if (is.null(full.phy)) stop("please provide a full phylogeny")
            full.data.match <- match(full.phy$tip.label, rownames(y))
            tips.no.data <- full.phy$tip.label[which(is.na(full.data.match))]
            phy <- dropTipPartial(full.phy, tips.no.data)
        }
        if (is.ultrametric(phy)) {
            phy.bd <- suppressWarnings(birthdeath(phy))
        } else {
            phy.bd <- suppressWarnings (birthdeath_motmot(phy))
        }
        mu_over_lambda <- phy.bd[[4]][1]
        lambda_minus_mu <- phy.bd[[4]][2]
        lambda.sp <- as.numeric(lambda_minus_mu/(1 - mu_over_lambda))
        mu.ext <- as.numeric(lambda_minus_mu/(1/mu_over_lambda - 
            1))
        if (mu.ext > 0) {
            phy <- sampleHiddenSp(phy, lambda.sp = lambda.sp, 
                mu.ext = mu.ext, useMean = useMean)
        } else {
            phy$hidden.speciation <- NULL
        }
        var.funpsi <- function(param) {
            if (length(param) != 2) lambda <- 1 else lambda <- param[2]
            psi <- param[1]
            lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, 
                model = "lambda", meserr = meserr)
            return(transformPhylo.ll(y = y, phy = lambdaPhy, 
                psi = psi, model = "psi", meserr = meserr, covPIC = covPIC, 
                lambda.sp = lambda.sp)[[2]])
        }
        vo <- optim(psi, var.funpsi, method = "L-BFGS-B", lower = lowerBound, 
            upper = upperBound, control = controlList)
        if (lambdaEst) lambda <- vo$par[2] else lambda <- 1
        lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, 
            model = "lambda", meserr = meserr)
        if (modelCIs == TRUE) {
            psi.fun <- function(param, chiSq = TRUE) {
                ll <- transformPhylo.ll(y, lambdaPhy, model = "psi", 
                  psi = param, meserr = meserr, covPIC = covPIC, 
                  lambda.sp = lambda.sp)$logLikelihood
                if (chiSq) return(ll - vo$value + 1.92) else return(ll)
            }
            if (psi.fun(lowerBound[1]) < 0) {
                LCI <- uniroot(psi.fun, interval = c(lowerBound[1], 
                  vo$par[1]))$root
            } else {
                LCI <- lowerBound[1]
                lower.function.warning()
            }
            if (psi.fun(upperBound[1]) < 0) {
                UCI <- uniroot(psi.fun, interval = c(vo$par[1], 
                  upperBound[1]))$root
            } else {
                UCI <- upperBound[1]
                upper.function.warning()
            }
        }
        if (profilePlot) {
            par(mar = c(5, 5, 5, 5), oma = c(0, 0, 0, 0))
            psiCurve <- Vectorize(psi.fun)
            curve(psiCurve(x, FALSE), from = lowerBound[1], to = upperBound[1], 
                xlab = expression(psi), ylab = "log-likelihood", 
                las = 1, main = "profile plot", lwd = 2)
            if (modelCIs) {
                abline(v = c(LCI, vo$par[1], UCI), lty = c(3, 
                  2, 3), lwd = 2, col = "#00000090")
            }
        }
        out <- list()
        out$MaximumLikelihood <- vo$value[1]
        if (modelCIs) {
            out$psi <- matrix(c(vo$par[1], LCI, UCI), 1, 3, byrow = TRUE)
            colnames(out$psi) <- c("MLpsi", "LowerCI", "UpperCI")
        } else {
            out$psi <- matrix(vo$par[1], 1, 1, byrow = TRUE)
            colnames(out$psi) <- c("MLpsi")
        }
        psiPhy <- transformPhylo(y = y, phy = lambdaPhy, model = "psi", 
            psi = vo$par[1], meserr = meserr, lambda.sp = lambda.sp)
        out$brownianVariance <- likTraitPhylo(y = y, phy = psiPhy, 
            covPIC = covPIC)$brownianVariance
        out$root.state <- apply(y, 2, function(col.y) as.numeric(as.numeric(ace(col.y, 
            phy = psiPhy, method = "pic")[[1]][1])))
        param <- 3
        if (lambdaEst) {
            out$lambda <- vo$par[2]
            param <- 4
        }
        out$AIC <- aic.fun(out$MaximumLikelihood, param)
        out$AICc <- aicc.fun(out$MaximumLikelihood, param, Ntip(phy))
        if (returnPhy) out$psiPhy <- psiPhy
        class(out) <- "psi.ML"
    }, multipsi = {
        if (is.null(branchLabels)) stop("for 'multipsi' model must provide branchLabels giving state for each branch")
        states <- levels(factor(branchLabels))
        if (is.null(lowerBound)) {
            lowerBound <- bounds[rep("psi", length(states)), 
                1]
            if (lambdaEst) lowerBound[length(states) + 1] <- bounds["lambda", 
                1]
        }
        if (is.null(upperBound)) {
            upperBound <- bounds[rep("psi", length(states)), 
                2]
            if (lambdaEst) upperBound[length(states) + 1] <- bounds["lambda", 
                2]
        }
        ran.start <- runif(1, lowerBound[1], upperBound[1])
        start <- rep(ran.start, length(states))
        if (lambdaEst) start <- c(start, runif(1, lowerBound[2], 
            upperBound[2]))
        if (hiddenSpeciation) {
            if (is.null(full.phy)) stop("please provide a full phylogeny")
            full.data.match <- match(full.phy$tip.label, rownames(y))
            tips.no.data <- full.phy$tip.label[which(is.na(full.data.match))]
            phy <- dropTipPartial(full.phy, tips.no.data)
        }
        if (is.ultrametric(phy)) {
            phy.bd <- birthdeath(phy)
        } else {
            phy.bd <- birthdeath_motmot(phy)
        }
        mu_over_lambda <- phy.bd[[4]][1]
        lambda_minus_mu <- phy.bd[[4]][2]
        lambda.sp <- as.numeric(lambda_minus_mu/(1 - mu_over_lambda))
        mu.ext <- as.numeric(lambda_minus_mu/(1/mu_over_lambda - 
            1))
        if (mu.ext > 0) {
            phy <- sampleHiddenSp(phy, lambda.sp = lambda.sp, 
                mu.ext = mu.ext, useMean = useMean)
        } else {
            phy$hidden.speciation <- NULL
        }
        var.funmultipsi <- function(param) {
            all.param <- length(param)
            if (lambdaEst) {
                lambda <- param[all.param]
                psi <- param[-all.param]
            } else {
                psi <- param
                lambda <- 1
            }
            lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, 
                model = "lambda", meserr = meserr)
            return(transformPhylo.ll(y = y, phy = lambdaPhy, 
                branchLabels = branchLabels, psi = psi, model = "multipsi", 
                meserr = meserr, covPIC = covPIC, lambda.sp = lambda.sp)[[2]])
        }
        vo <- optim(start, var.funmultipsi, method = "L-BFGS-B", 
            lower = lowerBound, upper = upperBound, control = controlList)
        if (lambdaEst) lambda <- tail(vo$par, 1) else lambda <- 1
        lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, 
            model = "lambda", meserr = meserr)
        out <- list()
        out$MaximumLikelihood <- vo$value[1]
        out$psi <- matrix(NA, length(states), 3, byrow = TRUE, 
            dimnames = list(states, c("MLpsi", "LowerCI", "UpperCI")))
        out$psi[, 1] <- vo$par[1:length(states)]
        if (modelCIs == TRUE) {
            psi.fun <- function(param, chiSq = TRUE) {
                psi <- as.numeric(vo$par)
                psi[i] <- param
                ll <- transformPhylo.ll(y, lambdaPhy, model = "multipsi", 
                  branchLabels = branchLabels, psi = psi, meserr = meserr, 
                  covPIC = covPIC, lambda.sp = lambda.sp)$logLikelihood
                if (chiSq) return(ll - vo$value + 1.92) else return(ll)
            }
            for (i in 1:length(states)) {
                if (psi.fun(lowerBound[i]) < 0) {
                  LCI <- uniroot(psi.fun, interval = c(lowerBound[i], 
                    vo$par[i]))$root
                } else {
                  LCI <- lowerBound[1]
                  lower.function.warning()
                }
                if (psi.fun(upperBound[i]) < 0) {
                  UCI <- uniroot(psi.fun, interval = c(vo$par[i], 
                    upperBound[i]))$root
                } else {
                  UCI <- upperBound[1]
                  upper.function.warning()
                }
                out$psi[i, 2:3] <- c(LCI, UCI)
            }
        }
        if (profilePlot) {
            par(mar = c(5, 5, 5, 5), oma = c(0, 0, 0, 0), mfrow = c(length(states), 
                length(states)))
            for (i in 1:length(states)) {
                psiCurve <- Vectorize(psi.fun)
                curve(psiCurve(x, FALSE), from = lowerBound[1], 
                  to = upperBound[1], xlab = expression(psi), 
                  ylab = "log-likelihood", las = 1, main = paste0("profile plot psi ", 
                    i), lwd = 2)
                if (modelCIs) {
                  abline(v = c(out$psi[i, 2], out$psi[i, 1], 
                    out$psi[i, 3]), lty = c(3, 2, 3), lwd = 2, 
                    col = "#00000090")
                }
            }
        }
        multipsiPhy <- transformPhylo(y = y, phy = lambdaPhy, 
            model = "multipsi", psi = vo$par[1:length(states)], 
            meserr = meserr, lambda.sp = lambda.sp, branchLabels = branchLabels)
        out$brownianVariance <- likTraitPhylo(y = y, phy = multipsiPhy, 
            covPIC = covPIC)$brownianVariance
        out$root.state <- apply(y, 2, function(col.y) as.numeric(as.numeric(ace(col.y, 
            phy = multipsiPhy, method = "pic")[[1]][1])))
        param <- length(states) + 2
        if (lambdaEst) {
            out$lambda <- vo$par[length(states) + 1]
            param <- param + 1
        }
        out$AIC <- aic.fun(out$MaximumLikelihood, param)
        out$AICc <- aicc.fun(out$MaximumLikelihood, param, Ntip(phy))
        if (returnPhy) out$psiPhy <- multipsiPhy
        class(out) <- "multipsi.ML"
    }, trend = {
        if (is.ultrametric(phy)) stop("trend model only makes sense with non-ultrametric trees")
        if (ncol(y) > 1) stop("only when trait at a one time, please run each trait individually")
        bm.model.var <- transformPhylo.ML(y, phy, model = "bm", 
            meserr = meserr)$brownianVariance[1, 1]
        phy <- reorder(phy)
        tip.age.from.root <- c(nodeTimes(phy)[1, 1] - nodeTimes(phy)[which(phy$edge[, 
            2] <= Ntip(phy)), 2])
        yy <- data.frame(y, tip.age.from.root)
        if (lambdaEst == FALSE) {
            estimates <- pic.pgls(y ~ tip.age.from.root, phy, 
                y = yy, lambda = 1, return.intercept.stat = TRUE, 
                meserr = meserr)
        } else {
            estimates <- pic.pgls(y ~ tip.age.from.root, phy, 
                y = yy, lambda = "ML", return.intercept.stat = TRUE, 
                meserr = meserr)
        }
        slope.model <- estimates$model.summary$coef[1]
        intercept.model <- estimates$intercept[1]
        y2 <- matrix((tip.age.from.root * slope.model) + intercept.model)
        cont <- pic.motmot(y2, phy)
        contr <- c(cont[[1]][, 1], 0)
        rawVariances <- c(cont$contr[, 2], cont$V)
        brCov <- 1/(Ntip(phy) + 1) * (sum(contr^2/rawVariances))
        brCov <- bm.model.var - brCov
        likelihood.trend <- likTraitPhylo(y, phy, brCov = brCov)[[2]]
        if (lambdaEst) lambda <- estimates$lambda else lambda <- 1
        lambdaPhy <- transformPhylo(y = y, phy = phy, lambda = lambda, 
            model = "lambda", meserr = meserr)
        output.slope <- estimates$model.summary$coef[1:2]
        output.intercept <- estimates$intercept[1:2]
        out <- list()
        out$MaximumLikelihood <- likelihood.trend
        if (modelCIs) {
            out$Trend <- matrix(output.slope[1:2], 1, 2, byrow = TRUE)
            colnames(out$Trend) <- c("MLSlope", "StdError")
            out$root.state <- matrix(output.intercept[1:2], 1, 
                2, byrow = TRUE)
            colnames(out$root.state) <- c("ancestral.state", 
                "StdError")
        } else {
            out$Trend <- matrix(output.slope[1], 1, 1, byrow = TRUE)
            colnames(out$Trend) <- c("MLSlope")
            out$root.state <- matrix(output.intercept[1], 1, 
                1, byrow = TRUE)
            colnames(out$root.state) <- c("ancestral.state")
        }
        out$brownianVariance <- matrix(brCov)
        param <- 3
        if (lambdaEst) {
            out$lambda <- estimates$lambda
            names(out)[4] <- "Lambda"
            param <- 4
        }
        out$AIC <- aic.fun(out$MaximumLikelihood, param)
        out$AICc <- aicc.fun(out$MaximumLikelihood, param, Ntip(phy))
        if (returnPhy) {
            out$TrendPhy <- lambdaPhy
        }
        class(out) <- "trend.ML"
    }, free = {
        if (is.null(lowerBound)) {
            lowerBound <- bounds["rate", 1]
        }
        if (is.null(upperBound)) {
            upperBound <- bounds["rate", 2]
        }
        branchRates <- rep(1, length(phy$edge.length))
        var.funfree <- function(branchRates) {
            return(transformPhylo.ll(y, phy, branchRates = branchRates, 
                model = "free", meserr = meserr, covPIC = covPIC)[[2]])
        }
        vo <- optim(branchRates, var.funfree, method = "L-BFGS-B", 
            lower = 0, control = c(fnscale = -1, maxit = 10, 
                factr = 1e+14))
        phy2 <- phy
        phy2$edge.length <- phy$edge.length * vo$par
        out <- vector(mode = "list", length = 3)
        output.par <- likTraitPhylo(y = y, phy = phy2, covPIC = covPIC)
        out$MaximumLikelihood <- output.par$logLikelihood
        out$brownianVariance <- output.par$brownianVariance
        out$Rates <- vo$par
        out$root.state <- apply(y, 2, function(col.y) as.numeric(as.numeric(ace(col.y, 
            phy = phy2, method = "pic")[[1]][1])))
        if (vo$convergence == 0) {
            out$Convergence <- "Successful"
        } else {
            out$Convergence <- "Failed"
        }
        param <- length(branchRates) + 2
        if (lambdaEst) {
            out$lambda <- vo$par[length(branchRates) + 1]
            param <- param + 1
        }
        out$AIC <- aic.fun(out$MaximumLikelihood, param)
        out$AICc <- aicc.fun(out$MaximumLikelihood, param, Ntip(phy))
        if (returnPhy) out$freePhy <- phy2
        class(out) <- "free.ML"
    }, clade = {
        if (is.null(lowerBound)) {
            lowerBound <- bounds["rate", 1]
        }
        if (is.null(upperBound)) {
            upperBound <- bounds["rate", 2]
        }
        if (is.null(rateType)) {
            rateType <- rep("clade", length(nodeIDs))
        } else {
            rateType <- rateType
        }
        cladeRates <- rep(1, length(nodeIDs))
        var.funclade <- function(param) {
            return(transformPhylo.ll(y, phy, nodeIDs = nodeIDs, 
                cladeRates = param, model = "clade", rateType = rateType)[[2]])
        }
        vo <- optim(cladeRates, var.funclade, method = "L-BFGS-B", 
            lower = lowerBound, upper = upperBound, control = controlList)
        phyCladeFull <- transformPhylo(phy, model = "clade", 
            nodeIDs = nodeIDs, rateType = rateType, cladeRates = vo$par, 
            y = y, meserr = meserr)
        out <- list()
        if (modelCIs == TRUE) {
            free.fun <- function(param) {
                ll <- transformPhylo.ll(y, phyClade, model = "clade", 
                  nodeIDs = SingleNode, rateType = whichRateType, 
                  cladeRates = param, meserr = meserr, covPIC = covPIC)$logLikelihood
                return(ll - vo$value + 1.92)
            }
            out$Rates <- matrix(vo$value, length(nodeIDs), 4, 
                byrow = TRUE)
            colnames(out$Rates) <- c("node", "MLRate", "LowerCI", 
                "UpperCI")
            for (i in 1:length(nodeIDs)) {
                SingleNode <- nodeIDs[i]
                whichRateType <- rateType[i]
                phyClade <- transformPhylo(phyCladeFull, model = "clade", 
                  nodeIDs = SingleNode, rateType = whichRateType, 
                  cladeRates = 1/vo$par[i], y = y, meserr = meserr)
                if (free.fun(lowerBound) < 0) {
                  LCI <- uniroot(free.fun, interval = c(lowerBound, 
                    vo$par[i]))$root
                } else {
                  LCI <- lowerBound[1]
                  lower.function.warning()
                }
                if (free.fun(upperBound) < 0) {
                  UCI <- uniroot(free.fun, interval = c(vo$par[i], 
                    upperBound))$root
                } else {
                  UCI <- upperBound[1]
                  upper.function.warning()
                }
                out$Rates[i, ] <- c(nodeIDs[i], vo$par[i], LCI, 
                  UCI)
            }
        } else {
            out$Rates <- matrix(NA, length(nodeIDs), 2, byrow = TRUE)
            colnames(out$Rates) <- c("node", "MLRate")
            out$Rates[, 1] <- nodeIDs
            out$Rates[, 2] <- vo$par
        }
        out$MaximumLikelihood <- vo$value
        out$brownianVariance <- likTraitPhylo(y = y, phy = phyCladeFull, 
            covPIC = covPIC)$brownianVariance
        out$root.state <- apply(y, 2, function(col.y) as.numeric(as.numeric(ace(col.y, 
            phy = phyCladeFull, method = "pic")[[1]][1])))
        param <- 2 + length(vo$par)
        out$AIC <- aic.fun(out$MaximumLikelihood, param)
        out$AICc <- aicc.fun(out$MaximumLikelihood, param, Ntip(phy))
        if (returnPhy) out$freePhy <- phyCladeFull
        class(out) <- "clade.ML"
    }, tm1 = {
        if (is.null(lowerBound)) {
            lowerBound <- bounds["rate", 1]
        }
        if (is.null(upperBound)) {
            upperBound <- bounds["rate", 2]
        }
        nodes <- unique(sort(phy$edge[, 2]))
        nodeDepth <- node.depth(phy)
        nodesByRichness <- cbind(richness = nodeDepth[nodes], 
            node = nodes)
        searchNode <- nodesByRichness[nodesByRichness[, 1] >= 
            minCladeSize, 2]
        if (is.null(restrictNode) == FALSE) {
            ngroups <- length(restrictNode)
            cm <- caper::clade.matrix(phy)
            colnames(cm$clade.matrix) <- cm$tip.label
            cmMat <- cm$clade.matrix
            skipNodes <- numeric()
            for (i in 1:ngroups) {
                matDrop <- cmMat[, restrictNode[[i]]]
                nodeDrop <- rowSums(matDrop) < length(restrictNode[[i]]) & 
                  rowSums(matDrop) >= 1
                skipNodes <- c(skipNodes, as.numeric(rownames(matDrop)[nodeDrop]))
            }
            skipNodes <- unique(c(skipNodes, phy$edge[phy$edge.length < 
                tol, 2]))
            searchNode <- setdiff(searchNode, skipNodes)
        } else {
            skipNodes <- unique(phy$edge[phy$edge.length < tol, 
                2])
            searchNode <- setdiff(searchNode, skipNodes)
        }
        n <- length(y)
        BERateOut <- matrix(NA, nrow = nSplits + 1, ncol = (6 + 
            nSplits))
        fullModelOut <- vector(mode = "list", length = nSplits)
        MLsingle <- transformPhylo.ll(y = y, phy = phy, model = "bm", 
            meserr = meserr, covPIC = covPIC)[[2]]
        AICsingle <- aic.fun(MLsingle, 2)
        AICcsingle <- aicc.fun(MLsingle, 2, n)
        colnames(BERateOut) <- 1:dim(BERateOut)[2]
        colnames(BERateOut)[1:6] <- c("node", "shiftPos", "lnL", 
            "n.params", "AIC", "AICc")
        colnames(BERateOut)[-c(1:6)] <- paste0("rate.", 1:nSplits)
        rownames(BERateOut) <- c("BM ", paste0("shift.", 1:nSplits))
        BERateOut[1, 1:6] <- c(0, 1, MLsingle, 2, AICsingle, 
            AICcsingle)
        BERateOut <- data.frame(BERateOut)
        cat("\n", "BM model")
        cat("\n")
        print(BERateOut[1, 1:6])
        for (k in 1:nSplits) {
            cladeMembers <- matrix(NA, ncol = k, nrow = length(phy$edge[, 
                1]))
            if (k == 1) {
                searchNode <- searchNode
            } else {
                searchNode <- searchNode[!(searchNode %in% bestNodes)]
            }
            for (i in searchNode) {
                if (k == 1) {
                  currentNodeIDs <- i
                } else {
                  currentNodeIDs <- c(bestNodes, i)
                }
                cladeRates <- rep(1, length(currentNodeIDs))
                var.funclade.tm1 <- function(cladeRates) {
                  return(transformPhylo.ll(y, phy, nodeIDs = currentNodeIDs, 
                    rateType = rep("clade", length(currentNodeIDs)), 
                    cladeRates = cladeRates, model = "clade", 
                    meserr = meserr, covPIC = covPIC)[[2]])
                }
                currentCladeModel <- optim(cladeRates, var.funclade.tm1, 
                  method = "L-BFGS-B", lower = lowerBound, upper = upperBound, 
                  control = controlList)
                fullModelOut[[k]] <- rbind(fullModelOut[[k]], 
                  c(node = as.integer(i), shiftPos = 1, ML = currentCladeModel$value, 
                    currentCladeModel$par))
                currentModel <- currentCladeModel
                shiftPos <- 1
                param <- (k * 2) + 1
                currentModel <- list(currentModel, i, cladeMembers)
                currentML <- currentModel[[1]]$value
                AIC <- aic.fun(currentModel[[1]]$value, param)
                AICc <- aicc.fun(currentModel[[1]]$value, param, 
                  n)
                if (i == min(searchNode)) {
                  BERateOut[k + 1, 1:(6 + k)] <- c(currentModel[[2]], 
                    shiftPos, currentModel[[1]]$value, param, 
                    AIC, AICc, rev(currentModel[[1]]$par))
                } else {
                  if (currentML > BERateOut[k + 1, 3]) {
                    BERateOut[k + 1, 1:(6 + k)] <- c(currentModel[[2]], 
                      shiftPos, currentModel[[1]]$value, param, 
                      AIC, AICc, rev(currentModel[[1]]$par))
                  }
                }
            }
            if (k == 1) bestNodes <- BERateOut[k + 1, 1] else bestNodes <- c(bestNodes, 
                BERateOut[k + 1, 1])
            if (sum(BERateOut[, "shiftPos"] == 1, na.rm = T) > 
                0) BERateOut[which(BERateOut[, "shiftPos"] == 
                1), "shiftPos"] <- "clade"
            cat("\n", "Shift", k)
            cat("\n")
            print(BERateOut[k + 1, c(1:(6 + k))])
        }
        out <- list(as.data.frame(BERateOut), fullModelOut, y, 
            phy)
        class(out) <- "traitMedusa"
    }, tm2 = {
        if (is.null(lowerBound)) lowerBound <- bounds["rate", 
            1]
        if (is.null(upperBound)) upperBound <- bounds["rate", 
            2]
        nodes <- unique(sort(phy$edge[, 2]))
        nodeDepth <- node.depth(phy)
        nodesByRichness <- cbind(richness = nodeDepth[nodes], 
            node = nodes)
        searchNode <- nodesByRichness[nodesByRichness[, 1] >= 
            minCladeSize, 2]
        searchNodeType <- c(searchNode, searchNode[searchNode > 
            Ntip(phy)])
        searchType <- c(rep("clade", length(searchNode)), rep("branch", 
            sum(searchNode > Ntip(phy))))
        searchNodeTypeAll <- data.frame(searchType, searchNodeType, 
            stringsAsFactors = FALSE)
        if (is.null(restrictNode) == FALSE) {
            ngroups <- length(restrictNode)
            cm <- caper::clade.matrix(phy)
            colnames(cm$clade.matrix) <- cm$tip.label
            cmMat <- cm$clade.matrix
            skipNodes <- numeric()
            for (i in 1:ngroups) {
                matDrop <- cmMat[, restrictNode[[i]]]
                nodeDrop <- rowSums(matDrop) < length(restrictNode[[i]]) & 
                  rowSums(matDrop) >= 1
                skipNodes <- c(skipNodes, as.numeric(rownames(matDrop)[nodeDrop]))
            }
            skipNodes <- unique(c(skipNodes, phy$edge[phy$edge.length < 
                tol, 2]))
            searchNode <- setdiff(searchNode, skipNodes)
        } else {
            skipNodes <- unique(phy$edge[phy$edge.length < tol, 
                2])
            searchNode <- setdiff(searchNode, skipNodes)
        }
        tm2.fun <- function(whichSearchNode) which(searchNodeTypeAll[, 
            2] == whichSearchNode)
        searchNodeTypeAll <- searchNodeTypeAll[sort(unlist(sapply(searchNode, 
            tm2.fun))), ]
        n <- length(y)
        BERateOut <- matrix(NA, nrow = nSplits + 1, ncol = (6 + 
            (nSplits)))
        fullModelOut <- vector(mode = "list", length = nSplits)
        bestModel <- matrix(NA, ncol = 2, nrow = nSplits)
        fullModelOut <- vector(mode = "list", length = nSplits)
        MLsingle <- transformPhylo.ll(y = y, phy = phy, model = "bm", 
            meserr = meserr, covPIC = covPIC)[[2]]
        AICsingle <- aic.fun(MLsingle, 2)
        AICcsingle <- aicc.fun(MLsingle, 2, Ntip(phy))
        BERateOut[1, 1:6] <- c(0, 1, MLsingle, 2, AICsingle, 
            AICcsingle)
        colnames(BERateOut) <- 1:dim(BERateOut)[2]
        colnames(BERateOut)[1:6] <- c("node", "shiftPos", "lnL", 
            "n.params", "AIC", "AICc")
        colnames(BERateOut)[-c(1:6)] <- paste0("rate.", 1:nSplits)
        rownames(BERateOut) <- c("BM ", paste0("shift.", 1:nSplits))
        BERateOut[1, 1:6] <- c(0, 1, MLsingle, 2, AICsingle, 
            AICcsingle)
        BERateOut <- data.frame(BERateOut)
        cat("\n", "BM model")
        cat("\n")
        print(BERateOut[1, 1:6])
        for (k in 1:nSplits) {
            cladeMembers <- matrix(NA, ncol = k, nrow = length(phy$edge[, 
                1]))
            bestModel[bestModel[, 2] == "1", 2] <- "clade"
            bestModel[bestModel[, 2] == "2", 2] <- "branch"
            if (k == 1) {
                searchNodeTypeAll <- searchNodeTypeAll
            } else {
                dropNodeRow <- which(searchNodeTypeAll[, 1] == 
                  bestModel[k - 1, 2] & searchNodeTypeAll[, 2] == 
                  as.numeric(bestModel[k - 1, 1]))
                rowDropLogical <- row(searchNodeTypeAll)[, 1] != 
                  dropNodeRow
                searchNodeTypeAll <- searchNodeTypeAll[rowDropLogical, 
                  ]
            }
            if (k == 1) currentNodeIDs <- NULL else currentNodeIDs <- na.omit(as.numeric(bestModel[, 
                1]))
            if (k == 1) currentRateType <- NULL else currentRateType <- na.omit(bestModel[, 
                2])
            foo2 <- function(x_foo, k, currentNodeIDs = currentNodeIDs, 
                currentRateType = currentRateType) {
                currentNodeIDs <- c(currentNodeIDs, x_foo[, 2])
                currentRateType <- c(currentRateType, x_foo[, 
                  1])
                cladeRates <- rep(1, length(currentNodeIDs))
                var.funclade.tm2 <- function(cladeRates) return(transformPhylo.ll(y, 
                  phy, nodeIDs = currentNodeIDs, rateType = currentRateType, 
                  cladeRates = cladeRates, model = "clade", meserr = meserr, 
                  covPIC = covPIC)[[2]])
                currentCladeModel <- optim(cladeRates, var.funclade.tm2, 
                  method = "L-BFGS-B", lower = lowerBound, upper = upperBound, 
                  control = controlList)
                if (x_foo[, 1] == "clade") shiftPos <- 1
                if (x_foo[, 1] == "branch") shiftPos <- 2
                currentModel <- currentCladeModel
                param <- k * 2 + 1
                currentModel <- list(currentModel, x_foo[2], 
                  cladeMembers)
                currentML <- currentModel[[1]]$value
                AIC <- aic.fun(currentModel[[1]]$value, param)
                AICc <- aicc.fun(currentModel[[1]]$value, param, 
                  n)
                out <- c(currentModel[[2]][, 1], shiftPos, currentModel[[1]]$value, 
                  param, AIC, AICc, currentModel[[1]]$par)
                return(out)
            }
            searchNodeList <- vector(mode = "list", length = length(searchNodeTypeAll[, 
                1]))
            for (i in 1:length(searchNodeTypeAll[, 1])) searchNodeList[[i]] <- searchNodeTypeAll[i, 
                ]
            all.output <- parallel::mclapply(searchNodeList, 
                FUN = foo2, k = k, currentNodeIDs = currentNodeIDs, 
                currentRateType = currentRateType, mc.cores = n.cores)
            all.output <- matrix(unlist(all.output), nrow = length(searchNodeList), 
                byrow = TRUE)
            best.out <- all.output[which.max(all.output[, 3]), 
                ]
            fullModelOut[[k]] <- all.output
            BERateOut[k + 1, 1:(6 + k)] <- best.out
            bestModel[k, ] <- unlist(BERateOut[k + 1, 1:2])
            if (sum(BERateOut[, "shiftPos"] == 1, na.rm = T) > 
                0) BERateOut[which(BERateOut[, "shiftPos"] == 
                1), "shiftPos"] <- "clade"
            if (sum(BERateOut[, "shiftPos"] == 2, na.rm = T) > 
                0) BERateOut[which(BERateOut[, "shiftPos"] == 
                2), "shiftPos"] <- "branch"
            cat("\n", "Shift", k)
            cat("\n")
            print(BERateOut[k + 1, c(1:(6 + k))])
        }
        out <- list(as.data.frame(BERateOut), fullModelOut, y, 
            phy)
        class(out) <- "traitMedusa"
    }, modeslice = {
        maxTime <- nodeTimes(phy)[1, 1]
        input <- lowerBound <- upperBound <- c()
        mode.order <- tolower(mode.order)
        count <- 1
        if (methods::is(splitTime)[1] == "NULL") stop("please provide a shift time(s)")
        if (lambdaEst) stop("sorry Lambda estimation not applicable with modeslice model, sorry")
        if ((length(splitTime) + 1) != length(mode.order)) stop("mismatch between number of nodes and shift times")
        transformation <- c()
        if (any(is.na(match(mode.order, c("bm", "ou", "kappa", 
            "acdc"))))) stop("mode.order must be a combination of 'bm', 'ou', 'kappa', and/or 'acdc'")
        for (x in 1:length(mode.order)) {
            if (mode.order[x] == "bm") {
                if (rate.var) {
                  input[count] <- log(1)
                  lowerBound[count] <- log(bounds["rate", 1])
                  upperBound[count] <- log(bounds["rate", 2])
                  count <- count + 1
                  transformation <- c(transformation, "log")
                }
            }
            if (mode.order[x] == "ou") {
                input[count] <- log(0.1)
                lowerBound[count] <- log(bounds["alpha", 1])
                upperBound[count] <- log(bounds["alpha", 2])
                count <- count + 1
                transformation <- c(transformation, "log")
            }
            if (mode.order[x] == "kappa") {
                input[count] <- 0.1
                lowerBound[count] <- log(bounds["kappa", 1])
                upperBound[count] <- log(bounds["kappa", 2])
                count <- count + 1
                transformation <- c(transformation, "log")
            }
            if (mode.order[x] == "acdc") {
                if (acdcScalar) {
                  input[count] <- log(1)
                  lowerBound[count] <- log(1)
                  upperBound[count] <- log(bounds["rate", 2])
                  count <- count + 1
                  transformation <- c(transformation, "log")
                }
                lowerBound[count] <- -10/nodeTimes(phy)[1, 1]
                upperBound[count] <- -1e-06
                input[count] <- runif(1, lowerBound[count], upperBound[count])
                count <- count + 1
                transformation <- c(transformation, "none")
            }
        }
        n <- Ntip(phy)
        anc.loc <- length(lowerBound) + 1
        lowerBound[anc.loc] <- upperBound[anc.loc] <- NA
        input[anc.loc] <- log(0.1)
        transformation <- c(transformation, "log")
        bm.loc <- length(lowerBound) + 1
        lowerBound[bm.loc] <- log(1e-08)
        upperBound[bm.loc] <- NA
        input[bm.loc] <- log(0.1)
        transformation <- c(transformation, "log")
        var.fun.modeslice <- function(param) {
            input.in <- param[-c(anc.loc, bm.loc)]
            transform <- transformation[-c(anc.loc, bm.loc)]
            for (u in 1:length(transform)) if (transform[u] == 
                "log") input.in[u] <- exp(input.in[u])
            return(transformPhylo.ll(y = y, phy = phy, mode.order = mode.order, 
                mode.param = input.in, model = "modeslice", mu = exp(param[anc.loc]), 
                sigma.sq = exp(param[bm.loc]), splitTime = splitTime, 
                covPIC = covPIC, rate.var = rate.var, cladeRates = acdcScalar, 
                meserr = meserr))
        }
        vo <- optim(input, var.fun.modeslice, method = "L-BFGS-B", 
            lower = lowerBound, upper = upperBound, control = controlList)
        res.par <- vo$par
        for (u in 1:length(transformation)) if (transformation[u] == 
            "log") res.par[u] <- exp(res.par[u])
        rm.param <- c(bm.loc, anc.loc)
        tre.tr <- function(mode.param.input) {
            transform <- transformation[-rm.param]
            for (u in 1:length(transform)) if (transform[u] == 
                "log") mode.param.input[u] <- exp(mode.param.input[u])
            vcv.matrix <- transformPhylo(y = y, phy = phy, mode.order = mode.order, 
                mode.param = mode.param.input, model = "modeslice", 
                splitTime = splitTime, rate.var = rate.var, cladeRates = acdcScalar, 
                meserr = meserr)
            mu <- mu.mean(vcv.matrix, y)[1, 1]
            sigma.sq <- sig.sq(mu, vcv.matrix, y)[1, 1]
            reml.lik <- mvtnorm::dmvnorm(y[, 1], mean = rep(mu, 
                ncol(vcv.matrix)), sigma = vcv.matrix * sigma.sq, 
                log = TRUE)
            return(list(reml.lik = reml.lik, reml.sigma.sq = sigma.sq, 
                reml.mu = mu))
        }
        reml.out <- tre.tr(mode.param.input = vo$par[-rm.param])
        vo$value[1] <- reml.out[[1]]
        if (modelCIs == TRUE) {
            ci.fun <- function(param.in, chiSq = TRUE) {
                param2 <- vo$par[-rm.param]
                param2[k] <- param.in
                ll <- tre.tr(param2)[[1]]
                if (chiSq) {
                  return(ll - vo$value + 1.92)
                } else {
                  return(ll)
                }
            }
            param <- vo$par[-rm.param]
            UCI <- LCI <- c()
            for (k in 1:length(param)) {
                lower.attempt <- try(uniroot(ci.fun, interval = c(lowerBound[k], 
                  param[k]))$root, silent = TRUE)
                if (is.numeric(lower.attempt)) {
                  LCI[k] <- suppressWarnings(uniroot(ci.fun, 
                    interval = c(lowerBound[k], param[k]))$root)
                } else {
                  LCI[k] <- lowerBound[k]
                  lower.function.warning()
                }
                upper.attempt <- try(uniroot(ci.fun, interval = c(param[k], 
                  upperBound[k]))$root, silent = TRUE)
                if (is.numeric(upper.attempt)) {
                  UCI[k] <- suppressWarnings(uniroot(ci.fun, 
                    interval = c(param[k], upperBound[k]))$root)
                } else {
                  UCI[k] <- upperBound[k]
                  upper.function.warning()
                }
            }
        }
        for (u in 1:length(param)) if (transformation[u] == "log") param[u] <- exp(param[u])
        for (u in 1:length(param)) if (transformation[u] == "log") LCI[u] <- exp(LCI[u])
        for (u in 1:length(param)) if (transformation[u] == "log") UCI[u] <- exp(UCI[u])
        out <- list()
        out$MaximumLikelihood <- vo$value
        out$brownianVariance <- reml.out$reml.sigma.sq
        out$root.state <- reml.out$reml.mu
        count <- 1
        for (x in 1:length(mode.order)) {
            if (mode.order[x] == "ou") x.name <- "alpha"
            if (mode.order[x] == "kappa") x.name <- "kappa"
            if (mode.order[x] == "acdc") x.name <- "acdc.rate"
            if (mode.order[x] == "bm") {
                if (rate.var) {
                  if (!modelCIs) {
                    x.temp <- matrix(param[count])
                    colnames(x.temp) <- "BM.rate"
                    out[[length(out) + 1]] <- x.temp
                    names(out)[length(out)] <- paste0("mode.", 
                      x, ".", mode.order[x])
                    count <- count + 1
                  } else {
                    x.temp <- matrix(c(param[count], LCI[count], 
                      UCI[count]), nrow = 1)
                    colnames(x.temp) <- c("BM.rate", "LCI", "UCI")
                    out[[length(out) + 1]] <- x.temp
                    names(out)[length(out)] <- paste0("mode.", 
                      x, ".", mode.order[x])
                    count <- count + 1
                  }
                }
            } else {
                if (!modelCIs) {
                  if (x.name == "acdc.rate" && acdcScalar) {
                    acdc <- param[count + 1]
                    acdc.scale <- param[count]
                    x.temp <- matrix(c(acdc, acdc.scale))
                    colnames(x.temp) <- c("acdc.rate", "acdc scalar")
                    out[[length(out) + 1]] <- x.temp
                    names(out)[length(out)] <- paste0("mode.", 
                      x, ".", mode.order[x])
                    count <- count + 2
                  } else {
                    x.temp <- matrix(param[count])
                    colnames(x.temp) <- x.name
                    out[[length(out) + 1]] <- x.temp
                    names(out)[length(out)] <- paste0("mode.", 
                      x, ".", mode.order[x])
                    count <- count + 1
                  }
                } else {
                  if (x.name == "acdc.rate" && acdcScalar) {
                    acdc <- param[count + 1]
                    acdc.lci <- LCI[count + 1]
                    acdc.uci <- UCI[count + 1]
                    scalar <- param[count]
                    scalar.lci <- LCI[count]
                    scalar.uci <- UCI[count]
                    x.temp <- matrix(c(acdc, acdc.lci, acdc.uci, 
                      scalar, scalar.lci, scalar.uci), nrow = 1)
                    colnames(x.temp) <- c("acdc.rate", "acdc.LCI", 
                      "acdc.UCI", "scalar", "scalar.LCI", "scalar.UCI")
                    out[[length(out) + 1]] <- x.temp
                    names(out)[length(out)] <- paste0("mode.", 
                      x, ".", mode.order[x])
                    count <- count + 2
                  } else {
                    x.temp <- matrix(c(param[count], LCI[count], 
                      UCI[count]), nrow = 1)
                    colnames(x.temp) <- c(x.name, "LCI", "UCI")
                    out[[length(out) + 1]] <- x.temp
                    names(out)[length(out)] <- paste0("mode.", 
                      x, ".", mode.order[x])
                    count <- count + 1
                  }
                }
            }
        }
        k <- 2 + length(param)
        out[length(out) + 1] <- aic.fun(vo$value, k)
        names(out)[length(out)] <- "AIC"
        out[length(out) + 1] <- aicc.fun(vo$value, k, n)
        names(out)[length(out)] <- "AICc"
        class(out) <- "modeSlice.ML"
        
    }, timeslice = {
    	
    	if (!is.null(splitTime) && !is.null(testShiftTimes))
    	  print("Both splitTime and testShiftTimes provided so testShiftTimes will be ignored. Please set splitTime to NULL for testShiftTimes to be modelled.") 
    	
    	
        maxTime <- nodeTimes(phy)[1, 1]
        n <- Ntip(phy)
        if (is.null(controlList["pgtol"])) controlList["pgtol"] <- 0.1
        if (is.null(lowerBound)) {
            lowerBound <- bounds["rate", 1]
        }
        if (is.null(upperBound)) {
            upperBound <- bounds["rate", 2]
        }
        output.all <- FALSE
        if (is.null(splitTime)) {
            if (is.null(testShiftTimes)) {
                boundaryAge <- sort(boundaryAge, decreasing = TRUE)
                if ((maxTime - boundaryAge[1]) < 0) stop("Boundary age bigger than the tree age.")
                if (length(boundaryAge) == 2) {
                	  if ((maxTime - boundaryAge[2]) < 0) stop("Boundary age bigger than the tree age.")
                  boundaryAge2 <- boundaryAge[2]
                  boundaryAge <- boundaryAge[1]
                } else {
                  boundaryAge2 <- boundaryAge
                }
                splitTime <- seq(maxTime - boundaryAge, boundaryAge2, -testAge)
                if (min(splitTime) > boundaryAge2) splitTime <- c(splitTime, boundaryAge2)
                splitTime.orig <- splitTime
            } else {
                splitTime <- unique(sort(testShiftTimes, decreasing = TRUE))
                nSplits <- length(testShiftTimes)
                splitTime.orig <- splitTime
            }
            output.mat <- matrix(NA, nrow = (nSplits + 1), ncol = (3 + 
                nSplits + (nSplits + 1)) + (2 * ncol(y)))
            bm.phy <- transformPhylo(phy = phy, model = "bm", 
                meserr = meserr, y = y)
            bm.model <- likTraitPhylo(y, bm.phy, covPIC = covPIC)
            log.lik <- as.numeric(bm.model[[2]])
            aic <- aic.fun(log.lik, 2)
            aicc <- aicc.fun(log.lik, 2, Ntip(phy))
            anc.state <- apply(y, 2, function(col.y) as.numeric(as.numeric(ace(col.y, 
                bm.phy, method = "pic")[[1]][1])))
            colnames(output.mat) <- 1:ncol(output.mat)
            colnames(output.mat)[1:3] <- c("lnL", "AIC", "AICc")
            c.y <- ncol(y)
            colnames(output.mat)[4:(4 + c.y - 1)] <- paste0("sigma.sq.", 
                1:ncol(y))
            n.c.ct <- (4 + c.y)
            colnames(output.mat)[n.c.ct:(n.c.ct + c.y - 1)] <- paste0("anc.state.", 
                1:ncol(y))
            n.c.ct <- (n.c.ct + c.y):(n.c.ct + c.y + nSplits)
            colnames(output.mat)[n.c.ct] <- paste0("rates", 1:(nSplits + 
                1))
            n.c.ct <- (max(n.c.ct) + 1):(max(n.c.ct) + nSplits)
            colnames(output.mat)[n.c.ct] <- paste0("time.split", 
                1:nSplits)
            to.here <- which(regexpr("rates", colnames(output.mat)) == 
                1)[1] - 1
            output.mat[1, 1:to.here] <- c(log.lik, aic, aicc, 
                as.numeric(diag(bm.model[[1]])), anc.state)
            print("BM model", quote = FALSE)
            print(output.mat[1, 1:to.here])
            fixed.time <- NULL
            vo.out <- list()
            full.model.out <- list()
            if (saveAll) output.all <- TRUE
            
            for (q in 1:nSplits) {
                all.models <- parallel::mclapply(splitTime, mc.cores = n.cores, 
                  function(x_times) {
                    splitTimeInt <- sort(c(x_times, fixed.time))
                    nSplitInt.n <- length(splitTimeInt) + 1
                    rateVec <- rep(1, nSplitInt.n)
                    var.timeslice <- function(rate.vec) transformPhylo.ll(y, 
                      phy, timeRates = rate.vec, splitTime = splitTimeInt, 
                      model = "timeSlice", meserr = meserr, covPIC = covPIC)[[2]]
                    vo <- optim(rateVec, var.timeslice, method = "L-BFGS-B", 
                      lower = lowerBound, upper = upperBound, 
                      control = controlList)
                    c(vo$par, vo$value)
                  })
                all.models <- sapply(all.models, cbind)
                colnames(all.models) <- splitTime
                rownames(all.models) <- c(paste0("rate ", 1:(q + 1)), "loglikelihood")
                
                full.model.out[[q]] <- all.models
                best.model.n <- which.max(all.models[nrow(all.models), 
                  ])
                shift.time <- splitTime[best.model.n]
                fixed.time <- c(fixed.time, shift.time)
                splitTime <- splitTime[-which(splitTime == shift.time)]
                log.lik <- tail(all.models[, best.model.n], 1)
                rates <- all.models[, best.model.n][-dim(all.models)[1]]
                phy.temp <- transformPhylo(y = y, phy = phy, 
                  timeRates = rates, splitTime = fixed.time, 
                  model = "timeSlice", meserr = meserr)
                anc.state <- apply(y, 2, function(col.y) as.numeric(as.numeric(ace(col.y, 
                  phy = phy.temp, method = "pic")[[1]][1])))
                bVar <- as.numeric(likTraitPhylo(y, phy.temp, 
                  covPIC = covPIC)[[1]])
                param <- length(rates) + 2
                aic <- aic.fun(log.lik, param)
                aicc <- aicc.fun(log.lik, param, Ntip(phy))
                output.mat[q + 1, 1:to.here] <- c(log.lik, aic, 
                  aicc, bVar, anc.state)
                output.mat[q + 1, (to.here + 1):(to.here + length(rates))] <- rates
                start.time <- tail(1:dim(output.mat)[2], nSplits)[1:q]
                output.mat[q + 1, start.time] <- sort(fixed.time)
                na.test <- which(is.na(output.mat[q + 1, ]))
                print(paste("shift", q), quote = F)
                na.test <- which(is.na(output.mat[q + 1, ]))
                if (length(na.test) > 0) {
                  print(output.mat[q + 1, -which(is.na(output.mat[q + 
                    1, ]))])
                } else {
                  print(output.mat[q + 1, ])
                }
                rateVec <- rep(1, length(rates))
                var.timeslice <- function(rate.vec) transformPhylo.ll(y, 
                  phy, timeRates = rate.vec, splitTime = fixed.time, 
                  model = "timeSlice", meserr = meserr, covPIC = covPIC)[[2]]
                vo.out[[q]] <- optim(rateVec, var.timeslice, 
                  method = "L-BFGS-B", lower = lowerBound, upper = upperBound, 
                  control = controlList)
            }
        } else {
            nSplits <- length(splitTime)
            rateVec <- rep(1, (nSplits + 1))
            output.mat <- matrix(NA, nrow = 2, ncol = (3 + nSplits + 
                length(rateVec)) + (2 * ncol(y)))
            bm.phy <- transformPhylo(phy = phy, model = "bm", 
                meserr = meserr, y = y)
            bm.model <- likTraitPhylo(y, bm.phy, covPIC = covPIC)
            log.lik <- as.numeric(bm.model[[2]])
            aic <- aic.fun(log.lik, 2)
            aicc <- aicc.fun(log.lik, 2, Ntip(phy))
            anc.state <- apply(y, 2, function(col.y) as.numeric(as.numeric(ace(col.y, 
                bm.phy, method = "pic")[[1]][1])))
            colnames(output.mat) <- 1:ncol(output.mat)
            colnames(output.mat)[1:3] <- c("lnL", "AIC", "AICc")
            c.y <- ncol(y)
            colnames(output.mat)[4:(4 + c.y - 1)] <- paste0("sigma.sq.", 
                1:ncol(y))
            n.c.ct <- (4 + c.y)
            colnames(output.mat)[n.c.ct:(n.c.ct + c.y - 1)] <- paste0("anc.state.", 
                1:ncol(y))
            n.c.ct <- (n.c.ct + c.y):(n.c.ct + c.y + nSplits)
            colnames(output.mat)[n.c.ct] <- paste0("rates", 1:(nSplits + 
                1))
            n.c.ct <- (max(n.c.ct) + 1):(max(n.c.ct) + nSplits)
            colnames(output.mat)[n.c.ct] <- paste0("time.split", 
                1:(nSplits))
            to.here <- which(regexpr("rates", colnames(output.mat)) == 
                1)[1] - 1
            output.mat[1, 1:to.here] <- c(log.lik, aic, aicc, 
                as.numeric(diag(bm.model[[1]])), anc.state)
            print("BM model")
            print(output.mat[1, 1:to.here])
            var.timeslice <- function(rate.vec) transformPhylo.ll(y, 
                phy, timeRates = rate.vec, splitTime = splitTime, 
                model = "timeSlice", meserr = meserr, covPIC = covPIC)[[2]]
            vo.out <- optim(rateVec, var.timeslice, method = "L-BFGS-B", 
                lower = lowerBound, upper = upperBound, control = controlList)
            log.lik <- vo.out$value
            rates <- vo.out$par
            param <- length(rates) + 2
            aic <- aic.fun(log.lik, param)
            aicc <- aicc.fun(log.lik, param, Ntip(phy))
            phyTemp <- transformPhylo(y = y, phy = phy, timeRates = rates, 
                splitTime = splitTime, model = "timeSlice", meserr = meserr)
            bVar <- as.numeric(likTraitPhylo(y, phyTemp, covPIC = covPIC)[[1]])
            anc.state <- apply(y, 2, function(col.y) as.numeric(ace(col.y, 
                phyTemp, method = "pic")[[1]][1]))
            output.mat[2, 1:to.here] <- c(log.lik, aic, aicc, 
                bVar, anc.state)
            output.mat[2, (to.here + 1):(to.here + length(rates))] <- rates
            rt <- max(which(regexpr("rates", colnames(output.mat)) == 
                1)) + 1
            full.rt <- ncol(output.mat)
            output.mat[2, rt:full.rt] <- splitTime
            print("shiftModel")
            print(output.mat[2, ])
        }
        out <- list()
        out$timeSlice <- output.mat
        out$optim.output <- vo.out
        out$phy <- phy
        out$y <- y
        out$meserr <- meserr
        out$covPIC <- covPIC
        if (output.all) 
          out$all.models <- full.model.out
        out$testShiftTimes <- testShiftTimes
        class(out) <- "timeSlice.ML"
    })
    return(out)
}

#' @title Identify nodes and tips descended from a node 
#' @description Obtains a vector of the tips and nodes subtending from a node in a phylogeny.
#' @note \code{as.rateMatrix} calls the CAIC function \code{vcv.array} multiple times and this can be slow for large phylogenies (though faster than using the \pkg{ape} equivalent \code{vcv.phylo}).
#' @param x A positive integer
#' @param phy An object of class \code{phylo} (see \pkg{ape}).
#' @param tip.labels Logical - output tip.labels?
#' @details This function is stolen from the clade.members function in the CAIC package but returns both node and tip id's.
#' @return Returns a vector of node and tip ids descended from the tip(s) "x". If tip.labels=TRUE then returns a list of node ids and tip labels.
#' @author Gavin Thomas, David Orme
#' @export

node.descendents <- function (x, phy, tip.labels = FALSE) 
{
	 
	if (x <= Ntip(phy)) { return(numeric()) } else {
		node <- x
		Ntip <- Ntip(phy)
		ROOT <- Ntip + 1
		Nedge <- dim(phy$edge)[1]
		wbl <- !is.null(phy$edge.length)
		
		phy <- reorder(phy)
		root.node <- which(phy$edge[, 2] == node)
		start <- root.node + 1
		anc <- phy$edge[root.node, 1]
		next.anc <- which(phy$edge[-(1:start), 1] <= anc)
		keep <- if (length(next.anc)) start + 0:(next.anc[1] - 1) else start:Nedge
		
		nodes <- phy$edge[keep, 2]    
		nodes <- unique(nodes)
		if (tip.labels == TRUE) {
			nodesTips <- vector(mode = "list", length = 2)
			nodesTips[[1]] <- nodes[nodes > Ntip]
			nodesTips[[2]] <- with(phy, tip.label[nodes[nodes <=	Ntip]])
			return(nodesTips)
		}
		else {
			return(nodes)
		}}
}

#' @title Maximum likelihood rate estimation for traits and phylogenies 
#' @description Function for the maximum likelihood estimation of rate parameters on a trait and phylogeny.
#' @param rateData an object of class \code{rateData}
#' @param rate a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal.
#' @param fixed A vector stating whether each parameter should be allowed to vary (either \code{FALSE} which results in a start value of 1, or a numeric start value) or should be fixed (\code{TRUE}).
#' @param rateMIN Minimum value for the rate parameters
#' @param rateMAX Maximum value for the rate parameters
#' @param common.mean a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=FALSE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates.
#' @param lambda.est Logical. Fit Pagel's lambda.
#' @param meserr Logical. Include measurement error.
#' @return MLRate Maximum likelihood estimates of the rate parameters
#' @return Max.lik  Maximum (log) likeihood
#' @return AIC AIC for maximum likelihood model
#' @return AICc  AICc for maximum likelihood model
#' @return convergence convergence value from \code{optim}
#' @return n.parameters Number of parameters in the model (how many means and rate categories)
#' @references Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.
#' @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @author Gavin Thomas
#' @examples
#' data(anolis.tree)
#' data(anolis.data)
#'
#' ## Convert data to class rateData with a rateMatrix object as input
#' anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)
#'
#' anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
#' rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)
#'
#' # A model with a different rate in each of the four groups. The 'fixed' command is used to determine
#' # whether a particular rate is to be constrained or not. Use '1' to fix a group and 'FALSE' to show
#' # that the parameter is not fixed and should be estimated. The values should be entered in the same 
#' # order as the ranking of the groups. That is, group 0 (small islands) takes position one in the 
#' # fixed vector, group 1 (large island trunk crown and trunk ground) takes position 2 and so on. 
#' # The default is to allow each group to take a different mean. 
#'
#' optim.likRatePhylo(anolis.rateData, rate=c(1,1,1,1), common.mean=TRUE, lambda.est=FALSE)
#' @export

optim.likRatePhylo <-
function(rateData, rate=NULL, fixed = NULL, rateMIN = 0.001, rateMAX = 50, common.mean=FALSE, lambda.est=TRUE, meserr=FALSE) {
		
		if(is.null(rate))  { rate <- c(rep(1,length(rateData$Vmat))) } else { rate <- rate }	
	
		if(is.null(fixed))  { op <- fixed <- c(TRUE, rep(FALSE,length(rateData$Vmat) - 1)) } else { op <- fixed }
				
		lower <- c(rep(rateMIN, length(rateData$Vmat)), 0.00001)
		upper <- c(rep(rateMAX, length(rateData$Vmat)), 1)
	
		mvl <- make.likRatePhylo(rateData, fixed=op, common.mean=common.mean, lambda.est=lambda.est, meserr=meserr)

			vo <- try(optim(c(rate, 1, 1), mvl, method = "L-BFGS-B", lower = lower, upper = upper))
			MLRate <- vo$par[1:length(fixed)]
			Lambda <- vo$par[1+length(fixed)]
			
			fixed[which(fixed==FALSE)] <- MLRate[which(fixed==FALSE)]
			MLRate <- fixed

			ML <- -vo$value
			convergence <- vo$convergence
			n <- length(rateData$y)
			
				if(length(op)!=length(which(op==FALSE))) {
					if(common.mean==TRUE) {k <- 2 + length(which(op==FALSE) + lambda.est + meserr)
							} else { k <- (length(which(op==FALSE)) +1 + length(op)) + lambda.est + meserr }
							
							} else {
					
					if(common.mean==TRUE) {k <- 1 + length(which(op==FALSE) + lambda.est + meserr)
							} else { k <- (length(which(op==FALSE)) + length(op)) + lambda.est + meserr}
							}
							
			aic <- -2 * ML + 2 * k
			aicc <- -2 * ML + 2 * k + ((2*k*(k+1))/(n-k-1))
			
			
	ML.RatePhylo <- list(MLRate = MLRate, Lambda = Lambda, Max.lik = ML, aic = aic, aicc = aicc, convergence=convergence, n.parameters = k)
	return(ML.RatePhylo)
	
}

#' @title Identify shifts in the rate of trait diversification
#' @description Summarises phenotypic rate variation on phylogenies.
#' @param object Output of a traitMedusa analysis in \code{\link{transformPhylo.ML}}.
#' @param ... Additional arguments to be passed to \code{\link{summary}}.
#' @param cutoff Cutoff value for differences in AIC scores when comparing models. More complex models with an AIC score more than this number of units lower than simpler models are retained (as per runMedusa in the \pkg{geiger}).
#' @param AICc If true, AICc is used instead of AIC.
#' @param lowerBound Minimum value for parameter estimates.
#' @param upperBound Maximum value for parameter estimates.
#' @param print.warnings Logical. If TRUE, warnings are issued if confidence intervals fall outside upper or lower bounds.
#' @details This functions summarises the output of a "medusa" model in transformPhylo.ML (see below). The best overall model is chosen based on AIC (or AICc if AICc=TRUE). The cut-off point for improvement in AIC score between successively more complex models can be defined using cutoff. The default cutoff is 4 but this is somewhat arbitrary and a "good" cut-off may well vary between data sets so it may well be worth exploring different cutoffs.
#' @return ModelFit Summary of the best optimal rate shift model.
#' @return Rates Summary of the rate parameters from the best rate shift model.
#' @return optimalTree A phylo object with branch lengths scaled relative to rate.
#' @seealso \code{\link{transformPhylo.ML}}
#' @references Alfaro ME, Santini F, Brock CD, Alamillo H, Dornburg A, Carnevale G, Rabosky D & Harmon LJ. 2009. Nine exceptional radiations plus high turnover explain species diversity in jawed vertebrates. PNAS 106, 13410-13414.
#' @references O'Meara BC, Ane C, Sanderson MJ & Wainwright PC. 2006. Testing for different rates of continuous trait evolution using likelihood. Evolution 60, 922-933
#' @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @author Gavin Thomas
#' @examples 
#' data(anolis.tree)
#' data(anolis.data)
#' attach(anolis.data)
#' male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
#' sortedData <- sortTraitData(anolis.tree, male.length)
#' phy <- sortedData$phy
#' male.length <- sortedData$trait
#' phy.clade <- extract.clade(phy, 182)
#' male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, rownames(male.length)),])
#' tm1 <- transformPhylo.ML(male.length.clade, phy=phy.clade, model="tm1", minCladeSize=10, nSplits=1)
#' tm1_out <- summary(tm1, cutoff=1)
#' @export summary.traitMedusa
#' @export

summary.traitMedusa <- function (object, ..., cutoff=4, AICc=TRUE, lowerBound=1e-8, upperBound=200, print.warnings=FALSE) {
	
  if (!inherits(object, "traitMedusa"))
    stop("Please sapply output from transformPhylo.ML with model = 'tm1' or model = 'tm2'.")
  
  traitMedusaObject <- object
  y <- traitMedusaObject[[3]]
  original.phy <- phy <- traitMedusaObject[[4]]
  breaks <- numeric()
  output.silent <- summary(y, ...)
  
  if (AICc == TRUE) {
    datCol = 6 
  } else {
  	datCol = 5
  }
  
  bestModel <- traitMedusaObject[[1]][1, ]
  rateType <- traitMedusaObject[[1]][2:nrow(traitMedusaObject[[1]]), 2]
  
  for (i in 2:dim(traitMedusaObject[[1]])[1]) {
  	if ((traitMedusaObject[[1]][i - 1, datCol] - traitMedusaObject[[1]][i, datCol]) < cutoff)
  	  break
  	bestModel <- traitMedusaObject[[1]][i,]
  	where.model <- i
  	}
  
  out <- vector(mode="list", length = 3)
  names(out) <- c("ModelFit", "Rates", "optimalTree")
  
  if (bestModel$node == 0) {
  	out$optimalTree <- phy 
  	out$ModelFit <- bestModel
  	out$Rates <- "Single rate"
  } else {
  	foo <- function(param) {
  	  ll <- transformPhylo.ll(y, phyClade, model = "clade", nodeIDs = SingleNode, cladeRates = param, rateType = whichRateType)$logLikelihood
  	  return(as.numeric(ll - as.numeric(bestModel["lnL"]) + 1.92))
  	  }
    nodeIDs <- traitMedusaObject[[1]][2:where.model, "node"]
    cladeRates <- traitMedusaObject[[1]][where.model, -c(1:6)]
    if(any(is.na(cladeRates)))
      cladeRates <- cladeRates[-which(is.na(cladeRates))]
    cladeRates <- as.numeric(cladeRates)
    rateType <- traitMedusaObject[[1]][2:where.model, "shiftPos"]
    optimalTree <- transformPhylo(phy, model="clade", nodeIDs=sort(nodeIDs), cladeRates=as.numeric(cladeRates), rateType=rateType)
    out$Rates <- matrix(NA, length(nodeIDs), 5, byrow = TRUE)
    out$Rates <- as.data.frame(out$Rates)
    colnames(out$Rates) <- c("node", "shiftPos", "MLRate", "LowerCI", "UpperCI")
    out$ModelFit <- bestModel[ ,3:6]
    out$optimalTree <- optimalTree
    for (i in 1:length(nodeIDs)) {
      SingleNode <- sort(nodeIDs[i])
      whichRateType <- rateType[i]
      phyClade <- transformPhylo(optimalTree, model = "clade", nodeIDs = SingleNode, cladeRates = 1 / cladeRates[i], rateType = whichRateType)
      LCI <- NULL
      UCI <- NULL
      out.lower <- try(uniroot(foo, interval = c(lowerBound, cladeRates[i]))$root, silent = TRUE)
      if (is.numeric(out.lower)) {
      	LCI <- uniroot(foo, interval = c(lowerBound, cladeRates[i]))$root 				
      } else {
      	LCI <- NA 
      }
      out.upper <- try(uniroot(foo, interval = c(cladeRates[i], upperBound))$root, silent = TRUE)
      if(is.numeric(out.upper)) {
      	UCI <- uniroot(foo, interval = c(cladeRates[i], upperBound))$root
      } else {
      	UCI <- NA
      }
      out$Rates[i,] <- aaa <- c(nodeIDs[i], rateType[i], cladeRates[i], LCI, UCI)
    }
		
	if (any(is.na(out$Rates[,3:4])) && print.warnings)
	  warning("Confidence limits fall outside the current parameter bounds - consider changing lowerBound and/or upperBound")
  }
  out$original.phy <- original.phy
  class(out) <- c("traitMedusa.model")
  return(out)
}

#' @title add a fossil to an interior branch of a time-scaled phylogeny
#'
#' @description the function takes a time-scaled phylogeny and adds a fossil at a branch in the past
#' @param phy Input phylogeny an object of class \code{phylo} (see \pkg{ape})
#' @param inGroup vector of tip labels defined as the ingroup - the fossil(s) will be placed on the stem branch leading to the 'inGroups' most recent common ancestor
#' @param fossil tip labels for the new fossil
#' @param fossilAge age of the fossil
#' @param minLength minimum length leading to the fossil 
#' @param maxLength maximum length leading to the fossil. If \code{NULL} (default) then the maximum bound is half the length of the branch leading to the crown node
#' @return the the time-scaled phylogeny with the fossil attached
#' @references Puttick, M. N., Kriwet, J., Wen, W., Hu, S., Thomas, G. H., & Benton, M. J. (2017). Body length of bony fishes was not a selective factor during the biggest mass extinction of all time. Palaeontology, 60, 727-741.
#' @author Mark Puttick
#' @examples
#' data(anolis.tree)
#' plot(anolis.tree)
#' nodelabels(214, 214)
#' # add fossil to node 214
#' in.groups <- node.descendents(x=214, phy=anolis.tree, tip.labels=TRUE)[[2]]
#' fossilPhy <- addFossilToPhy(anolis.tree, in.groups, fossil="fakeFossil", fossilAge=60)
#' plot(fossilPhy)
#' @export

addFossilToPhy <- function(phy, inGroup, fossil, fossilAge, minLength = 0.1, maxLength=NULL) {
	commonNode <- getMRCA(phy, inGroup)
	fossilTip <- read.tree(text="(fossilTip);")
	fossilTip$tip.label <- paste0(fossil)
	
	b.times <- nodeTimes(phy)
	edge.loc <- match(commonNode, phy$edge[,2])
	branchToCrown <- -diff(b.times[edge.loc,])
	minEdgeLength <- b.times[edge.loc, 2] -  fossilAge
	
	if(is.null(maxLength)) {
		maxLength <- branchToCrown
		} else {
		if(maxLength < branchToCrown) maxLength <- branchToCrown
	}
	
	if(minEdgeLength < 0) {
		minEdgeLength2 <- -minEdgeLength #+ minLength
		if(minEdgeLength2 > maxLength ) maxLength <- minEdgeLength2
		branchToCrown  <- runif(1, minEdgeLength2, maxLength)
		fossilTip$edge.length <- branchToCrown + b.times[edge.loc, 2] -  fossilAge
		return(bind.tree(phy, fossilTip, commonNode, branchToCrown))

	} else 
	{
		branchToCrown  <- runif(1, minLength, maxLength)
		fossilTip$edge.length <- branchToCrown + b.times[edge.loc, 2] -  fossilAge
		return(bind.tree(phy, fossilTip, commonNode, branchToCrown))
	}
}

#' @title Sample hidden speciation events along branches of a tree
#' @description Uses estimated speciation and extinction rates to sample the number of speciation events 'hidden' by subsequent extinction on each branch of a tree following Bokma (2008). For use with the \code{psi} and \code{multipsi} models. 
#' @param phy An object of class \code{phylo} (see \pkg{ape})
#' @param la Estimate of the rate of speciation "lambda"
#' @param mu Estimate of the rate of extinction "mu"
#' @param useMean A logical indicating whether to output the average or expected number of hidden speciation events per branch, which may be non-integer (if \code{TRUE}), or to sample an integer number on each branch from a Poisson distribution (if \code{FALSE}, the default)
#' @details The expected number of hidden speciation events are calculated for each branch given its start and end times, and estimates of lambda and mu which are assumed to be constant across the tree. To properly account for uncertainty in the effect of extinction on the number of nodes affecting each branch of a tree, it may be appropriate to repeat model-fitting on many realizations of \code{Sobs} on the tree of interest (similar to evaluating phylogenetic uncertainty)
#' @return Phylogenetic tree in \code{phylo} format, with an added element \code{Sobs}, a vector of numbers of hidden speciation events per branch, in the same order as the branches in the \code{phylo} object
#' @seealso \code{\link{transformPhylo.ML}}
#' @references Bokma, F. 2008. Detection of "punctuated equilibrium" by Bayesian estimation of speciation and extinction rates, ancestral character states, and rates of anagenetic and cladogenetic evolution on a molecular phylogeny. Evolution 62: 2718-2726.
#' @references Ingram, T. 2011. Speciation along a depth gradient in a marine adaptive radiation. Proc. R. Soc. B 278: 613-618.
#' @author Travis Ingram
#' @export

sampleShid <- function(phy, la = NULL, mu = NULL, useMean = FALSE)
{
	if(is.null(la)|is.null(mu))
    stop("Please provide values for lambda and mu ")
	phy$node.label<-NULL
	br<-branching.times(phy)
	to<-br[as.character(phy$edge[,1])]
	start<-br[as.character(phy$edge[,1])]
	te<-start-phy$edge.length
	te[te<0]<-0
	if(mu > 0){
		if(la == mu){
			expSh <- 2 * la * (to - te) + 2 * log((1 + la * te) / (1 + la * to))
		}else{
			expSh <- 2 * la * (to - te) + 2 * log((la * exp(te * (la - mu)) - mu) / (la * exp(to * (la - mu)) - mu))
		}
		if(useMean){
			Shid <- expSh
		}else{
			Shid <- rpois(length(expSh), expSh)
		}
	}else{
		Shid <- rep(0, length(to))
	}
	phy$Shid <- Shid
	phy
}

---
title: "MOTMOT: Models Of Trait Macroevolution On Trees"
author: "Mark Puttick"
date: "`r format(Sys.time(), '%d %B %Y')`"
output: html_document
vignette: >
  %\VignetteIndexEntry{MOTMOT: Models Of Trait Macroevolution On Trees}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo=FALSE, warning=FALSE, message=FALSE}
library(knitr)
opts_chunk$set(tidy.opts=list(width.cutoff=60))
```

```{r global_options, include=FALSE}
knitr::opts_chunk$set(fig.pos = 'H')
```

Models Of Trait Macroevolution On Trees (MOTMOT) is an R package that allows for testing of models of trait evolution (Thomas *et al.* 2012). 

- [Tree transformation](#models-of-trait-evolution) models estimated using Maximum likelihood: [Brownian motion](#brownian-motion), [Pagel's lambda](#pagels-lambda), [Delta](#delta), [Kappa](#kappa), [Ornstein-Uhlenbeck (OU)](#ornstein-uhlenbeck), [Acceleration-Deaceleration (ACDC) and early bursts](#acdc-and-early-burst), [psi and multispi](#psi-and-multispi), and [estimating lambda alongside other models](#estimate-pagels-lambda-alongside-other-modes)
- [Rate heterogeneous models of evolution](#rate-heterogeneous-models-of-evolution). Fit models in which the rate of evolution differs in clades selected [*a priori*](#rate-heterogeneity-selected-a-priori) (O'Meara *et al.* 2006; Thomas *et al.* 2006), and models with [no *a-priori* shift locations](#rate-heterogeneity-with-no-a-priori-information) (Thomas *et al.* 2012)
- [timeSlice](#timeslice-model) fit models in which all rates change at a specific time(s) by testing multiple shift times or those selected by the user
- [modeSlice](#modeslice-model) fit models in which modes change at a specific time(s) in an extension to models introduced by Slater (2013)
- [Nested Shift modes](#nested-models-of-evolution) Fit models models in which the ancestral BM rate switches to a 'nested' rate within a monophyletic clade in the phylogeny (Puttick 2018)
- [Bayesian estimation](#bayesian-estimation-of-tree-transformation-models) of tree transformation models
- [Character displacement models](#character-displacement-models) of inter-specific competition from Clarke *et al.* (2017)
- [Fast estimation of Phylogenetic Generalised Least Squares (PGLS)](#fast-estimation-of-phylogenetic-generalised-least-squares) using independent contrasts


# Introduction


First we install

```{r, warning=FALSE, message=FALSE, eval=FALSE}
install.packages("motmot")
```

and load MOTMOT
 
```{r, warning=FALSE, message=FALSE}
library(motmot)
```

For these examples we will use anoles lizard data available from MOTMOT. A time-calibrated phylogeny of *Anolis* species `anolis.tree`, and various trait and biogeographical trait data `anolis.data`.

```{r}
data(anolis.tree)
data(anolis.data)
attach(anolis.data)
anolis.tree
```

We will use the continuous trait data: male snout-ventral length `Male_SVL`. Here, we construct a matrix of just `Male_SVL` data, remove missing data, and log-transform the values. All this can be done using the function `sortTraitData`

```{r}
sortedData <- sortTraitData(phy=anolis.tree, y=anolis.data, data.name="Male_SVL", pass.ultrametric = TRUE)
phy <- sortedData$phy
male.length <- sortedData$trait
```

Finally, we will 'prune' the species from the tree using `drop.tip` from [APE](https://CRAN.R-project.org/package=ape). We plot our tree and data using the MOTMOT `traitData.plot` function.

```{r plot1, fig.cap = "Figure 1. TraitData showing the relative male snout-vent length at the tips", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200}
traitData.plot(y=male.length, phy, lwd.traits=2, col.label="#00008050", tck=-0.01, mgp=c(0,0.2,0), cex.axis=0.5, show.tips=FALSE)
```

For the sake of brevity, in the following examples we fit the models to a subset of these data: including the clade from node 182 only using the [APE](https://CRAN.R-project.org/package=ape) function `extract.clade`.

```{r}
## uncomment to view the tree
# plot(phy, show.tip.label=FALSE, no.margin=TRUE, edge.col="grey20")
# nodelabels(182, 182, bg="black", col="white")
phy.clade <- extract.clade(phy, 182)
male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, rownames(male.length)),])
```

# Models of trait evolution

We can now test various models of evolution using our trait data.

## Brownian motion

To start we will fit a simple Brownian motion model to the data, as the null hypothesis of phylogenetic trait evolution (Cavalli-Sforza and Edwards 1967; Felsenstein 1973; 1985). Brownian motion describes a process in which tip states are modelled under the assumption of a multi-variate normal distribution. On a phylogeny, the multi-variate mean of tip states is equal to the root state estimate, and variance accummulates linearly through time. Trait evolution is shared but following a split individual branches evolve and accummulate trait variance independently from their shared ancestral value.

The function `transformPhylo.ML` is used to fit Brownian motion models and its derivatives. Here we fit a simple Brownian motion model to the subset of anolis male SVL data to obtain the Brownian variance, ancestral estimate, log-likelihood, Akaike Information Criterion (AIC), and small-sample AIC (AICc). 

```{r}
bm.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="bm")
bm.ml
```

## Pagel's lambda

Here we fit models to test Pagel's lambda (Pagel 1997; 1999). Pagel's lambda is a measure of phylogenetic 'signal' in which the degree to which shared history of taxa has driven trait distributions at tips. In this model, internal branch lengths are transformed by the lambda parameter value. When the parameter lambda equals 1, branches are transformed by multiplying by 1 and so the model is equal to Brownian motion (high phylogenetic signal). Values of lambda under 1 suggest there has been less influence of shared history on trait values at the tips. Finally, a lambda value of 0 indicates no phylogenetic influence on trait distributions, and is equivalent to a 'star phylogeny' with no shared branch lengths.

The maximum likelihood of lambda can be estimated in MOTMOT.

```{r}
lambda.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="lambda")
lambda.ml
```

The maximum likelhood estimate of Pagel's lambda is equal to 0.84.

A new feature in MOTMOT allows for plotting of the likelihood profile for the branch-transformation parameter, in this case Pagel's lambda using the argument `profilePlot` in `transformPhylo.ML`.

```{r plot2, fig.cap = "Figure 2. Profile plot of ML estimation for Pagel's lambda", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200}
lambda.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="lambda", profilePlot=TRUE)
```

We now compare the relative fit of the BM and lambda models. Lambda has higher likelihood, but it also has more parameters. The root state and sigma-squared (rate) parameters are present in both models but the lambda model also requires an estimate of the parameter lambda. We can test whether the lambda model is a significant improvement over BM. First we test the relative fit by using the chi-squared distribution. The models differ in one degree of freedom: BM has 2 parameters and lambda has 3. We can use the `stats` function `pchisq` to obtain a p value by testing using a chi-squared distribution. The lambda is indeed a superior fit compared to BM when fit to these data (*p* < 0.05).

```{r}
p.value <- 1 - pchisq(lambda.ml$MaximumLikelihood - bm.ml$logLikelihood, 1)
p.value
```

Additionally there is a large small-sample Akaike Information Criterion (AICc) difference between the two models: BM has a higher AICc compared to lambda. The difference (11.09) is >4 which is traditionally seen as indication of a superior fit (Burnham and Anderson 2003). 

```{r}
bm.ml$AICc- lambda.ml$AICc
```

## Delta

Delta indicates a decrease or increase in the rate of trait evolution through time (Pagel 1997; 1999); a value of 1 is equivalent to Brownian motion, < 1 indicates a slow-down, and > 1 is indicates greater change closer to the present. Here we find a Maximum likelihood estimated for Delta of 2.23 but the CI spans < 1 to > 4, so it is not possible to conclusively support a change in the rate of evolution through time.

```{r}
delta.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="delta")
delta.ml
```

## Kappa

Kappa is used as a measure of punctuated evolution and spans values of 0-1 (Pagel 1997:1999). A Kappa value of 1 is equivalent to BM, and 0 indicates trait change occurs at events of speciation. Here there is evidence of punctuated evolution. `transformPhylo.ML` also allows users to see the the phylogeny transformed by model parameters. As an example, we show the original, BM model phylogeny and compare this with the phylogeny transformed by the Kappa phylogeny.

```{r plot3, fig.cap = "Figure 3. Comparison of BM and Kappa transformed trees.", echo = T, fig.height = 5, fig.width = 5, , fig.path='figures/', dev='png', dpi=200}
kappa.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="kappa", profilePlot=FALSE, returnPhy=TRUE)
par(mfrow=c(1,2))
plot.phylo(phy.clade, show.tip.label=FALSE, no.margin=TRUE)
mtext("Original phylogeny", 3, cex=0.7, line=-1)
plot.phylo(kappa.ml$kappaPhy, show.tip.label=FALSE, no.margin=TRUE)
mtext("Kappa model phylogeny", 3, cex=0.7, line=-1)
mtext("Kappa = 1e-8", 3, cex=0.7, line=-2)
```

## Ornstein-Uhlenbeck

The OU model allows for modelling of attraction to a optimum value, alpha (Hansen 1997; Butler and King 2004). This model again is similar to the Brownian motion model, but models the strength of attraction to alpha. The OU model can be difficult to interpret and care is advised in its use (Cooper *et al.* 2016).

In MOTMOT, as with most implements of the phylogenetic OU model, the value of the optimum is equal to the ancestral trait estimate. With all `transformPhylo.ML` functions it is possible to change the bounds on the estimated parameters. For example, here the value of *alpha* is constrained to 2 using the argument `upperBound`.

```{r plot4, fig.cap = "Figure 4. Profile plot to estimate alpha", echo = T, fig.height = 5, fig.width = 5, , fig.path='figures/', dev='png', dpi=200}
ou.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="OU", profilePlot=TRUE, upperBound=2)
ou.ml
```

The value of alpha is higher than zero, but very small (0.01692855). So the model is not equivalent to Brownian motion but there is little evidence from AICc that the model is an improvement, and the likelihood ratio test show a non-significant improvement so it does not have higher relative support compared to BM (*p* > 0.05).

```{r}
p.value <- 1 - pchisq(ou.ml$MaximumLikelihood - bm.ml$logLikelihood, 1)
p.value
bm.ml$AICc- ou.ml$AICc
```

## ACDC and Early Burst

A new addition to MOTMOT is the ACDC model (Blomberg *et al.* 2003). This model allows for exponential changes in the rate of evolution in the history of a clade. 

```{r plot5, fig.cap = "Figure 5. Profile plot to estimate the ACDC parameter", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200}
acdc.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="ACDC", profilePlot=TRUE)
acdc.ml
```

There is little evidence here of exponential decreases or increases in the rate of trait evolution - the ACDC exponential parameter is close to 0 (0.034). We can see this is not a significant improvement on BM.

```{r}
p.value.2 <- 1 - pchisq(acdc.ml$MaximumLikelihood - bm.ml$logLikelihood , 1)
p.value.2
```

As an example, here we constrain the 'upperBound' to < 0, this is equivalent to the Early Burst model (Harmon *et al.* 2010) fit in [geiger](https://CRAN.R-project.org/package=geiger).

```{r}
transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="ACDC", profilePlot=FALSE, upperBound=-1e-6, print.warning=FALSE)
```

The estimate of -1e-6 for the exponential decrease parameter, which means the model is effectively equivalent to Brownian motion.

## psi and multispi

The parameter psi is similar to the parameter Kappa in that it is a measure of the relative contribution of speciational (~punctuated) and gradual evolution to trait change on a phylogeny (Ingram 2011; Ingram *et al.* 2016). The parameter psi is based upon measures of evolution over time and at speciation, and can also account for 'hidden' nodes not seen in the input phylogeny. The parameter psi measures the proportion of total evolutionary change (speciational + gradual) that can be attributable to speciational evolution, so the estimation for psi between 0 (Brownian motion) and 1 (indicating equal branch lengths, ~speciational change).

In MOTMOT we can fit a simple psi model using the input tree. 

```{r plot6, fig.cap = "Figure 6. Profile plot to estimate the psi parameter", echo = TRUE, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200}
psi.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="psi", profilePlot=TRUE)
psi.ml
```

This indicates support for the psi model is a significant improvement on Brownian motion (*p* < 0.05).

```{r}
p.value.psi <- 1 - pchisq(psi.ml$MaximumLikelihood - bm.ml$logLikelihood , 1)
p.value.psi
```

We could also get a potentially more accurate of speciation rates by using the full tree, rather than the pruned tree to estimate speication and extinction rates as this will give more accurate estimates rather than using the taxa with complete data only. If extinction rates are larger than 0, then the estimates will differ from the simple model above. 

```{r}
psi_ext.est <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="psi", profilePlot=FALSE, hiddenSpeciation=TRUE, full.phy=phy)
all.equal(psi.ml, psi_ext.est)
```

In this case, there is no difference in the estimates as extinction rates are equal to 0. 

We can also apply multipsi model in which different regions of the tree have different estimates of the parameter psi. We can now fit the multispi model with these data. In MOTMOT, this model requires branch labels given *a priori* by the user to delimit the different regimes on the phylogeny. Note that these clades with potentially different psi regimes do not need to be monophyletic clades. Here we arbitarily assign two clades 'a' and 'b' to test differences between them. 

```{r plot7, fig.cap = "Figure 7. Two clades used in the multipsi model", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200}
plot(phy.clade, no.margin=TRUE, cex=0.8)
two.clade.labels <- c(rep("a", 17), rep("b",37))
edgelabels(two.clade.labels, col=c(rep("blue", 17), rep("red", 37)), bg="white")
```

Using these data we fit the model with `transformPhylo.ML`.

```{r}
transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="multipsi", branchLabels=c(rep("a", 17), rep("b",37)), hiddenSpeciation=TRUE, full.phy=phy)
```

In this model, the estimate of psi does not differ between the two regions of the tree

## Estimate Pagel's lambda alongside other modes

One way to deal with 'noisy' data is to estimate Pagel's lambda alongside a parameter of interest. By using Pagel's lambda alongside other models it may be possible to account for variation in the data that may be a result of errors in the phylogeny or trait data. In MOTMOT, Pagel's lambda can be estimated alongside the delta, kappa, OU, psi, and ACDC models. Here we look at example using ACDC. The model is fit with same function. `transformPhyo.ML` but with the argument `lambdaEst` set to `TRUE`.

```{r}
acdc.ml.lambda <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="ACDC", lambdaEst=TRUE)
# original ACDC model
acdc.ml
# ACDC model plus lambda
acdc.ml.lambda
```

We can see lambda is < 1, and this has affected the parameter estimation. The improvement in the model fit is significant compared to the ACDC model fit without estimating lambda and the null BM model.

```{r}
# p value of the ACDC and ACDC+lambda models. No significant improvement
1 - pchisq(acdc.ml.lambda$MaximumLikelihood - acdc.ml$MaximumLikelihood , df=1)
# p value of the BM and ACDC+lambda model comparison. No significant improvement
1 - pchisq(acdc.ml.lambda$MaximumLikelihood - bm.ml$logLikelihood, df=2)
```

# Rate heterogeneous models of evolution

## rate heterogeneity selected *a priori*

MOTMOT can test models of evolution in which pre-defined clades can vary in the rate of evolution. Here we fit a model in which the nodes descending from nodes 32 and 49 have a seperate rate of evolution. First, we can visualise these nodes on the phylogeny.

```{r plot8, fig.cap = "Figure 8. Lineages with different rates of evolution", echo = T, fig.height = 5, fig.width = 5, , fig.path='figures/', dev='png', dpi=200}
plot(phy.clade, show.tip.label=FALSE, no.margin=TRUE, edge.col="grey20")
nodelabels(c(32, 49), c(32, 49), bg="black", col="white")
```

We then fit the MOTMOT model, again using the function `transformPhylo.ML`. We use the argument `model=clade`. This fits the non-censored model of O'Meara *et al.* (2006). 

```{r}
cladeRate.ml <- transformPhylo.ML(phy=phy.clade, y=male.length.clade, model="clade", nodeIDs=c(32, 49))
cladeRate.ml
```

These results indicate that the two clades tend to have a lower rate of evolution compared to the background rate. However, the CIs indicate these decreases may not be robust.

## rate heterogeneity with no *a priori* information

We can also fit rate heterogeneous models without specifying where we expect shifts on the tree. We can use the arguments `model="tm1"` and `model="tm2"`. These models fit the `traitMedusa` model in which nodes are individually tested for rate increases or decreases (Thomas and Freckleton 2012), and the clade or branch with a rate change that produces the largest increase in likelihood is returned. Note, it is possible to exclude small nodes using the argument `minCladeSize`. As well as allowing clade differences in rate, the `tm2` also allows for branch-based increases or decreases in rate, whereas `tm1` only searches for clade-based rate changes. 

We can now fit the `tm2` algorithm. The output shows the log-likelihood, AIC, AICc, rate type (branch of clade), for the best-fitting model at each stage. This starts with the BM model, and then one shift model, two shift model, etc., 
```{r}
# tm1 algorithm not run
# tm1.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="tm1", minCladeSize=2, nSplits=3)
# trait.medusa.tm1.summary <- summary.traitMedusa(tm1.ml, cutoff=2, AICc=T)
# tm2 model
tm2.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="tm2", minCladeSize=5, nSplits=2)
```

We can summarise the analyses using `summary.traitMedusa` or just and `summary`plotting the shifts on the phylogeny using the function `plot.traitMedusa.model` or just `plot`. These results show a decrease at node 39 that we can visualise on the phylogeny. 

```{r plot9, fig.cap = "Figure 9. The subset of the tree showing the rate heterogeneity estimated from the traitMedusa model", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200}
trait.medusa.tm2.summary <- summary(tm2.ml, cutoff=2, AICc=TRUE)
trait.medusa.tm2.summary
colour_motmot <- plot(x = trait.medusa.tm2.summary, reconType = "rates", type = "fan", cex=0.5, edge.width=2)
```

Thomas and Freckleton (2012) showed the `tm2` algortihm has a high type-one error rate. One way to ameliorate this is to estimate the level a one shift is supported when we know BM is the true model. For example, we could simulate 1000 BM datasets on the tree, estimate a single shift using the `tm2` algortihm, and calculating the difference between the AICcs for each BM and one shift model. We can these use this difference to estimate the AICc 'penalty' the is needed to reduce the `tm2` type-one error rate to 0.05. We could use this penalty in the `cutoff` argument of the `summary.traitMedusa` (or `summary`) argument. 

This can all be calculated with the MOTMOT function `calcCutOff`. The function requires the tree and input from the model applied to the empirical data as well as the number of simulations. Here we calculated the AICc cut-off required for the `tm2` analysis from above (for brevity this is not run here, but should be run for each analysis individually).

```{r}
## uncomment to run
# set.seed(203);
# calcCutOff(phy.clade, n=1000, model="tm2", minCladeSize=5, nSplits=1);
##      95% 
## 5.698198 
```

Here if we repeat this analysis with the appropriate AICc cut-off (5.698) the we see that the single-rate Brownian motion is, in fact, supported. 

```{r}
summary(tm2.ml, cutoff=5.698198, AICc=TRUE)$Rates
```

# timeSlice model 

A new addition to motmot is a Maximum likelihood model that allows for heterogeneous rates in different time periods. These models are seperate from the models that allow for heterogeneous rates among lineages, as modelled by the `traitMedusa` algorithms.

The `timeSlice` model is implemented using the `transformPhylo.ML` function, using the argument `model='timeSlice'`. The function allows for two seperate models of evolution. In one, it is possible to test shifts in evolution at times selected *a priori*. Alternatively, the fit of models can be tested at a range of different times, and the function will return the best-fitting model

First we will test for a shift in the rate of evolution 10 million years ago. 

```{r}
timeSlice.10.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="timeSlice", splitTime=10)
```

We can use the function `plot.timeSlice.ML` or simply `plot` to summarise and plot the results. The output summarises the best model according to AICc fit. This function automatically plots the original tree showing the location of shift(s), and the colours show the relative rates in each time slice. The second plot below shows the same tree and colours, but with the branch lengths scaled to the ML optimised rates.
 
```{r plot10, fig.cap = "Figure 10. TimeSlice plot with a split at 10 Ma", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200}
outputSummary <- plot(timeSlice.10.ml, cutoff=0.001, cex=0.55, edge.width=2, cex.plot=0.8, colour.ramp=c("blue", "red"), label.offset=0.5)
```

We can also see other summarise information, such as the CI for each rate estimate. 

```{r}
outputSummary$RatesCI
```

Rather than testing the overall fit of each model, the model can search all shift times and returns the shift location or locations with the highest likelihood. The function automatically tests for all 1 Ma shifts between the age of the tree - 10 Ma, and the present + 10 Ma; all these presets can be customised using the ``boundaryAge`` argument that supplies a vector with the first age specifying the distance from the root and the second age specifying the age from the tips. The ``splitTime`` argument sets the ages at which all shifts will be tested for between the ``boundaryAge`` with the default testing all shifts at 1 Ma intervals. The model searches for *n* shifts set by the ``nSplits`` argument.

This model searches for the highest likelihood single shift by searching for the highest likelihood shift time between 62-8 Myrs.

```{r}
timeSlice.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="timeSlice", nSplits=1, boundaryAge=8)
```

And summarise the results. We can selected the cutoff AICc improvement needed to justify selecting the next model. Here we use the arbitary cut-off value of 1. We could test this formally by estimating the correct AICc value needed to reduced type-error > 5% by using BM simulated data (an example using the tm2 is shown above).

```{r plot11, fig.cap = "Figure 11. TimeSlice plot with Maximum likelihood estimation of split time", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200}
outputSummary <- plot(timeSlice.ml, cutoff=1, cex=0.2, edge.width=2, cex.plot=0.8, colour.ramp=c("blue", "red"), label.offset=0.5)
```

# modeSlice model

In a related extension, we have incorporated the new `modeSlice` model to the `transformPhylo.ML`. `modeSlice` incorporates and extends the methods of Slater (2013) by allowing for multiple shifts in various modes of evolution (BM, OU, EB, and Kappa) at different times in the phylogeny’s history. This is flexible as users can input multiple rate shift times with different combinations of modes. Furthermore, time bins with a BM mode of evolution can optionally vary in the rate of evolution compared to the background variance (`rate.var` argument), and users can include a rate scalar alongside EB modes. 

Here a model is fit with a shift from an EB model with associated rate scalar to an OU model 40 Ma and then to a BM rate shift model at 30 Ma to the present. The results indicate an ACDC/EB scalar (root age-40Ma), followed by a OU model with alpha of 1.75 (40-30Ma), followed by a rate increase (5.3x background from 30-0 Ma). However this model is not supported over Brownian motion.

```{r}
modeSlice.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="modeSlice", splitTime=c(40, 30), mode.order=c("ACDC", "OU", "BM"), rate.var=TRUE, acdcScalar=TRUE)
modeSlice.ml$AICc
bm.ml$AICc
```

# Nested models of evolution

We can also tested models of nested evolution in which an ancestral model of BM evolution changes to a alternative model (EB, OU, kappa, delta, psi) within the phylogeny (Puttick 2018).

Here we can show an example of BM -> OU and BM -> ACDC at node 44 of the phylogeny. However, neither of these is a significantly better relative fit than BM.

```{r}
bm.model <- transformPhylo.ML(male.length.clade, phy=phy.clade, model="bm")
nested.acdc <- transformPhylo.ML(male.length.clade, phy=phy.clade, model="ACDC", nodeIDs=44)
nested.ou <- transformPhylo.ML(male.length.clade, phy=phy.clade, model="OU", nodeIDs=44)

1 - pchisq(nested.acdc$MaximumLikelihood - bm.model$logLikelihood, 1)
1 - pchisq(nested.ou$MaximumLikelihood - bm.model$logLikelihood, 1)
```

# Bayesian estimation of tree transformation models

Parameters of various modes of evolution can be conducted using a simple Bayesian Markov Chain Monte Carlo (MCMC) algorithm in `transformPhylo.MCMC` which may better reflect probabilistic uncertainty in parameter estimates compared to Maximum likelihood estimation. By default the model places a uniform prior and new proposals using an indepdence sampler in that new proposed parameters are not dependent upon the current value of the chain.

After completion, the function returns convergence diagnostics (effective sample size, acceptance proportion ratio), MCMC chain, and the median value and 95% Highest Posterior Density of the estimated parameter.

The function 'transformPhylo.MCMC' allows for the estimation of model parameters using Bayesian statistics. Models of lambda, delta, kappa, OU, ACDC, psi, and multi-psi can currently be modelled using transformPhylo.MCMC. Additionally, Pagel's lambda can be optimised alongside parameters and nested modes in the same way as `transformPhylo.ML`.

We will run an MCMC chain of 1000 generations to estimate Pagel's lambda and discarding the first 10% ('200 generations ('burn.in = 0.1'). All the models use a 'uniform' prior for each of the parameters. For lambda, this is a uniform distribution between 0 and 1 (although lambda can reach slightly higher than one), meaning we think all potential values are equally likely. To obtain identical results wel will set 'random.start=FALSE', if this is set to TRUE a random start value is taken from the system time

```{r, results="hide"}
set.seed(12) # set seed so run will be identical - for example use only
lambda.mcmc <- transformPhylo.MCMC(y=male.length.clade, phy=phy.clade, model="lambda", mcmc.iteration=2000, burn.in=0.25, random.start=FALSE, sample.every=1)
```

We can know check the posterior estimate of lambda and convergence of the model. The median and 95 Highest Posterior Density (HPD) is output by the model. Some diagnostics are output as standard: Effective Sample Size (ESS) and acceptance rate. We aim for an ESS of at least 200 and an acceptance rate around 0.44

```{r}
lambda.mcmc[1:4]
```

Our lambda median value is 0.79 but there is a large 95% HPD (0.52-0.96). The ESS and acceptance rate look ok. We can also plot the trace from the MCMC chain - this could look better - running for more generations would help

```{r plot12, fig.cap = "Figure 12. MCMC trace for Pagel's lambda", echo = T, fig.height = 5, fig.width = 5, fig.path='figures/', dev='png', dpi=200}
mcmc.plot(lambda.mcmc)
```

# Character displacement models

Magnus Clarke *et al.* (2017) introduced a character displacement model in which inter-specific competition can drive trait change. This model estimates a parameter 'a' that drives the strength of inter-specific competition, alongside a Brownian motion model with parameter estimation of the trait variance. If a=0 the model is equivalent to Brownian motion, and larger values of a drive trait evolution away from the values of inter-specific competitors.

The character displacement model employs an approximate Bayesian computation (ABC) approach, in which many datasets are simulated based on the known tree using a range of parameter values for *a* and the trait variance. These simulations then are compared to the empirical data to estimate the 'best-fitting' parameters of the Brownian motion process variance, and the character displacement parameter *a*.

First data are simulated on the known tree, allowing for a range of variance (sigma) and *a* values with both sample from a uniform distribution between 0 and 8. For brevity, we will use 100 simulations only. For actual analyses, many more iterations would be required, perhaps 1 million (Clarke *et al* 2017). Note this process can be made parallel on Mac and Linux systems by using the 'mc.cores' argument, but here we will use one core only.

```{r}
data(finches)
emp.tree <- finch.tree
emp.data <- finch.data
param.simulation <- chr.disp.param(emp.tree, n.sim = 100, max.sigma = 8, max.a = 8, ntraits=1, mc.cores = 1)
```

We can then compare these simulated data with the empirical data using the function 'chr.disp.lrt'. We will use only 75 simulations from the posterior, this value can be guided by simulations (see Clarke et al. 2017)

```{r}
chr.disp.lrt(emp.tree=emp.tree, emp.data=emp.data, param.out=param.simulation, posteriorSize=75)
```

The output shows the 'estimates' for hypothesis 0 (Brownian motion) and hypothesis 1 (character displacement) with the variance and a values summarised (a is 0 in the Brownian motion model, by definition). The second list element 'likelihood.ratio.test' shows the likelihood of each model, the value of the likelihood-ratio test statistic, and the *p* value (here the character displacement is not supported over the character displacement model).

# Fast estimation of Phylogenetic Generalised Least Squares

The package *caper* (Orme *et al* 2018) offers an excellent model to run Phylogenetic Generalised Least Squares (PGLS) models, but these are based-upon Generalised Least Squares (using variance-covariance matrices) which are substantially slower than using indpendent contrasts (Freckleton 2012).

In motmot, code allows for continuous PGLS models can be estimated using contrasts - this gives identical results to *caper* but is substantially faster, as is shown below. At current only continuous data is allowed in the models for motmot, so if any of the input data are not continuous CAPER or similar should be used. Additionally motmot only estimates Pagel's lambda rather than other models, such as Kappa as offered by CAPER

```{r}
# Data and phylogeny
data(anolis.tree)
anolis.tree$node.label <- NULL
set.seed(3492)
lm.data <- transformPhylo.sim(phy=anolis.tree, n=2, model="bm")
dat <- data.frame(x = lm.data[,1], y = lm.data[,2], names = anolis.tree$tip, row.names = anolis.tree$tip)
# pgls from CAPER with matrix inversion
library(caper)
comp.dat <- comparative.data(anolis.tree, dat, names)
time.now <- Sys.time()
matrix.inv.caper <- pgls( y ~ x, data = comp.dat, lambda="ML")
pgls.time <- Sys.time() - time.now
pgls.time
time.now <- Sys.time()
picModel <- pic.pgls(formula=y ~  x, phy=anolis.tree, y = dat, lambda="ML", return.intercept.stat=FALSE)
pic.time <- Sys.time() - time.now
pic.time
```

The results are identical between the two methods

```{r}
# from caper
summary(matrix.inv.caper)
# from MOTMOT
picModel
```

**References**

* Blomberg SP, Garland T, and Ives AR. 2003. Testing for phylogenetic signal in comparative data: behavorial traits more labile. *Evolution* 57, 717–45.
* Butler MA, and King AA. 2004. Phylogenetic comparative analysis: a modeling approach for adaptive evolution. *The American Naturalist* 164, 683-695.
* Cavalli‐Sforza, LL, and Edwards AWF. 1967 Phylogenetic analysis: models and estimation procedures. *Evolution* 21, 550-570.
* Clarke M, Thomas GH, and Freckleton RP. 2017. Trait evolution in adaptive radiations: modeling and measuring interspecific competition on phylogenies. *The American Naturalist* 189, 121-137.
* Cooper N, Thomas GH, Venditti C, Meade A, & Freckleton RP. 2016. A cautionary note on the use of Ornstein Uhlenbeck models in macroevolutionary studies. *Biological Journal of the Linnean Society* 118, 64-77.
* Felsenstein J. 1973. Maximum-likelihood estimation of evolutionary trees from continuous characters.]*American journal of human genetics* 25, 471.
* Felsenstein J. 1985. Phylogenies and the comparative method.*The American Naturalist* 125, 1-15.
* Freckleton RP. 2012. Fast likelihood calculations for comparative analyses. *Methods in Ecology and Evolution* 3, 940-947.
* Hansen TF, 1997. Stabilizing selection and the comparative analysis of adaptation. *Evolution* 51, 1341-1351.
* Harmon LJ, *et al.* 2010. Early bursts of body size and shape evolution are rare in comparative data. *Evolution* 64, 2385–96.
* Ingram T. 2011. Speciation along a depth gradient in a marine adaptive radiation. *Proceedings of the Royal Society of London B: Biological Sciences* 278, 613-618.
* Ingram T *et al*. 2016. Comparative tests of the role of dewlap size in *Anolis* lizard speciation. *Proceedings of the Royal Society of London B: Biological Sciences*, 283, 20162199.
* O’Meara BC, Ané C, Sanderson MJ, and Wainwright PC. 2006. Testing for different rates of continuous trait evolution using likelihood. *Evolution* 60, 922–33.
* Orme D, Freckleton RP, Thomas GH, Petzoldt T, Fritz S, Isaac N, and Pearse W. 2018. caper: Comparative Analyses of Phylogenetics and Evolution in R. R package version 1.0.1.
* Paradis E, Schliep K, and Schwartz R. 2018. ape 5.0: an environment for modern phylogenetics and evolutionary analyses in R. *Bioinformatics*.
* Pagel, M. Inferring evolutionary processes from phylogenies. 1997. *Zoologica Scripta* 26, 331-348.
* Pagel, M. 1999. Inferring the historical patterns of biological evolution. *Nature* 401, 877.
* Puttick, MN. 2018. Mixed evidence for early bursts of morphological evolution in extant clades. *Journal of Evolutionary Biology* 31, 502-515.
* Slater GJ. 2013. Phylogenetic evidence for a shift in the mode of mammalian body size evolution at the Cretaceous‐Palaeogene boundary. Methods in Ecology and Evolution, 4, 734-744.
* Thomas GH, and Freckleton RP. 2012. MOTMOT: Models of trait macroevolution on trees. *Methods in Ecology and Evolution* 3, 145–51.
* Thomas GH, Freckleton RP, and Székely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. *Proceedings of the Royal Society B: Biological Sciences* 273, 1619–24.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pic.pgls.R
\name{pic.pgls}
\alias{pic.pgls}
\title{Fast PLGS estimation based on contrasts}
\usage{
pic.pgls(
  formula,
  phy,
  y,
  lambda = "ML",
  return.intercept.stat = FALSE,
  meserr = NULL
)
}
\arguments{
\item{formula}{A model formula with continuous variables only}

\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{y}{A matrix of trait values with rownames corresponding to the phy tip labels, and column names corresponding to the formula variable names}

\item{lambda}{Default is "ML" meaning the phylogenetic signal of the response variable will be estimated using restricted Maximum likelihood. If a numeric value between 0-1 is provided this will be used in the calculation of regression coefficients}

\item{return.intercept.stat}{Logical. If \code{TRUE} the standard errors, t value, and p value of the estimated Intercept is provided for comparison with output from \code{\link[caper]{pgls}} from \pkg{caper} etc.,. Default is \code{FALSE} as this slows the function as it involves constructing and calculating the inverse of the phy variance-covariance matrix, and based on contrasts the design matrix column of ones would have zero contrasts.}

\item{meserr}{A vector (or matrix) of measurement error for each tip. This is only applicable to univariate analyses.}
}
\value{
A list containing the model, model summary, intercept, estimate of Lambda, model log-Likelihood, model AIC
}
\description{
Estimates regression parameters for a phylogenetic generalised least-squares analysis using the fast constrasts method (Felsenstein 1973; 1985; Freckleton 2012). This implementation is applicable for continuous traits only and not factors
}
\examples{
# Data and phylogeny
data(anolis.tree)
anolis.tree$node.label <- NULL
lm.data <- transformPhylo.sim(phy=anolis.tree, n=2, model="bm")
dat <- data.frame(x = lm.data[,1], y = lm.data[,2],
names = anolis.tree$tip, row.names = anolis.tree$tip)
picModel <- pic.pgls(formula=y ~  x,
phy=anolis.tree, y = dat, lambda=1,
return.intercept.stat=FALSE)
}
\seealso{
\code{\link[caper]{pgls}}
}
\author{
Mark N Puttick and Rob Freckleton
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RData_finch_sympatric.R
\docType{data}
\name{sympatric.data}
\alias{sympatric.data}
\title{Pairwise sympatry matrix for finch species}
\format{
A numeric matrix with dimensions 13 x 13. Row and column names are species labels corresponding to `finch.data` and `finch.tree`. Entries are integers (typically 1 for sympatry, 0 for absence of sympatry).
}
\usage{
data(finches)
}
\description{
A 13 by 13 numeric matrix indicating sympatry between finch species included in this package.
}
\details{
This matrix provides pairwise sympatry information (presence/absence) among the finch species supplied with the package and can be used with functions that require sympatry/allopatry matrices.
}
\examples{
data(finches)
head(sympatric.data)
}
\references{
Clarke M, Thomas GH, Freckleton RP. 2017. Trait evolution in adaptive radiations: modelling and measuring interspecific competition on phylogenies. The American Naturalist. 189, 121-137.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phyloMean.R
\name{phyloMean}
\alias{phyloMean}
\title{Calculation of phylogenetically corrected mean.}
\usage{
phyloMean(
  rateData,
  rate = NULL,
  common.mean = FALSE,
  lambda.est = TRUE,
  lambda = 1,
  meserr = FALSE
)
}
\arguments{
\item{rateData}{an object of class \code{rateData}}

\item{rate}{a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal.}

\item{common.mean}{a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=FALSE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates.}

\item{lambda.est}{Logical. Fit Pagel's lambda.}

\item{lambda}{Numeric value for lambda from 0-1.}

\item{meserr}{Logical. Include measurement error.}
}
\value{
mu phylogenetically corrected mean
}
\description{
This function calculates the phylogenetic mean of the data given the tree and model of evolution
}
\note{
The means are output as treatment contrasts.
}
\examples{
 ## Read in phylogeny and data from Thomas et al. (2009)
data(anolis.tree)
data(anolis.data)

## Convert data to class rateData with a rateMatrix object as input
anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)

anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)

# A model with a different rate in each of the four groups. The 'fixed' command is used to determine
# whether a particular rate is to be constrained or not. Use '1' to fix a group and 'FALSE' to show
# that the parameter is not fixed and should be estimated. The values should be entered in the same 
# order as the ranking of the groups. That is, group 0 (small islands) takes position one in the 
# fixed vector, group 1 (large island trunk crown and trunk ground) takes position 2 and so on. 
# The default is to allow each group to take a different mean. 

phyloMean(anolis.rateData, rate=c(1,1,1,1), common.mean=FALSE)
# common mean for all groups
phyloMean(anolis.rateData, rate=c(1,1,1,1), common.mean=TRUE)
}
\references{
Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.

Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\author{
Gavin Thomas, Rob Freckleton
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RDataTree.R
\docType{data}
\name{anolis.tree}
\alias{anolis.tree}
\title{Anolis phylogeny}
\format{
An object of class \code{"phylo"}.
}
\usage{
data(anolis.tree)
}
\description{
Data on anolis phylogeny from Thomas et al. 2009
}
\examples{
data(anolis.tree)
anolis.tree
}
\references{
Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformPhylo.MCMC.R
\name{transformPhylo.MCMC}
\alias{transformPhylo.MCMC}
\title{Bayesian MCMC for models of trait evolution}
\usage{
transformPhylo.MCMC(
  y,
  phy,
  model,
  mcmc.iteration = 1000,
  burn.in = 0.1,
  hiddenSpeciation = FALSE,
  full.phy = NULL,
  lowerBound = NULL,
  upperBound = NULL,
  useMean = FALSE,
  random.start = TRUE,
  meserr = NULL,
  covPIC = TRUE,
  lambdaEst = FALSE,
  nodeIDs = NULL,
  branchLabels = NULL,
  acdcScalar = FALSE,
  sample.every = 10
)
}
\arguments{
\item{y}{A matrix of trait values.}

\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{model}{The model of trait evolution (see details).}

\item{mcmc.iteration}{Integer - the number of generations for which to run the MCMC chain}

\item{burn.in}{The proportion of the chain (as given by mcmc.iteration) which to discard as 'burn-in'}

\item{hiddenSpeciation}{Logical. If TRUE the psi model will include nodes that are on the 'full.phy' but not the tree pruned of trait data}

\item{full.phy}{The full phylogeny containing the species that do not contain trait data so are not included in 'phy'}

\item{lowerBound}{Minimum value for parameter estimates}

\item{upperBound}{Maximum value for parameter estimates}

\item{useMean}{Logical. Use the branch-based estimates of extinction of mean (TRUE, default) for the "psi" and "multispi" models only applicable if "hiddenSpeciation" = TRUE}

\item{random.start}{Use a random starting value for the MCMC run (TRUE), or use the environment set.seed() value}

\item{meserr}{A vector (or matrix) of measurement error for each tip. This is only applicable to univariate analyses}

\item{covPIC}{Logical. For multivariate analyses, allow for co-variance between traits rates (TRUE) or no covariance in trait rates (FALSE). If FALSE, only the trait variances not co-variances are used.}

\item{lambdaEst}{Logical.  Estimate lambda alongside parameter estimates to reduce data noise. Only applicable for models "kappa", "delta", "OU", "psi", "multispi", and "ACDC". Default=FALSE.}

\item{nodeIDs}{Integer - ancestral nodes of clades applicable to rate heterogenous and nested models of evolution (see details)}

\item{branchLabels}{Branches on which different psi parameters are estimated in the "multipsi" model}

\item{acdcScalar}{Logical.  For nested EB rate model, simultaneously estimated a rate scalar alongside EB model. Default=FALSE.}

\item{sample.every}{Number specifying the every nth that is sampled in the MCMC chain (default = 1).}
}
\value{
median The median estimate of the posterior for the parameter

95.HPD The 95 percent Highest Posterior Density for the parameter

ESS Effective Sample Size for the posterior

acceptance.rate The ratio for which new proposals were accepted during the MCMC chain

mcmc.chain Full MCMC chain containing all iterations (including burn-in)
}
\description{
Fits Bayesian models for various models of continuous character evolution using a Metropolis-Hastings Markov Chain Monte Carlo (MCMC) approach
}
\details{
The method estimates posterior probabilities using a Metropolis-Hastings MCMC approach that places a prior bounded uniform distribution on all parameters with an independence sampler. These prior distributions can be altered by changing the upperBound and lowerBound arguments. The MCMC model will estimate the posterior probability for the following models:
\describe{
\item{model="kappa"}{fits Pagel's kappa by raising all branch lengths to the power kappa. As kappa approaches zero, trait change becomes focused at branching events. For complete phylogenies, if kappa approaches zero this infers speciational trait change. Default bounds from ~0 - 1.}
\item{model="lambda"}{fits Pagel's lambda to estimate phylogenetic signal by multiplying all internal branches of the tree by lambda, leaving tip branches as their original length (root to tip distances are unchanged). Default bounds from ~0 - 1.}
\item{model="delta"}{fits Pagel's delta by raising all node depths to the power delta. If delta <1, trait evolution is concentrated early in the tree whereas if delta >1 trait evolution is concentrated towards the tips. Values of delta above one can be difficult to fit reliably. Default bounds from ~0 - 5.}
\item{model="OU"}{fits an Ornstein-Uhlenbeck model - a random walk with a central tendency proportional to alpha. High values of alpha can be interpreted as evidence of evolutionary constraints, stabilising selection or weak phylogenetic signal. It is often difficult to distinguish among these possibilities. Default bounds from ~0 - 10.}
\item{model="psi"}{fits a acceleration-deacceleration model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate (Ingram 2010).}
\item{model="ACDC"}{fits a model to in which rates can exponentially increased or decrease through time (Blomberg et al. 2003). If the upper bound is < 0, the model is equivalent to the 'Early Burst' model of Harmon et al. 2010. Default rate parameter bounds from ln(1e-10) ~ ln(20) divided by the root age.}
}
}
\examples{
data(anolis.tree)
data(anolis.data)
attach(anolis.data)
male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
sortedData <- sortTraitData(anolis.tree, male.length)
phy <- sortedData$phy
male.length <- sortedData$trait
phy.clade <- extract.clade(phy, 182)
male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, rownames(male.length)),])
## please note, this model will be need to run for longer to achieve convergence
lambda.mcmc <- transformPhylo.MCMC(y=male.length.clade, phy=phy.clade, 
model="lambda", mcmc.iteration=100, burn.in=0.1)
}
\seealso{
\code{\link{transformPhylo.ML}}, \code{\link{transformPhylo.ll}}, \code{\link{transformPhylo}}
}
\author{
Mark Puttick, Gavin Thomas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcCutOff.R
\name{calcCutOff}
\alias{calcCutOff}
\title{Calculate multiple-test cut-off}
\usage{
calcCutOff(
  phy,
  n = 1000,
  mc.cores = 1,
  model,
  measure = "AICc",
  alpha.error = 0.05,
  ...
)
}
\arguments{
\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{n}{Number of simulations}

\item{mc.cores}{Number of cores for parallel processing for linux-type systems (not applicable to Windows)}

\item{model}{Evolutionary model, typically "tm1", "tm2", or "timeSlice", which is used to test the empirical data}

\item{measure}{Measure used to summarise the model. One of "lnL" (log-likelihood), "AIC", or "AICc"}

\item{alpha.error}{Target for the desired type-one error rate for the model (default 0.05)}

\item{...}{Arguments to be passed to \code{\link{transformPhylo.ML}} which should be identical to the model applied to empirical data}
}
\value{
The cut-off requred to produce an type-one error rate equal to quantile.cut.off (default = 0.05) when data are simulated under Brownian motion, and these data are analysed under the appropriate model.
}
\description{
Calculate the log-likelihood, AIC, or AICc cut-off necessary for type-one error to reach acceptable levels
}
\examples{
data(anolis.tree)
set.seed(393)
# calculated necessary AICc cut-off to reduce type-one error to 5\% 
# for a timeSlice model with a split at 30Ma (only 5 simulations used,
# it's recommend to use 1000 for analyses)
calcCutOff(anolis.tree, n=5, model="timeSlice", splitTime=30)
}
\seealso{
\code{\link{transformPhylo.ML}}, \code{\link{transformPhylo.ll}}, \code{\link{transformPhylo}}, \code{\link{transformPhylo.MCMC}}
}
\author{
Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likRatePhylo.R
\name{likRatePhylo}
\alias{likRatePhylo}
\title{Log-likelihood rate estimation for traits and phylogenies}
\usage{
likRatePhylo(
  rateData,
  rate = NULL,
  common.mean = FALSE,
  lambda.est = TRUE,
  lambda = 1,
  meserr = FALSE,
  sigmaScale = NULL
)
}
\arguments{
\item{rateData}{an object of class \code{rateData}}

\item{rate}{a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal.}

\item{common.mean}{a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=FALSE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates.#'}

\item{lambda.est}{Logical. Fit Pagel's lambda.}

\item{lambda}{Logical. Numeric value for lambda from 0-1.}

\item{meserr}{Logical. Logical. Include measurement error.}

\item{sigmaScale}{Logical. Scalar for measurement error relative to tree.}
}
\value{
ll log-likelihood of the model

mu phylogenetically corrected mean(s)

s2 Brownian variance
}
\description{
This function calculates the log-likelihood, phylogenetic mean, and Brownian variance for a trait and a phylogeny transformed according to variation in relative rates.
}
\note{
The means are output as treatment contrasts.
}
\examples{
data(anolis.tree)
data(anolis.data)

## Convert data to class rateData with a rateMatrix object as input
anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)

anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)

## Calculate phylogenetic mean, variance, log likelihood for a model where the first 

# mean only
phyloMean(rateData=anolis.rateData, rate = c(1,2,0.1,1), common.mean = FALSE)

# variance only
phyloVar(rateData=anolis.rateData, rate = c(1,2,0.1,1), common.mean = FALSE) 

# mean, variance and log-likelihood
likRatePhylo(rateData=anolis.rateData, rate = c(1,2,0.1,1), common.mean = FALSE) 
}
\references{
Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.

Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\author{
Gavin Thomas, Rob Freckleton
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contemporaryPhy.R
\name{contemporaryPhy}
\alias{contemporaryPhy}
\title{prune tree and data to lineages present in a time bin in the past}
\usage{
contemporaryPhy(
  phy,
  maxBin,
  minBin,
  reScale = 0,
  allTraits,
  closest.min = TRUE,
  traits.from.tip = TRUE
)
}
\arguments{
\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{maxBin}{the start age (older time in myr from present) of the time bin in which lineages are preserved}

\item{minBin}{the final age (younger time in myr from present) of the time bin in which lineages are preserved}

\item{reScale}{if the most recent tip is not from the present, the age needed to add so the phylogeny is in 'real time'}

\item{allTraits}{a vector of trait data corresponding to the phy$edge object. The trait data represent tip and internal node data for the phylogeny}

\item{closest.min}{Logical. Should new tip values for lineages that span the bin be taken from the node nearest the \code{minBin} age (\code{closest.min=TRUE}, default) or the \code{maxBin} age \code{closest.min=FALSE}}

\item{traits.from.tip}{Logical. Should tip values for pendant edges in the bin be taken from the original tip value or the reconstructed node value (if it is closer than the tip value)}
}
\value{
the pruned phylogeny. The object \code{descendants} refers to the lineages the branch in the time bin gave rise to before it was pruned. If traits are included a vector of trait values representing species at the tips.
}
\description{
the function takes a full tree and returns a pruned phylogeny with only tips and lineages found within a time bin preserved. If trait data are supplied the function will return tip states based either on the original tips found in the bin, or tip states inferred from ancestral states
}
\examples{
## prune a random tree to taxa present between 4 and 2 units before present
# generate tree
set.seed(20)
tree <- rtree(20)
# generate traits
traits <- rnorm(20)
# plot tree and timeframe
plot(tree)
max.age <- nodeTimes(tree)[1,1]
abline(v=max.age - c(4, 2))
# prune tree to timeframe
cont.tree <- contemporaryPhy(phy=tree, maxBin=4, minBin=2, allTraits=traits)
plot(cont.tree$phy)
}
\references{
Puttick, M. N., Kriwet, J., Wen, W., Hu, S., Thomas, G. H., & Benton, M. J. (2017). Body length of bony fishes was not a selective factor during the biggest mass extinction of all time. Palaeontology, 60, 727-741.
}
\author{
Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sortTraitData.R
\name{sortTraitData}
\alias{sortTraitData}
\title{Sort data and remove missing entries for tree and trait data}
\usage{
sortTraitData(
  phy,
  y,
  data.name = NULL,
  log.trait = TRUE,
  pass.ultrametric = FALSE
)
}
\arguments{
\item{phy}{An object of class \code{phylo} or \code{multiPhylo}  (see \pkg{ape})}

\item{y}{A matrix of trait values with taxon names as rownames. Missing values should be NA}

\item{data.name}{If null the first column of y is assummed as the trait, otherwise if y is a matrix with more than one column either the name of the column or the number of the column must be supplied by data.name}

\item{log.trait}{Logical. If \code{TRUE}, data are log-transformed}

\item{pass.ultrametric}{Although trees that are believed to be ultrametric to pass the function \code{is.ultrametric} in \pkg{ape}}
}
\value{
phy Tree with missing data pruned

trait Rearranged data with missing species removed
}
\description{
Plots a phylogeny with lines representing the value of a continuous trait
}
\examples{
data(anolis.tree)
data(anolis.data)
attach(anolis.data)
male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
any(is.na(male.length[,1]))
data.sorted <- sortTraitData(anolis.tree, male.length)
phy <- data.sorted[[1]]
male.length <- data.sorted[[2]]
}
\author{
Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RData_finch_tree.R
\docType{data}
\name{finch.tree}
\alias{finch.tree}
\title{Finch tree}
\format{
An object of class \code{"phylo"}
}
\usage{
data(finches)
}
\description{
Finch data from Clarke et al. 2017
}
\examples{
data(finches)
head(finch.tree)
}
\references{
Clarke M, Thomas GH, Freckleton RP. 2017. Trait evolution in adaptive radiations: modelling and measuring interspecific competition on phylogenies. The American Naturalist. 189, 121-137.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chr.disp.sim.R
\name{chr.disp.sim}
\alias{chr.disp.sim}
\title{Simulate character displacement data}
\usage{
chr.disp.sim(
  phy,
  n.steps = 1000,
  sigma = 1,
  a = 0,
  ntraits = 1,
  sympatry = NA,
  allopatry = NA,
  trait.lim = NA
)
}
\arguments{
\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{n.steps}{Number of time steps the for the simulation (default = 1000 time steps).}

\item{sigma}{The value of Brownian variance in the simulation}

\item{a}{The strength of competition between inter-specific lineages}

\item{ntraits}{Number of traits to be simulated}

\item{sympatry}{an optional matrix giving the time that each pair of species starts to interact}

\item{allopatry}{an optional matrix giving the times when species stop interacting}

\item{trait.lim}{an optional parameter that puts limits on the available trait-space, preventing trait values with magnitude greater than the value of lim}
}
\value{
A list containing the the simulated data (tval) showing the sigma, a, mean gap and gap standard deviation. Additionally, if used, the user input sympatry (symp) and/or allopatry (allo) matrices
}
\description{
Simulates data under a Brownian motion or character displacement model
}
\examples{
## import finch data form Clarke et al. (2017)
data(finches)
emp.tree <- finch.tree
emp.data <- finch.data
## simulate small amount of data 
## (example only - many more datasets are required for accuracy)
sim.data <- chr.disp.sim(emp.tree, n.steps=100,
sigma=1, a=2, ntraits=1, sympatry=NA, allopatry=NA, trait.lim=NA)
}
\references{
Clarke M, Thomas GH, Freckleton RP. 2017. Trait evolution in adaptive radiations: modelling and measuring interspecific competition on phylogenies. The American Naturalist. 189, 121-137.
}
\seealso{
\code{\link{chr.disp.param}}, \code{\link{chr.disp.lrt}}
}
\author{
Magnus Clarke and Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.rateData.R
\name{as.rateData}
\alias{as.rateData}
\title{Conversion among data and phylogeny objects}
\usage{
as.rateData(
  y,
  x,
  rateMatrix = NULL,
  phy = NULL,
  data,
  meserr.col = NULL,
  meserr.propn = NULL,
  log.y = FALSE,
  report_prune = FALSE
)
}
\arguments{
\item{y}{The response variable - typically a continuous trait. Specified as a column name or index}

\item{x}{The explanatory (discrete) variable used to define the hypothesised rate categories. Specified as a column name or index.}

\item{rateMatrix}{A "rateMatrix" object or NULL}

\item{phy}{An object of class "phylo" (see \pkg{ape}).}

\item{data}{A data frame containing (minimally) the x and y variables as columns with species names as rownames.}

\item{meserr.col}{Column name or index containing measurement error for species means.}

\item{meserr.propn}{Single value specifying the proportional measurement to be applied across all species.}

\item{log.y}{Logical, natural log transform response variable.}

\item{report_prune}{Logical. Prints a list of dropped species if TRUE}
}
\value{
rateData An object of class "rateData" which is a list containing the response (y) and explanatory (x) variable along with a list of variance-covaraince matrices.
}
\description{
Function to generate a "rateData" object containing the discrete explanatory variable, continuous response variable and set of variance co-variance matrices. It loads the trait data and removes species with missing data from the data and vcv matrix. 
\code{as.rateData} requires either a set of matrices in rateMatrix format created using \code{as.rateMatrix} or, if no rateMatrix object is input then it requires a phylogeny in "phylo" format. If  a "phylo" object is used  \code{as.rateData} will call \code{as.rateMatrix} internally.
\code{as.rateMatrix} calls the "ape" function \code{vcv.phylo} multiple times and this can be slow for large phylogenies. It will often be more efficient to use \code{as.rateMatrix} first to create a "rateMatrix" object to pass to \code{as.rateData}, particularly if there are many response traits of interest to be fitted to the same phylogeny and set of reconstructed ancestral states.
}
\examples{
## Read in phylogeny and data from Thomas et al. (2009)
data(anolis.tree)
data(anolis.data)

## Convert data to class rateData with a rateMatrix object as input
anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)

anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)  

## Convert data to class rateData with a phylo object as input 
anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
rateMatrix = NULL, phy=anolis.tree, data=anolis.data, log.y=TRUE)
}
\references{
Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\author{
Gavin Thomas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformPhylo.ll.R
\name{transformPhylo.ll}
\alias{transformPhylo.ll}
\title{Log-likelhood for models of trait evolution.}
\usage{
transformPhylo.ll(
  y = NULL,
  phy,
  model = NULL,
  meserr = NULL,
  kappa = NULL,
  lambda = NULL,
  delta = NULL,
  alpha = NULL,
  psi = NULL,
  lambda.sp = NULL,
  nodeIDs = NULL,
  rateType = NULL,
  branchRates = NULL,
  cladeRates = NULL,
  timeRates = NULL,
  splitTime = NULL,
  branchLabels = NULL,
  acdcRate = NULL,
  covPIC = TRUE,
  cophenetic.dist = NULL,
  vcv.matrix = NULL,
  mode.order = NULL,
  mode.param = NULL,
  rate.var = NULL,
  mu = NULL,
  sigma.sq = NULL
)
}
\arguments{
\item{y}{A matrix of trait values.}

\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{model}{The model of trait evolution (see details).}

\item{meserr}{A vector (or matrix) of measurement error for each tip. This is only applicable to univariate analyses.}

\item{kappa}{Value of kappa transform.}

\item{lambda}{Value of lambda transform.}

\item{delta}{Value of delta transform.}

\item{alpha}{Value of alpha (OU) transform.}

\item{psi}{Value of psi transform.}

\item{lambda.sp}{Speciation rate estimate for the tree.}

\item{nodeIDs}{Integer - ancestral nodes of clades.}

\item{rateType}{If model="clade", a vector specifying if rate shift occurs in a clade ("clade") or on the single branch leading to a clade ("branch").}

\item{branchRates}{Numeric vector specifying relative rates for individual branches}

\item{cladeRates}{Numeric vector specifying relative rates for clades or logical to indicate scalar is included in the 'modeslice' model (the scalar value is included in the mode.param argument with the 'modeslice' model).}

\item{timeRates}{The rates (from ancient to recent) for the timeSlice model}

\item{splitTime}{A split time (measured from the present, or most recent species) at which a shift in the rate occurs for the "timeSlice" model}

\item{branchLabels}{Branches on which different psi parameters are estimated in the "multipsi" model.}

\item{acdcRate}{Value of ACDC transform}

\item{covPIC}{Logical. For multivariate analyses, allow for co-variance between traits rates (TRUE) or no covariance in trait rates (FALSE). If FALSE, only the trait variances not co-variances are used.}

\item{cophenetic.dist}{a cophenetic distance matrix showing the absolute distance between taxa - only applicable for OU models run on non-ultrmetric trees. If null will be calculated internally, but supplying the data can speed up run time}

\item{vcv.matrix}{a variance-covariance matrix - only applicable for OU models run on non-ultrmetric trees. If null will be calculated internally, but supplying the data can speed up run time}

\item{mode.order}{The order of modes for the 'modeslice' model. Any combination of 'BM', 'OU', 'acdc', and 'kappa'}

\item{mode.param}{Parameters for the modes of evoluton in the 'modeslice' model}

\item{rate.var}{Allows rate variation in BM modes in the 'modeslice' model}

\item{mu}{Phylogenetic mean estimate, Mainly for internal use when using the variance-covariance method to calculate likelihood for non-ultrametric trees with the OU model}

\item{sigma.sq}{Brownian variance estimate, mainly for internal use when using the variance-covariance method to calculate likelihood for non-ultrametric trees with the OU model}
}
\value{
brownianVariance Brownian variance (or covariance for multiple traits) given the data and phylogeny

logLikelihood The log-likelihood of the model and data
}
\description{
Fits likelihood models for various models of continuous character evolution.
}
\details{
This function fits likelihood models (see below) for continuous character evolution where the parameter values are set a priori. The function returns the log-likihood and the Brownian variance (or variance covariance matrix).
- `model="bm"`: Brownian motion (constant rates random walk).
- `model="kappa"`: fits Pagel's kappa by raising all branch lengths to the power kappa. As kappa approaches zero, trait change becomes focused at branching events. For complete phylogenies, if kappa approaches zero this infers speciational trait change. Default bounds from ~0 - 1.
- `model="lambda"`: fits Pagel's lambda to estimate phylogenetic signal by multiplying all internal branches of the tree by lambda, leaving tip branches as their original length (root to tip distances are unchanged). Default bounds from ~0 - 1.
- `model="delta"`: fits Pagel's delta by raising all node depths to the power delta. If delta <1, trait evolution is concentrated early in the tree whereas if delta >1 trait evolution is concentrated towards the tips. Values of delta above one can be difficult to fit reliably. If a nodeIDs is supplied, the model will fit a delta model nested within a clade, with a BM fit to the rest of the tree. Default bounds from ~0 - 5.
- `model="OU"`: fits an Ornstein-Uhlenbeck model - a random walk with a central tendency proportional to alpha. High values of alpha can be interpreted as evidence of evolutionary constraints, stabilising selection or weak phylogenetic signal. It is often difficult to distinguish among these possibilities. If a nodeIDs is supplied, the model will fit a OU model nested within a clade, with a BM fit to the rest of the tree. For OU models, alternative optimisation are performed with different starting values (1e-8, 0.01, 0.1, 1, 5). Default bounds from ~0 - 10.
- `model="ACDC"`: fits a model to in which rates can exponentially increased or decrease through time (Blomberg et al. 2003). If the upper bound is < 0, the model is equivalent to the 'Early Burst' model of Harmon et al. 2010. If a nodeIDs is supplied, the model will fit a ACDC model nested within a clade, with a BM fit to the rest of the tree. Default rate parameter bounds from ln(1e-10) ~ ln(20) divided by the root age. Note this process starts on the stem branch leading to the MRCA of the common node, unlike the other methods that start at the common node.
- `model="trend"`: fits a model in which the expectated mean change through time is non-zero, signifying a directional evolution to a larger or smaller trait value. This model is only appliacble to non-ultrametric trees.
- `model="psi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate (Ingram 2010). Note that the algorithm will automatically estimate speciation and extinction estimates, and will incorporate estimates of 'hidden' speciation if death estimates are greater than 0.
- `model="multipsi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate but allows seperate values of psi fitted to seperate branches (Ingram 2010; Ingram et al. 2016). Note that the algorithm will automatically estimate speciation and extinction estimates, and will incorporate estimates of 'hidden' speciation if death estimates are greater than 0.
- `model="free"`: fits Mooers et al's free model where each branch has its own rate of trait evolution. This can be a useful exploratory analysis but it is slow due to the number of parameters, particularly for large trees. Default rate parameter bounds from ~0 - 200.
- `model="clade"`: fits a model where particular clades are a priori hypothesised to have different rates of trait evolution (see O'Meara et al. 2006; Thomas et al. 2006, 2009). Clades are specified using nodeIDs and are defined as the mrca node. Default rate parameter bounds from ~0 - 200.
- `model="tm1"`: fits "clade" models without any a priori assertion of the location of phenotypic diversification rate shifts. It uses the same AIC approach as the runMedusa function in the geiger package (runMedusa tests for shifts in the rate of lineage diversification). The algorithm first fits a constant-rate Brownian model to the data, it then works iteratively through the phylogeny fitting a two-rate model at each node in turn. Each two-rate model is compared to the constant rate model and the best two-rate model is retained. Keeping the location of this rate shift intact, it then repeats the procedure for a three-rate model and so on. The maximum number of rate shifts can be specified a priori using nSplits. Limits can be applied to the size (species richness) of clades on which to infer new rate shifts using minCladeSize. This can be useful to enable large trees to be handled but should be used cautiously since specifiying a large minimum clade size may result in biologically interesting nested rate shifts being missed. Equally, very small clade sizes may provide poor estimates of rate that may not be informative. Limits on the search can also be placed using restrictNode. This requires a list where each element of the list is a vector of tip names that define monophyletic groups. Rate shifts will not be searched for within any of the defined groups. Default rate parameter bounds from ~0 - 1000.
- `model="tm2"`: this model is similar to "tm1", however, at each node it assesses the fit of two models. The first model is exactly as per "tm1". The second model infers a rate shift on the single branch descending directly from a node but not on any of the descending branches thereafter. Only the best fitting single-branch or whole clade model is retained for the next iteration. If a single-branch shift is favoured, this infers either that there was a rapid shift in trait value along the stem leading to the crown group, or that the members of the clade have undergone parallel shifts. In either case, this can be considered as a change in mean, though separating a single early shift from a clade-parallel shift is not possible with this method.
- `model="timeSlice"`: A model in which all branch rates change at a time or times set a priori by the user. If splitTime=NULL, all 1 Ma (as defined by test Age) intervals from the root of the tree - 10 and the youngest tip + 10 will be included in the search. The +/- 10 Ma age can be modified using the argument boundaryAge. At each stage the best fitting model will be stored, and the search will continue until n shifts, with n shifts defined by nSplits. If a single value or vector is used for splitTime, only these ages are included in the search.
- `model="modeslice"`: A model in which all branch modes change at a time or times set a priori by the user.
}
\examples{
# Data and phylogeny
data(anolis.tree)
data(anolis.data)

# anolis.data is not matrix and contains missing data so put together matrix of
# relevant traits (here female and male snout vent lengths) and remove species 
# with missing data from the matrix and phylogeny
sorted.traits <- sortTraitData(anolis.tree, anolis.data,
c("Female_SVL", "Male_SVL"), log.trait=TRUE, pass.ultrametric=TRUE)

tree <- sorted.traits$phy
traits <- sorted.traits$trait

# log likelihood of kappa = 0.1 or 1
transformPhylo.ll(traits, phy=tree, model="kappa", kappa=0.1)
transformPhylo.ll(traits, phy=tree, model="kappa", kappa=1)

# log likelihood of lambda = 0.01 or 1
transformPhylo.ll(traits, phy=tree, model="lambda", lambda=0.01)
transformPhylo.ll(traits, phy=tree, model="lambda", lambda=1)

# log likelihood of delta = 1.5 or 1
transformPhylo.ll(traits, phy=tree, model="delta", delta=1.5)
transformPhylo.ll(traits, phy=tree, model="delta", delta=1)

# log likelihood of alpha = 0.001 or 2
transformPhylo.ll(traits, phy=tree, model="OU", alpha=0.001)
transformPhylo.ll(traits, phy=tree, model="OU", alpha=2)

}
\references{
Felsenstein J. 1973. Maximum-likelihood estimation of evolutionary trees from continuous characters. Am. J. Hum. Genet. 25, 471-492.
Felsenstein J. 1985. Phylogenies and the comparative method. American Naturalist 125, 1-15.
Freckleton RP & Jetz W. 2009. Space versus phylogeny: disentangling phylogenetic and spatial signals in comparative data. Proc. Roy. Soc. B 276, 21-30.

Ingram T. 2011. Speciation along a depth gradient in a marine adaptive radiation. Proc. Roy. Soc. B. 278, 613-618.

Ingram T,  Harrison AD, Mahler L, Castaneda MdR, Glor RE, Herrel A, Stuart YE, and Losos JB. 2016. Comparative tests of the role of dewlap size in Anolis lizard speciation. Proc. Roy. Soc. B. 283, 20162199. #' Mooers AO, Vamosi S, & Schluter D. 1999. Using phylogenies to test macroevolutionary models of trait evolution: sexual selection and speciation in Cranes (Gruinae). American Naturalist 154, 249-259.
O'Meara BC, Ane C, Sanderson MJ & Wainwright PC. 2006. Testing for different rates of continuous trait evolution using likelihood. Evolution 60, 922-933
Pagel M. 1997. Inferring evolutionary processes from phylogenies. Zoologica Scripta 26, 331-348.
Pagel M. 1999 Inferring the historical patterns of biological evolution. Nature 401, 877-884.
Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\seealso{
\code{\link{transformPhylo.ML}},\code{\link{transformPhylo.MCMC}}, \code{\link{transformPhylo.sim}}
}
\author{
Gavin Thomas, Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ML.RatePhylo.R
\name{ML.RatePhylo}
\alias{ML.RatePhylo}
\title{Maximum likelihood rate estimation for traits and phylogenies}
\usage{
ML.RatePhylo(
  rateData,
  rate = NULL,
  fixed = NULL,
  pretty = TRUE,
  rateMIN = 0.001,
  rateMAX = 50,
  common.mean = FALSE,
  lambda.est = TRUE,
  est.CI = FALSE,
  meserr = FALSE,
  file = NULL
)
}
\arguments{
\item{rateData}{an object of class \code{rateData}}

\item{rate}{a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal.}

\item{fixed}{A vector stating whether each parameter should be allowed to vary (either \code{FALSE} which results in a start value of 1, or a numeric start value) or should be fixed (\code{TRUE}).}

\item{pretty}{Display the output nicely (\code{pretty=TRUE}) or as a list (\code{pretty=FALSE})}

\item{rateMIN}{Minimum value for the rate parameters.}

\item{rateMAX}{Maximum value for the rate parameters}

\item{common.mean}{a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=TRUE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates.}

\item{lambda.est}{Logical. Estimate Pagel's lambda.}

\item{est.CI}{Logical. Estimate approximate confidence intervals for rate parameters.}

\item{meserr}{Logical. Incorporate measurement error.}

\item{file}{File string for output. Only used if \code{pretty=TRUE}.}
}
\value{
If \code{pretty=FALSE}, returns a list containing:
\describe{
\item{MLRate}{Maximum likelihood estimates of the rate parameters}
\item{Lambda}{Maximum likelihood estimate of lambda}
\item{LCI}{Approximate lower confidence intervals for rate}
\item{UCI}{Approximate upper confidence intervals for rate parameters}
\item{means}{Means for each category}
\item{nParam}{Number of parameters in the model (how many means and rate categories)}
\item{Max.lik}{Maximum (log) likeihood}
\item{AIC}{for maximum likelihood model}
\item{AICc}{for maximum likelihood model}
\item{LambdaSingle}{Maximum likelihood estimate of lambda for the single rate model}
\item{Lik1}{Likelihood of the equivalent single rate model}
\item{Likelihood}{ratio statistic of "Max.lik" vs "Lik1"}
\item{P}{P values for the LR statistic}
\item{df}{Degrees of freedom for the LR statistic}
\item{AIC.rate1}{AIC for single rate model}
\item{AICc.rate1}{AICc for single rate model}
}

If \code{pretty=TRUE}, prints a nice version of the list to screen. If \code{file} is specified the pretty output will be sent to file, not the console.
}
\description{
Full function for maximum likelihood estimation of rate parameters and comparison to a single rate model.
}
\note{
Unlike phyloMean and likRatePhylo (that use treatment contrasts), the means reported here are the actual values
}
\examples{
## Read in phylogeny and data from Thomas et al. (2009)
data(anolis.tree)
data(anolis.data)
## Convert data to class rateData with a rateMatrix object as input
anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph",
data=anolis.data)
anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)
# A model with a different rate in one of the four groups. The 'fixed' command is used to determine
# whether a particular rate is to be constrained or not. Use '1' to fix a group and 'FALSE' to show
# that the parameter is not fixed and should be estimated. The values should be entered in the same 
# order as the ranking of the groups. That is, group 0 (small islands) takes position one in the 
# fixed vector, group 1 (large island trunk crown and trunk ground) takes position 2 and so on. 
# The default is to allow each group to take a different mean. 
ML.RatePhylo(anolis.rateData, fixed=c(1, FALSE, FALSE, FALSE), pretty=TRUE, lambda.est = FALSE)
}
\references{
Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.

Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\author{
Gavin Thomas, Rob Freckleton
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RatePhylo.allCI.R
\name{RatePhylo.allCI}
\alias{RatePhylo.allCI}
\title{Confidence intervals for rate parameters}
\usage{
RatePhylo.allCI(
  rateData,
  MLrate = NULL,
  fixed = NULL,
  rateMIN = 0.001,
  rateMAX = 50,
  common.mean = FALSE,
  lambda.est = TRUE
)
}
\arguments{
\item{rateData}{an object of class \code{rateData}}

\item{MLrate}{a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal. Normally these will be the maximum likelihood rate estimates.}

\item{fixed}{A vector stating whether each parameter should be allowed to vary (either \code{FALSE} which results in a start value of 1, or a numeric start value) or should be fixed (\code{TRUE}).}

\item{rateMIN}{Minimum value for the rate parameters}

\item{rateMAX}{Maximum value for the rate parameters}

\item{common.mean}{a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=FALSE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates.}

\item{lambda.est}{Logical. Estimate Pagel's lambda.}
}
\value{
rateLci Lower confidence interval for rate estimate

rateUci Upper confidence interval for rate estimate
}
\description{
Calculates approximate confidence intervals for all rate parameters. CIs are esimated for one rate parameters while fixing others at a given value (usually the maximum likelihood estimate).

These are reliable (given the asympotic assumptions of the chi-square distribution) if only two groups are being compared but should be regarded only as a rough approximation for =>3 different rate categories. If the rates are correlated the CIs may be underestimated.
}
\examples{
data(anolis.data)
data(anolis.tree)
## Convert data to class rateData with a rateMatrix object as input
anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)

anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)

# A model with a different rate in each of the four groups. The 'fixed' command is used to determine
# whether a particular rate is to be constrained or not. Use '1' to fix a group and 'FALSE' to show 
# that the parameter is not fixed and should be estimated. The values should be entered in the same
# order as the ranking of the groups. That is, group 0 (small islands) takes position one in the 
# fixed vector, group 1 (large island trunk crown and trunk ground) takes position 2 and so on. The 
# default is to allow each group to take a different mean. 

anole.ML <- optim.likRatePhylo(rateData=anolis.rateData, rate=NULL,
fixed=c(FALSE,FALSE,FALSE, FALSE),
common.mean=FALSE, lambda.est=FALSE)

# Confidence intervals for the first two parameters

RatePhylo.allCI(rateData=anolis.rateData, MLrate = anole.ML$MLRate, 
fixed=c(FALSE, TRUE, TRUE, TRUE), rateMIN = 0.001, rateMAX = 50, 
common.mean = FALSE)
}
\references{
Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.

Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\author{
Gavin Thomas, Rob Freckleton
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampleShid.R
\name{sampleShid}
\alias{sampleShid}
\title{Sample hidden speciation events along branches of a tree}
\usage{
sampleShid(phy, la = NULL, mu = NULL, useMean = FALSE)
}
\arguments{
\item{phy}{An object of class \code{phylo} (see \pkg{ape})}

\item{la}{Estimate of the rate of speciation "lambda"}

\item{mu}{Estimate of the rate of extinction "mu"}

\item{useMean}{A logical indicating whether to output the average or expected number of hidden speciation events per branch, which may be non-integer (if \code{TRUE}), or to sample an integer number on each branch from a Poisson distribution (if \code{FALSE}, the default)}
}
\value{
Phylogenetic tree in \code{phylo} format, with an added element \code{Sobs}, a vector of numbers of hidden speciation events per branch, in the same order as the branches in the \code{phylo} object
}
\description{
Uses estimated speciation and extinction rates to sample the number of speciation events 'hidden' by subsequent extinction on each branch of a tree following Bokma (2008). For use with the \code{psi} and \code{multipsi} models.
}
\details{
The expected number of hidden speciation events are calculated for each branch given its start and end times, and estimates of lambda and mu which are assumed to be constant across the tree. To properly account for uncertainty in the effect of extinction on the number of nodes affecting each branch of a tree, it may be appropriate to repeat model-fitting on many realizations of \code{Sobs} on the tree of interest (similar to evaluating phylogenetic uncertainty)
}
\references{
Bokma, F. 2008. Detection of "punctuated equilibrium" by Bayesian estimation of speciation and extinction rates, ancestral character states, and rates of anagenetic and cladogenetic evolution on a molecular phylogeny. Evolution 62: 2718-2726.

Ingram, T. 2011. Speciation along a depth gradient in a marine adaptive radiation. Proc. R. Soc. B 278: 613-618.
}
\seealso{
\code{\link{transformPhylo.ML}}
}
\author{
Travis Ingram
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package-motmot.R
\docType{package}
\name{motmot-package}
\alias{motmot-package}
\alias{motmot}
\title{Models Of Trait Macroevolution On Trees}
\description{
Functions for fitting models of trait evolution on phylogenies for continuous
traits. The majority of functions described in Thomas and Freckleton (2012)
and include functions that allow for tests of variation in the rates of trait
evolution.
}
\references{
Thomas G. H., & Freckleton R. 2012.
 MOTMOT: models of trait macroevolution on trees.
 _Methods in Ecology and Evolution_ 3, 145-151.
 \doi{10.1111/j.2041-210X.2011.00132.x}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://puttickbiology.wordpress.com/motmot/}
}

}
\author{
Mark Puttick <marknputtick@gmail.com>.

Gavin Thomas.

Travis Ingram.

Magnus Clarke.

Rob Freckleton.

David Orme.

Emmanuel Paradis.
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformPhylo.R
\name{transformPhylo}
\alias{transformPhylo}
\title{Phylogenetic tree transformations}
\usage{
transformPhylo(
  phy,
  model = NULL,
  y = NULL,
  meserr = NULL,
  kappa = NULL,
  lambda = NULL,
  delta = NULL,
  alpha = NULL,
  psi = NULL,
  lambda.sp = NULL,
  nodeIDs = NULL,
  rateType = NULL,
  branchRates = NULL,
  cladeRates = NULL,
  splitTime = NULL,
  timeRates = NULL,
  acdcRate = NULL,
  branchLabels = NULL,
  cophenetic.dist = NULL,
  vcv.matrix = NULL,
  mode.order = NULL,
  mode.param = NULL,
  rate.var = NULL
)
}
\arguments{
\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{model}{The model of trait evolution (see details).}

\item{y}{A matrix of trait values.}

\item{meserr}{A vector (or matrix) of measurement error for each tip. This is only applicable to univariate analyses. Largely untested - please use cautiously}

\item{kappa}{Value of kappa transform.}

\item{lambda}{Value of lambda transform.}

\item{delta}{Value of delta transform.}

\item{alpha}{Value of alpha (OU) transform.}

\item{psi}{Value of psi transform.}

\item{lambda.sp}{Estimate of speciation (lambda) for the psi models}

\item{nodeIDs}{Integer - ancestral nodes of clades.}

\item{rateType}{If model="clade", a vector specifying if rate shift occurs in a clade ("clade") or on the single branch leading to a clade ("branch").}

\item{branchRates}{Numeric vector specifying relative rates for individual branches}

\item{cladeRates}{Numeric vector specifying telative rates for clades or logical to indicate scalar is included in the 'modeslice' model (the scalar is included in the mode.param argument with the 'modeslice' model).}

\item{splitTime}{A split time (measured from the present, or most recent species) at which a shift in the rate occurs for the "timeSlice" model}

\item{timeRates}{The rates (from ancient to recent) for the timeSlice model}

\item{acdcRate}{Value of ACDC transform.}

\item{branchLabels}{A vector of length equal to the number of internal branches, signifying the which "multiPsi" class it belongs to}

\item{cophenetic.dist}{a cophenetic distance matrix showing the absolute distance between taxa - only applicable for OU models run on non-ultrmetric trees. If null will be calculated internally, but supplying the data can speed up run time}

\item{vcv.matrix}{a variance-covariance matrix - only applicable for OU models run on non-ultrmetric trees. If null will be calculated internally, but supplying the data can speed up run time}

\item{mode.order}{The order of modes for the 'modeslice' model. Any combination of 'BM', 'OU', 'acdc', and 'kappa'}

\item{mode.param}{Parameters for the modes of evoluton in the 'modeslice' model}

\item{rate.var}{Allows rate variation in BM modes in the 'modeslice' model}
}
\value{
phy  A phylo object with branch lengths scaled according to the given model and parameters
}
\description{
Transforms the branch lengths of a phylo object according to a model of trait evolution (see details).
}
\details{
Transforms the branch lengths of a phylo object according to one of the following models:
- `model="bm"`: Brownian motion (constant rates random walk)
- `model="kappa"`: fits Pagel's kappa by raising all branch lengths to the power kappa. As kappa approaches zero, trait change becomes focused at branching events. For complete phylogenies, if kappa approaches zero this infers speciational trait change.
- `model="lambda"`: fits Pagel's lambda to estimate phylogenetic signal by multiplying all internal branches of the tree by lambda, leaving tip branches as their original length (root to tip distances are unchanged).
- `model="delta"`: fits Pagel's delta by raising all node depths to the power delta. If delta <1, trait evolution is concentrated early in the tree whereas if delta >1 trait evolution is concentrated towards the tips. Values of delta above one can be difficult to fit reliably.
- `model="free"`: fits Mooers et al's (1999) free model where each branch has its own rate of trait evolution. This can be a useful exploratory analysis but it is slow due to the number of parameters, particularly for large trees.
- `model="clade"`: fits a model where particular clades are a priori hypothesised to have different rates of trait evolution (see O'Meara et al. 2006; Thomas et al. 2006, 2009). Clades are specified using nodeIDs and are defined as the mrca node. Unique rates for each clade are specified using cladeRates. rateType specifies whether the rate shift occurs in the stem clade or on the single branch leading to the clade.
- `model="OU"`: fits an Ornstein-Uhlenbeck model - a random walk with a central tendency proportional to alpha. High values of alpha can be interpreted as evidence of evolutionary constraints, stabilising selection or weak phylogenetic signal.
- `model="psi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate (Ingram 2010). Note that 'original nodes' from the full phylogeny can be included as an element on the phylogeny (e.g., phy$orig.node) as well as estimates of 'hidden' speciation (e.g., phy$hidden.speciation) if estimates of extinction (mu) are > 0.
- `model="multiPsi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate but allows seperate values of psi fitted to seperate branches (Ingram 2010; Ingram et al. 2016). Note that 'original nodes' from the full phylogeny can be included as an element on the phylogeny (e.g., phy$orig.node) as well as estimates of 'hidden' speciation (e.g., phy$hidden.speciation) if estimates of extinction (mu) are > 0.
- `model="ACDC"`: fits a model to in which rates can exponentially increased or decrease through time (Blomberg et al. 2003). If the upper bound is < 0, the model is equivalent to the 'Early Burst' model of Harmon et al. 2010. If a nodeIDs is supplied, the model will fit a ACDC model nested within a clade, with a BM fit to the rest of the tree.
- `model="timeSlice"`: A model in which all branch rates change at time(s) in the past.
- `model="modeSlice"`: A model in which all branch modes change at a time or times set a priori by the user.
}
\examples{
data(anolis.tree)
anolis.treeDelta <- transformPhylo(phy=anolis.tree, model="delta", delta=0.5)
}
\references{
Ingram T. 2011. Speciation along a depth gradient in a marine adaptive radiation. Proc. Roy. Soc. B. 278, 613-618.

Ingram T,  Harrison AD, Mahler L, Castaneda MdR, Glor RE, Herrel A, Stuart YE, and Losos JB. Comparative tests of the role of dewlap size in Anolis lizard speciation. Proc. Roy. Soc. B. 283, 20162199.

Mooers AO, Vamosi S, & Schluter D. 1999. Using phylogenies to test macroevolutionary models of trait evolution: sexual selection and speciation in Cranes (Gruinae). American Naturalist 154, 249-259.

O'Meara BC, Ane C, Sanderson MJ & Wainwright PC. 2006. Testing for different rates of continuous trait evolution using likelihood. Evolution 60, 922-933

Pagel M. 1997. Inferring evolutionary processes from phylogenies. Zoologica Scripta 26, 331-348.

Pagel M. 1999 Inferring the historical patterns of biological evolution. Nature 401, 877-884.

Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\seealso{
\code{\link{transformPhylo.ML}}, \code{\link{transformPhylo.ll}}, \code{\link{transformPhylo.MCMC}}
}
\author{
Gavin Thomas, Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.rateMatrix.R
\name{as.rateMatrix}
\alias{as.rateMatrix}
\title{Conversion among data and phylogeny objects}
\usage{
as.rateMatrix(phy, x, data)
}
\arguments{
\item{phy}{An object of class "phylo" (see \pkg{ape}).}

\item{x}{The explanatory (discrete) variable used to define the hypothesised rate categories. Can be specified as a column number or column name}

\item{data}{The explanatory (discrete) variable used to define the hypothesised rate categories. Can be specified as a column number or column name}
}
\value{
rateMatrix An object of class "rateMatrix" - a list of matrices describing the expected variances and covariances of between species. Each matrix refers to the variances and covariances for a given state of x (see Thomas et al. 2006).
}
\description{
Function to generate a \code{rateMatrix} object containing a set of variance covariance matrices. 
Note that \code{as.rateMatrix} calls the CAIC function \code{vcv.array} multiple times and this can be slow for large phylogenies (though faster than using the \pkg{ape} equivalent \code{vcv.phylo}).
}
\examples{
## Read in phylogeny and data from Thomas et al. (2009)
data(anolis.tree)
data(anolis.data)

## Convert data to class rateMatrix
anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)
}
\references{
Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.
}
\author{
Gavin Thomas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phyloCovar.R
\name{phyloCovar}
\alias{phyloCovar}
\title{Calculation of Brownian (co)variance using independent contrasts.}
\usage{
phyloCovar(x, phy, estimator = "unbiased")
}
\arguments{
\item{x}{A continuous trait}

\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{estimator}{Should Brownian variance (or covariance) be based on the unbiased ("unbiased" - default) or maximum likelihood ("ML") estimator.}
}
\value{
brownianVariance Brownian variance (or covariance for multiple traits) given the data and phylogeny
}
\description{
Calculates the Brownian variance (single trait) or variance-covariance matrix (mutliple traits) using phylogenetically independent contrasts.
}
\examples{
data(anolis.tree)
data(anolis.data)
## calculate Brownian variance of female SVL
female.svl <- matrix(anolis.data[,"Female_SVL"], 
dimnames=list(rownames(anolis.data)))
input.data <- sortTraitData(phy=anolis.tree, y=female.svl, log.trait=TRUE)
phyloCovar(x=input.data$trait, phy = input.data$phy)
}
\references{
Felsenstein J. 1973. Maximum-likelihood estimation of evolutionary trees from continuous characters. Am. J. Hum. Genet. 25, 471-492.

Felsenstein J. 1985. Phylogenies and the comparative method. American Naturalist 125, 1-15.

Freckleton RP & Jetz W. 2009. Space versus phylogeny: disentangling phylogenetic and spatial signals in comparative data. Proc. Roy. Soc. B 276, 21-30.
}
\author{
Gavin Thomas, Rob Freckleton
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformRateMatrix.R
\name{transformRateMatrix}
\alias{transformRateMatrix}
\title{Conversion among data and phylogeny objects}
\usage{
transformRateMatrix(rateData, rate = NULL)
}
\arguments{
\item{rateData}{an object of class \code{rateData}}

\item{rate}{a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal.}
}
\value{
retMat Rate-transformed variance covariance matrix
}
\description{
Transforms the expected variance and covariances among species according to hypotheses of rate variation between lineages.
}
\examples{
 data(anolis.tree)
 data(anolis.data)
 ## Convert data to class rateData with a rateMatrix object as input
 anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)
 anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
 rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)
 
 # Tranform the expected variance covariance matrix so that the rates in the first and last 
 # categories are equal (both 1) whereas the rate in the second category is twice as fast (2) and 
 # the rate in the third category is ten times slower.
 
 trans.anolis.rateData <- transformRateMatrix(rateData=anolis.rateData, rate = c(1,2,0.1,1))
}
\references{
Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.

Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\author{
Gavin Thomas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmc.plot.R
\name{mcmc.plot}
\alias{mcmc.plot}
\title{plot the output from transformPhylo.MCMC}
\usage{
mcmc.plot(
  mcmc.input,
  y.limit = NULL,
  x.limit = NULL,
  label.text = NULL,
  cex.axis = 1,
  cex.labels = 0.7,
  col.hist = "green4",
  col.trace = "navy"
)
}
\arguments{
\item{mcmc.input}{an object of class "mcmc.motmot" output from \code{\link{transformPhylo.MCMC}}}

\item{y.limit}{the limits for the y axes for the plots}

\item{x.limit}{the limits for the x axes for the plots}

\item{label.text}{the labels for the two plots defaults to '(a)' etc., for the histogram and '(b)' etc., for the trace plot}

\item{cex.axis}{character expansion for the plot axis labels}

\item{cex.labels}{character expansion for the plot axis names}

\item{col.hist}{colour for the histogram bars}

\item{col.trace}{colour for the trace plot}
}
\value{
Two plots showing the histogram of the estimated parameter value and a trace of the MCMC estimation
}
\description{
Plots a histogram of the estimated parameter and a trace of the results
}
\examples{
library(motmot)
data(anolis.tree)
data(anolis.data)
attach(anolis.data)
male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
sortedData <- sortTraitData(anolis.tree, male.length)
phy <- sortedData$phy
male.length <- sortedData$trait
phy.clade <- extract.clade(phy, 182)
male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, rownames(male.length)),])
## not run
# please note, this model will be need to run for longer to achieve convergence
# lambda.mcmc <- transformPhylo.MCMC(y=male.length.clade, phy=phy.clade, 
# model="lambda", mcmc.iteration=100, burn.in=0.1)
# mcmc.plot(lambda.mcmc)
}
\author{
Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addFossilToPhy.R
\name{addFossilToPhy}
\alias{addFossilToPhy}
\title{add a fossil to an interior branch of a time-scaled phylogeny}
\usage{
addFossilToPhy(
  phy,
  inGroup,
  fossil,
  fossilAge,
  minLength = 0.1,
  maxLength = NULL
)
}
\arguments{
\item{phy}{Input phylogeny an object of class \code{phylo} (see \pkg{ape})}

\item{inGroup}{vector of tip labels defined as the ingroup - the fossil(s) will be placed on the stem branch leading to the 'inGroups' most recent common ancestor}

\item{fossil}{tip labels for the new fossil}

\item{fossilAge}{age of the fossil}

\item{minLength}{minimum length leading to the fossil}

\item{maxLength}{maximum length leading to the fossil. If \code{NULL} (default) then the maximum bound is half the length of the branch leading to the crown node}
}
\value{
the the time-scaled phylogeny with the fossil attached
}
\description{
the function takes a time-scaled phylogeny and adds a fossil at a branch in the past
}
\examples{
data(anolis.tree)
plot(anolis.tree)
nodelabels(214, 214)
# add fossil to node 214
in.groups <- node.descendents(x=214, phy=anolis.tree, tip.labels=TRUE)[[2]]
fossilPhy <- addFossilToPhy(anolis.tree, in.groups, fossil="fakeFossil", fossilAge=60)
plot(fossilPhy)
}
\references{
Puttick, M. N., Kriwet, J., Wen, W., Hu, S., Thomas, G. H., & Benton, M. J. (2017). Body length of bony fishes was not a selective factor during the biggest mass extinction of all time. Palaeontology, 60, 727-741.
}
\author{
Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/node.descendents.R
\name{node.descendents}
\alias{node.descendents}
\title{Identify nodes and tips descended from a node}
\usage{
node.descendents(x, phy, tip.labels = FALSE)
}
\arguments{
\item{x}{A positive integer}

\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{tip.labels}{Logical - output tip.labels?}
}
\value{
Returns a vector of node and tip ids descended from the tip(s) "x". If tip.labels=TRUE then returns a list of node ids and tip labels.
}
\description{
Obtains a vector of the tips and nodes subtending from a node in a phylogeny.
}
\details{
This function is stolen from the clade.members function in the CAIC package but returns both node and tip id's.
}
\note{
\code{as.rateMatrix} calls the CAIC function \code{vcv.array} multiple times and this can be slow for large phylogenies (though faster than using the \pkg{ape} equivalent \code{vcv.phylo}).
}
\author{
Gavin Thomas, David Orme
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blomberg.k.R
\name{blomberg.k}
\alias{blomberg.k}
\title{Blomberg's K
Estimate Blomberg's K (Blomberg et al. 2003)}
\usage{
blomberg.k(phy, y)
}
\arguments{
\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{y}{A matrix of trait values.}
}
\value{
The estimate of the K statistic
}
\description{
Blomberg's K
Estimate Blomberg's K (Blomberg et al. 2003)
}
\references{
Blomberg SP, Garland T, & Ives AR. 2003. Testing for phylogenetic signal in comparative data: behavioral traits are more labile. Evolution 57, 717-745.
}
\seealso{
\code{\link{transformPhylo.ML}}, the Picante package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.traitMedusa.R
\name{summary.traitMedusa}
\alias{summary.traitMedusa}
\title{Identify shifts in the rate of trait diversification}
\usage{
\method{summary}{traitMedusa}(
  object,
  ...,
  cutoff = 4,
  AICc = TRUE,
  lowerBound = 1e-08,
  upperBound = 200,
  print.warnings = FALSE
)
}
\arguments{
\item{object}{Output of a traitMedusa analysis in \code{\link{transformPhylo.ML}}.}

\item{...}{Additional arguments to be passed to \code{\link{summary}}.}

\item{cutoff}{Cutoff value for differences in AIC scores when comparing models. More complex models with an AIC score more than this number of units lower than simpler models are retained (as per runMedusa in the \pkg{geiger}).}

\item{AICc}{If true, AICc is used instead of AIC.}

\item{lowerBound}{Minimum value for parameter estimates.}

\item{upperBound}{Maximum value for parameter estimates.}

\item{print.warnings}{Logical. If TRUE, warnings are issued if confidence intervals fall outside upper or lower bounds.}
}
\value{
ModelFit Summary of the best optimal rate shift model.

Rates Summary of the rate parameters from the best rate shift model.

optimalTree A phylo object with branch lengths scaled relative to rate.
}
\description{
Summarises phenotypic rate variation on phylogenies.
}
\details{
This functions summarises the output of a "medusa" model in transformPhylo.ML (see below). The best overall model is chosen based on AIC (or AICc if AICc=TRUE). The cut-off point for improvement in AIC score between successively more complex models can be defined using cutoff. The default cutoff is 4 but this is somewhat arbitrary and a "good" cut-off may well vary between data sets so it may well be worth exploring different cutoffs.
}
\examples{
data(anolis.tree)
data(anolis.data)
attach(anolis.data)
male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
sortedData <- sortTraitData(anolis.tree, male.length)
phy <- sortedData$phy
male.length <- sortedData$trait
phy.clade <- extract.clade(phy, 182)
male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, rownames(male.length)),])
tm1 <- transformPhylo.ML(male.length.clade, phy=phy.clade, model="tm1", minCladeSize=10, nSplits=1)
tm1_out <- summary(tm1, cutoff=1)
}
\references{
Alfaro ME, Santini F, Brock CD, Alamillo H, Dornburg A, Carnevale G, Rabosky D & Harmon LJ. 2009. Nine exceptional radiations plus high turnover explain species diversity in jawed vertebrates. PNAS 106, 13410-13414.

O'Meara BC, Ane C, Sanderson MJ & Wainwright PC. 2006. Testing for different rates of continuous trait evolution using likelihood. Evolution 60, 922-933

Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\seealso{
\code{\link{transformPhylo.ML}}
}
\author{
Gavin Thomas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chr.disp.lrt.R
\name{chr.disp.lrt}
\alias{chr.disp.lrt}
\title{Character displacement likelihood ratio test}
\usage{
chr.disp.lrt(emp.tree, emp.data, param.out, posteriorSize = 500)
}
\arguments{
\item{emp.tree}{An empirical phylogeny - a object of class \code{phylo} (see \pkg{ape}).}

\item{emp.data}{Continuous trait data matrix}

\item{param.out}{simulated data from the function \code{chr.disp.sim}}

\item{posteriorSize}{The number of samples to use in the likelihood-ratio test}
}
\value{
List containing element of 'estimates' with the estimates of sigma and a, with the Brownian motion (a = 0) summarised in column one and the character displacement (a > 0) in column two. 'likelihood' contains the likelihood of the Brownian motion model and the character displacement model, and the likelihood ratio test estimate. If used, there is an estimate of Blomberg's K for the empirical and simulated data.
}
\description{
Conducts a likelihood ratio test between empirical data (phylogeny and trait data), and simumlations from the function chr.disp.sim using an approximate Bayesian computation (ABC) approach (Clarke et al. 2017)
}
\examples{
## import finch data form Clarke et al. (2017)
data(finches)
## simulate small amount of data 
## (example only - many more datasets are required for accuracy)
param.simulation <- chr.disp.param(finch.tree, n.sim = 100, n.steps=100,
max.sigma = 8, max.a = 8, ntraits=1, 
allopatry=as.matrix(allopatric.data), mc.cores = 1)
chr.disp.lrt(finch.tree, finch.data, param.simulation, 50)
}
\references{
Clarke M, Thomas GH, Freckleton RP. 2017. Trait evolution in adaptive radiations: modelling and measuring interspecific competition on phylogenies. The American Naturalist. 189, 121-137.
}
\seealso{
\code{\link{chr.disp.sim}}, \code{\link{chr.disp.param}}
}
\author{
Magnus Clarke and Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dropTipPartial.R
\name{dropTipPartial}
\alias{dropTipPartial}
\title{Drop tips from a phylogenetic tree while preserving deleted nodes}
\usage{
dropTipPartial(phy, tip)
}
\arguments{
\item{phy}{Phylogenetic tree in \code{phylo} format}

\item{tip}{A vector of mode numeric or character specifying the tips to delete, to be passed to \code{drop.tip}}
}
\value{
Phylogenetic tree in \code{phylo} format, with an added element \code{Shid}, a vector of numbers of observed but "missing" speciation events per branch, in the same order as the branches in the \code{phylo} object
}
\description{
Wrapper for the \code{ape} function \code{drop.tip} that preserves the number of nodes affecting each branch. For use with the \code{psi} and \code{multipsi} models.
}
\examples{
## Read in phylogeny and data from Thomas et al. (2009)
data(anolis.tree)
data(anolis.data)
## identify tips to drop
tips.to.go <- anolis.tree$tip.label[1:30]
dropTipPartial(phy=anolis.tree, tip=tips.to.go)
}
\references{
Ingram, T. 2011. Speciation along a depth gradient in a marine adaptive radiation. Proc. R. Soc. B 278: 613-618.
}
\seealso{
\code{\link{transformPhylo.ML}}
}
\author{
Travis Ingram
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/traitData.plot.R
\name{traitData.plot}
\alias{traitData.plot}
\title{plot a univariate continuous trait data on a phylogeny}
\usage{
traitData.plot(
  y,
  phy,
  col.label = "red",
  col.tree = "black",
  col.hist = "navy",
  cex.plot = 0.7,
  cex.tips = 0.7,
  show.tips = FALSE,
  include.hist = FALSE,
  n.split = 5,
  lwd.traits = 1,
  show.axis = TRUE,
  axis.text = NULL,
  transform.axis.label = NULL,
  at = NULL,
  labels = NULL,
  axis.text.line = 1,
  offset.bars = 1,
  ...
)
}
\arguments{
\item{y}{A matrix of trait values with taxon names as rownames.}

\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{col.label}{colour labels for the traits at the tips and in the histogram}

\item{col.tree}{colour for the edge labels on the tree}

\item{col.hist}{colour of the histogram}

\item{cex.plot}{Numeric. The size of labels for the histogram axis labels}

\item{cex.tips}{Numeric. The size of the phylogeny tip labels}

\item{show.tips}{Logical. If FALSE (default), no tip labels are shown. If TRUE, tip labels are shown}

\item{include.hist}{Logical. Include a histrogram alongside the plot of the tree?}

\item{n.split}{Numeric. The number of splits for the axis labels and shading for the trait values}

\item{lwd.traits}{Line widths of traits shown on the plot}

\item{show.axis}{Logical. If TRUE, shows the axis of trait values on the plot.}

\item{axis.text}{text shown above the trait label axis. If NULL (default), nothing is displayed}

\item{transform.axis.label}{If the data are provided as logarithms the labels for trait axis can be transformed to their original values by calculating the exponential function of the natural (transform.axis.label="exp") or base 10 logarithm. The default (NULL) leaves the labels un-transformed.}

\item{at}{Axis tick point locations for if show.axis is TRUE.}

\item{labels}{Axis labels locations for if show.axis is TRUE.}

\item{axis.text.line}{The location of the label for the trait axis beside the plot.}

\item{offset.bars}{The distance of trait plot lines from the phylogeny.}

\item{...}{further arguments passed to the axis function}
}
\value{
A plot with the trait values shown at the tips, and a histrogram of the trait values
}
\description{
Plots a phylogeny with lines representing the value of a continuous trait
}
\examples{
data(anolis.tree)
data(anolis.data)
attach(anolis.data)
male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
sortedData <- sortTraitData(anolis.tree, male.length)
phy <- sortedData$phy
male.length <- sortedData$trait
traitData.plot(y=male.length, phy)
}
\author{
Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fairProportions.R
\name{fairProportions}
\alias{fairProportions}
\title{Calculate fair proportions phylogenetic diversity metric}
\usage{
fairProportions(phy, nodeCount = FALSE)
}
\arguments{
\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{nodeCount}{Logical - should root to tip node counts be returned (default is \code{FALSE})}
}
\value{
Returns a matrix of fair proportion for all tips in phylogeny and node counts if selected.
}
\description{
Calculate fair proportions phylogenetic diversity metric
Note that \code{as.rateMatrix} calls the CAIC function \code{vcv.array} multiple times and this can be slow for large phylogenies (though faster than using the "ape" equivalent \code{vcv.phylo}).
}
\examples{
data(anolis.tree)

fp <- fairProportions(anolis.tree)
fpNodes <- fairProportions(anolis.tree, nodeCount=TRUE)
}
\references{
Redding, D.W. and Mooers, A.O. (2006). Incorporating evolutionary measures into conservation prioritisation. Conservation Biology, 20, 1670-1678.

Isaac, N.J.B., Turvey, S.T., Collen, B., Waterman, C. and Baillie, J.E.M. (2007). Mammals on the EDGE: conservation priorities based on threat and phylogeny. PLoS ONE, 2, e296.
}
\author{
Gavin Thomas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chr.disp.param.R
\name{chr.disp.param}
\alias{chr.disp.param}
\title{Simulate character displacement data wrapper}
\usage{
chr.disp.param(
  phy,
  n.sim = 100,
  n.steps = 1000,
  max.sigma = 8,
  max.a = 8,
  est.blomberg.k = FALSE,
  ntraits = 1,
  sympatry = NA,
  allopatry = NA,
  trait.lim = NA,
  mc.cores = 1
)
}
\arguments{
\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{n.sim}{Number of replications to simulate data}

\item{n.steps}{Number of time steps the for the simulation (default = 1000 time steps).}

\item{max.sigma}{The maximum value of Brownian variance in the simulation sampled from a U(0, max.sigma) distribution for each iteration}

\item{max.a}{The maximum value of the strength of competition between inter-specific lineages sampled from a U(0, max.a) distribution for each iteration}

\item{est.blomberg.k}{Logical. If TRUE, Blomberg's K is simultaneously estimated}

\item{ntraits}{Number of traits to be simulated}

\item{sympatry}{an optional matrix giving the time that each pair of species starts to interact}

\item{allopatry}{an optional matrix giving the times when species stop interacting}

\item{trait.lim}{an optional parameter that puts limits on the available trait-space, preventing trait values with magnitude greater than the value of lim}

\item{mc.cores}{Numeric. The number of parallel cores to be used in simulations. Only applicable on Linux and Mac systems}
}
\value{
List containing the simulated data 'simulated.param': a matrix with each row represented an iteration, the sigma (Brownian variance) used in the iteration, the 'a' value used in each iteration, the mean and standard deviation between neighbouring trait values. The 'input.arguments' from the model, the 'input.phy' from the model, and the input 'sympatry' and 'allopatry' matrices.
}
\description{
Simulates phylogenetic trait data under a character displacement model (Clarke et al. 2017) in which traits interact inter-specifically, with competition between sympatric lineages driving trait values apart
}
\examples{
## import finch data form Clarke et al. (2017)
data(finches)
## simulate small amount of data 
## (example only - many more datasets are required for accuracy)
param.simulation <- chr.disp.param(finch.tree, n.sim = 3, n.steps=100,  
max.sigma = 8, max.a = 8, ntraits=1, 
allopatry=as.matrix(allopatric.data), mc.cores = 1)
}
\references{
Clarke M, Thomas GH, & Freckleton RP. 2017. Trait evolution in adaptive radiations: modeling and measuring interspecific competition on phylogenies. The American Naturalist 189, 121-137.
}
\seealso{
\code{\link{chr.disp.sim}}, \code{\link{chr.disp.lrt}}
}
\author{
Magnus Clarke and Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformPhylo.ML.R
\name{transformPhylo.ML}
\alias{transformPhylo.ML}
\title{Maximum likelihood for models of trait evoluion}
\usage{
transformPhylo.ML(
  y,
  phy,
  model = NULL,
  modelCIs = TRUE,
  nodeIDs = NULL,
  rateType = NULL,
  minCladeSize = 1,
  nSplits = 2,
  splitTime = NULL,
  boundaryAge = 10,
  testAge = 1,
  restrictNode = NULL,
  lambdaEst = FALSE,
  acdcScalar = FALSE,
  branchLabels = NULL,
  hiddenSpeciation = FALSE,
  full.phy = NULL,
  useMean = FALSE,
  profilePlot = FALSE,
  lowerBound = NULL,
  upperBound = NULL,
  covPIC = TRUE,
  n.cores = 1,
  tol = NULL,
  meserr = NULL,
  controlList = c(fnscale = -1, maxit = 100, factr = 1e-07, pgtol = 0, type = 2, lmm = 5),
  returnPhy = FALSE,
  print.warnings = FALSE,
  mode.order = NULL,
  rate.var = FALSE,
  testShiftTimes = NULL,
  saveAll = TRUE
)
}
\arguments{
\item{y}{A matrix of trait values.}

\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{model}{The model of trait evolution (see details).}

\item{modelCIs}{Logical - estimate confidence intervals for parameter estimates.}

\item{nodeIDs}{Integer - ancestral nodes of clades applicable to rate heterogenous and nested models of evolution (see details)}

\item{rateType}{If model="clade", a vector specifying if rate shift occurs in a clade ("clade") or on the single branch leading to a clade ("branch").}

\item{minCladeSize}{Integer - minimum clade size for inferred rate shift (where model="medusa").}

\item{nSplits}{Integer - number of rate shifts to apply for model="medusa" and "timeSlice".}

\item{splitTime}{A split time (measured from the present, or most recent species) at which a shift in the rate occurs for the "timeSlice" model. If splitTime=NULL, then all ages between 1 million year intervals from the root age - 10 Ma to the present + 10 Ma will be included in the search. The best model will be retained in each search, and will be used as a fixed age in the next search. The model will calculate the likelihood for the number of shifts defined by 'nSplits'.}

\item{boundaryAge}{Only applicable if splitTime=NULL, the age distance from the tips and and youngest tip for which to search for rate shifts. For example, if boundaryAge=10, only ages between the root age - 10 and the latest tip + 10 will be included in the search. If one value is given this will be used for upper and lower ages, but if a vector with two ages is provided the first is used for the upper age boundary and the second for the lower age boundary. Set to zero to allow testing of all ages.}

\item{testAge}{If splitTime=NULL, the interval between ages to be tested. For example, if testAge=1, all 1 Ma ages between the ages defined by 'boundaryAge' will be tested. If you would like to sequentially test specific shift times only, please use the argument "specificShiftTimes".}

\item{restrictNode}{List defining monophyletic groups within which no further rate shifts are searched.}

\item{lambdaEst}{Logical.Estimate lambda alongside parameter estimates to reduce data noise. Only applicable for models "kappa", "delta", "OU", "psi", "multispi", and "ACDC". Default=FALSE.}

\item{acdcScalar}{Logical.For nested EB rate model, simultaneously estimated a rate scalar alongside EB model. Default=FALSE. Only applicable to 'nested mode' and 'modeSlice' models.}

\item{branchLabels}{Branches on which different psi parameters are estimated in the "multipsi" model}

\item{hiddenSpeciation}{Logical. If TRUE the psi model will include nodes that are on the 'full.phy' but not the tree pruned of trait data}

\item{full.phy}{The full phylogeny containing the species that do not contain trait data so are not included in 'phy'}

\item{useMean}{Logical. Use the branch-based estimates of extinction of mean (TRUE, default) for the "psi" and "multispi" models. Only applicable if "hiddenSpeciation"=TRUE. If FALSE, this will generate a single realisation of the numbers of hidden speciation events on each branch}

\item{profilePlot}{Logical. For the single parameter models "kappa", "lambda", "delta", "OU", "psi", "multipsi", and "ACDC", plot the profile of the likelihood.}

\item{lowerBound}{Minimum value for parameter estimates}

\item{upperBound}{Maximum value for parameter estimates}

\item{covPIC}{Logical. For multivariate analyses, allow for co-variance between traits rates (TRUE) or no covariance in trait rates (FALSE). If FALSE, only the trait variances not co-variances are used.}

\item{n.cores}{Integer. Set number of computing cores when running model="traitMedusa" (tm1 and tm2 models)}

\item{tol}{Tolerance (minimum branch length) to exclude branches from trait MEDUSA search. Primarily intended to prevent inference of rate shifts at randomly resolved polytomies.}

\item{meserr}{A vector (or matrix) of measurement error for each tip. This is only applicable to univariate analyses.}

\item{controlList}{List. Specify fine-tune parameters for the optim likelihood search}

\item{returnPhy}{Logical. In TRUE the phylogeny with branch lengths transformed by the ML model parameters is returned}

\item{print.warnings}{Logical. If TRUE, warnings are issued if confidence intervals fall outside upper or lower bounds}

\item{mode.order}{The order of modes for the 'modeslice' model. Any combination of 'BM', 'OU', 'acdc', and 'kappa'}

\item{rate.var}{Allows rate variation in BM modes in the 'modeslice model'}

\item{testShiftTimes}{A vector of times to be used in the search for split times. For use in the timeSlice model when splitTime=NULL. This allows users to specify ages that are test suquentially, rather than all shifts optimised simultaneously as is done when ages are provided in the argument 'splitTime'.}

\item{saveAll}{Logical. If TRUE, saves all the outputs from traitMedusa search in timeSlice (i.e, the log-likelihood and rate estimates for all considered shifts, not just the best fitting shift model). This can be used for model averaging with the function \code{\link{plot.timeSlice.ML}}}
}
\value{
Returns the maximum log-likelihood and parameter estimates (with 95 percent confidence intervals if specified). If model="bm" instead returns the Brownian (co)variance and log-likelihood. Also returned are the root estimate, the AIC, and AICc.

traitMedusaObject A list in which the first element contains a matrix summarising the parameter estimates and node ids, log-likelihoods, number of parameters (k), AIC and AICc for the best one-rate model, two-rate model, three rate model and so on. The second element is a sub-list where the first element contains all two-rate models, the second element contains all three-rate models and so on. This can be summarised using traitMedusaSummary. The third element is the input trait data. The fourth element is the input phylogeny.
}
\description{
Fits likelihood models for various models of continuous character evolution. Model fitting is based on maximum-likelihood evaluation using phylogenetically independent contrasts. This is exactly equivalent to, but substantially faster than, GLS approaches.
}
\details{
This function finds the maximum likelihood parameter values for continuous character evolution. For "kappa", "delta", "OU", "multipsi", and "ACDC" it is possible to fit a 'nested' model of evolution in which the ancestral rate of BM switches to a different node, as specified by nodeIDs or branchLabels for multipsi. The function returns the maximum-likelihood parameter estimates for the following models.
- `model="bm"`: Brownian motion (constant rates random walk).
- `model="kappa"`: fits Pagel's kappa by raising all branch lengths to the power kappa. As kappa approaches zero, trait change becomes focused at branching events. For complete phylogenies, if kappa approaches zero this infers speciational trait change. Default bounds from ~0 - 1.
- `model="lambda"`: fits Pagel's lambda to estimate phylogenetic signal by multiplying all internal branches of the tree by lambda, leaving tip branches as their original length (root to tip distances are unchanged). Default bounds from ~0 - 1.
- `model="delta"`: fits Pagel's delta by raising all node depths to the power delta. If delta <1, trait evolution is concentrated early in the tree whereas if delta >1 trait evolution is concentrated towards the tips. Values of delta above one can be difficult to fit reliably. If a nodeIDs is supplied, the model will fit a delta model nested within a clade, with a BM fit to the rest of the tree. Default bounds from ~0 - 5.
- `model="OU"`: fits an Ornstein-Uhlenbeck model - a random walk with a central tendency proportional to alpha. High values of alpha can be interpreted as evidence of evolutionary constraints, stabilising selection or weak phylogenetic signal. It is often difficult to distinguish among these possibilities. If a nodeIDs is supplied, the model will fit a OU model nested within a clade, with a BM fit to the rest of the tree. For OU models, alternative optimisation are performed with different starting values (1e-8, 0.01, 0.1, 1, 5). Default bounds from ~0 - 10.
- `model="ACDC"`: fits a model to in which rates can exponentially increased or decrease through time (Blomberg et al. 2003). If the upper bound is < 0, the model is equivalent to the 'Early Burst' model of Harmon et al. 2010. If a nodeIDs is supplied, the model will fit a ACDC model nested within a clade, with a BM fit to the rest of the tree. Default rate parameter bounds from ln(1e-10) ~ ln(20) divided by the root age. Note this process starts on the stem branch leading to the MRCA of the common node, unlike the other methods that start at the common node.
- `model="trend"`: fits a model in which the expectated mean change through time is non-zero, signifying a directional evolution to a larger or smaller trait value. This model is only appliacble to non-ultrametric trees.
- `model="psi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate (Ingram 2010). Note that the algorithm will automatically estimate speciation and extinction estimates, and will incorporate estimates of 'hidden' speciation if death estimates are greater than 0.
- `model="multipsi"`: fits a model to assess to the relative contributions of speciation and gradual evolution to a trait's evolutionary rate but allows seperate values of psi fitted to seperate branches (Ingram 2010; Ingram et al. 2016). Note that the algorithm will automatically estimate speciation and extinction estimates, and will incorporate estimates of 'hidden' speciation if death estimates are greater than 0.
- `model="free"`: fits Mooers et al's free model where each branch has its own rate of trait evolution. This can be a useful exploratory analysis but it is slow due to the number of parameters, particularly for large trees. Default rate parameter bounds from ~0 - 200.
- `model="clade"`: fits a model where particular clades are a priori hypothesised to have different rates of trait evolution (see O'Meara et al. 2006; Thomas et al. 2006, 2009). Clades are specified using nodeIDs and are defined as the mrca node. Default rate parameter bounds from ~0 - 200.
- `model="tm1"`: fits "clade" models without any a priori assertion of the location of phenotypic diversification rate shifts. It uses the same AIC approach as the runMedusa function in the geiger package (runMedusa tests for shifts in the rate of lineage diversification). The algorithm first fits a constant-rate Brownian model to the data, it then works iteratively through the phylogeny fitting a two-rate model at each node in turn. Each two-rate model is compared to the constant rate model and the best two-rate model is retained. Keeping the location of this rate shift intact, it then repeats the procedure for a three-rate model and so on. The maximum number of rate shifts can be specified a priori using nSplits. Limits can be applied to the size (species richness) of clades on which to infer new rate shifts using minCladeSize. This can be useful to enable large trees to be handled but should be used cautiously since specifiying a large minimum clade size may result in biologically interesting nested rate shifts being missed. Equally, very small clade sizes may provide poor estimates of rate that may not be informative. Limits on the search can also be placed using restrictNode. This requires a list where each element of the list is a vector of tip names that define monophyletic groups. Rate shifts will not be searched for within any of the defined groups. Default rate parameter bounds from ~0 - 1000.
- `model="tm2"`: this model is similar to "tm1", however, at each node it assesses the fit of two models. The first model is exactly as per "tm1". The second model infers a rate shift on the single branch descending directly from a node but not on any of the descending branches thereafter. Only the best fitting single-branch or whole clade model is retained for the next iteration. If a single-branch shift is favoured, this infers either that there was a rapid shift in trait value along the stem leading to the crown group, or that the members of the clade have undergone parallel shifts. In either case, this can be considered as a change in mean, though separating a single early shift from a clade-parallel shift is not possible with this method.
- `model="timeSlice"`: A model in which all branch rates change at a time or times set a priori by the user. If splitTime=NULL, all 1 Ma (as defined by test Age) intervals from the root of the tree - 10 and the youngest tip + 10 will be included in the search. The +/- 10 Ma age can be modified using the argument boundaryAge. At each stage the best fitting model will be stored, and the search will continue until n shifts, with n shifts defined by nSplits. If a single value or vector is used for splitTime, only these ages are included in the search.
- `model="modeslice"`: A model in which all branch modes change at a time or times set a priori by the user.
}
\note{
Confidence intervals are based on the assumption of an asymptotic Chi-square distribution. For multi-parameter models (e.g. rate shift models with more than two rates) the confidence intervals are approximate and are calculated for each parameter in turn while holding all other parameters at their maximum likelihood value.
}
\examples{
# Data and phylogeny
data(anolis.tree)
data(anolis.data)
sortedData <- sortTraitData(anolis.tree, anolis.data,
data.name="Male_SVL", log.trait=TRUE, pass.ultrametric=TRUE)
phy <- sortedData$phy
male.length <- sortedData$trait
phy.clade <- extract.clade(phy, 182)
male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, rownames(male.length)),])

# Brownian motion model
transformPhylo.ML(male.length.clade , phy=phy.clade, model="bm")

# Delta
transformPhylo.ML(male.length.clade , phy=phy.clade, model="delta", upperBound=2)

# The upper confidence interval for kappa is outside the bounds so try increasing
# the upper bound

transformPhylo.ML(male.length.clade , phy=phy.clade, model="delta", upperBound=5)

# Test for different rates in different clades - here with 2 hypothesised
# unusual rates compared to the background

# This fits the non-censored model of O'Meara et al. (2006)
phy.clade$node.label[which(phy.clade$node.label == "3")] <- 2
transformPhylo.ML(male.length.clade, phy=phy.clade, model="clade", nodeIDs=c(49, 54))

# Identify rate shifts and print and plot results with upto three rate shifts
# and minimum clade size of 20.
anolisSVL_MEDUSA <- transformPhylo.ML(male.length.clade, phy=phy.clade, model="tm1",
minCladeSize=10, nSplits=2)
}
\references{
Alfaro ME, Santini F, Brock CD, Alamillo H, Dornburg A, Carnevale G, Rabosky D & Harmon LJ. 2009. Nine exceptional radiations plus high turnover explain species diversity in jawed vertebrates. PNAS 106, 13410-13414.

Blomberg SP, Garland T & Ives AR 2003. Testing for phylogenetic signal in comparative data: behavioral traits are more labile. Evolution 57, 717-745.

Felsenstein J. 1973. Maximum-likelihood estimation of evolutionary trees from continuous characters. Am. J. Hum. Genet. 25, 471-492.

Felsenstein J. 1985. Phylogenies and the comparative method. American Naturalist 125, 1-15.

Freckleton RP & Jetz W. 2009. Space versus phylogeny: disentangling phylogenetic and spatial signals in comparative data. Proc. Roy. Soc. B 276, 21-30.

Harmon LJ et al. 2010. Early bursts of body size and shape evolution are rare in comparative data. Evolution 57, 717-745.

Ingram T. 2011. Speciation along a depth gradient in a marine adaptive radiation. Proc. Roy. Soc. B. 278, 613-618.

Ingram T,Harrison AD, Mahler L, Castaneda MdR, Glor RE, Herrel A, Stuart YE, and Losos JB. 2016. Comparative tests of the role of dewlap size in Anolis lizard speciation. Proc. Roy. Soc. B. 283, 20162199.

Mooers AO, Vamosi S, & Schluter D. 1999. Using phylogenies to test macroevolutionary models of trait evolution: sexual selection and speciation in Cranes (Gruinae). American Naturalist 154, 249-259.

O'Meara BC, Ane C, Sanderson MJ & Wainwright PC. 2006. Testing for different rates of continuous trait evolution using likelihood. Evolution 60, 922-933

Pagel M. 1997. Inferring evolutionary processes from phylogenies. Zoologica Scripta 26, 331-348.

Pagel M. 1999 Inferring the historical patterns of biological evolution. Nature 401, 877-884.

Pagel M. 1999 Inferring the historical patterns of biological evolution. Nature 401, 877-884.

Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\seealso{
\code{\link{transformPhylo.MCMC}}, \code{\link{transformPhylo}}, \code{\link{transformPhylo.ll}}, \code{\link{blomberg.k}}
}
\author{
Gavin Thomas, Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.timeSlice.ML.R
\name{plot.timeSlice.ML}
\alias{plot.timeSlice.ML}
\title{Identify shifts in the rate of trait diversification through time}
\usage{
\method{plot}{timeSlice.ML}(
  x,
  ...,
  cutoff = 4,
  AICc = TRUE,
  lowerBound = 1e-08,
  upperBound = 1000,
  phylo.plot = TRUE,
  colour.ramp = c("blue", "red"),
  cex.plot = 1,
  model.average = FALSE
)
}
\arguments{
\item{x}{Output of a timeSlice analysis in \code{\link{transformPhylo.ML}}}

\item{...}{Other functions to pass to \code{\link[ape]{plot.phylo}}}

\item{cutoff}{Value for differences in AIC scores when comparing models. More complex models with an AIC score more than this number of units lower than simpler models are retained (as per runMedusa in the geiger package).}

\item{AICc}{If \code{TRUE}, AICc is used instead of AIC.}

\item{lowerBound}{Minimum value for parameter estimates.}

\item{upperBound}{Maximum value for parameter estimates.}

\item{phylo.plot}{Logical. If \code{TRUE}, the phylogeny is plotted}

\item{colour.ramp}{The colours signifying different rates from low (first colour) to high (second colour)}

\item{cex.plot}{Character expansion for the plot of rates through time}

\item{model.average}{Logical only applicable to "timeSlice" models. Will return the model averaged timeSlice for models in which multiple shifts were considered (i.e, when splitTime is NULL). If TRUE, the function returns a plot showing the relative weights of each shift time and the model-averaged rates through time that are weighted by their relative weights. If TRUE, plot.phylo is ignored.}
}
\value{
ModelFit Summary of the best optimal rate shift model or the model average of each split time (if model averaging was used).

Rates Summary of the rate parameters from the best rate shift model or the model averaged rates through time.

optimalTree A phylo object with branch lengths scaled relative to rate and a plot of estimated rates through time with their associated CIs.
}
\description{
Summarises phenotypic rate variation on phylogenies through
}
\details{
This functions summarises the output of a "timeSlice" model in \code{\link{transformPhylo.ML}} (see below). The best overall model is chosen based on AIC (or AICc if AICc=TRUE). The cut-off point for improvement in AIC score between successively more complex models can be defined using cutoff. The default cutoff is 4 but this is somewhat arbitrary and a "good" cut-off may well vary between data sets so it may well be worth exploring different cutoffs.
}
\examples{
data(anolis.tree)
data(anolis.data)
attach(anolis.data)
male.length <- matrix(Male_SVL, dimnames=list(rownames(anolis.data)))
sortedData <- sortTraitData(anolis.tree, male.length)
phy <- sortedData$phy
male.length <- sortedData$trait
phy.clade <- extract.clade(phy, 182)
male.length.clade <- as.matrix(male.length[match(phy.clade$tip.label, 
rownames(male.length)),])
timeSlice.10.ml <- transformPhylo.ML(y=male.length.clade, phy=phy.clade, model="timeSlice", 
splitTime=c(10))
outputSummary <- plot(timeSlice.10.ml, cutoff=0.001, cex=0.5, 
colour.ramp=c("blue", "red"))
}
\references{
To Add
}
\seealso{
\code{\link{transformPhylo.ML}}
}
\author{
Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.traitMedusa.model.R
\name{plot.traitMedusa.model}
\alias{plot.traitMedusa.model}
\title{Tree plotting for rates}
\usage{
\method{plot}{traitMedusa.model}(x, y = NULL, ..., reconType = "rates", palette = "hotspot.colors")
}
\arguments{
\item{x}{Output from \code{\link{summary.traitMedusa}}.}

\item{y}{A matrix of trait values.}

\item{...}{Other functions to pass to \code{\link[ape]{plot.phylo}}}

\item{reconType}{Colour branches according to rate shifts ("rates" - requires traitMedusaObject) or ancestral state reconstruction ("picReconstruction"  - requires x).}

\item{palette}{Defines the colour scheme with four options: hotspot.colors (red to blue), heat.colors (yellow to red), cool.colors (blues), combi.colors (yellows to reds and blues)}
}
\value{
Returns a data frame of colours used in plot along with rate (or ancestral state) range for each colour.
}
\description{
Plots trees with colours based on rates of trait evolution. Also provides simple coloured plotting for trait values using the \code{\link[ape]{ace}} function in the \pkg{ape} library.
}
\examples{
# Data and phylogeny
data(anolis.tree)
data(anolis.data)

# female SVL data
female.svl <- matrix(anolis.data[,"Female_SVL"], 
dimnames=list(rownames(anolis.data)))
input.data <- sortTraitData(phy=anolis.tree, y=female.svl, log.trait=TRUE)

# arbitarily reduce data size for speed in this example
phy.clade <- extract.clade(input.data[[1]], 182)
male.length.clade <- as.matrix(input.data[[2]][match(input.data[[1]]$tip.label, 
rownames(input.data[[2]])),])
# Identify rate shifts and print and plot results with up to one rate shifts 
# and minimum clade size of 10.
anolisSVL_MEDUSA <- transformPhylo.ML(male.length.clade, phy=phy.clade, 
model="tm1",minCladeSize=10, nSplits=1)
anolisSVL_MEDUSA_out <- summary(anolisSVL_MEDUSA, cutoff=1, AICc=FALSE)
colours <- plot(x = anolisSVL_MEDUSA_out,
reconType = "rates", type = "fan", cex=0.6, edge.width=3)
}
\seealso{
\code{\link{transformPhylo.ML}}, \code{\link{summary.traitMedusa}}.
}
\author{
Gavin Thomas, Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optim.likRatePhylo.R
\name{optim.likRatePhylo}
\alias{optim.likRatePhylo}
\title{Maximum likelihood rate estimation for traits and phylogenies}
\usage{
optim.likRatePhylo(
  rateData,
  rate = NULL,
  fixed = NULL,
  rateMIN = 0.001,
  rateMAX = 50,
  common.mean = FALSE,
  lambda.est = TRUE,
  meserr = FALSE
)
}
\arguments{
\item{rateData}{an object of class \code{rateData}}

\item{rate}{a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated. If \code{rate=NULL} then rates are equal.}

\item{fixed}{A vector stating whether each parameter should be allowed to vary (either \code{FALSE} which results in a start value of 1, or a numeric start value) or should be fixed (\code{TRUE}).}

\item{rateMIN}{Minimum value for the rate parameters}

\item{rateMAX}{Maximum value for the rate parameters}

\item{common.mean}{a logical specififying whether each rate category should have its own mean (\code{common.mean=FALSE}) or all categories should have the same mean (\code{common.mean=FALSE}). See Thomas et al. (2009) for a discussion on the impact of assumptions about mean on rate estimates.}

\item{lambda.est}{Logical. Fit Pagel's lambda.}

\item{meserr}{Logical. Include measurement error.}
}
\value{
MLRate Maximum likelihood estimates of the rate parameters

Max.lik  Maximum (log) likeihood

AIC AIC for maximum likelihood model

AICc  AICc for maximum likelihood model

convergence convergence value from \code{optim}

n.parameters Number of parameters in the model (how many means and rate categories)
}
\description{
Function for the maximum likelihood estimation of rate parameters on a trait and phylogeny.
}
\examples{
data(anolis.tree)
data(anolis.data)

## Convert data to class rateData with a rateMatrix object as input
anolis.rateMatrix <- as.rateMatrix(phy=anolis.tree, x="geo_ecomorph", data=anolis.data)

anolis.rateData <- as.rateData(y="Female_SVL", x="geo_ecomorph", 
rateMatrix = anolis.rateMatrix, phy=NULL, data=anolis.data, log.y=TRUE)

# A model with a different rate in each of the four groups. The 'fixed' command is used to determine
# whether a particular rate is to be constrained or not. Use '1' to fix a group and 'FALSE' to show
# that the parameter is not fixed and should be estimated. The values should be entered in the same 
# order as the ranking of the groups. That is, group 0 (small islands) takes position one in the 
# fixed vector, group 1 (large island trunk crown and trunk ground) takes position 2 and so on. 
# The default is to allow each group to take a different mean. 

optim.likRatePhylo(anolis.rateData, rate=c(1,1,1,1), common.mean=TRUE, lambda.est=FALSE)
}
\references{
Thomas GH, Freckleton RP, & Szekely T. 2006. Comparative analyses of the influence of developmental mode on phenotypic diversification rates in shorebirds. Proceedings of the Royal Society B 273, 1619-1624.

Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
}
\author{
Gavin Thomas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nodeTimes.R
\name{nodeTimes}
\alias{nodeTimes}
\title{Get times for nodes and tips}
\usage{
nodeTimes(phy)
}
\arguments{
\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}
}
\value{
Returns a matrix corresponging the phy "edge" matrix showning internal and external node times
}
\description{
Produces branching and tip times for ultrametric and non-ultrametric trees
}
\note{
\code{nodeTimes} is essentially a re-written version of the \pkg{ape} \code{branching.times}.
}
\examples{
## Read in phylogeny from Thomas et al. (2009)
data(anolis.tree)
anolis.node.times <- nodeTimes(phy=anolis.tree)
}
\author{
Mark Puttick, Emmanuel Paradis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformPhylo.sim.R
\name{transformPhylo.sim}
\alias{transformPhylo.sim}
\title{Phylogenetic tree transformations}
\usage{
transformPhylo.sim(
  phy,
  n = 1,
  x = NULL,
  model = NULL,
  returnNodes = FALSE,
  kappa = NULL,
  lambda = NULL,
  delta = NULL,
  alpha = NULL,
  psi = NULL,
  acdcRate = NULL,
  lambda.sp = NULL,
  trend = NULL,
  trend.anc.state = 0,
  nodeIDs = NULL,
  rateType = NULL,
  cladeRates = NULL,
  branchRates = NULL,
  rate = NULL,
  group.means = NULL,
  splitTime = NULL,
  timeRates = NULL,
  branchLabels = NULL,
  rate.var = NULL,
  mode.order = NULL
)
}
\arguments{
\item{phy}{An object of class \code{phylo} (see \pkg{ape}).}

\item{n}{Number of simulations}

\item{x}{Vector, matrix or data.frame (with taxon names as names or rownames) of categories for each species. Only applicable if model="mixedRate"}

\item{model}{The model of trait evolution (see details).}

\item{returnNodes}{Logical. If TRUE, alongside the tip values all node values are returned corresponding to APE's edge.matrix for the tree.}

\item{kappa}{Value of kappa transform.}

\item{lambda}{Value of lambda transform.}

\item{delta}{Value of delta transform.}

\item{alpha}{Value of alpha (OU) transform.}

\item{psi}{Value of psi transform.  Note that 'original nodes' from the full phylogeny can be included as an element on the phylogeny (e.g., phy$orig.node) as well as estimates of 'hidden' speciation (e.g., phy$hidden.speciation) if estimates of extinction (mu) are > 0.}

\item{acdcRate}{Value of ACDC transform.}

\item{lambda.sp}{Estimate of speciation (lambda) for the psi models}

\item{trend}{value of the expectation mean change through time}

\item{trend.anc.state}{the expected ancestal state for the trend model (default is 0)}

\item{nodeIDs}{Integer - ancestral nodes of clades.}

\item{rateType}{If model="clade", a vector specifying if rate shift occurs in a clade ("clade") or on the single branch leading to a clade ("branch").}

\item{cladeRates}{Numeric vector specifying telative rates for clades or logical to indicate scalar is included in the 'modeslice' model (the scalar is included in the mode.param argument with the 'modeslice' model).}

\item{branchRates}{Numeric vector specifying relative rates for individual branches}

\item{rate}{a vector of relative rate parameters. The length of the vector is equal to the number of rates being estimated.}

\item{group.means}{a vector of the relative difference in means between rate categories, expressed as a scalar applied to the expected standard deviation (see Ricklefs 2006)}

\item{splitTime}{A split time (measured from the present, or most recent species) at which a shift in the rate occurs for the "timeSlice" model}

\item{timeRates}{The rates (from ancient to recent) for the timeSlice model}

\item{branchLabels}{Branches on which different psi parameters are estimated in the "multipsi" model.}

\item{rate.var}{Allows rate variation in BM modes in the 'modeslice' model}

\item{mode.order}{The order of modes for the 'modeslice' model. Any combination of 'BM', 'OU', 'acdc', and 'kappa'}
}
\value{
Returns a matrix of simulated dated with taxon names as rownames (number of columns=n).
}
\description{
Simulates trait data on a tree using a specified model of evolution (see details).
}
\examples{
data(anolis.tree)
data(anolis.data)

# Simulate 10 sets of data with kappa=0.1 using the anolis tree
sim.dat1 <- transformPhylo.sim(phy=anolis.tree, n=10, model="kappa", kappa=0.1)

# Simulate 10 sets of data where rates and means differ between to the categories defined by "x"
x <- anolis.data$geo_ecomorph
names(x) <-  rownames(anolis.data)
sim.dat2 <- transformPhylo.sim(phy=anolis.tree, n=10, x=x, model="mixedRate", rate=c(1,1,2,4),
group.means=c(0,5,0,0))
}
\references{
Ricklefs RE. 2006. Time, species, and the generation of trait variation in clades. Systematic Biology 55, 151-159.

Ricklefs RE. 2006. Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030
}
\seealso{
\code{\link{transformPhylo.ML}}, \code{\link{transformPhylo.ll}}, \code{\link{transformPhylo}}, \code{\link{transformPhylo.MCMC}}
}
\author{
Gavin Thomas, Mark Puttick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likTraitPhylo.R
\name{likTraitPhylo}
\alias{likTraitPhylo}
\title{Log-likelihood estimation for traits and phylogenies}
\usage{
likTraitPhylo(y, phy, covPIC = TRUE, brCov = NULL)
}
\arguments{
\item{y}{A matrix of trait data. Rownames must be included and match the taxon names in the phylogeny. Can accept single traits (calculates variance) or multiple traits (calculates variance-covariance matrix).}

\item{phy}{An object of class "phylo" (see \pkg{ape}).}

\item{covPIC}{Logical - allow for covariance between multivariate traits (\code{TRUE}), or assume not covariance (\code{FALSE}). Only applicable to multivariate traits}

\item{brCov}{If \code{NULL} (the default), Brownian covariance is analytically estimated. If a user-supplied numerical value is suppied the likelihood is calculate given this value}
}
\value{
brownianVariance Brownian variance (or covariance for multiple traits) given the data and phylogeny

logLikelihood The log-likelihood of the model and data
}
\description{
This function calculates the log-likelihood and Brownian (co)variance for a trait(s) and a phylogeny using phylogenetically independent contrasts
Note that \code{as.rateMatrix} calls the CAIC function \code{vcv.array} multiple times and this can be slow for large phylogenies (though faster than using the "ape" equivalent \code{vcv.phylo}).
}
\details{
The \code{phylo} object must be rooted and fully dichotomous
}
\examples{
data(anolis.tree)
data(anolis.data)
## calculate Brownian variance log-likelihood of female SVL
female.svl <- matrix(anolis.data[,"Female_SVL"], 
dimnames=list(rownames(anolis.data)))
input.data <- sortTraitData(phy=anolis.tree, y=female.svl, log.trait=TRUE)
likTraitPhylo(phy = input.data$phy, y=input.data$trait)
}
\references{
Felsenstein J. 1973. Maximum-likelihood estimation of evolutionary trees from continuous characters. Am. J. Hum. Genet. 25, 471-492.

Felsenstein J. 1985. Phylogenies and the comparative method. American Naturalist 125, 1-15.

Freckleton RP & Jetz W. 2009. Space versus phylogeny: disentangling phylogenetic and spatial signals in comparative data. Proc. Roy. Soc. B 276, 21-30.
}
\author{
Gavin Thomas, Rob Freckleton
}
